# DeclPacker - Declarative Programming Packer

Prolog placement logic for the [Vodafone open dataset](http://retis.sssup.it/~tommaso/papers/ic2e22.php).
An extended description of this software can be found in the paper published at the CLOSER'24 conference:

- R. Andreoli, S. Forti, L. Pannocchi, T. Cucinotta, A. Brogi. "[A logic programming approach to VM placement](https://retis.santannapisa.it/~tommaso/papers/closer24.php)," 14th International Conference on Cloud Computing and Services Science (CLOSER 2024), May 2-4, 2024, Angers, France. 

## Authors

Stefano Forti, Remo Andreoli, Tommaso Cucinotta, Antonio Brogi

## License

This software is released under a GPLv3 license. Please, see LICENSE for details.

# Steps for Reproducibility

## System Requirements
- A Linux system makes it easier to follow all the steps
- `Python 3.7` (or higher). 
- `SWI-Prolog` (here we assume version 9.0.4).
- `CPLEX ILOG Optimization Studio` full version (here we assume version 22.1.1) with Python APIs installed. CPLEX is free for academics, download [here](https://www.ibm.com/academic/home). A guide on how to download CPLEX [here](https://github.com/IBMDecisionOptimization/docplex-examples/issues/45). The installation procedure will guide you through the steps to install CPLEX on your PC, as well as the Python APIs.

## Python Requirements
```
pip install -r ./requirements.txt
```
Or simply use the venv environment within the repository.

## Usage
1. Make sure you are on the root folder of the project:
```console
$> pwd
/path/to/prologplacer
```
2. Generate rules for a given vBom and host configuration (both are .JSON files).

```console
$> ./generate_rules.py ./vboms/vbom7/vbom7.json ./host_config.json

[...]
Output files in /path/to/prologplacer/vboms/vbom7/*.pl

```
Please do **not** use any other input arguments (`generate_rules.py -h`) for the purpose of the reproducibility artifact, as they are meant for debug only and will conflict with the next steps

3. Collect the suggested number of hosts and solve time for a (set of) generated vbom(s). The output format is csv-compliant:

```console
$> ./prolog-solve.py ./vboms/vbom7/problem_mediaclustervbom_dc*.pl --placertype blindff

problem,num_vms,num_hosts,solve_time(s)
vbom7_mediaclustervbom_dc1_DC1_0.pl,32,0.025
vbom7_mediaclustervbom_dc1_DC1_1.pl,46,0.034
vbom7_mediaclustervbom_dc1_DC1_2.pl,51,0.038
vbom7_mediaclustervbom_dc1_DC1_3.pl,51,0.042
vbom7_mediaclustervbom_dc2_DC2_0.pl,33,0.030
vbom7_mediaclustervbom_dc2_DC2_1.pl,48,0.045
vbom7_mediaclustervbom_dc2_DC2_2.pl,54,0.040
vbom7_mediaclustervbom_dc2_DC2_3.pl,55,0.046
vbom7_mediaclustervbom_dc3_DC3_0.pl,29,0.015
vbom7_mediaclustervbom_dc3_DC3_1.pl,33,0.016
vbom7_mediaclustervbom_dc3_DC3_2.pl,35,0.020
vbom7_mediaclustervbom_dc3_DC3_3.pl,39,0.022
vbom7_mediaclustervbom_dc4_DC4_0.pl,32,0.025
vbom7_mediaclustervbom_dc4_DC4_1.pl,35,0.024
vbom7_mediaclustervbom_dc4_DC4_2.pl,37,0.029
vbom7_mediaclustervbom_dc4_DC4_3.pl,41,0.038
```
Save the output in a csv file (such as `blindff_0.csv`) using bash stdout redirection `>` 

4. If you wish to re-run the MILP-based formulations, use:
```console
$> ./cplex-solve.py ./vboms/vbom7/problem_mediaclustervbom_dc*.lp

[...]
```

5. If you wish to change the upper-bound for the MILP-based formulations, use `change-host-upperbound.py`. For instance, the next command will change the host upperbound for every mod file in vbom7 using the number of suggested hosts in a csv file:
```console
$> python3.7 ./change-host-upperbound.py ./vboms/vbom7/*.mod --ub ./vboms/vbom7/blindff_0.csv
```
The output will be new lp files to be run on CPLEX as show in step 4.
Option `--b` accepts a csv file or a simple scalare value. Make sure that the csv file follows the format of step 3.

# Note for Mac and Power users
The Prolog, Python and Bash scripts assume that you are using the default installation paths (as of Ubuntu 23.10) for each interpreter:
```console
#!/usr/bin/swipl
#!/usr/bin/python
#!/bin/bash
```
If that's not the case, remember to prepend the correct path before every script invocation.


# Related Projects

See also the FogBrainX project at: https://github.com/di-unipi-socc/fogbrainx/blob/main/placers/placerOfficial.pl
