#!/usr/bin/python

from pathlib import Path
import argparse
import pandas as pd

# Parse input
def general_description():
    return """
Convert the execution report's format to the one used in this project, merging each input file in one single csv file.
"""
parser = argparse.ArgumentParser(description="Convert the execution report's csv format to the one used in this project")
parser.add_argument("csvpath", nargs="+", help="Path to *_exe_report.csv file(s) -- accepts wildcars for multi-convert")
parser.add_argument("--no-merge", action="store_true", help="Skip merge step")
args = parser.parse_args()

# Parse
csvpaths = []
for csvpath in args.csvpath:
    csvpath = Path(csvpath)
    assert csvpath.is_file() and csvpath.suffix == ".csv", "Pass a valid .csv file for csvpath"
    csvpaths.append(csvpath)


# Convert
dfs = []
for csvpath in csvpaths:
    df = pd.read_csv(csvpath)

    try:
        df['problem'] = "problem_" + df['cluster'] + "_" + df['site'] + "_" + df["fy"].astype(str) + ".lp"
        df['problem'] = df['problem'].map(lambda p : (csvpath.parent / Path(p)).resolve())

        df.rename(columns={'warm_start_time': 'solve_time(s)',
                           'warm_start_hosts': 'num_hosts'}, inplace=True)

        if 'num_vms' not in df.columns:
            df['num_vms'] = "None"

        # cleanups
        df.drop(['cluster', 'site', 'fy', 'milp_time'], axis=1, inplace=True)
        df = df[['problem', 'num_vms', 'num_hosts', 'solve_time(s)']]

        print(f"Convert output in {csvpath.resolve()}")
        df.to_csv(csvpath, index=False)
    except KeyError as e:
        print(f"KeyError -- Did you convert {csvpath.resolve()} already?? Skipping conversion phase...")
    
    dfs.append(df)

if not args.no_merge and len(dfs) > 1:
    mergepath = csvpath.parent / "lga_wrmst.csv"

    merge_df = pd.concat(dfs, ignore_index=True)
    merge_df.to_csv(mergepath, index=False)
    print(f"Merged outputs in {mergepath.resolve()}")
else:
    print("Merge phase skipped...")

# Unused
def cleanup_whitelines(csvpath):
    with open(csvpath, 'r') as f:
        lines = f.readlines()

    lines = [line.replace(' ', '') for line in lines]

    with open(csvpath, 'w') as f:
        f.writelines(lines)
