#!/usr/bin/python
import docplex.mp.model_reader as mr

from pathlib import Path
import argparse
import json
import os 
# Parse input
parser = argparse.ArgumentParser(description="Run a series of .lp programs, optionally warm-started, using CPLEX")
parser.add_argument("lppath", nargs='+', help="Path to .lp file(s) -- accepts wildcards for multi-runs")
parser.add_argument("--wspath", nargs='+', help="Path to .json file(s) for warm-starting the corresponding .lp file -- accepts wildcards for multi-runs", default=[])
parser.add_argument("--timeout", type=int, help="Set time-limit for Cplex", default=0)
parser.add_argument("--log", action='store_true', help="Show Cplex output log", default=False)
args = parser.parse_args()

lppaths = []
for lppath in args.lppath:
    lppath = Path(lppath)

    if lppath.is_file() and lppath.suffix == ".lp":
        lppaths.append(lppath)
    else:
        print("Not a .lp file -- skipped")

wspaths = []
for wspath in args.wspath:
    wspath = Path(wspath)
    assert wspath.is_file() and wspath.suffix == ".json", "Not a valid .json file"

    with open(wspath, 'r') as f:
        wspaths.append((wspath, json.load(f)))
wspaths += [None] * (len(lppaths) - len(wspaths))


# Solve
cpx = mr.ModelReader()

os.system("sudo ./cpu_setup.sh > /dev/null")
print("problem,warmstart,num_vms,num_hosts,solve_time(s)")
for lppath,ws_tup in zip(lppaths, wspaths):
    mdl = cpx.read_model(str(lppath))
    
    # Optional warm-start
    wspath,ws = None, None
    if ws_tup:
        wspath,ws = ws_tup
        wspath = wspath.resolve()
        warmstart = mdl.new_solution()
        
        warmstart.update(ws)
        mdl.add_mip_start(warmstart)
    
    if args.timeout > 0:
        mdl.set_time_limit(args.timeout)
    sol = mdl.solve(log_output=args.log)
  
    # Ugly way to calculate the total number of vms
    vms = 0
    while mdl.get_var_by_name(f"x({vms},0)"):
        vms += 1

    print(f"{lppath.resolve()},{wspath}", end=",")
    if sol:
        print(f'{vms},{sol.get_value("used_hosts")},{round(mdl.solve_details.time,2)}')
    else:
        print(f"{vms},-1,{round(mdl.solve_details.time,2)}")
os.system("sudo ./cpu_teardown.sh > /dev/null")
