:- consult('config.pl').
:- consult('placer.pl'). 
:- consult('ga.pl').

declPlace(Strategy, Placement, NumberOfHosts) :-
    toPlace(VMList),
    declPlace(Strategy, VMList, Placement, NumberOfHosts).

declPlace(exhaustive, VMList, Placement, Hosts) :-
    splitXconstraints(), cleanupInvalidAffinities(),
    findall((L,P), (place(VMList, [], P), length(P,L)), Ps),
    sort(Ps, [(Hosts,BestP)|_]), humanReadable(BestP, 1, [], Placement).
declPlace(blindff, VMList, Placement, Hosts) :-
    splitXconstraints(), cleanupInvalidAffinities(),
    place(VMList, [], P),
    length(P, Hosts), humanReadable(P, 1, [], Placement).
declPlace(random, VMList, Placement, Hosts):-
    splitXconstraints(), cleanupInvalidAffinities(),
    random_permutation(VMList, VMs), place(VMs, [], P),
    length(P,Hosts), humanReadable(P, 1, [], Placement).
declPlace(genetic, VMList, Placement, Hosts) :-
    splitXconstraints(), cleanupInvalidAffinities(),
    length(VMList,N), assertz(vmNumber(N)),
    geneticAlgorithm(VMList),
    maxFitness(Placement, _, _),
    distinctHosts(Placement,Hosts), 
    retractall(maxFitness(_,_,_)).
declPlace((heuristic, Mode, Order), VMList, Placement, Hosts) :-
    splitXconstraints(), cleanupInvalidAffinities(),
    sortVMs(VMList, Mode, Order, SortedVMs), place(SortedVMs, [], P), 
    length(P,Hosts), humanReadable(P, 1, [], Placement).

toPlace([VM|List]) :- findall(V, vm(V,_,_), [VM|List]).
