

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..38;
# Set of Hosts
set Hs := 0..7;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_1_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_0_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_1_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {16,17}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm7_0_{h in Hs}: sum{v in {20}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm7_1_{h in Hs}: sum{v in {21}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {22}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_1_{h in Hs}: sum{v in {23}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {24..31}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm9_0_{h in Hs}: sum{v in {32}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm10_0_{h in Hs}: sum{v in {33,34}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm11_0_{h in Hs}: sum{v in {35..37}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm12_0_{h in Hs}: sum{v in {38}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 16
5 16
6 16
7 16
8 2
9 2
10 2
11 2
12 2
13 2
14 2
15 2
16 8
17 8
18 2
19 2
20 2
21 2
22 2
23 2
24 16
25 16
26 16
27 16
28 16
29 16
30 16
31 16
32 16
33 8
34 8
35 4
36 4
37 4
38 1;
param: MEM := 
0 16
1 16
2 16
3 16
4 32
5 32
6 32
7 32
8 4
9 4
10 4
11 4
12 4
13 4
14 4
15 4
16 16
17 16
18 2
19 2
20 6
21 6
22 4
23 4
24 50
25 50
26 50
27 50
28 50
29 50
30 50
31 50
32 64
33 32
34 32
35 8
36 8
37 8
38 2;
param: NET := 
0 20
1 20
2 20
3 20
4 100
5 100
6 100
7 100
8 70
9 70
10 70
11 70
12 70
13 70
14 70
15 70
16 80
17 80
18 100
19 100
20 0
21 0
22 0
23 0
24 0
25 0
26 0
27 0
28 0
29 0
30 0
31 0
32 1000
33 1000
34 1000
35 1000
36 1000
37 1000
38 1000;


######################
# SOLVE 
######################
end;
