

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..54;
# Set of Hosts
set Hs := 0..10;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {22..24}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_1_{h in Hs}: sum{v in {25..27}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm6_0_{h in Hs}: sum{v in {28}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm6_1_{h in Hs}: sum{v in {29}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm7_0_{h in Hs}: sum{v in {30,31}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm7_1_{h in Hs}: sum{v in {32,33}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm8_0_{h in Hs}: sum{v in {34}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm8_1_{h in Hs}: sum{v in {35}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm9_0_{h in Hs}: sum{v in {36,37}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm9_1_{h in Hs}: sum{v in {38,39}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm10_0_{h in Hs}: sum{v in {40,41}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm2_0_{h in Hs}: sum{v in {42,43}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm5_0_{h in Hs}: sum{v in {51..54}} x[v, h]<= 1;

#=== Soft Affinity Variables ===
var vnf2_dot_vm3_0{h in Hs}, binary;
s.t. or_sum_vnf2_dot_vm3_0{h in Hs}: vnf2_dot_vm3_0[h] <= sum{v in {8..11}} x[v, h];
s.t. or_ineq_vnf2_dot_vm3_0{v in {8..11}, h in Hs}: vnf2_dot_vm3_0[h] >= x[v, h];
var vnf2_dot_vm3_1{h in Hs}, binary;
s.t. or_sum_vnf2_dot_vm3_1{h in Hs}: vnf2_dot_vm3_1[h] <= sum{v in {12..15}} x[v, h];
s.t. or_ineq_vnf2_dot_vm3_1{v in {12..15}, h in Hs}: vnf2_dot_vm3_1[h] >= x[v, h];
var vnf2_dot_vm4_0{h in Hs}, binary;
s.t. or_sum_vnf2_dot_vm4_0{h in Hs}: vnf2_dot_vm4_0[h] <= sum{v in {16..18}} x[v, h];
s.t. or_ineq_vnf2_dot_vm4_0{v in {16..18}, h in Hs}: vnf2_dot_vm4_0[h] >= x[v, h];
var vnf2_dot_vm4_1{h in Hs}, binary;
s.t. or_sum_vnf2_dot_vm4_1{h in Hs}: vnf2_dot_vm4_1[h] <= sum{v in {19..21}} x[v, h];
s.t. or_ineq_vnf2_dot_vm4_1{v in {19..21}, h in Hs}: vnf2_dot_vm4_1[h] >= x[v, h];
var vnf6_dot_vm3_0{h in Hs}, binary;
s.t. or_sum_vnf6_dot_vm3_0{h in Hs}: vnf6_dot_vm3_0[h] <= sum{v in {44..47}} x[v, h];
s.t. or_ineq_vnf6_dot_vm3_0{v in {44..47}, h in Hs}: vnf6_dot_vm3_0[h] >= x[v, h];
var vnf6_dot_vm4_0{h in Hs}, binary;
s.t. or_sum_vnf6_dot_vm4_0{h in Hs}: vnf6_dot_vm4_0[h] <= sum{v in {48..50}} x[v, h];
s.t. or_ineq_vnf6_dot_vm4_0{v in {48..50}, h in Hs}: vnf6_dot_vm4_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf2_dot_vm3_0[h]) - 1 + (sum{h in Hs} vnf2_dot_vm3_1[h]) - 1 + (sum{h in Hs} vnf2_dot_vm4_0[h]) - 1 + (sum{h in Hs} vnf2_dot_vm4_1[h]) - 1 + (sum{h in Hs} vnf6_dot_vm3_0[h]) - 1 + (sum{h in Hs} vnf6_dot_vm4_0[h]) - 1) / 21;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 16
23 16
24 16
25 16
26 16
27 16
28 2
29 2
30 4
31 4
32 4
33 4
34 4
35 4
36 4
37 4
38 4
39 4
40 4
41 4
42 8
43 8
44 8
45 8
46 8
47 8
48 8
49 8
50 8
51 11
52 11
53 11
54 11;
param: MEM := 
0 6
1 6
2 6
3 6
4 16
5 16
6 16
7 16
8 42
9 42
10 42
11 42
12 42
13 42
14 42
15 42
16 16
17 16
18 16
19 16
20 16
21 16
22 8
23 8
24 8
25 8
26 8
27 8
28 2
29 2
30 8
31 8
32 8
33 8
34 4
35 4
36 8
37 8
38 8
39 8
40 8
41 8
42 16
43 16
44 48
45 48
46 48
47 48
48 16
49 16
50 16
51 3
52 3
53 3
54 3;
param: NET := 
0 0
1 0
2 0
3 0
4 171
5 171
6 171
7 171
8 535
9 535
10 535
11 535
12 535
13 535
14 535
15 535
16 0
17 0
18 0
19 0
20 0
21 0
22 1600
23 1600
24 1600
25 1600
26 1600
27 1600
28 0
29 0
30 0
31 0
32 0
33 0
34 0
35 0
36 0
37 0
38 0
39 0
40 0
41 0
42 675
43 675
44 535
45 535
46 535
47 535
48 0
49 0
50 0
51 1600
52 1600
53 1600
54 1600;


######################
# SOLVE 
######################
end;
