

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..15;
# Set of Hosts
set Hs := 0..1;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_2_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_3_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_2_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_3_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 2
1 2
2 2
3 2
4 2
5 2
6 2
7 2
8 2
9 2
10 2
11 2
12 2
13 2
14 2
15 2;
param: MEM := 
0 6
1 6
2 6
3 6
4 6
5 6
6 6
7 6
8 12
9 12
10 12
11 12
12 12
13 12
14 12
15 12;
param: NET := 
0 155
1 155
2 155
3 155
4 155
5 155
6 155
7 155
8 95
9 95
10 95
11 95
12 95
13 95
14 95
15 95;


######################
# SOLVE 
######################
end;
