

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..82;
# Set of Hosts
set Hs := 0..15;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm5_0_{h in Hs}: sum{v in {14..16}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm6_0_{h in Hs}: sum{v in {17..20}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm7_0_{h in Hs}: sum{v in {21,22}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm8_0_{h in Hs}: sum{v in {23..26}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm1_0_{h in Hs}: sum{v in {27..29}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm5_0_{h in Hs}: sum{v in {36,37}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm5_1_{h in Hs}: sum{v in {38,39}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm9_0_{h in Hs}: sum{v in {40,41}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm9_1_{h in Hs}: sum{v in {42,43}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm10_0_{h in Hs}: sum{v in {44..46}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm10_1_{h in Hs}: sum{v in {47..49}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm11_0_{h in Hs}: sum{v in {50,51}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm12_0_{h in Hs}: sum{v in {52..55}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm13_0_{h in Hs}: sum{v in {56..58}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm1_0_{h in Hs}: sum{v in {59..62}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm4_0_{h in Hs}: sum{v in {65..68}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm5_0_{h in Hs}: sum{v in {69..72}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm14_0_{h in Hs}: sum{v in {73,74}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm14_0_{h in Hs}: sum{v in {75,76}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm15_0_{h in Hs}: sum{v in {77,78}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm15_0_{h in Hs}: sum{v in {79..82}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {2,3}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {2,3}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {4,5}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {4,5}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;

#=== Soft Affinity Variables ===
var vnf5_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm2_0{h in Hs}: vnf5_dot_vm2_0[h] <= sum{v in {10..13}} x[v, h];
s.t. or_ineq_vnf5_dot_vm2_0{v in {10..13}, h in Hs}: vnf5_dot_vm2_0[h] >= x[v, h];
var vnf10_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf10_dot_vm2_0{h in Hs}: vnf10_dot_vm2_0[h] <= sum{v in {30..32}} x[v, h];
s.t. or_ineq_vnf10_dot_vm2_0{v in {30..32}, h in Hs}: vnf10_dot_vm2_0[h] >= x[v, h];
var vnf10_dot_vm4_0{h in Hs}, binary;
s.t. or_sum_vnf10_dot_vm4_0{h in Hs}: vnf10_dot_vm4_0[h] <= sum{v in {33..35}} x[v, h];
s.t. or_ineq_vnf10_dot_vm4_0{v in {33..35}, h in Hs}: vnf10_dot_vm4_0[h] >= x[v, h];
var vnf13_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf13_dot_vm2_0{h in Hs}: vnf13_dot_vm2_0[h] <= sum{v in {63,64}} x[v, h];
s.t. or_ineq_vnf13_dot_vm2_0{v in {63,64}, h in Hs}: vnf13_dot_vm2_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf5_dot_vm2_0[h]) - 1 + (sum{h in Hs} vnf10_dot_vm2_0[h]) - 1 + (sum{h in Hs} vnf10_dot_vm4_0[h]) - 1 + (sum{h in Hs} vnf13_dot_vm2_0[h]) - 1) / 12;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 2.0
7 2.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 11.0
18 11.0
19 11.0
20 11.0
21 8.0
22 8.0
23 8.0
24 8.0
25 8.0
26 8.0
27 7.0
28 7.0
29 7.0
30 4.0
31 4.0
32 4.0
33 4.0
34 4.0
35 4.0
36 4.0
37 4.0
38 4.0
39 4.0
40 4.0
41 4.0
42 4.0
43 4.0
44 6.0
45 6.0
46 6.0
47 6.0
48 6.0
49 6.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 11.0
60 11.0
61 11.0
62 11.0
63 12.0
64 12.0
65 12.0
66 12.0
67 12.0
68 12.0
69 12.0
70 12.0
71 12.0
72 12.0
73 12.0
74 12.0
75 12.0
76 12.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0;
param: MEM := 
0 16.0
1 16.0
2 48.0
3 48.0
4 48.0
5 48.0
6 16.0
7 16.0
8 16.0
9 16.0
10 42.0
11 42.0
12 42.0
13 42.0
14 16.0
15 16.0
16 16.0
17 8.0
18 8.0
19 8.0
20 8.0
21 16.0
22 16.0
23 24.0
24 24.0
25 24.0
26 24.0
27 8.0
28 8.0
29 8.0
30 16.0
31 16.0
32 16.0
33 16.0
34 16.0
35 16.0
36 10.0
37 10.0
38 10.0
39 10.0
40 8.0
41 8.0
42 8.0
43 8.0
44 32.0
45 32.0
46 32.0
47 32.0
48 32.0
49 32.0
50 16.0
51 16.0
52 48.0
53 48.0
54 48.0
55 48.0
56 16.0
57 16.0
58 16.0
59 3.0
60 3.0
61 3.0
62 3.0
63 60.0
64 60.0
65 60.0
66 60.0
67 60.0
68 60.0
69 60.0
70 60.0
71 60.0
72 60.0
73 60.0
74 60.0
75 60.0
76 60.0
77 16.0
78 16.0
79 48.0
80 48.0
81 48.0
82 48.0;
param: NET := 
0 675.0
1 675.0
2 535.0
3 535.0
4 535.0
5 535.0
6 0.0
7 0.0
8 171.0
9 171.0
10 535.0
11 535.0
12 535.0
13 535.0
14 0.0
15 0.0
16 0.0
17 1600.0
18 1600.0
19 1600.0
20 1600.0
21 171.0
22 171.0
23 535.0
24 535.0
25 535.0
26 535.0
27 1600.0
28 1600.0
29 1600.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 675.0
51 675.0
52 535.0
53 535.0
54 535.0
55 535.0
56 0.0
57 0.0
58 0.0
59 1600.0
60 1600.0
61 1600.0
62 1600.0
63 100.0
64 100.0
65 300.0
66 300.0
67 300.0
68 300.0
69 300.0
70 300.0
71 300.0
72 300.0
73 300.0
74 300.0
75 300.0
76 300.0
77 171.0
78 171.0
79 535.0
80 535.0
81 535.0
82 535.0;


######################
# SOLVE 
######################
end;
