

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..84;
# Set of Hosts
set Hs := 0..16;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm1_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {14..16}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {17..19}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {20,21}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm5_0_{h in Hs}: sum{v in {26..28}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm6_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm7_0_{h in Hs}: sum{v in {31,32}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm8_0_{h in Hs}: sum{v in {33..35}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm1_0_{h in Hs}: sum{v in {36,37}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm5_0_{h in Hs}: sum{v in {45..48}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm9_0_{h in Hs}: sum{v in {49,50}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm10_0_{h in Hs}: sum{v in {51..54}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm11_0_{h in Hs}: sum{v in {55..58}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm12_0_{h in Hs}: sum{v in {59,60}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm13_0_{h in Hs}: sum{v in {61,62}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm1_0_{h in Hs}: sum{v in {63,64}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm4_0_{h in Hs}: sum{v in {69..71}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm5_0_{h in Hs}: sum{v in {72..74}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm14_0_{h in Hs}: sum{v in {75,76}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm14_0_{h in Hs}: sum{v in {77,78}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm15_0_{h in Hs}: sum{v in {79..81}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm15_0_{h in Hs}: sum{v in {82..84}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {2,3}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {2,3}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {4,5}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {4,5}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;

#=== Soft Affinity Variables ===
var vnf3_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf3_dot_vm2_0{h in Hs}: vnf3_dot_vm2_0[h] <= sum{v in {10..13}} x[v, h];
s.t. or_ineq_vnf3_dot_vm2_0{v in {10..13}, h in Hs}: vnf3_dot_vm2_0[h] >= x[v, h];
var vnf5_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm2_0{h in Hs}: vnf5_dot_vm2_0[h] <= sum{v in {22..25}} x[v, h];
s.t. or_ineq_vnf5_dot_vm2_0{v in {22..25}, h in Hs}: vnf5_dot_vm2_0[h] >= x[v, h];
var vnf10_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf10_dot_vm2_0{h in Hs}: vnf10_dot_vm2_0[h] <= sum{v in {38..41}} x[v, h];
s.t. or_ineq_vnf10_dot_vm2_0{v in {38..41}, h in Hs}: vnf10_dot_vm2_0[h] >= x[v, h];
var vnf10_dot_vm4_0{h in Hs}, binary;
s.t. or_sum_vnf10_dot_vm4_0{h in Hs}: vnf10_dot_vm4_0[h] <= sum{v in {42..44}} x[v, h];
s.t. or_ineq_vnf10_dot_vm4_0{v in {42..44}, h in Hs}: vnf10_dot_vm4_0[h] >= x[v, h];
var vnf13_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf13_dot_vm2_0{h in Hs}: vnf13_dot_vm2_0[h] <= sum{v in {65..68}} x[v, h];
s.t. or_ineq_vnf13_dot_vm2_0{v in {65..68}, h in Hs}: vnf13_dot_vm2_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf3_dot_vm2_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm2_0[h]) - 1 + (sum{h in Hs} vnf10_dot_vm2_0[h]) - 1 + (sum{h in Hs} vnf10_dot_vm4_0[h]) - 1 + (sum{h in Hs} vnf13_dot_vm2_0[h]) - 1) / 19;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 2
7 2
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 11
18 11
19 11
20 8
21 8
22 8
23 8
24 8
25 8
26 7
27 7
28 7
29 4
30 4
31 4
32 4
33 6
34 6
35 6
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 11
46 11
47 11
48 11
49 12
50 12
51 12
52 12
53 12
54 12
55 12
56 12
57 12
58 12
59 12
60 12
61 12
62 12
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 11
73 11
74 11
75 8
76 8
77 8
78 8
79 4
80 4
81 4
82 4
83 4
84 4;
param: MEM := 
0 16
1 16
2 48
3 48
4 48
5 48
6 16
7 16
8 16
9 16
10 42
11 42
12 42
13 42
14 16
15 16
16 16
17 8
18 8
19 8
20 16
21 16
22 24
23 24
24 24
25 24
26 8
27 8
28 8
29 10
30 10
31 8
32 8
33 32
34 32
35 32
36 16
37 16
38 48
39 48
40 48
41 48
42 16
43 16
44 16
45 3
46 3
47 3
48 3
49 60
50 60
51 60
52 60
53 60
54 60
55 60
56 60
57 60
58 60
59 60
60 60
61 60
62 60
63 16
64 16
65 42
66 42
67 42
68 42
69 16
70 16
71 16
72 8
73 8
74 8
75 6
76 6
77 6
78 6
79 16
80 16
81 16
82 16
83 16
84 16;
param: NET := 
0 675
1 675
2 535
3 535
4 535
5 535
6 0
7 0
8 171
9 171
10 535
11 535
12 535
13 535
14 0
15 0
16 0
17 1600
18 1600
19 1600
20 171
21 171
22 535
23 535
24 535
25 535
26 1600
27 1600
28 1600
29 0
30 0
31 0
32 0
33 0
34 0
35 0
36 675
37 675
38 535
39 535
40 535
41 535
42 0
43 0
44 0
45 1600
46 1600
47 1600
48 1600
49 100
50 100
51 300
52 300
53 300
54 300
55 300
56 300
57 300
58 300
59 300
60 300
61 300
62 300
63 171
64 171
65 535
66 535
67 535
68 535
69 0
70 0
71 0
72 1600
73 1600
74 1600
75 0
76 0
77 0
78 0
79 0
80 0
81 0
82 0
83 0
84 0;


######################
# SOLVE 
######################
end;
