

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..117;
# Set of Hosts
set Hs := 0..20;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {12..15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {16..19}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {20}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {21}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {22,23}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {24,25}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {26,27}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm1_0_{h in Hs}: sum{v in {28,29}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm1_1_{h in Hs}: sum{v in {30,31}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {32,33}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_1_{h in Hs}: sum{v in {34,35}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {36,37}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_1_{h in Hs}: sum{v in {38,39}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {40..43}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_1_{h in Hs}: sum{v in {44..47}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm1_0_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm1_1_{h in Hs}: sum{v in {50,51}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm1_0_{h in Hs}: sum{v in {52,53}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm2_0_{h in Hs}: sum{v in {54..57}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm1_0_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm2_0_{h in Hs}: sum{v in {60..63}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm3_0_{h in Hs}: sum{v in {64}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm4_0_{h in Hs}: sum{v in {65}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm5_0_{h in Hs}: sum{v in {66,67}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm6_0_{h in Hs}: sum{v in {68,69}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm7_0_{h in Hs}: sum{v in {70,71}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm1_0_{h in Hs}: sum{v in {72,73}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm2_0_{h in Hs}: sum{v in {74,75}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm1_0_{h in Hs}: sum{v in {76,77}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm2_0_{h in Hs}: sum{v in {78..81}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm1_0_{h in Hs}: sum{v in {82,83}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm1_1_{h in Hs}: sum{v in {84,85}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm8_0_{h in Hs}: sum{v in {86,87}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm8_1_{h in Hs}: sum{v in {88,89}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm9_0_{h in Hs}: sum{v in {90,91}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm9_1_{h in Hs}: sum{v in {92,93}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm10_0_{h in Hs}: sum{v in {94,95}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm10_1_{h in Hs}: sum{v in {96,97}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm11_0_{h in Hs}: sum{v in {98,99}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm11_1_{h in Hs}: sum{v in {100,101}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm1_0_{h in Hs}: sum{v in {102,103}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm1_1_{h in Hs}: sum{v in {104,105}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm2_0_{h in Hs}: sum{v in {106,107}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm2_1_{h in Hs}: sum{v in {108,109}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm1_0_{h in Hs}: sum{v in {110,111}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm1_1_{h in Hs}: sum{v in {112,113}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm2_0_{h in Hs}: sum{v in {114,115}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm2_1_{h in Hs}: sum{v in {116,117}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 2.0
21 2.0
22 2.0
23 2.0
24 2.0
25 2.0
26 2.0
27 2.0
28 14.0
29 14.0
30 14.0
31 14.0
32 14.0
33 14.0
34 14.0
35 14.0
36 4.0
37 4.0
38 4.0
39 4.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 8.0
47 8.0
48 4.0
49 4.0
50 4.0
51 4.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 2.0
65 2.0
66 2.0
67 2.0
68 2.0
69 2.0
70 2.0
71 2.0
72 14.0
73 14.0
74 14.0
75 14.0
76 4.0
77 4.0
78 8.0
79 8.0
80 8.0
81 8.0
82 4.0
83 4.0
84 4.0
85 4.0
86 10.0
87 10.0
88 10.0
89 10.0
90 10.0
91 10.0
92 10.0
93 10.0
94 10.0
95 10.0
96 10.0
97 10.0
98 14.0
99 14.0
100 14.0
101 14.0
102 6.0
103 6.0
104 6.0
105 6.0
106 10.0
107 10.0
108 10.0
109 10.0
110 6.0
111 6.0
112 6.0
113 6.0
114 10.0
115 10.0
116 10.0
117 10.0;
param: MEM := 
0 54.0
1 54.0
2 54.0
3 54.0
4 54.0
5 54.0
6 54.0
7 54.0
8 20.0
9 20.0
10 20.0
11 20.0
12 54.0
13 54.0
14 54.0
15 54.0
16 54.0
17 54.0
18 54.0
19 54.0
20 8.0
21 8.0
22 12.0
23 12.0
24 16.0
25 16.0
26 8.0
27 8.0
28 40.0
29 40.0
30 40.0
31 40.0
32 8.0
33 8.0
34 8.0
35 8.0
36 12.0
37 12.0
38 12.0
39 12.0
40 32.0
41 32.0
42 32.0
43 32.0
44 32.0
45 32.0
46 32.0
47 32.0
48 16.0
49 16.0
50 16.0
51 16.0
52 54.0
53 54.0
54 54.0
55 54.0
56 54.0
57 54.0
58 20.0
59 20.0
60 54.0
61 54.0
62 54.0
63 54.0
64 8.0
65 8.0
66 12.0
67 12.0
68 16.0
69 16.0
70 8.0
71 8.0
72 40.0
73 40.0
74 8.0
75 8.0
76 12.0
77 12.0
78 32.0
79 32.0
80 32.0
81 32.0
82 16.0
83 16.0
84 16.0
85 16.0
86 10.0
87 10.0
88 10.0
89 10.0
90 10.0
91 10.0
92 10.0
93 10.0
94 10.0
95 10.0
96 10.0
97 10.0
98 8.0
99 8.0
100 8.0
101 8.0
102 24.0
103 24.0
104 24.0
105 24.0
106 24.0
107 24.0
108 24.0
109 24.0
110 24.0
111 24.0
112 24.0
113 24.0
114 24.0
115 24.0
116 24.0
117 24.0;
param: NET := 
0 36.0
1 36.0
2 36.0
3 36.0
4 87.0
5 87.0
6 87.0
7 87.0
8 36.0
9 36.0
10 36.0
11 36.0
12 97.0
13 97.0
14 97.0
15 97.0
16 97.0
17 97.0
18 97.0
19 97.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 360.0
29 360.0
30 360.0
31 360.0
32 20.0
33 20.0
34 20.0
35 20.0
36 25.0
37 25.0
38 25.0
39 25.0
40 370.0
41 370.0
42 370.0
43 370.0
44 370.0
45 370.0
46 370.0
47 370.0
48 0.0
49 0.0
50 0.0
51 0.0
52 36.0
53 36.0
54 87.0
55 87.0
56 87.0
57 87.0
58 36.0
59 36.0
60 97.0
61 97.0
62 97.0
63 97.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 360.0
73 360.0
74 90.0
75 90.0
76 40.0
77 40.0
78 370.0
79 370.0
80 370.0
81 370.0
82 40.0
83 40.0
84 40.0
85 40.0
86 126.0
87 126.0
88 126.0
89 126.0
90 126.0
91 126.0
92 126.0
93 126.0
94 126.0
95 126.0
96 126.0
97 126.0
98 40.0
99 40.0
100 40.0
101 40.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0;


######################
# SOLVE 
######################
end;
