

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..139;
# Set of Hosts
set Hs := 0..25;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {12..15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {16..19}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {20}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {21}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {22,23}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {24,25}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {26,27}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {28,29}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_1_{h in Hs}: sum{v in {30,31}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {32..35}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_1_{h in Hs}: sum{v in {36..39}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm1_0_{h in Hs}: sum{v in {40,41}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm1_1_{h in Hs}: sum{v in {42,43}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm1_0_{h in Hs}: sum{v in {44,45}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm1_1_{h in Hs}: sum{v in {46,47}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm2_0_{h in Hs}: sum{v in {48..51}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm2_1_{h in Hs}: sum{v in {52..55}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm1_0_{h in Hs}: sum{v in {56,57}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm1_1_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm2_0_{h in Hs}: sum{v in {60,61}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm2_1_{h in Hs}: sum{v in {62,63}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm3_0_{h in Hs}: sum{v in {64}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm4_0_{h in Hs}: sum{v in {65}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm5_0_{h in Hs}: sum{v in {66,67}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm6_0_{h in Hs}: sum{v in {68,69}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm7_0_{h in Hs}: sum{v in {70,71}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm1_0_{h in Hs}: sum{v in {72,73}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm1_1_{h in Hs}: sum{v in {74,75}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm2_0_{h in Hs}: sum{v in {76,77}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm2_1_{h in Hs}: sum{v in {78,79}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm1_0_{h in Hs}: sum{v in {80,81}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm1_1_{h in Hs}: sum{v in {82,83}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm2_0_{h in Hs}: sum{v in {84..87}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm2_1_{h in Hs}: sum{v in {88..91}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm1_0_{h in Hs}: sum{v in {92,93}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm1_1_{h in Hs}: sum{v in {94,95}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm8_0_{h in Hs}: sum{v in {96,97}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm8_1_{h in Hs}: sum{v in {98,99}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm9_0_{h in Hs}: sum{v in {100,101}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm9_1_{h in Hs}: sum{v in {102,103}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm10_0_{h in Hs}: sum{v in {104,105}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm10_1_{h in Hs}: sum{v in {106,107}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm11_0_{h in Hs}: sum{v in {108,109}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm11_1_{h in Hs}: sum{v in {110,111}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm1_0_{h in Hs}: sum{v in {112,113}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm1_1_{h in Hs}: sum{v in {114,115}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm2_0_{h in Hs}: sum{v in {116,117}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm2_1_{h in Hs}: sum{v in {118,119}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm1_0_{h in Hs}: sum{v in {120,121}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm1_1_{h in Hs}: sum{v in {122,123}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm2_0_{h in Hs}: sum{v in {124,125}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm2_1_{h in Hs}: sum{v in {126,127}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm1_0_{h in Hs}: sum{v in {128,129}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm2_0_{h in Hs}: sum{v in {130,131}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm1_0_{h in Hs}: sum{v in {132,133}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm1_1_{h in Hs}: sum{v in {134,135}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm2_0_{h in Hs}: sum{v in {136,137}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm2_1_{h in Hs}: sum{v in {138,139}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 2.0
21 2.0
22 2.0
23 2.0
24 2.0
25 2.0
26 2.0
27 2.0
28 4.0
29 4.0
30 4.0
31 4.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 8.0
39 8.0
40 4.0
41 4.0
42 4.0
43 4.0
44 8.0
45 8.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 2.0
65 2.0
66 2.0
67 2.0
68 2.0
69 2.0
70 2.0
71 2.0
72 14.0
73 14.0
74 14.0
75 14.0
76 14.0
77 14.0
78 14.0
79 14.0
80 4.0
81 4.0
82 4.0
83 4.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 4.0
93 4.0
94 4.0
95 4.0
96 10.0
97 10.0
98 10.0
99 10.0
100 10.0
101 10.0
102 10.0
103 10.0
104 10.0
105 10.0
106 10.0
107 10.0
108 14.0
109 14.0
110 14.0
111 14.0
112 6.0
113 6.0
114 6.0
115 6.0
116 10.0
117 10.0
118 10.0
119 10.0
120 6.0
121 6.0
122 6.0
123 6.0
124 10.0
125 10.0
126 10.0
127 10.0
128 8.0
129 8.0
130 8.0
131 8.0
132 14.0
133 14.0
134 14.0
135 14.0
136 14.0
137 14.0
138 14.0
139 14.0;
param: MEM := 
0 54.0
1 54.0
2 54.0
3 54.0
4 54.0
5 54.0
6 54.0
7 54.0
8 20.0
9 20.0
10 20.0
11 20.0
12 54.0
13 54.0
14 54.0
15 54.0
16 54.0
17 54.0
18 54.0
19 54.0
20 8.0
21 8.0
22 12.0
23 12.0
24 16.0
25 16.0
26 8.0
27 8.0
28 12.0
29 12.0
30 12.0
31 12.0
32 32.0
33 32.0
34 32.0
35 32.0
36 32.0
37 32.0
38 32.0
39 32.0
40 16.0
41 16.0
42 16.0
43 16.0
44 54.0
45 54.0
46 54.0
47 54.0
48 54.0
49 54.0
50 54.0
51 54.0
52 54.0
53 54.0
54 54.0
55 54.0
56 20.0
57 20.0
58 20.0
59 20.0
60 54.0
61 54.0
62 54.0
63 54.0
64 8.0
65 8.0
66 12.0
67 12.0
68 16.0
69 16.0
70 8.0
71 8.0
72 40.0
73 40.0
74 40.0
75 40.0
76 8.0
77 8.0
78 8.0
79 8.0
80 12.0
81 12.0
82 12.0
83 12.0
84 32.0
85 32.0
86 32.0
87 32.0
88 32.0
89 32.0
90 32.0
91 32.0
92 16.0
93 16.0
94 16.0
95 16.0
96 10.0
97 10.0
98 10.0
99 10.0
100 10.0
101 10.0
102 10.0
103 10.0
104 10.0
105 10.0
106 10.0
107 10.0
108 8.0
109 8.0
110 8.0
111 8.0
112 24.0
113 24.0
114 24.0
115 24.0
116 24.0
117 24.0
118 24.0
119 24.0
120 24.0
121 24.0
122 24.0
123 24.0
124 24.0
125 24.0
126 24.0
127 24.0
128 20.0
129 20.0
130 54.0
131 54.0
132 40.0
133 40.0
134 40.0
135 40.0
136 8.0
137 8.0
138 8.0
139 8.0;
param: NET := 
0 36.0
1 36.0
2 36.0
3 36.0
4 87.0
5 87.0
6 87.0
7 87.0
8 36.0
9 36.0
10 36.0
11 36.0
12 97.0
13 97.0
14 97.0
15 97.0
16 97.0
17 97.0
18 97.0
19 97.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 25.0
29 25.0
30 25.0
31 25.0
32 370.0
33 370.0
34 370.0
35 370.0
36 370.0
37 370.0
38 370.0
39 370.0
40 0.0
41 0.0
42 0.0
43 0.0
44 36.0
45 36.0
46 36.0
47 36.0
48 87.0
49 87.0
50 87.0
51 87.0
52 87.0
53 87.0
54 87.0
55 87.0
56 36.0
57 36.0
58 36.0
59 36.0
60 97.0
61 97.0
62 97.0
63 97.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 360.0
73 360.0
74 360.0
75 360.0
76 90.0
77 90.0
78 90.0
79 90.0
80 40.0
81 40.0
82 40.0
83 40.0
84 370.0
85 370.0
86 370.0
87 370.0
88 370.0
89 370.0
90 370.0
91 370.0
92 40.0
93 40.0
94 40.0
95 40.0
96 126.0
97 126.0
98 126.0
99 126.0
100 126.0
101 126.0
102 126.0
103 126.0
104 126.0
105 126.0
106 126.0
107 126.0
108 40.0
109 40.0
110 40.0
111 40.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 36.0
129 36.0
130 97.0
131 97.0
132 300.0
133 300.0
134 300.0
135 300.0
136 20.0
137 20.0
138 20.0
139 20.0;


######################
# SOLVE 
######################
end;
