

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..122;
# Set of Hosts
set Hs := 0..19;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm1_0_{h in Hs}: sum{v in {27,28}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm2_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm1_0_{h in Hs}: sum{v in {31,32}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm2_0_{h in Hs}: sum{v in {33,34}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm11_0_{h in Hs}: sum{v in {35}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm12_0_{h in Hs}: sum{v in {36}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm13_0_{h in Hs}: sum{v in {37,38}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm14_0_{h in Hs}: sum{v in {39,40}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm15_0_{h in Hs}: sum{v in {41,42}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm1_0_{h in Hs}: sum{v in {43,44}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm2_0_{h in Hs}: sum{v in {45,46}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm1_0_{h in Hs}: sum{v in {47,48}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm2_0_{h in Hs}: sum{v in {49,50}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm16_0_{h in Hs}: sum{v in {51}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm16_1_{h in Hs}: sum{v in {52}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm17_0_{h in Hs}: sum{v in {53}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm18_0_{h in Hs}: sum{v in {54}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm18_1_{h in Hs}: sum{v in {55}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm19_0_{h in Hs}: sum{v in {56}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm19_1_{h in Hs}: sum{v in {57}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm20_0_{h in Hs}: sum{v in {58}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm20_1_{h in Hs}: sum{v in {59}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm21_0_{h in Hs}: sum{v in {60}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm21_1_{h in Hs}: sum{v in {61}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm22_0_{h in Hs}: sum{v in {62}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm22_1_{h in Hs}: sum{v in {63}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm23_0_{h in Hs}: sum{v in {64}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm23_1_{h in Hs}: sum{v in {65}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm24_0_{h in Hs}: sum{v in {66}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm24_1_{h in Hs}: sum{v in {67}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm25_0_{h in Hs}: sum{v in {68}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm25_1_{h in Hs}: sum{v in {69}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm26_0_{h in Hs}: sum{v in {70..73}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm27_0_{h in Hs}: sum{v in {74,75}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm28_0_{h in Hs}: sum{v in {76..78}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm29_0_{h in Hs}: sum{v in {79,80}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm30_0_{h in Hs}: sum{v in {81,82}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm31_0_{h in Hs}: sum{v in {83..85}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm32_0_{h in Hs}: sum{v in {86}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm33_0_{h in Hs}: sum{v in {87..89}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm34_0_{h in Hs}: sum{v in {90..92}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm35_0_{h in Hs}: sum{v in {93..95}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm36_0_{h in Hs}: sum{v in {96..98}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm37_0_{h in Hs}: sum{v in {107..110}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm37_1_{h in Hs}: sum{v in {111..114}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm37_0_{h in Hs}: sum{v in {115..118}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm37_1_{h in Hs}: sum{v in {119..122}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {2,3}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {2,3}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {4,5}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {4,5}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;

#=== Soft Affinity Variables ===
var vnf4_dot_vm8_0{h in Hs}, binary;
s.t. or_sum_vnf4_dot_vm8_0{h in Hs}: vnf4_dot_vm8_0[h] <= sum{v in {16,17}} x[v, h];
s.t. or_ineq_vnf4_dot_vm8_0{v in {16,17}, h in Hs}: vnf4_dot_vm8_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf4_dot_vm8_0[h]) - 1) / 2;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 4
7 4
8 4
9 4
10 2
11 2
12 4
13 4
14 2
15 2
16 16
17 16
18 16
19 12
20 12
21 12
22 2
23 2
24 2
25 2
26 2
27 4
28 4
29 4
30 4
31 2
32 2
33 2
34 2
35 2
36 2
37 2
38 2
39 2
40 2
41 2
42 2
43 14
44 14
45 14
46 14
47 4
48 4
49 8
50 8
51 8
52 8
53 8
54 2
55 2
56 1
57 1
58 1
59 1
60 12
61 12
62 24
63 24
64 16
65 16
66 2
67 2
68 4
69 4
70 16
71 16
72 16
73 16
74 16
75 16
76 16
77 16
78 16
79 16
80 16
81 16
82 16
83 4
84 4
85 4
86 2
87 6
88 6
89 6
90 12
91 12
92 12
93 8
94 8
95 8
96 8
97 8
98 8
99 4
100 4
101 4
102 4
103 4
104 4
105 4
106 4
107 4
108 4
109 4
110 4
111 4
112 4
113 4
114 4
115 4
116 4
117 4
118 4
119 4
120 4
121 4
122 4;
param: MEM := 
0 54
1 54
2 54
3 54
4 54
5 54
6 8
7 8
8 8
9 8
10 4
11 4
12 8
13 8
14 4
15 4
16 32
17 32
18 64
19 64
20 64
21 64
22 32
23 32
24 32
25 32
26 32
27 20
28 20
29 20
30 20
31 8
32 8
33 24
34 24
35 8
36 8
37 12
38 12
39 16
40 16
41 8
42 8
43 20
44 20
45 8
46 8
47 12
48 12
49 24
50 24
51 8
52 8
53 16
54 4
55 4
56 8
57 8
58 4
59 4
60 32
61 32
62 64
63 64
64 64
65 64
66 8
67 8
68 4
69 4
70 64
71 64
72 64
73 64
74 64
75 64
76 64
77 64
78 64
79 64
80 64
81 64
82 64
83 16
84 16
85 16
86 8
87 10
88 10
89 10
90 64
91 64
92 64
93 64
94 64
95 64
96 64
97 64
98 64
99 32
100 32
101 32
102 32
103 32
104 32
105 32
106 32
107 32
108 32
109 32
110 32
111 32
112 32
113 32
114 32
115 32
116 32
117 32
118 32
119 32
120 32
121 32
122 32;
param: NET := 
0 36.0
1 36.0
2 87.0
3 87.0
4 87.0
5 87.0
6 8.313786916666666
7 8.313786916666666
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 36.0
28 36.0
29 87.0
30 87.0
31 36.0
32 36.0
33 97.0
34 97.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 360.0
44 360.0
45 90.0
46 90.0
47 40.0
48 40.0
49 370.0
50 370.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 500.0
71 500.0
72 500.0
73 500.0
74 500.0
75 500.0
76 500.0
77 500.0
78 500.0
79 500.0
80 500.0
81 500.0
82 500.0
83 500.0
84 500.0
85 500.0
86 500.0
87 500.0
88 500.0
89 500.0
90 900.0
91 900.0
92 900.0
93 300.0
94 300.0
95 300.0
96 350.0
97 350.0
98 350.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0;


######################
# SOLVE 
######################
end;
