

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..35;
# Set of Hosts
set Hs := 0..7;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_1_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {8}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {9}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_0_{h in Hs}: sum{v in {10..17}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm6_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm7_0_{h in Hs}: sum{v in {20..22}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm8_0_{h in Hs}: sum{v in {23}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm9_0_{h in Hs}: sum{v in {24,25}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm4_0_{h in Hs}: sum{v in {26,27}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm10_0_{h in Hs}: sum{v in {28,29}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm11_0_{h in Hs}: sum{v in {30,31}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm12_0_{h in Hs}: sum{v in {32,33}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm13_0_{h in Hs}: sum{v in {34,35}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 16
1 16
2 4
3 4
4 4
5 4
6 4
7 4
8 2
9 2
10 16
11 16
12 16
13 16
14 16
15 16
16 16
17 16
18 8
19 8
20 4
21 4
22 4
23 1
24 8
25 8
26 16
27 16
28 2
29 2
30 2
31 2
32 8
33 8
34 2
35 2;
param: MEM := 
0 64
1 64
2 8
3 8
4 8
5 8
6 8
7 8
8 6
9 4
10 50
11 50
12 50
13 50
14 50
15 50
16 50
17 50
18 32
19 32
20 8
21 8
22 8
23 2
24 16
25 16
26 32
27 32
28 4
29 4
30 4
31 4
32 16
33 16
34 2
35 2;
param: NET := 
0 1000
1 1000
2 200
3 200
4 200
5 200
6 200
7 200
8 0
9 0
10 0
11 0
12 0
13 0
14 0
15 0
16 0
17 0
18 1000
19 1000
20 1000
21 1000
22 1000
23 1000
24 20
25 20
26 100
27 100
28 70
29 70
30 70
31 70
32 0
33 0
34 0
35 0;


######################
# SOLVE 
######################
end;
