

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..142;
# Set of Hosts
set Hs := 0..35;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2..13}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {14..16}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {17..47}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {58..61}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm6_0_{h in Hs}: sum{v in {62,63}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm7_0_{h in Hs}: sum{v in {64..67}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {68,69}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm9_0_{h in Hs}: sum{v in {70..73}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm10_0_{h in Hs}: sum{v in {74..78}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm10_0_{h in Hs}: sum{v in {79}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm10_0_{h in Hs}: sum{v in {80..82}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm11_0_{h in Hs}: sum{v in {83,84}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm11_0_{h in Hs}: sum{v in {85,86}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm12_0_{h in Hs}: sum{v in {87,88}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm13_0_{h in Hs}: sum{v in {89,90}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm14_0_{h in Hs}: sum{v in {91..106}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm15_0_{h in Hs}: sum{v in {107..122}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm16_0_{h in Hs}: sum{v in {123..126}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm17_0_{h in Hs}: sum{v in {127..130}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm18_0_{h in Hs}: sum{v in {131..133}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm19_0_{h in Hs}: sum{v in {134,135}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm20_0_{h in Hs}: sum{v in {136..142}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_master{h in Hs}: ni_vnf3_dot_vm1_0_master[h] <= sum{v in {48}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_master{v in {48}, h in Hs}: ni_vnf3_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_slave{h in Hs}: ni_vnf3_dot_vm1_0_slave[h] <= sum{v in {49}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_slave{v in {49}, h in Hs}: ni_vnf3_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf3_dot_vm1_0_master[h] + ni_vnf3_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_master{h in Hs}: ni_vnf3_dot_vm2_0_master[h] <= sum{v in {50..53}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_master{v in {50..53}, h in Hs}: ni_vnf3_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_slave{h in Hs}: ni_vnf3_dot_vm2_0_slave[h] <= sum{v in {54..57}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_slave{v in {54..57}, h in Hs}: ni_vnf3_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf3_dot_vm2_0_master[h] + ni_vnf3_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 16
18 16
19 16
20 16
21 16
22 16
23 16
24 16
25 16
26 16
27 16
28 16
29 16
30 16
31 16
32 16
33 16
34 16
35 16
36 16
37 16
38 16
39 16
40 16
41 16
42 16
43 16
44 16
45 16
46 16
47 16
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 4
59 4
60 4
61 4
62 7
63 7
64 2
65 2
66 2
67 2
68 6
69 6
70 4
71 4
72 4
73 4
74 16
75 16
76 16
77 16
78 16
79 16
80 4
81 4
82 4
83 4
84 4
85 2
86 2
87 12
88 12
89 12
90 12
91 12
92 12
93 12
94 12
95 12
96 12
97 12
98 12
99 12
100 12
101 12
102 12
103 12
104 12
105 12
106 12
107 12
108 12
109 12
110 12
111 12
112 12
113 12
114 12
115 12
116 12
117 12
118 12
119 12
120 12
121 12
122 12
123 12
124 12
125 12
126 12
127 12
128 12
129 12
130 12
131 8
132 8
133 8
134 5
135 5
136 16
137 16
138 16
139 16
140 16
141 16
142 16;
param: MEM := 
0 16
1 16
2 42
3 42
4 42
5 42
6 42
7 42
8 42
9 42
10 42
11 42
12 42
13 42
14 16
15 16
16 16
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8
28 8
29 8
30 8
31 8
32 8
33 8
34 8
35 8
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 8
46 8
47 8
48 16
49 16
50 48
51 48
52 48
53 48
54 48
55 48
56 48
57 48
58 32
59 32
60 32
61 32
62 10
63 10
64 32
65 32
66 32
67 32
68 4
69 4
70 10
71 10
72 10
73 10
74 64
75 64
76 64
77 64
78 64
79 64
80 16
81 16
82 16
83 32
84 32
85 8
86 8
87 60
88 60
89 60
90 60
91 60
92 60
93 60
94 60
95 60
96 60
97 60
98 60
99 60
100 60
101 60
102 60
103 60
104 60
105 60
106 60
107 60
108 60
109 60
110 60
111 60
112 60
113 60
114 60
115 60
116 60
117 60
118 60
119 60
120 60
121 60
122 60
123 60
124 60
125 60
126 60
127 60
128 60
129 60
130 60
131 10
132 10
133 10
134 12
135 12
136 85
137 85
138 85
139 85
140 85
141 85
142 85;
param: NET := 
0 171
1 171
2 535
3 535
4 535
5 535
6 535
7 535
8 535
9 535
10 535
11 535
12 535
13 535
14 0
15 0
16 0
17 1600
18 1600
19 1600
20 1600
21 1600
22 1600
23 1600
24 1600
25 1600
26 1600
27 1600
28 1600
29 1600
30 1600
31 1600
32 1600
33 1600
34 1600
35 1600
36 1600
37 1600
38 1600
39 1600
40 1600
41 1600
42 1600
43 1600
44 1600
45 1600
46 1600
47 1600
48 675
49 675
50 535
51 535
52 535
53 535
54 535
55 535
56 535
57 535
58 0
59 0
60 0
61 0
62 0
63 0
64 0
65 0
66 0
67 0
68 0
69 0
70 0
71 0
72 0
73 0
74 120
75 120
76 120
77 120
78 120
79 0
80 35
81 35
82 35
83 25
84 25
85 15
86 15
87 100
88 100
89 100
90 100
91 300
92 300
93 300
94 300
95 300
96 300
97 300
98 300
99 300
100 300
101 300
102 300
103 300
104 300
105 300
106 300
107 300
108 300
109 300
110 300
111 300
112 300
113 300
114 300
115 300
116 300
117 300
118 300
119 300
120 300
121 300
122 300
123 300
124 300
125 300
126 300
127 300
128 300
129 300
130 300
131 0
132 0
133 0
134 0
135 0
136 0
137 0
138 0
139 0
140 0
141 0
142 0;


######################
# SOLVE 
######################
end;
