

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..142;
# Set of Hosts
set Hs := 0..36;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2..13}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {14..19}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {20..50}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm6_0_{h in Hs}: sum{v in {61,62}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm7_0_{h in Hs}: sum{v in {63..67}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {68,69}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm9_0_{h in Hs}: sum{v in {70..73}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm10_0_{h in Hs}: sum{v in {74..78}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm10_0_{h in Hs}: sum{v in {79}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm10_0_{h in Hs}: sum{v in {80..82}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm11_0_{h in Hs}: sum{v in {83,84}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm11_0_{h in Hs}: sum{v in {85,86}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm12_0_{h in Hs}: sum{v in {87,88}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm13_0_{h in Hs}: sum{v in {89,90}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm14_0_{h in Hs}: sum{v in {91..106}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm15_0_{h in Hs}: sum{v in {107..122}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm16_0_{h in Hs}: sum{v in {123..126}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm17_0_{h in Hs}: sum{v in {127..130}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm18_0_{h in Hs}: sum{v in {131..133}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm19_0_{h in Hs}: sum{v in {134,135}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm20_0_{h in Hs}: sum{v in {136..142}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_master{h in Hs}: ni_vnf3_dot_vm1_0_master[h] <= sum{v in {51}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_master{v in {51}, h in Hs}: ni_vnf3_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_slave{h in Hs}: ni_vnf3_dot_vm1_0_slave[h] <= sum{v in {52}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_slave{v in {52}, h in Hs}: ni_vnf3_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf3_dot_vm1_0_master[h] + ni_vnf3_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_master{h in Hs}: ni_vnf3_dot_vm2_0_master[h] <= sum{v in {53..56}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_master{v in {53..56}, h in Hs}: ni_vnf3_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_slave{h in Hs}: ni_vnf3_dot_vm2_0_slave[h] <= sum{v in {57..60}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_slave{v in {57..60}, h in Hs}: ni_vnf3_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf3_dot_vm2_0_master[h] + ni_vnf3_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 16.0
21 16.0
22 16.0
23 16.0
24 16.0
25 16.0
26 16.0
27 16.0
28 16.0
29 16.0
30 16.0
31 16.0
32 16.0
33 16.0
34 16.0
35 16.0
36 16.0
37 16.0
38 16.0
39 16.0
40 16.0
41 16.0
42 16.0
43 16.0
44 16.0
45 16.0
46 16.0
47 16.0
48 16.0
49 16.0
50 16.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 7.0
62 7.0
63 2.0
64 2.0
65 2.0
66 2.0
67 2.0
68 6.0
69 6.0
70 4.0
71 4.0
72 4.0
73 4.0
74 16.0
75 16.0
76 16.0
77 16.0
78 16.0
79 16.0
80 4.0
81 4.0
82 4.0
83 4.0
84 4.0
85 2.0
86 2.0
87 12.0
88 12.0
89 12.0
90 12.0
91 12.0
92 12.0
93 12.0
94 12.0
95 12.0
96 12.0
97 12.0
98 12.0
99 12.0
100 12.0
101 12.0
102 12.0
103 12.0
104 12.0
105 12.0
106 12.0
107 12.0
108 12.0
109 12.0
110 12.0
111 12.0
112 12.0
113 12.0
114 12.0
115 12.0
116 12.0
117 12.0
118 12.0
119 12.0
120 12.0
121 12.0
122 12.0
123 12.0
124 12.0
125 12.0
126 12.0
127 12.0
128 12.0
129 12.0
130 12.0
131 8.0
132 8.0
133 8.0
134 5.0
135 5.0
136 16.0
137 16.0
138 16.0
139 16.0
140 16.0
141 16.0
142 16.0;
param: MEM := 
0 16.0
1 16.0
2 42.0
3 42.0
4 42.0
5 42.0
6 42.0
7 42.0
8 42.0
9 42.0
10 42.0
11 42.0
12 42.0
13 42.0
14 16.0
15 16.0
16 16.0
17 16.0
18 16.0
19 16.0
20 8.0
21 8.0
22 8.0
23 8.0
24 8.0
25 8.0
26 8.0
27 8.0
28 8.0
29 8.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 8.0
39 8.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 16.0
52 16.0
53 48.0
54 48.0
55 48.0
56 48.0
57 48.0
58 48.0
59 48.0
60 48.0
61 10.0
62 10.0
63 32.0
64 32.0
65 32.0
66 32.0
67 32.0
68 4.0
69 4.0
70 10.0
71 10.0
72 10.0
73 10.0
74 64.0
75 64.0
76 64.0
77 64.0
78 64.0
79 64.0
80 16.0
81 16.0
82 16.0
83 32.0
84 32.0
85 8.0
86 8.0
87 60.0
88 60.0
89 60.0
90 60.0
91 60.0
92 60.0
93 60.0
94 60.0
95 60.0
96 60.0
97 60.0
98 60.0
99 60.0
100 60.0
101 60.0
102 60.0
103 60.0
104 60.0
105 60.0
106 60.0
107 60.0
108 60.0
109 60.0
110 60.0
111 60.0
112 60.0
113 60.0
114 60.0
115 60.0
116 60.0
117 60.0
118 60.0
119 60.0
120 60.0
121 60.0
122 60.0
123 60.0
124 60.0
125 60.0
126 60.0
127 60.0
128 60.0
129 60.0
130 60.0
131 10.0
132 10.0
133 10.0
134 12.0
135 12.0
136 85.0
137 85.0
138 85.0
139 85.0
140 85.0
141 85.0
142 85.0;
param: NET := 
0 171.0
1 171.0
2 535.0
3 535.0
4 535.0
5 535.0
6 535.0
7 535.0
8 535.0
9 535.0
10 535.0
11 535.0
12 535.0
13 535.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 1600.0
21 1600.0
22 1600.0
23 1600.0
24 1600.0
25 1600.0
26 1600.0
27 1600.0
28 1600.0
29 1600.0
30 1600.0
31 1600.0
32 1600.0
33 1600.0
34 1600.0
35 1600.0
36 1600.0
37 1600.0
38 1600.0
39 1600.0
40 1600.0
41 1600.0
42 1600.0
43 1600.0
44 1600.0
45 1600.0
46 1600.0
47 1600.0
48 1600.0
49 1600.0
50 1600.0
51 675.0
52 675.0
53 535.0
54 535.0
55 535.0
56 535.0
57 535.0
58 535.0
59 535.0
60 535.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 120.0
75 120.0
76 120.0
77 120.0
78 120.0
79 0.0
80 35.0
81 35.0
82 35.0
83 25.0
84 25.0
85 15.0
86 15.0
87 100.0
88 100.0
89 100.0
90 100.0
91 300.0
92 300.0
93 300.0
94 300.0
95 300.0
96 300.0
97 300.0
98 300.0
99 300.0
100 300.0
101 300.0
102 300.0
103 300.0
104 300.0
105 300.0
106 300.0
107 300.0
108 300.0
109 300.0
110 300.0
111 300.0
112 300.0
113 300.0
114 300.0
115 300.0
116 300.0
117 300.0
118 300.0
119 300.0
120 300.0
121 300.0
122 300.0
123 300.0
124 300.0
125 300.0
126 300.0
127 300.0
128 300.0
129 300.0
130 300.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0;


######################
# SOLVE 
######################
end;
