

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..274;
# Set of Hosts
set Hs := 0..50;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {4..25}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {26..47}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {50,51}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {52..83}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {84..115}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {116}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {117}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {118..122}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {123..135}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {136,137}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm1_0_{h in Hs}: sum{v in {138,139}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {140,141}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {142,143}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {144..193}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm1_0_{h in Hs}: sum{v in {194,195}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm2_0_{h in Hs}: sum{v in {196..220}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm1_0_{h in Hs}: sum{v in {221,222}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm1_0_{h in Hs}: sum{v in {248,249}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm2_0_{h in Hs}: sum{v in {250..274}} x[v, h]<= 1;

#=== Soft Affinity Variables ===
var vnf7_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf7_dot_vm2_0{h in Hs}: vnf7_dot_vm2_0[h] <= sum{v in {223..247}} x[v, h];
s.t. or_ineq_vnf7_dot_vm2_0{v in {223..247}, h in Hs}: vnf7_dot_vm2_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf7_dot_vm2_0[h]) - 1) / 25;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8
28 8
29 8
30 8
31 8
32 8
33 8
34 8
35 8
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 8
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 8
73 8
74 8
75 8
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 8
89 8
90 8
91 8
92 8
93 8
94 8
95 8
96 8
97 8
98 8
99 8
100 8
101 8
102 8
103 8
104 8
105 8
106 8
107 8
108 8
109 8
110 8
111 8
112 8
113 8
114 8
115 8
116 2
117 2
118 2
119 2
120 2
121 2
122 2
123 2
124 2
125 2
126 2
127 2
128 2
129 2
130 2
131 2
132 2
133 2
134 2
135 2
136 2
137 2
138 14
139 14
140 14
141 14
142 4
143 4
144 8
145 8
146 8
147 8
148 8
149 8
150 8
151 8
152 8
153 8
154 8
155 8
156 8
157 8
158 8
159 8
160 8
161 8
162 8
163 8
164 8
165 8
166 8
167 8
168 8
169 8
170 8
171 8
172 8
173 8
174 8
175 8
176 8
177 8
178 8
179 8
180 8
181 8
182 8
183 8
184 8
185 8
186 8
187 8
188 8
189 8
190 8
191 8
192 8
193 8
194 6
195 6
196 10
197 10
198 10
199 10
200 10
201 10
202 10
203 10
204 10
205 10
206 10
207 10
208 10
209 10
210 10
211 10
212 10
213 10
214 10
215 10
216 10
217 10
218 10
219 10
220 10
221 6
222 6
223 10
224 10
225 10
226 10
227 10
228 10
229 10
230 10
231 10
232 10
233 10
234 10
235 10
236 10
237 10
238 10
239 10
240 10
241 10
242 10
243 10
244 10
245 10
246 10
247 10
248 6
249 6
250 10
251 10
252 10
253 10
254 10
255 10
256 10
257 10
258 10
259 10
260 10
261 10
262 10
263 10
264 10
265 10
266 10
267 10
268 10
269 10
270 10
271 10
272 10
273 10
274 10;
param: MEM := 
0 54
1 54
2 54
3 54
4 54
5 54
6 54
7 54
8 54
9 54
10 54
11 54
12 54
13 54
14 54
15 54
16 54
17 54
18 54
19 54
20 54
21 54
22 54
23 54
24 54
25 54
26 54
27 54
28 54
29 54
30 54
31 54
32 54
33 54
34 54
35 54
36 54
37 54
38 54
39 54
40 54
41 54
42 54
43 54
44 54
45 54
46 54
47 54
48 20
49 20
50 20
51 20
52 54
53 54
54 54
55 54
56 54
57 54
58 54
59 54
60 54
61 54
62 54
63 54
64 54
65 54
66 54
67 54
68 54
69 54
70 54
71 54
72 54
73 54
74 54
75 54
76 54
77 54
78 54
79 54
80 54
81 54
82 54
83 54
84 54
85 54
86 54
87 54
88 54
89 54
90 54
91 54
92 54
93 54
94 54
95 54
96 54
97 54
98 54
99 54
100 54
101 54
102 54
103 54
104 54
105 54
106 54
107 54
108 54
109 54
110 54
111 54
112 54
113 54
114 54
115 54
116 8
117 8
118 12
119 12
120 12
121 12
122 12
123 16
124 16
125 16
126 16
127 16
128 16
129 16
130 16
131 16
132 16
133 16
134 16
135 16
136 8
137 8
138 40
139 40
140 8
141 8
142 12
143 12
144 32
145 32
146 32
147 32
148 32
149 32
150 32
151 32
152 32
153 32
154 32
155 32
156 32
157 32
158 32
159 32
160 32
161 32
162 32
163 32
164 32
165 32
166 32
167 32
168 32
169 32
170 32
171 32
172 32
173 32
174 32
175 32
176 32
177 32
178 32
179 32
180 32
181 32
182 32
183 32
184 32
185 32
186 32
187 32
188 32
189 32
190 32
191 32
192 32
193 32
194 24
195 24
196 24
197 24
198 24
199 24
200 24
201 24
202 24
203 24
204 24
205 24
206 24
207 24
208 24
209 24
210 24
211 24
212 24
213 24
214 24
215 24
216 24
217 24
218 24
219 24
220 24
221 24
222 24
223 24
224 24
225 24
226 24
227 24
228 24
229 24
230 24
231 24
232 24
233 24
234 24
235 24
236 24
237 24
238 24
239 24
240 24
241 24
242 24
243 24
244 24
245 24
246 24
247 24
248 24
249 24
250 24
251 24
252 24
253 24
254 24
255 24
256 24
257 24
258 24
259 24
260 24
261 24
262 24
263 24
264 24
265 24
266 24
267 24
268 24
269 24
270 24
271 24
272 24
273 24
274 24;
param: NET := 
0 36
1 36
2 36
3 36
4 87
5 87
6 87
7 87
8 87
9 87
10 87
11 87
12 87
13 87
14 87
15 87
16 87
17 87
18 87
19 87
20 87
21 87
22 87
23 87
24 87
25 87
26 87
27 87
28 87
29 87
30 87
31 87
32 87
33 87
34 87
35 87
36 87
37 87
38 87
39 87
40 87
41 87
42 87
43 87
44 87
45 87
46 87
47 87
48 36
49 36
50 36
51 36
52 97
53 97
54 97
55 97
56 97
57 97
58 97
59 97
60 97
61 97
62 97
63 97
64 97
65 97
66 97
67 97
68 97
69 97
70 97
71 97
72 97
73 97
74 97
75 97
76 97
77 97
78 97
79 97
80 97
81 97
82 97
83 97
84 97
85 97
86 97
87 97
88 97
89 97
90 97
91 97
92 97
93 97
94 97
95 97
96 97
97 97
98 97
99 97
100 97
101 97
102 97
103 97
104 97
105 97
106 97
107 97
108 97
109 97
110 97
111 97
112 97
113 97
114 97
115 97
116 0
117 0
118 0
119 0
120 0
121 0
122 0
123 0
124 0
125 0
126 0
127 0
128 0
129 0
130 0
131 0
132 0
133 0
134 0
135 0
136 0
137 0
138 360
139 360
140 90
141 90
142 25
143 25
144 370
145 370
146 370
147 370
148 370
149 370
150 370
151 370
152 370
153 370
154 370
155 370
156 370
157 370
158 370
159 370
160 370
161 370
162 370
163 370
164 370
165 370
166 370
167 370
168 370
169 370
170 370
171 370
172 370
173 370
174 370
175 370
176 370
177 370
178 370
179 370
180 370
181 370
182 370
183 370
184 370
185 370
186 370
187 370
188 370
189 370
190 370
191 370
192 370
193 370
194 2
195 2
196 95
197 95
198 95
199 95
200 95
201 95
202 95
203 95
204 95
205 95
206 95
207 95
208 95
209 95
210 95
211 95
212 95
213 95
214 95
215 95
216 95
217 95
218 95
219 95
220 95
221 2
222 2
223 216
224 216
225 216
226 216
227 216
228 216
229 216
230 216
231 216
232 216
233 216
234 216
235 216
236 216
237 216
238 216
239 216
240 216
241 216
242 216
243 216
244 216
245 216
246 216
247 216
248 2
249 2
250 152
251 152
252 152
253 152
254 152
255 152
256 152
257 152
258 152
259 152
260 152
261 152
262 152
263 152
264 152
265 152
266 152
267 152
268 152
269 152
270 152
271 152
272 152
273 152
274 152;


######################
# SOLVE 
######################
end;
