

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..285;
# Set of Hosts
set Hs := 0..52;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {4..25}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {26..47}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {50,51}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {52..83}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {84..115}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {116}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {117}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {118..122}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {123..135}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {136,137}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {138,139}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {140..189}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm1_0_{h in Hs}: sum{v in {190,191}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm2_0_{h in Hs}: sum{v in {192..221}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm1_0_{h in Hs}: sum{v in {222,223}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm1_0_{h in Hs}: sum{v in {254,255}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm2_0_{h in Hs}: sum{v in {256..285}} x[v, h]<= 1;

#=== Soft Affinity Variables ===
var vnf7_dot_vm2_0{h in Hs}, binary;
s.t. or_sum_vnf7_dot_vm2_0{h in Hs}: vnf7_dot_vm2_0[h] <= sum{v in {224..253}} x[v, h];
s.t. or_ineq_vnf7_dot_vm2_0{v in {224..253}, h in Hs}: vnf7_dot_vm2_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf7_dot_vm2_0[h]) - 1) / 30;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 8.0
21 8.0
22 8.0
23 8.0
24 8.0
25 8.0
26 8.0
27 8.0
28 8.0
29 8.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 8.0
39 8.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 8.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 8.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0
83 8.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 8.0
93 8.0
94 8.0
95 8.0
96 8.0
97 8.0
98 8.0
99 8.0
100 8.0
101 8.0
102 8.0
103 8.0
104 8.0
105 8.0
106 8.0
107 8.0
108 8.0
109 8.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 2.0
117 2.0
118 2.0
119 2.0
120 2.0
121 2.0
122 2.0
123 2.0
124 2.0
125 2.0
126 2.0
127 2.0
128 2.0
129 2.0
130 2.0
131 2.0
132 2.0
133 2.0
134 2.0
135 2.0
136 2.0
137 2.0
138 4.0
139 4.0
140 8.0
141 8.0
142 8.0
143 8.0
144 8.0
145 8.0
146 8.0
147 8.0
148 8.0
149 8.0
150 8.0
151 8.0
152 8.0
153 8.0
154 8.0
155 8.0
156 8.0
157 8.0
158 8.0
159 8.0
160 8.0
161 8.0
162 8.0
163 8.0
164 8.0
165 8.0
166 8.0
167 8.0
168 8.0
169 8.0
170 8.0
171 8.0
172 8.0
173 8.0
174 8.0
175 8.0
176 8.0
177 8.0
178 8.0
179 8.0
180 8.0
181 8.0
182 8.0
183 8.0
184 8.0
185 8.0
186 8.0
187 8.0
188 8.0
189 8.0
190 6.0
191 6.0
192 10.0
193 10.0
194 10.0
195 10.0
196 10.0
197 10.0
198 10.0
199 10.0
200 10.0
201 10.0
202 10.0
203 10.0
204 10.0
205 10.0
206 10.0
207 10.0
208 10.0
209 10.0
210 10.0
211 10.0
212 10.0
213 10.0
214 10.0
215 10.0
216 10.0
217 10.0
218 10.0
219 10.0
220 10.0
221 10.0
222 6.0
223 6.0
224 10.0
225 10.0
226 10.0
227 10.0
228 10.0
229 10.0
230 10.0
231 10.0
232 10.0
233 10.0
234 10.0
235 10.0
236 10.0
237 10.0
238 10.0
239 10.0
240 10.0
241 10.0
242 10.0
243 10.0
244 10.0
245 10.0
246 10.0
247 10.0
248 10.0
249 10.0
250 10.0
251 10.0
252 10.0
253 10.0
254 6.0
255 6.0
256 10.0
257 10.0
258 10.0
259 10.0
260 10.0
261 10.0
262 10.0
263 10.0
264 10.0
265 10.0
266 10.0
267 10.0
268 10.0
269 10.0
270 10.0
271 10.0
272 10.0
273 10.0
274 10.0
275 10.0
276 10.0
277 10.0
278 10.0
279 10.0
280 10.0
281 10.0
282 10.0
283 10.0
284 10.0
285 10.0;
param: MEM := 
0 54.0
1 54.0
2 54.0
3 54.0
4 54.0
5 54.0
6 54.0
7 54.0
8 54.0
9 54.0
10 54.0
11 54.0
12 54.0
13 54.0
14 54.0
15 54.0
16 54.0
17 54.0
18 54.0
19 54.0
20 54.0
21 54.0
22 54.0
23 54.0
24 54.0
25 54.0
26 54.0
27 54.0
28 54.0
29 54.0
30 54.0
31 54.0
32 54.0
33 54.0
34 54.0
35 54.0
36 54.0
37 54.0
38 54.0
39 54.0
40 54.0
41 54.0
42 54.0
43 54.0
44 54.0
45 54.0
46 54.0
47 54.0
48 20.0
49 20.0
50 20.0
51 20.0
52 54.0
53 54.0
54 54.0
55 54.0
56 54.0
57 54.0
58 54.0
59 54.0
60 54.0
61 54.0
62 54.0
63 54.0
64 54.0
65 54.0
66 54.0
67 54.0
68 54.0
69 54.0
70 54.0
71 54.0
72 54.0
73 54.0
74 54.0
75 54.0
76 54.0
77 54.0
78 54.0
79 54.0
80 54.0
81 54.0
82 54.0
83 54.0
84 54.0
85 54.0
86 54.0
87 54.0
88 54.0
89 54.0
90 54.0
91 54.0
92 54.0
93 54.0
94 54.0
95 54.0
96 54.0
97 54.0
98 54.0
99 54.0
100 54.0
101 54.0
102 54.0
103 54.0
104 54.0
105 54.0
106 54.0
107 54.0
108 54.0
109 54.0
110 54.0
111 54.0
112 54.0
113 54.0
114 54.0
115 54.0
116 8.0
117 8.0
118 12.0
119 12.0
120 12.0
121 12.0
122 12.0
123 16.0
124 16.0
125 16.0
126 16.0
127 16.0
128 16.0
129 16.0
130 16.0
131 16.0
132 16.0
133 16.0
134 16.0
135 16.0
136 8.0
137 8.0
138 12.0
139 12.0
140 32.0
141 32.0
142 32.0
143 32.0
144 32.0
145 32.0
146 32.0
147 32.0
148 32.0
149 32.0
150 32.0
151 32.0
152 32.0
153 32.0
154 32.0
155 32.0
156 32.0
157 32.0
158 32.0
159 32.0
160 32.0
161 32.0
162 32.0
163 32.0
164 32.0
165 32.0
166 32.0
167 32.0
168 32.0
169 32.0
170 32.0
171 32.0
172 32.0
173 32.0
174 32.0
175 32.0
176 32.0
177 32.0
178 32.0
179 32.0
180 32.0
181 32.0
182 32.0
183 32.0
184 32.0
185 32.0
186 32.0
187 32.0
188 32.0
189 32.0
190 24.0
191 24.0
192 24.0
193 24.0
194 24.0
195 24.0
196 24.0
197 24.0
198 24.0
199 24.0
200 24.0
201 24.0
202 24.0
203 24.0
204 24.0
205 24.0
206 24.0
207 24.0
208 24.0
209 24.0
210 24.0
211 24.0
212 24.0
213 24.0
214 24.0
215 24.0
216 24.0
217 24.0
218 24.0
219 24.0
220 24.0
221 24.0
222 24.0
223 24.0
224 24.0
225 24.0
226 24.0
227 24.0
228 24.0
229 24.0
230 24.0
231 24.0
232 24.0
233 24.0
234 24.0
235 24.0
236 24.0
237 24.0
238 24.0
239 24.0
240 24.0
241 24.0
242 24.0
243 24.0
244 24.0
245 24.0
246 24.0
247 24.0
248 24.0
249 24.0
250 24.0
251 24.0
252 24.0
253 24.0
254 24.0
255 24.0
256 24.0
257 24.0
258 24.0
259 24.0
260 24.0
261 24.0
262 24.0
263 24.0
264 24.0
265 24.0
266 24.0
267 24.0
268 24.0
269 24.0
270 24.0
271 24.0
272 24.0
273 24.0
274 24.0
275 24.0
276 24.0
277 24.0
278 24.0
279 24.0
280 24.0
281 24.0
282 24.0
283 24.0
284 24.0
285 24.0;
param: NET := 
0 36.0
1 36.0
2 36.0
3 36.0
4 87.0
5 87.0
6 87.0
7 87.0
8 87.0
9 87.0
10 87.0
11 87.0
12 87.0
13 87.0
14 87.0
15 87.0
16 87.0
17 87.0
18 87.0
19 87.0
20 87.0
21 87.0
22 87.0
23 87.0
24 87.0
25 87.0
26 87.0
27 87.0
28 87.0
29 87.0
30 87.0
31 87.0
32 87.0
33 87.0
34 87.0
35 87.0
36 87.0
37 87.0
38 87.0
39 87.0
40 87.0
41 87.0
42 87.0
43 87.0
44 87.0
45 87.0
46 87.0
47 87.0
48 36.0
49 36.0
50 36.0
51 36.0
52 97.0
53 97.0
54 97.0
55 97.0
56 97.0
57 97.0
58 97.0
59 97.0
60 97.0
61 97.0
62 97.0
63 97.0
64 97.0
65 97.0
66 97.0
67 97.0
68 97.0
69 97.0
70 97.0
71 97.0
72 97.0
73 97.0
74 97.0
75 97.0
76 97.0
77 97.0
78 97.0
79 97.0
80 97.0
81 97.0
82 97.0
83 97.0
84 97.0
85 97.0
86 97.0
87 97.0
88 97.0
89 97.0
90 97.0
91 97.0
92 97.0
93 97.0
94 97.0
95 97.0
96 97.0
97 97.0
98 97.0
99 97.0
100 97.0
101 97.0
102 97.0
103 97.0
104 97.0
105 97.0
106 97.0
107 97.0
108 97.0
109 97.0
110 97.0
111 97.0
112 97.0
113 97.0
114 97.0
115 97.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 25.0
139 25.0
140 370.0
141 370.0
142 370.0
143 370.0
144 370.0
145 370.0
146 370.0
147 370.0
148 370.0
149 370.0
150 370.0
151 370.0
152 370.0
153 370.0
154 370.0
155 370.0
156 370.0
157 370.0
158 370.0
159 370.0
160 370.0
161 370.0
162 370.0
163 370.0
164 370.0
165 370.0
166 370.0
167 370.0
168 370.0
169 370.0
170 370.0
171 370.0
172 370.0
173 370.0
174 370.0
175 370.0
176 370.0
177 370.0
178 370.0
179 370.0
180 370.0
181 370.0
182 370.0
183 370.0
184 370.0
185 370.0
186 370.0
187 370.0
188 370.0
189 370.0
190 2.0
191 2.0
192 95.0
193 95.0
194 95.0
195 95.0
196 95.0
197 95.0
198 95.0
199 95.0
200 95.0
201 95.0
202 95.0
203 95.0
204 95.0
205 95.0
206 95.0
207 95.0
208 95.0
209 95.0
210 95.0
211 95.0
212 95.0
213 95.0
214 95.0
215 95.0
216 95.0
217 95.0
218 95.0
219 95.0
220 95.0
221 95.0
222 2.0
223 2.0
224 216.0
225 216.0
226 216.0
227 216.0
228 216.0
229 216.0
230 216.0
231 216.0
232 216.0
233 216.0
234 216.0
235 216.0
236 216.0
237 216.0
238 216.0
239 216.0
240 216.0
241 216.0
242 216.0
243 216.0
244 216.0
245 216.0
246 216.0
247 216.0
248 216.0
249 216.0
250 216.0
251 216.0
252 216.0
253 216.0
254 2.0
255 2.0
256 152.0
257 152.0
258 152.0
259 152.0
260 152.0
261 152.0
262 152.0
263 152.0
264 152.0
265 152.0
266 152.0
267 152.0
268 152.0
269 152.0
270 152.0
271 152.0
272 152.0
273 152.0
274 152.0
275 152.0
276 152.0
277 152.0
278 152.0
279 152.0
280 152.0
281 152.0
282 152.0
283 152.0
284 152.0
285 152.0;


######################
# SOLVE 
######################
end;
