

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..54;
# Set of Hosts
set Hs := 0..10;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm10_0_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm11_0_{h in Hs}: sum{v in {8}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm11_1_{h in Hs}: sum{v in {9}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_0_{h in Hs}: sum{v in {10}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm13_0_{h in Hs}: sum{v in {11}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm13_1_{h in Hs}: sum{v in {12}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm14_0_{h in Hs}: sum{v in {13}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm14_1_{h in Hs}: sum{v in {14}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm15_0_{h in Hs}: sum{v in {15}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm15_1_{h in Hs}: sum{v in {16}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm16_0_{h in Hs}: sum{v in {17}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm17_0_{h in Hs}: sum{v in {18}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm18_0_{h in Hs}: sum{v in {19}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm19_0_{h in Hs}: sum{v in {20}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm20_0_{h in Hs}: sum{v in {21}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm21_0_{h in Hs}: sum{v in {22..24}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm22_0_{h in Hs}: sum{v in {25..27}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm23_0_{h in Hs}: sum{v in {28..30}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm24_0_{h in Hs}: sum{v in {31..34}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm24_1_{h in Hs}: sum{v in {35..38}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm27_0_{h in Hs}: sum{v in {43,44}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm28_0_{h in Hs}: sum{v in {45,46}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm27_0_{h in Hs}: sum{v in {47,48}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm28_0_{h in Hs}: sum{v in {49,50}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm29_0_{h in Hs}: sum{v in {51,52}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm29_1_{h in Hs}: sum{v in {53,54}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4.0
1 4.0
2 10.0
3 10.0
4 10.0
5 10.0
6 14.0
7 14.0
8 8.0
9 8.0
10 8.0
11 2.0
12 2.0
13 1.0
14 1.0
15 4.0
16 4.0
17 12.0
18 24.0
19 16.0
20 2.0
21 4.0
22 12.0
23 12.0
24 12.0
25 8.0
26 8.0
27 8.0
28 8.0
29 8.0
30 8.0
31 4.0
32 4.0
33 4.0
34 4.0
35 4.0
36 4.0
37 4.0
38 4.0
39 4.0
40 4.0
41 4.0
42 4.0
43 16.0
44 16.0
45 16.0
46 16.0
47 16.0
48 16.0
49 16.0
50 16.0
51 8.0
52 8.0
53 8.0
54 8.0;
param: MEM := 
0 16.0
1 16.0
2 10.0
3 10.0
4 10.0
5 10.0
6 8.0
7 8.0
8 8.0
9 8.0
10 16.0
11 4.0
12 4.0
13 8.0
14 8.0
15 4.0
16 4.0
17 32.0
18 64.0
19 64.0
20 8.0
21 4.0
22 64.0
23 64.0
24 64.0
25 64.0
26 64.0
27 64.0
28 64.0
29 64.0
30 64.0
31 32.0
32 32.0
33 32.0
34 32.0
35 32.0
36 32.0
37 32.0
38 32.0
39 32.0
40 32.0
41 32.0
42 32.0
43 32.0
44 32.0
45 32.0
46 32.0
47 32.0
48 32.0
49 32.0
50 32.0
51 16.0
52 16.0
53 16.0
54 16.0;
param: NET := 
0 40.0
1 40.0
2 126.0
3 126.0
4 126.0
5 126.0
6 40.0
7 40.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 900.0
23 900.0
24 900.0
25 300.0
26 300.0
27 300.0
28 600.0
29 600.0
30 600.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 200.0
44 200.0
45 200.0
46 200.0
47 200.0
48 200.0
49 200.0
50 200.0
51 200.0
52 200.0
53 200.0
54 200.0;


######################
# SOLVE 
######################
end;
