

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..72;
# Set of Hosts
set Hs := 0..14;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm6_0_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm7_0_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm8_0_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm9_0_{h in Hs}: sum{v in {16,17}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm10_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm11_0_{h in Hs}: sum{v in {20}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm11_1_{h in Hs}: sum{v in {21}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_0_{h in Hs}: sum{v in {22}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm13_0_{h in Hs}: sum{v in {23}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm13_1_{h in Hs}: sum{v in {24}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm14_0_{h in Hs}: sum{v in {25}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm14_1_{h in Hs}: sum{v in {26}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm15_0_{h in Hs}: sum{v in {27}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm15_1_{h in Hs}: sum{v in {28}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm16_0_{h in Hs}: sum{v in {29}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm17_0_{h in Hs}: sum{v in {30}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm18_0_{h in Hs}: sum{v in {31}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm19_0_{h in Hs}: sum{v in {32}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm20_0_{h in Hs}: sum{v in {33}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm21_0_{h in Hs}: sum{v in {34..36}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm22_0_{h in Hs}: sum{v in {37..39}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm23_0_{h in Hs}: sum{v in {40..42}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm24_0_{h in Hs}: sum{v in {43..46}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm24_1_{h in Hs}: sum{v in {47..50}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm1_0_{h in Hs}: sum{v in {55,56}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm25_0_{h in Hs}: sum{v in {57,58}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm26_0_{h in Hs}: sum{v in {59}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm26_1_{h in Hs}: sum{v in {60}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm27_0_{h in Hs}: sum{v in {61,62}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm28_0_{h in Hs}: sum{v in {63,64}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm27_0_{h in Hs}: sum{v in {65,66}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm28_0_{h in Hs}: sum{v in {67,68}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm29_0_{h in Hs}: sum{v in {69,70}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm29_1_{h in Hs}: sum{v in {71,72}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4
1 4
2 10
3 10
4 10
5 10
6 10
7 10
8 10
9 10
10 10
11 10
12 10
13 10
14 10
15 10
16 10
17 10
18 14
19 14
20 8
21 8
22 8
23 2
24 2
25 1
26 1
27 4
28 4
29 12
30 24
31 16
32 2
33 4
34 12
35 12
36 12
37 8
38 8
39 8
40 8
41 8
42 8
43 4
44 4
45 4
46 4
47 4
48 4
49 4
50 4
51 4
52 4
53 4
54 4
55 14
56 14
57 14
58 14
59 8
60 8
61 16
62 16
63 16
64 16
65 16
66 16
67 16
68 16
69 8
70 8
71 8
72 8;
param: MEM := 
0 16
1 16
2 10
3 10
4 10
5 10
6 10
7 10
8 10
9 10
10 10
11 10
12 10
13 10
14 10
15 10
16 10
17 10
18 8
19 8
20 8
21 8
22 16
23 4
24 4
25 8
26 8
27 4
28 4
29 32
30 64
31 64
32 8
33 4
34 64
35 64
36 64
37 64
38 64
39 64
40 64
41 64
42 64
43 32
44 32
45 32
46 32
47 32
48 32
49 32
50 32
51 32
52 32
53 32
54 32
55 40
56 40
57 8
58 8
59 32
60 32
61 32
62 32
63 32
64 32
65 32
66 32
67 32
68 32
69 16
70 16
71 16
72 16;
param: NET := 
0 40
1 40
2 126
3 126
4 126
5 126
6 126
7 126
8 126
9 126
10 126
11 126
12 126
13 126
14 126
15 126
16 126
17 126
18 40
19 40
20 0
21 0
22 0
23 0
24 0
25 0
26 0
27 0
28 0
29 0
30 0
31 0
32 0
33 0
34 900
35 900
36 900
37 300
38 300
39 300
40 600
41 600
42 600
43 0
44 0
45 0
46 0
47 0
48 0
49 0
50 0
51 0
52 0
53 0
54 0
55 300
56 300
57 20
58 20
59 0
60 0
61 200
62 200
63 200
64 200
65 200
66 200
67 200
68 200
69 200
70 200
71 200
72 200;


######################
# SOLVE 
######################
end;
