

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..30;
# Set of Hosts
set Hs := 0..5;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {10..15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {16,17}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm6_0_{h in Hs}: sum{v in {20,21}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {22}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_1_{h in Hs}: sum{v in {23}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {24}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm1_0_{h in Hs}: sum{v in {25,26}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {27,28}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {2..5}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {2..5}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {6..9}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {6..9}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 2.0
17 2.0
18 4.0
19 4.0
20 2.0
21 2.0
22 8.0
23 8.0
24 8.0
25 14.0
26 14.0
27 14.0
28 14.0
29 8.0
30 8.0;
param: MEM := 
0 54.0
1 54.0
2 54.0
3 54.0
4 54.0
5 54.0
6 54.0
7 54.0
8 54.0
9 54.0
10 56.0
11 56.0
12 56.0
13 56.0
14 56.0
15 56.0
16 4.0
17 4.0
18 8.0
19 8.0
20 4.0
21 4.0
22 8.0
23 8.0
24 16.0
25 48.0
26 48.0
27 48.0
28 48.0
29 32.0
30 32.0;
param: NET := 
0 36.0
1 36.0
2 87.0
3 87.0
4 87.0
5 87.0
6 87.0
7 87.0
8 87.0
9 87.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0;


######################
# SOLVE 
######################
end;
