

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..39;
# Set of Hosts
set Hs := 0..7;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {10..15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {16,17}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm6_0_{h in Hs}: sum{v in {20,21}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {22}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_1_{h in Hs}: sum{v in {23}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {24}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm1_0_{h in Hs}: sum{v in {25,26}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {27,28}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm6_0_{h in Hs}: sum{v in {31..33}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm4_0_{h in Hs}: sum{v in {34,35}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm9_0_{h in Hs}: sum{v in {36,37}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm10_0_{h in Hs}: sum{v in {38,39}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {2..5}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {2..5}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {6..9}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {6..9}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 2
17 2
18 4
19 4
20 2
21 2
22 8
23 8
24 8
25 14
26 14
27 14
28 14
29 8
30 8
31 6
32 6
33 6
34 16
35 16
36 2
37 2
38 10
39 10;
param: MEM := 
0 54
1 54
2 54
3 54
4 54
5 54
6 54
7 54
8 54
9 54
10 56
11 56
12 56
13 56
14 56
15 56
16 4
17 4
18 8
19 8
20 4
21 4
22 8
23 8
24 16
25 48
26 48
27 48
28 48
29 32
30 32
31 8
32 8
33 8
34 56
35 56
36 4
37 4
38 20
39 20;
param: NET := 
0 36
1 36
2 87
3 87
4 87
5 87
6 87
7 87
8 87
9 87
10 0
11 0
12 0
13 0
14 0
15 0
16 0
17 0
18 0
19 0
20 0
21 0
22 0
23 0
24 0
25 0
26 0
27 0
28 0
29 0
30 0
31 0
32 0
33 0
34 0
35 0
36 0
37 0
38 0
39 0;


######################
# SOLVE 
######################
end;
