

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..143;
# Set of Hosts
set Hs := 0..25;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {1,2}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {3..9}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_0_{h in Hs}: sum{v in {11,12}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm6_0_{h in Hs}: sum{v in {13,14}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_0_{h in Hs}: sum{v in {17..20}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm10_0_{h in Hs}: sum{v in {21..24}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm11_0_{h in Hs}: sum{v in {25,26}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm12_0_{h in Hs}: sum{v in {27,28}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm13_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm14_0_{h in Hs}: sum{v in {31..34}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm19_0_{h in Hs}: sum{v in {95..97}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm20_0_{h in Hs}: sum{v in {98..100}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm21_0_{h in Hs}: sum{v in {101..103}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm22_0_{h in Hs}: sum{v in {104..106}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm23_0_{h in Hs}: sum{v in {107..109}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm24_0_{h in Hs}: sum{v in {110..117}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm25_0_{h in Hs}: sum{v in {118,119}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm26_0_{h in Hs}: sum{v in {120,121}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm27_0_{h in Hs}: sum{v in {122..124}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm29_0_{h in Hs}: sum{v in {127..129}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm30_0_{h in Hs}: sum{v in {130,131}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm33_0_{h in Hs}: sum{v in {137..142}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm32_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm32_0_master{h in Hs}: ni_vnf9_dot_vm32_0_master[h] <= sum{v in {135}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm32_0_master{v in {135}, h in Hs}: ni_vnf9_dot_vm32_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm32_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm32_0_slave{h in Hs}: ni_vnf9_dot_vm32_0_slave[h] <= sum{v in {136}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm32_0_slave{v in {136}, h in Hs}: ni_vnf9_dot_vm32_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf9_dot_vm32_0_master[h] + ni_vnf9_dot_vm32_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 2.0
1 3.0
2 3.0
3 16.0
4 16.0
5 16.0
6 16.0
7 16.0
8 16.0
9 16.0
10 6.0
11 3.0
12 3.0
13 5.0
14 5.0
15 20.0
16 6.0
17 4.0
18 4.0
19 4.0
20 4.0
21 1.0
22 1.0
23 1.0
24 1.0
25 4.0
26 4.0
27 24.0
28 24.0
29 14.0
30 14.0
31 8.0
32 8.0
33 8.0
34 8.0
35 1.0
36 1.0
37 8.0
38 8.0
39 8.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 8.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 8.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0
83 8.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 8.0
93 8.0
94 8.0
95 4.0
96 4.0
97 4.0
98 4.0
99 4.0
100 4.0
101 4.0
102 4.0
103 4.0
104 4.0
105 4.0
106 4.0
107 4.0
108 4.0
109 4.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 8.0
117 8.0
118 4.0
119 4.0
120 4.0
121 4.0
122 4.0
123 4.0
124 4.0
125 4.0
126 4.0
127 8.0
128 8.0
129 8.0
130 8.0
131 8.0
132 8.0
133 8.0
134 8.0
135 14.0
136 14.0
137 14.0
138 14.0
139 14.0
140 14.0
141 14.0
142 14.0
143 0.0;
param: MEM := 
0 5.0
1 6.0
2 6.0
3 17.0
4 17.0
5 17.0
6 17.0
7 17.0
8 17.0
9 17.0
10 12.0
11 5.0
12 5.0
13 8.0
14 8.0
15 23.0
16 10.0
17 8.0
18 8.0
19 8.0
20 8.0
21 4.0
22 4.0
23 4.0
24 4.0
25 6.0
26 6.0
27 32.0
28 32.0
29 16.0
30 16.0
31 8.0
32 8.0
33 8.0
34 8.0
35 1.0
36 2.0
37 32.0
38 32.0
39 32.0
40 32.0
41 32.0
42 32.0
43 32.0
44 32.0
45 32.0
46 32.0
47 32.0
48 32.0
49 32.0
50 32.0
51 32.0
52 32.0
53 32.0
54 32.0
55 32.0
56 32.0
57 32.0
58 32.0
59 32.0
60 32.0
61 32.0
62 32.0
63 32.0
64 32.0
65 32.0
66 32.0
67 32.0
68 32.0
69 32.0
70 32.0
71 32.0
72 32.0
73 32.0
74 32.0
75 32.0
76 32.0
77 32.0
78 32.0
79 32.0
80 32.0
81 32.0
82 32.0
83 32.0
84 32.0
85 32.0
86 32.0
87 32.0
88 32.0
89 32.0
90 32.0
91 32.0
92 32.0
93 32.0
94 32.0
95 16.0
96 16.0
97 16.0
98 8.0
99 8.0
100 8.0
101 16.0
102 16.0
103 16.0
104 16.0
105 16.0
106 16.0
107 16.0
108 16.0
109 16.0
110 16.0
111 16.0
112 16.0
113 16.0
114 16.0
115 16.0
116 16.0
117 16.0
118 8.0
119 8.0
120 8.0
121 8.0
122 16.0
123 16.0
124 16.0
125 8.0
126 8.0
127 32.0
128 32.0
129 32.0
130 32.0
131 32.0
132 64.0
133 64.0
134 64.0
135 12.0
136 12.0
137 12.0
138 12.0
139 12.0
140 12.0
141 12.0
142 12.0
143 0.0;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0
143 0.0;


######################
# SOLVE 
######################
end;
