

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..44;
# Set of Hosts
set Hs := 0..8;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2..10}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {11..19}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_0_{h in Hs}: sum{v in {20..24}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_0_{h in Hs}: sum{v in {25..27}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm7_0_{h in Hs}: sum{v in {30..35}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm11_0_{h in Hs}: sum{v in {42..44}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_master{h in Hs}: ni_vnf1_dot_vm6_0_master[h] <= sum{v in {28}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_master{v in {28}, h in Hs}: ni_vnf1_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_slave{h in Hs}: ni_vnf1_dot_vm6_0_slave[h] <= sum{v in {29}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_slave{v in {29}, h in Hs}: ni_vnf1_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm6_0_master[h] + ni_vnf1_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm8_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm8_0_master{h in Hs}: ni_vnf1_dot_vm8_0_master[h] <= sum{v in {36}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm8_0_master{v in {36}, h in Hs}: ni_vnf1_dot_vm8_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm8_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm8_0_slave{h in Hs}: ni_vnf1_dot_vm8_0_slave[h] <= sum{v in {37}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm8_0_slave{v in {37}, h in Hs}: ni_vnf1_dot_vm8_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf1_dot_vm8_0_master[h] + ni_vnf1_dot_vm8_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm9_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm9_0_master{h in Hs}: ni_vnf1_dot_vm9_0_master[h] <= sum{v in {38}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm9_0_master{v in {38}, h in Hs}: ni_vnf1_dot_vm9_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm9_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm9_0_slave{h in Hs}: ni_vnf1_dot_vm9_0_slave[h] <= sum{v in {39}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm9_0_slave{v in {39}, h in Hs}: ni_vnf1_dot_vm9_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf1_dot_vm9_0_master[h] + ni_vnf1_dot_vm9_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm10_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm10_0_master{h in Hs}: ni_vnf1_dot_vm10_0_master[h] <= sum{v in {40}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm10_0_master{v in {40}, h in Hs}: ni_vnf1_dot_vm10_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm10_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm10_0_slave{h in Hs}: ni_vnf1_dot_vm10_0_slave[h] <= sum{v in {41}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm10_0_slave{v in {41}, h in Hs}: ni_vnf1_dot_vm10_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf1_dot_vm10_0_master[h] + ni_vnf1_dot_vm10_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 4.0
3 4.0
4 4.0
5 4.0
6 4.0
7 4.0
8 4.0
9 4.0
10 4.0
11 4.0
12 4.0
13 4.0
14 4.0
15 4.0
16 4.0
17 4.0
18 4.0
19 4.0
20 2.0
21 2.0
22 2.0
23 2.0
24 2.0
25 4.0
26 4.0
27 4.0
28 2.0
29 2.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 2.0
39 2.0
40 2.0
41 2.0
42 4.0
43 4.0
44 4.0;
param: MEM := 
0 32.0
1 32.0
2 12.0
3 12.0
4 12.0
5 12.0
6 12.0
7 12.0
8 12.0
9 12.0
10 12.0
11 12.0
12 12.0
13 12.0
14 12.0
15 12.0
16 12.0
17 12.0
18 12.0
19 12.0
20 24.0
21 24.0
22 24.0
23 24.0
24 24.0
25 24.0
26 24.0
27 24.0
28 8.0
29 8.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 16.0
37 16.0
38 8.0
39 8.0
40 10.0
41 10.0
42 40.0
43 40.0
44 40.0;
param: NET := 
0 339.0
1 339.0
2 433.0
3 433.0
4 433.0
5 433.0
6 433.0
7 433.0
8 433.0
9 433.0
10 433.0
11 433.0
12 433.0
13 433.0
14 433.0
15 433.0
16 433.0
17 433.0
18 433.0
19 433.0
20 35.0
21 35.0
22 35.0
23 35.0
24 35.0
25 40.0
26 40.0
27 40.0
28 31.0
29 31.0
30 47.0
31 47.0
32 47.0
33 47.0
34 47.0
35 47.0
36 1292.0
37 1292.0
38 72.0
39 72.0
40 10.0
41 10.0
42 104.0
43 104.0
44 104.0;


######################
# SOLVE 
######################
end;
