

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..293;
# Set of Hosts
set Hs := 0..33;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2..15}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {16..29}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_0_{h in Hs}: sum{v in {30..36}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_0_{h in Hs}: sum{v in {37..40}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm7_0_{h in Hs}: sum{v in {45..54}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm11_0_{h in Hs}: sum{v in {63..66}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm19_0_{h in Hs}: sum{v in {165..168}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm19_1_{h in Hs}: sum{v in {169..172}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm19_2_{h in Hs}: sum{v in {173..176}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm19_3_{h in Hs}: sum{v in {177..180}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm21_0_{h in Hs}: sum{v in {189..198}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm21_1_{h in Hs}: sum{v in {199..208}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm21_2_{h in Hs}: sum{v in {209..218}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm21_3_{h in Hs}: sum{v in {219..228}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {231..234}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm4_0_{h in Hs}: sum{v in {235,236}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm7_0_{h in Hs}: sum{v in {239,240}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_0_{h in Hs}: sum{v in {243,244}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm11_0_{h in Hs}: sum{v in {249,250}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm2_0_{h in Hs}: sum{v in {253..256}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm23_0_{h in Hs}: sum{v in {257..269}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm24_0_{h in Hs}: sum{v in {270..277}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm6_0_{h in Hs}: sum{v in {278..283}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm11_0_{h in Hs}: sum{v in {292,293}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_master{h in Hs}: ni_vnf1_dot_vm6_0_master[h] <= sum{v in {41,42}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_master{v in {41,42}, h in Hs}: ni_vnf1_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_slave{h in Hs}: ni_vnf1_dot_vm6_0_slave[h] <= sum{v in {43,44}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_slave{v in {43,44}, h in Hs}: ni_vnf1_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm6_0_master[h] + ni_vnf1_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm8_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm8_0_master{h in Hs}: ni_vnf1_dot_vm8_0_master[h] <= sum{v in {55}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm8_0_master{v in {55}, h in Hs}: ni_vnf1_dot_vm8_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm8_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm8_0_slave{h in Hs}: ni_vnf1_dot_vm8_0_slave[h] <= sum{v in {56}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm8_0_slave{v in {56}, h in Hs}: ni_vnf1_dot_vm8_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf1_dot_vm8_0_master[h] + ni_vnf1_dot_vm8_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm9_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm9_0_master{h in Hs}: ni_vnf1_dot_vm9_0_master[h] <= sum{v in {57,58}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm9_0_master{v in {57,58}, h in Hs}: ni_vnf1_dot_vm9_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm9_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm9_0_slave{h in Hs}: ni_vnf1_dot_vm9_0_slave[h] <= sum{v in {59,60}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm9_0_slave{v in {59,60}, h in Hs}: ni_vnf1_dot_vm9_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf1_dot_vm9_0_master[h] + ni_vnf1_dot_vm9_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm10_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm10_0_master{h in Hs}: ni_vnf1_dot_vm10_0_master[h] <= sum{v in {61}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm10_0_master{v in {61}, h in Hs}: ni_vnf1_dot_vm10_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm10_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm10_0_slave{h in Hs}: ni_vnf1_dot_vm10_0_slave[h] <= sum{v in {62}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm10_0_slave{v in {62}, h in Hs}: ni_vnf1_dot_vm10_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf1_dot_vm10_0_master[h] + ni_vnf1_dot_vm10_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm12_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm12_0_master{h in Hs}: ni_vnf2_dot_vm12_0_master[h] <= sum{v in {67..71}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm12_0_master{v in {67..71}, h in Hs}: ni_vnf2_dot_vm12_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm12_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm12_0_slave{h in Hs}: ni_vnf2_dot_vm12_0_slave[h] <= sum{v in {72..76}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm12_0_slave{v in {72..76}, h in Hs}: ni_vnf2_dot_vm12_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf2_dot_vm12_0_master[h] + ni_vnf2_dot_vm12_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm12_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm12_1_master{h in Hs}: ni_vnf2_dot_vm12_1_master[h] <= sum{v in {77..81}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm12_1_master{v in {77..81}, h in Hs}: ni_vnf2_dot_vm12_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm12_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm12_1_slave{h in Hs}: ni_vnf2_dot_vm12_1_slave[h] <= sum{v in {82..86}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm12_1_slave{v in {82..86}, h in Hs}: ni_vnf2_dot_vm12_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_6_{h in Hs}: ni_vnf2_dot_vm12_1_master[h] + ni_vnf2_dot_vm12_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm12_2_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm12_2_master{h in Hs}: ni_vnf2_dot_vm12_2_master[h] <= sum{v in {87..91}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm12_2_master{v in {87..91}, h in Hs}: ni_vnf2_dot_vm12_2_master[h] >= x[v, h];
var ni_vnf2_dot_vm12_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm12_2_slave{h in Hs}: ni_vnf2_dot_vm12_2_slave[h] <= sum{v in {92..96}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm12_2_slave{v in {92..96}, h in Hs}: ni_vnf2_dot_vm12_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_7_{h in Hs}: ni_vnf2_dot_vm12_2_master[h] + ni_vnf2_dot_vm12_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm13_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm13_0_master{h in Hs}: ni_vnf2_dot_vm13_0_master[h] <= sum{v in {97}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm13_0_master{v in {97}, h in Hs}: ni_vnf2_dot_vm13_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm13_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm13_0_slave{h in Hs}: ni_vnf2_dot_vm13_0_slave[h] <= sum{v in {98}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm13_0_slave{v in {98}, h in Hs}: ni_vnf2_dot_vm13_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_8_{h in Hs}: ni_vnf2_dot_vm13_0_master[h] + ni_vnf2_dot_vm13_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm13_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm13_1_master{h in Hs}: ni_vnf2_dot_vm13_1_master[h] <= sum{v in {99}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm13_1_master{v in {99}, h in Hs}: ni_vnf2_dot_vm13_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm13_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm13_1_slave{h in Hs}: ni_vnf2_dot_vm13_1_slave[h] <= sum{v in {100}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm13_1_slave{v in {100}, h in Hs}: ni_vnf2_dot_vm13_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_9_{h in Hs}: ni_vnf2_dot_vm13_1_master[h] + ni_vnf2_dot_vm13_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm13_2_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm13_2_master{h in Hs}: ni_vnf2_dot_vm13_2_master[h] <= sum{v in {101}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm13_2_master{v in {101}, h in Hs}: ni_vnf2_dot_vm13_2_master[h] >= x[v, h];
var ni_vnf2_dot_vm13_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm13_2_slave{h in Hs}: ni_vnf2_dot_vm13_2_slave[h] <= sum{v in {102}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm13_2_slave{v in {102}, h in Hs}: ni_vnf2_dot_vm13_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_10_{h in Hs}: ni_vnf2_dot_vm13_2_master[h] + ni_vnf2_dot_vm13_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm14_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm14_0_master{h in Hs}: ni_vnf2_dot_vm14_0_master[h] <= sum{v in {103}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm14_0_master{v in {103}, h in Hs}: ni_vnf2_dot_vm14_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm14_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm14_0_slave{h in Hs}: ni_vnf2_dot_vm14_0_slave[h] <= sum{v in {104}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm14_0_slave{v in {104}, h in Hs}: ni_vnf2_dot_vm14_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_11_{h in Hs}: ni_vnf2_dot_vm14_0_master[h] + ni_vnf2_dot_vm14_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm14_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm14_1_master{h in Hs}: ni_vnf2_dot_vm14_1_master[h] <= sum{v in {105}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm14_1_master{v in {105}, h in Hs}: ni_vnf2_dot_vm14_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm14_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm14_1_slave{h in Hs}: ni_vnf2_dot_vm14_1_slave[h] <= sum{v in {106}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm14_1_slave{v in {106}, h in Hs}: ni_vnf2_dot_vm14_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_12_{h in Hs}: ni_vnf2_dot_vm14_1_master[h] + ni_vnf2_dot_vm14_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm14_2_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm14_2_master{h in Hs}: ni_vnf2_dot_vm14_2_master[h] <= sum{v in {107}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm14_2_master{v in {107}, h in Hs}: ni_vnf2_dot_vm14_2_master[h] >= x[v, h];
var ni_vnf2_dot_vm14_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm14_2_slave{h in Hs}: ni_vnf2_dot_vm14_2_slave[h] <= sum{v in {108}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm14_2_slave{v in {108}, h in Hs}: ni_vnf2_dot_vm14_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_13_{h in Hs}: ni_vnf2_dot_vm14_2_master[h] + ni_vnf2_dot_vm14_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm15_0_master{h in Hs}: ni_vnf2_dot_vm15_0_master[h] <= sum{v in {109}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm15_0_master{v in {109}, h in Hs}: ni_vnf2_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm15_0_slave{h in Hs}: ni_vnf2_dot_vm15_0_slave[h] <= sum{v in {110}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm15_0_slave{v in {110}, h in Hs}: ni_vnf2_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_14_{h in Hs}: ni_vnf2_dot_vm15_0_master[h] + ni_vnf2_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm15_1_master{h in Hs}: ni_vnf2_dot_vm15_1_master[h] <= sum{v in {111}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm15_1_master{v in {111}, h in Hs}: ni_vnf2_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm15_1_slave{h in Hs}: ni_vnf2_dot_vm15_1_slave[h] <= sum{v in {112}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm15_1_slave{v in {112}, h in Hs}: ni_vnf2_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_15_{h in Hs}: ni_vnf2_dot_vm15_1_master[h] + ni_vnf2_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm15_2_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm15_2_master{h in Hs}: ni_vnf2_dot_vm15_2_master[h] <= sum{v in {113}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm15_2_master{v in {113}, h in Hs}: ni_vnf2_dot_vm15_2_master[h] >= x[v, h];
var ni_vnf2_dot_vm15_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm15_2_slave{h in Hs}: ni_vnf2_dot_vm15_2_slave[h] <= sum{v in {114}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm15_2_slave{v in {114}, h in Hs}: ni_vnf2_dot_vm15_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_16_{h in Hs}: ni_vnf2_dot_vm15_2_master[h] + ni_vnf2_dot_vm15_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm16_0_master{h in Hs}: ni_vnf2_dot_vm16_0_master[h] <= sum{v in {115}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm16_0_master{v in {115}, h in Hs}: ni_vnf2_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm16_0_slave{h in Hs}: ni_vnf2_dot_vm16_0_slave[h] <= sum{v in {116}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm16_0_slave{v in {116}, h in Hs}: ni_vnf2_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_17_{h in Hs}: ni_vnf2_dot_vm16_0_master[h] + ni_vnf2_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm16_1_master{h in Hs}: ni_vnf2_dot_vm16_1_master[h] <= sum{v in {117}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm16_1_master{v in {117}, h in Hs}: ni_vnf2_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm16_1_slave{h in Hs}: ni_vnf2_dot_vm16_1_slave[h] <= sum{v in {118}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm16_1_slave{v in {118}, h in Hs}: ni_vnf2_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_18_{h in Hs}: ni_vnf2_dot_vm16_1_master[h] + ni_vnf2_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm16_2_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm16_2_master{h in Hs}: ni_vnf2_dot_vm16_2_master[h] <= sum{v in {119}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm16_2_master{v in {119}, h in Hs}: ni_vnf2_dot_vm16_2_master[h] >= x[v, h];
var ni_vnf2_dot_vm16_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm16_2_slave{h in Hs}: ni_vnf2_dot_vm16_2_slave[h] <= sum{v in {120}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm16_2_slave{v in {120}, h in Hs}: ni_vnf2_dot_vm16_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_19_{h in Hs}: ni_vnf2_dot_vm16_2_master[h] + ni_vnf2_dot_vm16_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm17_0_master{h in Hs}: ni_vnf2_dot_vm17_0_master[h] <= sum{v in {121}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm17_0_master{v in {121}, h in Hs}: ni_vnf2_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm17_0_slave{h in Hs}: ni_vnf2_dot_vm17_0_slave[h] <= sum{v in {122}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm17_0_slave{v in {122}, h in Hs}: ni_vnf2_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_20_{h in Hs}: ni_vnf2_dot_vm17_0_master[h] + ni_vnf2_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm17_1_master{h in Hs}: ni_vnf2_dot_vm17_1_master[h] <= sum{v in {123}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm17_1_master{v in {123}, h in Hs}: ni_vnf2_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm17_1_slave{h in Hs}: ni_vnf2_dot_vm17_1_slave[h] <= sum{v in {124}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm17_1_slave{v in {124}, h in Hs}: ni_vnf2_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_21_{h in Hs}: ni_vnf2_dot_vm17_1_master[h] + ni_vnf2_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm17_2_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm17_2_master{h in Hs}: ni_vnf2_dot_vm17_2_master[h] <= sum{v in {125}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm17_2_master{v in {125}, h in Hs}: ni_vnf2_dot_vm17_2_master[h] >= x[v, h];
var ni_vnf2_dot_vm17_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm17_2_slave{h in Hs}: ni_vnf2_dot_vm17_2_slave[h] <= sum{v in {126}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm17_2_slave{v in {126}, h in Hs}: ni_vnf2_dot_vm17_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_22_{h in Hs}: ni_vnf2_dot_vm17_2_master[h] + ni_vnf2_dot_vm17_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm18_0_master{h in Hs}: ni_vnf2_dot_vm18_0_master[h] <= sum{v in {127}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm18_0_master{v in {127}, h in Hs}: ni_vnf2_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm18_0_slave{h in Hs}: ni_vnf2_dot_vm18_0_slave[h] <= sum{v in {128}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm18_0_slave{v in {128}, h in Hs}: ni_vnf2_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_23_{h in Hs}: ni_vnf2_dot_vm18_0_master[h] + ni_vnf2_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm18_1_master{h in Hs}: ni_vnf2_dot_vm18_1_master[h] <= sum{v in {129}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm18_1_master{v in {129}, h in Hs}: ni_vnf2_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm18_1_slave{h in Hs}: ni_vnf2_dot_vm18_1_slave[h] <= sum{v in {130}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm18_1_slave{v in {130}, h in Hs}: ni_vnf2_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_24_{h in Hs}: ni_vnf2_dot_vm18_1_master[h] + ni_vnf2_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm18_2_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm18_2_master{h in Hs}: ni_vnf2_dot_vm18_2_master[h] <= sum{v in {131}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm18_2_master{v in {131}, h in Hs}: ni_vnf2_dot_vm18_2_master[h] >= x[v, h];
var ni_vnf2_dot_vm18_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm18_2_slave{h in Hs}: ni_vnf2_dot_vm18_2_slave[h] <= sum{v in {132}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm18_2_slave{v in {132}, h in Hs}: ni_vnf2_dot_vm18_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_25_{h in Hs}: ni_vnf2_dot_vm18_2_master[h] + ni_vnf2_dot_vm18_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm12_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm12_0_master{h in Hs}: ni_vnf3_dot_vm12_0_master[h] <= sum{v in {133,134}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm12_0_master{v in {133,134}, h in Hs}: ni_vnf3_dot_vm12_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm12_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm12_0_slave{h in Hs}: ni_vnf3_dot_vm12_0_slave[h] <= sum{v in {135,136}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm12_0_slave{v in {135,136}, h in Hs}: ni_vnf3_dot_vm12_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_26_{h in Hs}: ni_vnf3_dot_vm12_0_master[h] + ni_vnf3_dot_vm12_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm12_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm12_1_master{h in Hs}: ni_vnf3_dot_vm12_1_master[h] <= sum{v in {137,138}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm12_1_master{v in {137,138}, h in Hs}: ni_vnf3_dot_vm12_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm12_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm12_1_slave{h in Hs}: ni_vnf3_dot_vm12_1_slave[h] <= sum{v in {139,140}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm12_1_slave{v in {139,140}, h in Hs}: ni_vnf3_dot_vm12_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_27_{h in Hs}: ni_vnf3_dot_vm12_1_master[h] + ni_vnf3_dot_vm12_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm13_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm13_0_master{h in Hs}: ni_vnf3_dot_vm13_0_master[h] <= sum{v in {141}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm13_0_master{v in {141}, h in Hs}: ni_vnf3_dot_vm13_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm13_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm13_0_slave{h in Hs}: ni_vnf3_dot_vm13_0_slave[h] <= sum{v in {142}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm13_0_slave{v in {142}, h in Hs}: ni_vnf3_dot_vm13_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_28_{h in Hs}: ni_vnf3_dot_vm13_0_master[h] + ni_vnf3_dot_vm13_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm13_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm13_1_master{h in Hs}: ni_vnf3_dot_vm13_1_master[h] <= sum{v in {143}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm13_1_master{v in {143}, h in Hs}: ni_vnf3_dot_vm13_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm13_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm13_1_slave{h in Hs}: ni_vnf3_dot_vm13_1_slave[h] <= sum{v in {144}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm13_1_slave{v in {144}, h in Hs}: ni_vnf3_dot_vm13_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_29_{h in Hs}: ni_vnf3_dot_vm13_1_master[h] + ni_vnf3_dot_vm13_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm14_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm14_0_master{h in Hs}: ni_vnf3_dot_vm14_0_master[h] <= sum{v in {145}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm14_0_master{v in {145}, h in Hs}: ni_vnf3_dot_vm14_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm14_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm14_0_slave{h in Hs}: ni_vnf3_dot_vm14_0_slave[h] <= sum{v in {146}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm14_0_slave{v in {146}, h in Hs}: ni_vnf3_dot_vm14_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_30_{h in Hs}: ni_vnf3_dot_vm14_0_master[h] + ni_vnf3_dot_vm14_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm14_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm14_1_master{h in Hs}: ni_vnf3_dot_vm14_1_master[h] <= sum{v in {147}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm14_1_master{v in {147}, h in Hs}: ni_vnf3_dot_vm14_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm14_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm14_1_slave{h in Hs}: ni_vnf3_dot_vm14_1_slave[h] <= sum{v in {148}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm14_1_slave{v in {148}, h in Hs}: ni_vnf3_dot_vm14_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_31_{h in Hs}: ni_vnf3_dot_vm14_1_master[h] + ni_vnf3_dot_vm14_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_0_master{h in Hs}: ni_vnf3_dot_vm15_0_master[h] <= sum{v in {149}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_0_master{v in {149}, h in Hs}: ni_vnf3_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_0_slave{h in Hs}: ni_vnf3_dot_vm15_0_slave[h] <= sum{v in {150}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_0_slave{v in {150}, h in Hs}: ni_vnf3_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_32_{h in Hs}: ni_vnf3_dot_vm15_0_master[h] + ni_vnf3_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_1_master{h in Hs}: ni_vnf3_dot_vm15_1_master[h] <= sum{v in {151}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_1_master{v in {151}, h in Hs}: ni_vnf3_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_1_slave{h in Hs}: ni_vnf3_dot_vm15_1_slave[h] <= sum{v in {152}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_1_slave{v in {152}, h in Hs}: ni_vnf3_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_33_{h in Hs}: ni_vnf3_dot_vm15_1_master[h] + ni_vnf3_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_0_master{h in Hs}: ni_vnf3_dot_vm16_0_master[h] <= sum{v in {153}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_0_master{v in {153}, h in Hs}: ni_vnf3_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_0_slave{h in Hs}: ni_vnf3_dot_vm16_0_slave[h] <= sum{v in {154}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_0_slave{v in {154}, h in Hs}: ni_vnf3_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_34_{h in Hs}: ni_vnf3_dot_vm16_0_master[h] + ni_vnf3_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_1_master{h in Hs}: ni_vnf3_dot_vm16_1_master[h] <= sum{v in {155}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_1_master{v in {155}, h in Hs}: ni_vnf3_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_1_slave{h in Hs}: ni_vnf3_dot_vm16_1_slave[h] <= sum{v in {156}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_1_slave{v in {156}, h in Hs}: ni_vnf3_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_35_{h in Hs}: ni_vnf3_dot_vm16_1_master[h] + ni_vnf3_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_0_master{h in Hs}: ni_vnf3_dot_vm17_0_master[h] <= sum{v in {157}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_0_master{v in {157}, h in Hs}: ni_vnf3_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_0_slave{h in Hs}: ni_vnf3_dot_vm17_0_slave[h] <= sum{v in {158}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_0_slave{v in {158}, h in Hs}: ni_vnf3_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_36_{h in Hs}: ni_vnf3_dot_vm17_0_master[h] + ni_vnf3_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_1_master{h in Hs}: ni_vnf3_dot_vm17_1_master[h] <= sum{v in {159}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_1_master{v in {159}, h in Hs}: ni_vnf3_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_1_slave{h in Hs}: ni_vnf3_dot_vm17_1_slave[h] <= sum{v in {160}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_1_slave{v in {160}, h in Hs}: ni_vnf3_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_37_{h in Hs}: ni_vnf3_dot_vm17_1_master[h] + ni_vnf3_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_0_master{h in Hs}: ni_vnf3_dot_vm18_0_master[h] <= sum{v in {161}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_0_master{v in {161}, h in Hs}: ni_vnf3_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_0_slave{h in Hs}: ni_vnf3_dot_vm18_0_slave[h] <= sum{v in {162}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_0_slave{v in {162}, h in Hs}: ni_vnf3_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_38_{h in Hs}: ni_vnf3_dot_vm18_0_master[h] + ni_vnf3_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_1_master{h in Hs}: ni_vnf3_dot_vm18_1_master[h] <= sum{v in {163}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_1_master{v in {163}, h in Hs}: ni_vnf3_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_1_slave{h in Hs}: ni_vnf3_dot_vm18_1_slave[h] <= sum{v in {164}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_1_slave{v in {164}, h in Hs}: ni_vnf3_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_39_{h in Hs}: ni_vnf3_dot_vm18_1_master[h] + ni_vnf3_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf4_dot_vm20_0_master{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_0_master{h in Hs}: ni_vnf4_dot_vm20_0_master[h] <= sum{v in {181}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_0_master{v in {181}, h in Hs}: ni_vnf4_dot_vm20_0_master[h] >= x[v, h];
var ni_vnf4_dot_vm20_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_0_slave{h in Hs}: ni_vnf4_dot_vm20_0_slave[h] <= sum{v in {182}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_0_slave{v in {182}, h in Hs}: ni_vnf4_dot_vm20_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_40_{h in Hs}: ni_vnf4_dot_vm20_0_master[h] + ni_vnf4_dot_vm20_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf4_dot_vm20_1_master{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_1_master{h in Hs}: ni_vnf4_dot_vm20_1_master[h] <= sum{v in {183}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_1_master{v in {183}, h in Hs}: ni_vnf4_dot_vm20_1_master[h] >= x[v, h];
var ni_vnf4_dot_vm20_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_1_slave{h in Hs}: ni_vnf4_dot_vm20_1_slave[h] <= sum{v in {184}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_1_slave{v in {184}, h in Hs}: ni_vnf4_dot_vm20_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_41_{h in Hs}: ni_vnf4_dot_vm20_1_master[h] + ni_vnf4_dot_vm20_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf4_dot_vm20_2_master{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_2_master{h in Hs}: ni_vnf4_dot_vm20_2_master[h] <= sum{v in {185}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_2_master{v in {185}, h in Hs}: ni_vnf4_dot_vm20_2_master[h] >= x[v, h];
var ni_vnf4_dot_vm20_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_2_slave{h in Hs}: ni_vnf4_dot_vm20_2_slave[h] <= sum{v in {186}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_2_slave{v in {186}, h in Hs}: ni_vnf4_dot_vm20_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_42_{h in Hs}: ni_vnf4_dot_vm20_2_master[h] + ni_vnf4_dot_vm20_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf4_dot_vm20_3_master{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_3_master{h in Hs}: ni_vnf4_dot_vm20_3_master[h] <= sum{v in {187}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_3_master{v in {187}, h in Hs}: ni_vnf4_dot_vm20_3_master[h] >= x[v, h];
var ni_vnf4_dot_vm20_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm20_3_slave{h in Hs}: ni_vnf4_dot_vm20_3_slave[h] <= sum{v in {188}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm20_3_slave{v in {188}, h in Hs}: ni_vnf4_dot_vm20_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_43_{h in Hs}: ni_vnf4_dot_vm20_3_master[h] + ni_vnf4_dot_vm20_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm1_0_master{h in Hs}: ni_vnf5_dot_vm1_0_master[h] <= sum{v in {229}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm1_0_master{v in {229}, h in Hs}: ni_vnf5_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm1_0_slave{h in Hs}: ni_vnf5_dot_vm1_0_slave[h] <= sum{v in {230}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm1_0_slave{v in {230}, h in Hs}: ni_vnf5_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_44_{h in Hs}: ni_vnf5_dot_vm1_0_master[h] + ni_vnf5_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm6_0_master{h in Hs}: ni_vnf5_dot_vm6_0_master[h] <= sum{v in {237}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm6_0_master{v in {237}, h in Hs}: ni_vnf5_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm6_0_slave{h in Hs}: ni_vnf5_dot_vm6_0_slave[h] <= sum{v in {238}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm6_0_slave{v in {238}, h in Hs}: ni_vnf5_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_45_{h in Hs}: ni_vnf5_dot_vm6_0_master[h] + ni_vnf5_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm22_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm22_0_master{h in Hs}: ni_vnf5_dot_vm22_0_master[h] <= sum{v in {241}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm22_0_master{v in {241}, h in Hs}: ni_vnf5_dot_vm22_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm22_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm22_0_slave{h in Hs}: ni_vnf5_dot_vm22_0_slave[h] <= sum{v in {242}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm22_0_slave{v in {242}, h in Hs}: ni_vnf5_dot_vm22_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_46_{h in Hs}: ni_vnf5_dot_vm22_0_master[h] + ni_vnf5_dot_vm22_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm9_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm9_0_master{h in Hs}: ni_vnf5_dot_vm9_0_master[h] <= sum{v in {245}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm9_0_master{v in {245}, h in Hs}: ni_vnf5_dot_vm9_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm9_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm9_0_slave{h in Hs}: ni_vnf5_dot_vm9_0_slave[h] <= sum{v in {246}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm9_0_slave{v in {246}, h in Hs}: ni_vnf5_dot_vm9_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_47_{h in Hs}: ni_vnf5_dot_vm9_0_master[h] + ni_vnf5_dot_vm9_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm10_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm10_0_master{h in Hs}: ni_vnf5_dot_vm10_0_master[h] <= sum{v in {247}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm10_0_master{v in {247}, h in Hs}: ni_vnf5_dot_vm10_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm10_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm10_0_slave{h in Hs}: ni_vnf5_dot_vm10_0_slave[h] <= sum{v in {248}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm10_0_slave{v in {248}, h in Hs}: ni_vnf5_dot_vm10_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_48_{h in Hs}: ni_vnf5_dot_vm10_0_master[h] + ni_vnf5_dot_vm10_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf6_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm1_0_master{h in Hs}: ni_vnf6_dot_vm1_0_master[h] <= sum{v in {251}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm1_0_master{v in {251}, h in Hs}: ni_vnf6_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf6_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm1_0_slave{h in Hs}: ni_vnf6_dot_vm1_0_slave[h] <= sum{v in {252}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm1_0_slave{v in {252}, h in Hs}: ni_vnf6_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_49_{h in Hs}: ni_vnf6_dot_vm1_0_master[h] + ni_vnf6_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf6_dot_vm25_0_master{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm25_0_master{h in Hs}: ni_vnf6_dot_vm25_0_master[h] <= sum{v in {284}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm25_0_master{v in {284}, h in Hs}: ni_vnf6_dot_vm25_0_master[h] >= x[v, h];
var ni_vnf6_dot_vm25_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm25_0_slave{h in Hs}: ni_vnf6_dot_vm25_0_slave[h] <= sum{v in {285}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm25_0_slave{v in {285}, h in Hs}: ni_vnf6_dot_vm25_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_50_{h in Hs}: ni_vnf6_dot_vm25_0_master[h] + ni_vnf6_dot_vm25_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf6_dot_vm26_0_master{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm26_0_master{h in Hs}: ni_vnf6_dot_vm26_0_master[h] <= sum{v in {286,287}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm26_0_master{v in {286,287}, h in Hs}: ni_vnf6_dot_vm26_0_master[h] >= x[v, h];
var ni_vnf6_dot_vm26_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm26_0_slave{h in Hs}: ni_vnf6_dot_vm26_0_slave[h] <= sum{v in {288,289}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm26_0_slave{v in {288,289}, h in Hs}: ni_vnf6_dot_vm26_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_51_{h in Hs}: ni_vnf6_dot_vm26_0_master[h] + ni_vnf6_dot_vm26_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf6_dot_vm10_0_master{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm10_0_master{h in Hs}: ni_vnf6_dot_vm10_0_master[h] <= sum{v in {290}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm10_0_master{v in {290}, h in Hs}: ni_vnf6_dot_vm10_0_master[h] >= x[v, h];
var ni_vnf6_dot_vm10_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf6_dot_vm10_0_slave{h in Hs}: ni_vnf6_dot_vm10_0_slave[h] <= sum{v in {291}} x[v, h];
s.t. or_ineq_ni_vnf6_dot_vm10_0_slave{v in {291}, h in Hs}: ni_vnf6_dot_vm10_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_52_{h in Hs}: ni_vnf6_dot_vm10_0_master[h] + ni_vnf6_dot_vm10_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 4
3 4
4 4
5 4
6 4
7 4
8 4
9 4
10 4
11 4
12 4
13 4
14 4
15 4
16 4
17 4
18 4
19 4
20 4
21 4
22 4
23 4
24 4
25 4
26 4
27 4
28 4
29 4
30 2
31 2
32 2
33 2
34 2
35 2
36 2
37 4
38 4
39 4
40 4
41 2
42 2
43 2
44 2
45 8
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 2
58 2
59 2
60 2
61 2
62 2
63 4
64 4
65 4
66 4
67 4
68 4
69 4
70 4
71 4
72 4
73 4
74 4
75 4
76 4
77 4
78 4
79 4
80 4
81 4
82 4
83 4
84 4
85 4
86 4
87 4
88 4
89 4
90 4
91 4
92 4
93 4
94 4
95 4
96 4
97 2
98 2
99 2
100 2
101 2
102 2
103 8
104 8
105 8
106 8
107 8
108 8
109 4
110 4
111 4
112 4
113 4
114 4
115 8
116 8
117 8
118 8
119 8
120 8
121 4
122 4
123 4
124 4
125 4
126 4
127 2
128 2
129 2
130 2
131 2
132 2
133 4
134 4
135 4
136 4
137 4
138 4
139 4
140 4
141 2
142 2
143 2
144 2
145 8
146 8
147 8
148 8
149 4
150 4
151 4
152 4
153 8
154 8
155 8
156 8
157 4
158 4
159 4
160 4
161 2
162 2
163 2
164 2
165 9
166 9
167 9
168 9
169 9
170 9
171 9
172 9
173 9
174 9
175 9
176 9
177 9
178 9
179 9
180 9
181 4
182 4
183 4
184 4
185 4
186 4
187 4
188 4
189 9
190 9
191 9
192 9
193 9
194 9
195 9
196 9
197 9
198 9
199 9
200 9
201 9
202 9
203 9
204 9
205 9
206 9
207 9
208 9
209 9
210 9
211 9
212 9
213 9
214 9
215 9
216 9
217 9
218 9
219 9
220 9
221 9
222 9
223 9
224 9
225 9
226 9
227 9
228 9
229 8
230 8
231 4
232 4
233 4
234 4
235 2
236 2
237 2
238 2
239 8
240 8
241 8
242 8
243 4
244 4
245 2
246 2
247 2
248 2
249 4
250 4
251 8
252 8
253 4
254 4
255 4
256 4
257 2
258 2
259 2
260 2
261 2
262 2
263 2
264 2
265 2
266 2
267 2
268 2
269 2
270 2
271 2
272 2
273 2
274 2
275 2
276 2
277 2
278 2
279 2
280 2
281 2
282 2
283 2
284 4
285 4
286 2
287 2
288 2
289 2
290 2
291 2
292 0
293 0;
param: MEM := 
0 32
1 32
2 12
3 12
4 12
5 12
6 12
7 12
8 12
9 12
10 12
11 12
12 12
13 12
14 12
15 12
16 12
17 12
18 12
19 12
20 12
21 12
22 12
23 12
24 12
25 12
26 12
27 12
28 12
29 12
30 24
31 24
32 24
33 24
34 24
35 24
36 24
37 24
38 24
39 24
40 24
41 8
42 8
43 8
44 8
45 8
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 16
56 16
57 8
58 8
59 8
60 8
61 10
62 10
63 40
64 40
65 40
66 40
67 24
68 24
69 24
70 24
71 24
72 24
73 24
74 24
75 24
76 24
77 24
78 24
79 24
80 24
81 24
82 24
83 24
84 24
85 24
86 24
87 24
88 24
89 24
90 24
91 24
92 24
93 24
94 24
95 24
96 24
97 8
98 8
99 8
100 8
101 8
102 8
103 16
104 16
105 16
106 16
107 16
108 16
109 16
110 16
111 16
112 16
113 16
114 16
115 32
116 32
117 32
118 32
119 32
120 32
121 16
122 16
123 16
124 16
125 16
126 16
127 8
128 8
129 8
130 8
131 8
132 8
133 24
134 24
135 24
136 24
137 24
138 24
139 24
140 24
141 8
142 8
143 8
144 8
145 16
146 16
147 16
148 16
149 16
150 16
151 16
152 16
153 32
154 32
155 32
156 32
157 16
158 16
159 16
160 16
161 8
162 8
163 8
164 8
165 16
166 16
167 16
168 16
169 16
170 16
171 16
172 16
173 16
174 16
175 16
176 16
177 16
178 16
179 16
180 16
181 8
182 8
183 8
184 8
185 8
186 8
187 8
188 8
189 16
190 16
191 16
192 16
193 16
194 16
195 16
196 16
197 16
198 16
199 16
200 16
201 16
202 16
203 16
204 16
205 16
206 16
207 16
208 16
209 16
210 16
211 16
212 16
213 16
214 16
215 16
216 16
217 16
218 16
219 16
220 16
221 16
222 16
223 16
224 16
225 16
226 16
227 16
228 16
229 32
230 32
231 12
232 12
233 12
234 12
235 24
236 24
237 8
238 8
239 8
240 8
241 16
242 16
243 24
244 24
245 8
246 8
247 10
248 10
249 40
250 40
251 32
252 32
253 12
254 12
255 12
256 12
257 20
258 20
259 20
260 20
261 20
262 20
263 20
264 20
265 20
266 20
267 20
268 20
269 20
270 8
271 8
272 8
273 8
274 8
275 8
276 8
277 8
278 8
279 8
280 8
281 8
282 8
283 8
284 12
285 12
286 16
287 16
288 16
289 16
290 10
291 10
292 40
293 40;
param: NET := 
0 339.0
1 339.0
2 433.0
3 433.0
4 433.0
5 433.0
6 433.0
7 433.0
8 433.0
9 433.0
10 433.0
11 433.0
12 433.0
13 433.0
14 433.0
15 433.0
16 433.0
17 433.0
18 433.0
19 433.0
20 433.0
21 433.0
22 433.0
23 433.0
24 433.0
25 433.0
26 433.0
27 433.0
28 433.0
29 433.0
30 35.0
31 35.0
32 35.0
33 35.0
34 35.0
35 35.0
36 35.0
37 40.0
38 40.0
39 40.0
40 40.0
41 31.0
42 31.0
43 31.0
44 31.0
45 47.0
46 47.0
47 47.0
48 47.0
49 47.0
50 47.0
51 47.0
52 47.0
53 47.0
54 47.0
55 1292.0
56 1292.0
57 72.0
58 72.0
59 72.0
60 72.0
61 10.0
62 10.0
63 104.0
64 104.0
65 104.0
66 104.0
67 375.0
68 375.0
69 375.0
70 375.0
71 375.0
72 375.0
73 375.0
74 375.0
75 375.0
76 375.0
77 375.0
78 375.0
79 375.0
80 375.0
81 375.0
82 375.0
83 375.0
84 375.0
85 375.0
86 375.0
87 375.0
88 375.0
89 375.0
90 375.0
91 375.0
92 375.0
93 375.0
94 375.0
95 375.0
96 375.0
97 80.0
98 80.0
99 80.0
100 80.0
101 80.0
102 80.0
103 1100.0
104 1100.0
105 1100.0
106 1100.0
107 1100.0
108 1100.0
109 120.0
110 120.0
111 120.0
112 120.0
113 120.0
114 120.0
115 235.0
116 235.0
117 235.0
118 235.0
119 235.0
120 235.0
121 235.0
122 235.0
123 235.0
124 235.0
125 235.0
126 235.0
127 10.0
128 10.0
129 10.0
130 10.0
131 10.0
132 10.0
133 375.0
134 375.0
135 375.0
136 375.0
137 375.0
138 375.0
139 375.0
140 375.0
141 80.0
142 80.0
143 80.0
144 80.0
145 1100.0
146 1100.0
147 1100.0
148 1100.0
149 120.0
150 120.0
151 120.0
152 120.0
153 235.0
154 235.0
155 235.0
156 235.0
157 235.0
158 235.0
159 235.0
160 235.0
161 10.0
162 10.0
163 10.0
164 10.0
165 3200.0
166 3200.0
167 3200.0
168 3200.0
169 3200.0
170 3200.0
171 3200.0
172 3200.0
173 3200.0
174 3200.0
175 3200.0
176 3200.0
177 3200.0
178 3200.0
179 3200.0
180 3200.0
181 310.0
182 310.0
183 310.0
184 310.0
185 310.0
186 310.0
187 310.0
188 310.0
189 800.0
190 800.0
191 800.0
192 800.0
193 800.0
194 800.0
195 800.0
196 800.0
197 800.0
198 800.0
199 800.0
200 800.0
201 800.0
202 800.0
203 800.0
204 800.0
205 800.0
206 800.0
207 800.0
208 800.0
209 800.0
210 800.0
211 800.0
212 800.0
213 800.0
214 800.0
215 800.0
216 800.0
217 800.0
218 800.0
219 800.0
220 800.0
221 800.0
222 800.0
223 800.0
224 800.0
225 800.0
226 800.0
227 800.0
228 800.0
229 3100.0
230 3100.0
231 1062.0
232 1062.0
233 1062.0
234 1062.0
235 210.0
236 210.0
237 200.0
238 200.0
239 720.0
240 720.0
241 1350.0
242 1350.0
243 200.0
244 200.0
245 1000.0
246 1000.0
247 600.0
248 600.0
249 900.0
250 900.0
251 2800.0
252 2800.0
253 2801.0
254 2801.0
255 2801.0
256 2801.0
257 130.0
258 130.0
259 130.0
260 130.0
261 130.0
262 130.0
263 130.0
264 130.0
265 130.0
266 130.0
267 130.0
268 130.0
269 130.0
270 500.0
271 500.0
272 500.0
273 500.0
274 500.0
275 500.0
276 500.0
277 500.0
278 200.0
279 200.0
280 200.0
281 200.0
282 200.0
283 200.0
284 1350.0
285 1350.0
286 200.0
287 200.0
288 200.0
289 200.0
290 0.0
291 0.0
292 0.0
293 0.0;


######################
# SOLVE 
######################
end;
