

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..203;
# Set of Hosts
set Hs := 0..44;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {4..7}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_2_{h in Hs}: sum{v in {8..11}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_3_{h in Hs}: sum{v in {12..15}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_4_{h in Hs}: sum{v in {16..19}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_5_{h in Hs}: sum{v in {20..23}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_6_{h in Hs}: sum{v in {24..27}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_7_{h in Hs}: sum{v in {28..31}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_8_{h in Hs}: sum{v in {32..35}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_9_{h in Hs}: sum{v in {36..39}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_10_{h in Hs}: sum{v in {40..43}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_11_{h in Hs}: sum{v in {44..47}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {72..82}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_1_{h in Hs}: sum{v in {83..93}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_2_{h in Hs}: sum{v in {94..104}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_3_{h in Hs}: sum{v in {105..115}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_4_{h in Hs}: sum{v in {116..126}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_5_{h in Hs}: sum{v in {127..137}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_6_{h in Hs}: sum{v in {138..148}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_7_{h in Hs}: sum{v in {149..159}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_8_{h in Hs}: sum{v in {160..170}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_9_{h in Hs}: sum{v in {171..181}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_10_{h in Hs}: sum{v in {182..192}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_11_{h in Hs}: sum{v in {193..203}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {48}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {48}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {49}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {49}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_1_master{h in Hs}: ni_vnf1_dot_vm2_1_master[h] <= sum{v in {50}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_1_master{v in {50}, h in Hs}: ni_vnf1_dot_vm2_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_1_slave{h in Hs}: ni_vnf1_dot_vm2_1_slave[h] <= sum{v in {51}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_1_slave{v in {51}, h in Hs}: ni_vnf1_dot_vm2_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_1_master[h] + ni_vnf1_dot_vm2_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_2_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_2_master{h in Hs}: ni_vnf1_dot_vm2_2_master[h] <= sum{v in {52}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_2_master{v in {52}, h in Hs}: ni_vnf1_dot_vm2_2_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_2_slave{h in Hs}: ni_vnf1_dot_vm2_2_slave[h] <= sum{v in {53}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_2_slave{v in {53}, h in Hs}: ni_vnf1_dot_vm2_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf1_dot_vm2_2_master[h] + ni_vnf1_dot_vm2_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_3_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_3_master{h in Hs}: ni_vnf1_dot_vm2_3_master[h] <= sum{v in {54}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_3_master{v in {54}, h in Hs}: ni_vnf1_dot_vm2_3_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_3_slave{h in Hs}: ni_vnf1_dot_vm2_3_slave[h] <= sum{v in {55}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_3_slave{v in {55}, h in Hs}: ni_vnf1_dot_vm2_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf1_dot_vm2_3_master[h] + ni_vnf1_dot_vm2_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_4_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_4_master{h in Hs}: ni_vnf1_dot_vm2_4_master[h] <= sum{v in {56}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_4_master{v in {56}, h in Hs}: ni_vnf1_dot_vm2_4_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_4_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_4_slave{h in Hs}: ni_vnf1_dot_vm2_4_slave[h] <= sum{v in {57}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_4_slave{v in {57}, h in Hs}: ni_vnf1_dot_vm2_4_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf1_dot_vm2_4_master[h] + ni_vnf1_dot_vm2_4_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_5_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_5_master{h in Hs}: ni_vnf1_dot_vm2_5_master[h] <= sum{v in {58}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_5_master{v in {58}, h in Hs}: ni_vnf1_dot_vm2_5_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_5_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_5_slave{h in Hs}: ni_vnf1_dot_vm2_5_slave[h] <= sum{v in {59}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_5_slave{v in {59}, h in Hs}: ni_vnf1_dot_vm2_5_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf1_dot_vm2_5_master[h] + ni_vnf1_dot_vm2_5_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_6_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_6_master{h in Hs}: ni_vnf1_dot_vm2_6_master[h] <= sum{v in {60}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_6_master{v in {60}, h in Hs}: ni_vnf1_dot_vm2_6_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_6_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_6_slave{h in Hs}: ni_vnf1_dot_vm2_6_slave[h] <= sum{v in {61}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_6_slave{v in {61}, h in Hs}: ni_vnf1_dot_vm2_6_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_6_{h in Hs}: ni_vnf1_dot_vm2_6_master[h] + ni_vnf1_dot_vm2_6_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_7_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_7_master{h in Hs}: ni_vnf1_dot_vm2_7_master[h] <= sum{v in {62}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_7_master{v in {62}, h in Hs}: ni_vnf1_dot_vm2_7_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_7_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_7_slave{h in Hs}: ni_vnf1_dot_vm2_7_slave[h] <= sum{v in {63}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_7_slave{v in {63}, h in Hs}: ni_vnf1_dot_vm2_7_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_7_{h in Hs}: ni_vnf1_dot_vm2_7_master[h] + ni_vnf1_dot_vm2_7_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_8_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_8_master{h in Hs}: ni_vnf1_dot_vm2_8_master[h] <= sum{v in {64}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_8_master{v in {64}, h in Hs}: ni_vnf1_dot_vm2_8_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_8_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_8_slave{h in Hs}: ni_vnf1_dot_vm2_8_slave[h] <= sum{v in {65}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_8_slave{v in {65}, h in Hs}: ni_vnf1_dot_vm2_8_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_8_{h in Hs}: ni_vnf1_dot_vm2_8_master[h] + ni_vnf1_dot_vm2_8_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_9_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_9_master{h in Hs}: ni_vnf1_dot_vm2_9_master[h] <= sum{v in {66}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_9_master{v in {66}, h in Hs}: ni_vnf1_dot_vm2_9_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_9_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_9_slave{h in Hs}: ni_vnf1_dot_vm2_9_slave[h] <= sum{v in {67}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_9_slave{v in {67}, h in Hs}: ni_vnf1_dot_vm2_9_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_9_{h in Hs}: ni_vnf1_dot_vm2_9_master[h] + ni_vnf1_dot_vm2_9_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_10_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_10_master{h in Hs}: ni_vnf1_dot_vm2_10_master[h] <= sum{v in {68}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_10_master{v in {68}, h in Hs}: ni_vnf1_dot_vm2_10_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_10_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_10_slave{h in Hs}: ni_vnf1_dot_vm2_10_slave[h] <= sum{v in {69}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_10_slave{v in {69}, h in Hs}: ni_vnf1_dot_vm2_10_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_10_{h in Hs}: ni_vnf1_dot_vm2_10_master[h] + ni_vnf1_dot_vm2_10_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_11_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_11_master{h in Hs}: ni_vnf1_dot_vm2_11_master[h] <= sum{v in {70}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_11_master{v in {70}, h in Hs}: ni_vnf1_dot_vm2_11_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_11_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_11_slave{h in Hs}: ni_vnf1_dot_vm2_11_slave[h] <= sum{v in {71}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_11_slave{v in {71}, h in Hs}: ni_vnf1_dot_vm2_11_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_11_{h in Hs}: ni_vnf1_dot_vm2_11_master[h] + ni_vnf1_dot_vm2_11_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9
1 9
2 9
3 9
4 9
5 9
6 9
7 9
8 9
9 9
10 9
11 9
12 9
13 9
14 9
15 9
16 9
17 9
18 9
19 9
20 9
21 9
22 9
23 9
24 9
25 9
26 9
27 9
28 9
29 9
30 9
31 9
32 9
33 9
34 9
35 9
36 9
37 9
38 9
39 9
40 9
41 9
42 9
43 9
44 9
45 9
46 9
47 9
48 4
49 4
50 4
51 4
52 4
53 4
54 4
55 4
56 4
57 4
58 4
59 4
60 4
61 4
62 4
63 4
64 4
65 4
66 4
67 4
68 4
69 4
70 4
71 4
72 9
73 9
74 9
75 9
76 9
77 9
78 9
79 9
80 9
81 9
82 9
83 9
84 9
85 9
86 9
87 9
88 9
89 9
90 9
91 9
92 9
93 9
94 9
95 9
96 9
97 9
98 9
99 9
100 9
101 9
102 9
103 9
104 9
105 9
106 9
107 9
108 9
109 9
110 9
111 9
112 9
113 9
114 9
115 9
116 9
117 9
118 9
119 9
120 9
121 9
122 9
123 9
124 9
125 9
126 9
127 9
128 9
129 9
130 9
131 9
132 9
133 9
134 9
135 9
136 9
137 9
138 9
139 9
140 9
141 9
142 9
143 9
144 9
145 9
146 9
147 9
148 9
149 9
150 9
151 9
152 9
153 9
154 9
155 9
156 9
157 9
158 9
159 9
160 9
161 9
162 9
163 9
164 9
165 9
166 9
167 9
168 9
169 9
170 9
171 9
172 9
173 9
174 9
175 9
176 9
177 9
178 9
179 9
180 9
181 9
182 9
183 9
184 9
185 9
186 9
187 9
188 9
189 9
190 9
191 9
192 9
193 9
194 9
195 9
196 9
197 9
198 9
199 9
200 9
201 9
202 9
203 9;
param: MEM := 
0 16
1 16
2 16
3 16
4 16
5 16
6 16
7 16
8 16
9 16
10 16
11 16
12 16
13 16
14 16
15 16
16 16
17 16
18 16
19 16
20 16
21 16
22 16
23 16
24 16
25 16
26 16
27 16
28 16
29 16
30 16
31 16
32 16
33 16
34 16
35 16
36 16
37 16
38 16
39 16
40 16
41 16
42 16
43 16
44 16
45 16
46 16
47 16
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 16
73 16
74 16
75 16
76 16
77 16
78 16
79 16
80 16
81 16
82 16
83 16
84 16
85 16
86 16
87 16
88 16
89 16
90 16
91 16
92 16
93 16
94 16
95 16
96 16
97 16
98 16
99 16
100 16
101 16
102 16
103 16
104 16
105 16
106 16
107 16
108 16
109 16
110 16
111 16
112 16
113 16
114 16
115 16
116 16
117 16
118 16
119 16
120 16
121 16
122 16
123 16
124 16
125 16
126 16
127 16
128 16
129 16
130 16
131 16
132 16
133 16
134 16
135 16
136 16
137 16
138 16
139 16
140 16
141 16
142 16
143 16
144 16
145 16
146 16
147 16
148 16
149 16
150 16
151 16
152 16
153 16
154 16
155 16
156 16
157 16
158 16
159 16
160 16
161 16
162 16
163 16
164 16
165 16
166 16
167 16
168 16
169 16
170 16
171 16
172 16
173 16
174 16
175 16
176 16
177 16
178 16
179 16
180 16
181 16
182 16
183 16
184 16
185 16
186 16
187 16
188 16
189 16
190 16
191 16
192 16
193 16
194 16
195 16
196 16
197 16
198 16
199 16
200 16
201 16
202 16
203 16;
param: NET := 
0 3200
1 3200
2 3200
3 3200
4 3200
5 3200
6 3200
7 3200
8 3200
9 3200
10 3200
11 3200
12 3200
13 3200
14 3200
15 3200
16 3200
17 3200
18 3200
19 3200
20 3200
21 3200
22 3200
23 3200
24 3200
25 3200
26 3200
27 3200
28 3200
29 3200
30 3200
31 3200
32 3200
33 3200
34 3200
35 3200
36 3200
37 3200
38 3200
39 3200
40 3200
41 3200
42 3200
43 3200
44 3200
45 3200
46 3200
47 3200
48 310
49 310
50 310
51 310
52 310
53 310
54 310
55 310
56 310
57 310
58 310
59 310
60 310
61 310
62 310
63 310
64 310
65 310
66 310
67 310
68 310
69 310
70 310
71 310
72 800
73 800
74 800
75 800
76 800
77 800
78 800
79 800
80 800
81 800
82 800
83 800
84 800
85 800
86 800
87 800
88 800
89 800
90 800
91 800
92 800
93 800
94 800
95 800
96 800
97 800
98 800
99 800
100 800
101 800
102 800
103 800
104 800
105 800
106 800
107 800
108 800
109 800
110 800
111 800
112 800
113 800
114 800
115 800
116 800
117 800
118 800
119 800
120 800
121 800
122 800
123 800
124 800
125 800
126 800
127 800
128 800
129 800
130 800
131 800
132 800
133 800
134 800
135 800
136 800
137 800
138 800
139 800
140 800
141 800
142 800
143 800
144 800
145 800
146 800
147 800
148 800
149 800
150 800
151 800
152 800
153 800
154 800
155 800
156 800
157 800
158 800
159 800
160 800
161 800
162 800
163 800
164 800
165 800
166 800
167 800
168 800
169 800
170 800
171 800
172 800
173 800
174 800
175 800
176 800
177 800
178 800
179 800
180 800
181 800
182 800
183 800
184 800
185 800
186 800
187 800
188 800
189 800
190 800
191 800
192 800
193 800
194 800
195 800
196 800
197 800
198 800
199 800
200 800
201 800
202 800
203 800;


######################
# SOLVE 
######################
end;
