

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..32;
# Set of Hosts
set Hs := 0..8;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {2..7}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm14_0_{h in Hs}: sum{v in {31,32}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 2
1 2
2 32
3 32
4 32
5 32
6 32
7 32
8 4
9 6
10 4
11 4
12 8
13 8
14 8
15 8
16 8
17 8
18 1
19 1
20 1
21 1
22 1
23 8
24 8
25 1
26 1
27 16
28 16
29 8
30 8
31 12
32 12;
param: MEM := 
0 32
1 32
2 152
3 152
4 152
5 152
6 152
7 152
8 24
9 24
10 32
11 32
12 16
13 64
14 64
15 64
16 64
17 64
18 2
19 2
20 2
21 2
22 2
23 32
24 32
25 2
26 2
27 64
28 64
29 64
30 64
31 16
32 16;
param: NET := 
0 2.0
1 2.0
2 1100.0
3 1100.0
4 1100.0
5 1100.0
6 1100.0
7 1100.0
8 2600.0
9 2600.0
10 200.0
11 200.0
12 100.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 2000.0
24 2000.0
25 200.0
26 200.0
27 0.0
28 0.0
29 0.0
30 0.0
31 2000.0
32 2000.0;


######################
# SOLVE 
######################
end;
