

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..18;
# Set of Hosts
set Hs := 0..2;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4
1 8
2 8
3 8
4 8
5 8
6 8
7 6
8 6
9 1
10 4
11 8
12 8
13 8
14 8
15 4
16 4
17 4
18 4;
param: MEM := 
0 16
1 48
2 48
3 48
4 48
5 48
6 48
7 24
8 24
9 1
10 16
11 48
12 48
13 48
14 48
15 16
16 16
17 16
18 16;
param: NET := 
0 2.0
1 2.0
2 2.0
3 2.0
4 2.0
5 2.0
6 2.0
7 2.0
8 2.0
9 2.0
10 10.0
11 20.0
12 20.0
13 25.0
14 50.0
15 20.0
16 0.0
17 20.0
18 1500.0;


######################
# SOLVE 
######################
end;
