

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..21;
# Set of Hosts
set Hs := 0..2;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_1_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_2_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {10..12}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_1_{h in Hs}: sum{v in {13..15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_2_{h in Hs}: sum{v in {16..18}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {1}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {2}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {2}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4
1 4
2 4
3 4
4 8
5 8
6 8
7 8
8 8
9 8
10 4
11 4
12 4
13 4
14 4
15 4
16 4
17 4
18 4
19 1
20 1
21 1;
param: MEM := 
0 8
1 6
2 6
3 6
4 32
5 32
6 32
7 32
8 32
9 32
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 2
20 2
21 2;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0;


######################
# SOLVE 
######################
end;
