

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..144;
# Set of Hosts
set Hs := 0..14;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {6}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_1_{h in Hs}: sum{v in {7}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_0_{h in Hs}: sum{v in {10..14}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_1_{h in Hs}: sum{v in {15..19}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {22,23}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_1_{h in Hs}: sum{v in {24,25}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm8_0_{h in Hs}: sum{v in {26..31}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm8_1_{h in Hs}: sum{v in {32..37}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm9_0_{h in Hs}: sum{v in {38,39}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm9_1_{h in Hs}: sum{v in {40,41}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm10_0_{h in Hs}: sum{v in {42,43}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm10_1_{h in Hs}: sum{v in {44,45}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm11_0_{h in Hs}: sum{v in {46,47}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm11_1_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm15_0_{h in Hs}: sum{v in {56,57}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm15_1_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_0_{h in Hs}: sum{v in {64..68}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_1_{h in Hs}: sum{v in {69..73}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm7_0_{h in Hs}: sum{v in {76,77}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm7_1_{h in Hs}: sum{v in {78,79}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {80..84}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_1_{h in Hs}: sum{v in {85..89}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm20_0_{h in Hs}: sum{v in {100,101}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm24_0_{h in Hs}: sum{v in {108..110}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm25_0_{h in Hs}: sum{v in {111..113}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm26_0_{h in Hs}: sum{v in {114..116}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm27_0_{h in Hs}: sum{v in {117..119}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm28_0_{h in Hs}: sum{v in {120..122}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm29_0_{h in Hs}: sum{v in {123,124}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm30_0_{h in Hs}: sum{v in {125,126}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm31_0_{h in Hs}: sum{v in {127,128}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm32_0_{h in Hs}: sum{v in {129..131}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm19_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm19_0_master{h in Hs}: ni_vnf9_dot_vm19_0_master[h] <= sum{v in {98}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm19_0_master{v in {98}, h in Hs}: ni_vnf9_dot_vm19_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm19_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm19_0_slave{h in Hs}: ni_vnf9_dot_vm19_0_slave[h] <= sum{v in {99}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm19_0_slave{v in {99}, h in Hs}: ni_vnf9_dot_vm19_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf9_dot_vm19_0_master[h] + ni_vnf9_dot_vm19_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 2.0
1 2.0
2 3.0
3 3.0
4 3.0
5 3.0
6 4.0
7 4.0
8 6.0
9 6.0
10 3.0
11 3.0
12 3.0
13 3.0
14 3.0
15 3.0
16 3.0
17 3.0
18 3.0
19 3.0
20 6.0
21 6.0
22 4.0
23 4.0
24 4.0
25 4.0
26 1.0
27 1.0
28 1.0
29 1.0
30 1.0
31 1.0
32 1.0
33 1.0
34 1.0
35 1.0
36 1.0
37 1.0
38 10.0
39 10.0
40 10.0
41 10.0
42 10.0
43 10.0
44 10.0
45 10.0
46 8.0
47 8.0
48 8.0
49 8.0
50 1.0
51 1.0
52 1.0
53 1.0
54 2.0
55 2.0
56 3.0
57 3.0
58 3.0
59 3.0
60 4.0
61 4.0
62 6.0
63 6.0
64 3.0
65 3.0
66 3.0
67 3.0
68 3.0
69 3.0
70 3.0
71 3.0
72 3.0
73 3.0
74 6.0
75 6.0
76 4.0
77 4.0
78 4.0
79 4.0
80 1.0
81 1.0
82 1.0
83 1.0
84 1.0
85 1.0
86 1.0
87 1.0
88 1.0
89 1.0
90 10.0
91 10.0
92 10.0
93 10.0
94 8.0
95 8.0
96 1.0
97 1.0
98 8.0
99 8.0
100 8.0
101 8.0
102 0.0
103 8.0
104 8.0
105 8.0
106 8.0
107 8.0
108 4.0
109 4.0
110 4.0
111 4.0
112 4.0
113 4.0
114 4.0
115 4.0
116 4.0
117 4.0
118 4.0
119 4.0
120 4.0
121 4.0
122 4.0
123 8.0
124 8.0
125 4.0
126 4.0
127 4.0
128 4.0
129 4.0
130 4.0
131 4.0
132 4.0
133 4.0
134 8.0
135 8.0
136 8.0
137 8.0
138 8.0
139 8.0
140 8.0
141 8.0
142 8.0
143 8.0
144 8.0;
param: MEM := 
0 5.0
1 5.0
2 6.0
3 6.0
4 6.0
5 6.0
6 5.0
7 5.0
8 12.0
9 12.0
10 5.0
11 5.0
12 5.0
13 5.0
14 5.0
15 5.0
16 5.0
17 5.0
18 5.0
19 5.0
20 10.0
21 10.0
22 8.0
23 8.0
24 8.0
25 8.0
26 4.0
27 4.0
28 4.0
29 4.0
30 4.0
31 4.0
32 4.0
33 4.0
34 4.0
35 4.0
36 4.0
37 4.0
38 16.0
39 16.0
40 16.0
41 16.0
42 16.0
43 16.0
44 16.0
45 16.0
46 8.0
47 8.0
48 8.0
49 8.0
50 1.0
51 1.0
52 2.0
53 2.0
54 5.0
55 5.0
56 6.0
57 6.0
58 6.0
59 6.0
60 5.0
61 5.0
62 12.0
63 12.0
64 5.0
65 5.0
66 5.0
67 5.0
68 5.0
69 5.0
70 5.0
71 5.0
72 5.0
73 5.0
74 10.0
75 10.0
76 8.0
77 8.0
78 8.0
79 8.0
80 4.0
81 4.0
82 4.0
83 4.0
84 4.0
85 4.0
86 4.0
87 4.0
88 4.0
89 4.0
90 16.0
91 16.0
92 16.0
93 16.0
94 8.0
95 8.0
96 2.0
97 2.0
98 12.0
99 12.0
100 12.0
101 12.0
102 0.0
103 32.0
104 32.0
105 32.0
106 32.0
107 32.0
108 16.0
109 16.0
110 16.0
111 8.0
112 8.0
113 8.0
114 16.0
115 16.0
116 16.0
117 16.0
118 16.0
119 16.0
120 16.0
121 16.0
122 16.0
123 16.0
124 16.0
125 8.0
126 8.0
127 8.0
128 8.0
129 16.0
130 16.0
131 16.0
132 8.0
133 8.0
134 32.0
135 32.0
136 32.0
137 32.0
138 32.0
139 32.0
140 32.0
141 32.0
142 64.0
143 64.0
144 64.0;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0
143 0.0
144 0.0;


######################
# SOLVE 
######################
end;
