

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..103;
# Set of Hosts
set Hs := 0..11;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm8_0_{h in Hs}: sum{v in {28,29}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm8_1_{h in Hs}: sum{v in {30,31}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm10_0_{h in Hs}: sum{v in {36,37}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm10_1_{h in Hs}: sum{v in {38,39}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm8_0_{h in Hs}: sum{v in {54,55}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm10_0_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm12_0_{h in Hs}: sum{v in {64..67}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm12_1_{h in Hs}: sum{v in {68..71}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm13_0_{h in Hs}: sum{v in {72,73}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm13_1_{h in Hs}: sum{v in {74,75}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm15_0_{h in Hs}: sum{v in {80,81}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm15_1_{h in Hs}: sum{v in {82,83}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm17_0_{h in Hs}: sum{v in {88,89}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm17_1_{h in Hs}: sum{v in {90,91}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm20_0_{h in Hs}: sum{v in {100,101}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm20_1_{h in Hs}: sum{v in {102,103}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_1_master{h in Hs}: ni_vnf1_dot_vm1_1_master[h] <= sum{v in {2}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_1_master{v in {2}, h in Hs}: ni_vnf1_dot_vm1_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_1_slave{h in Hs}: ni_vnf1_dot_vm1_1_slave[h] <= sum{v in {3}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_1_slave{v in {3}, h in Hs}: ni_vnf1_dot_vm1_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm1_1_master[h] + ni_vnf1_dot_vm1_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {4}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {4}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {5}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {5}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_1_master{h in Hs}: ni_vnf1_dot_vm2_1_master[h] <= sum{v in {6}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_1_master{v in {6}, h in Hs}: ni_vnf1_dot_vm2_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_1_slave{h in Hs}: ni_vnf1_dot_vm2_1_slave[h] <= sum{v in {7}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_1_slave{v in {7}, h in Hs}: ni_vnf1_dot_vm2_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf1_dot_vm2_1_master[h] + ni_vnf1_dot_vm2_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm3_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_master{h in Hs}: ni_vnf1_dot_vm3_0_master[h] <= sum{v in {8}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_master{v in {8}, h in Hs}: ni_vnf1_dot_vm3_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm3_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_slave{h in Hs}: ni_vnf1_dot_vm3_0_slave[h] <= sum{v in {9}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_slave{v in {9}, h in Hs}: ni_vnf1_dot_vm3_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf1_dot_vm3_0_master[h] + ni_vnf1_dot_vm3_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm3_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_1_master{h in Hs}: ni_vnf1_dot_vm3_1_master[h] <= sum{v in {10}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_1_master{v in {10}, h in Hs}: ni_vnf1_dot_vm3_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm3_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_1_slave{h in Hs}: ni_vnf1_dot_vm3_1_slave[h] <= sum{v in {11}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_1_slave{v in {11}, h in Hs}: ni_vnf1_dot_vm3_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf1_dot_vm3_1_master[h] + ni_vnf1_dot_vm3_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm4_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_0_master{h in Hs}: ni_vnf1_dot_vm4_0_master[h] <= sum{v in {12}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_0_master{v in {12}, h in Hs}: ni_vnf1_dot_vm4_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm4_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_0_slave{h in Hs}: ni_vnf1_dot_vm4_0_slave[h] <= sum{v in {13}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_0_slave{v in {13}, h in Hs}: ni_vnf1_dot_vm4_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_6_{h in Hs}: ni_vnf1_dot_vm4_0_master[h] + ni_vnf1_dot_vm4_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm4_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_1_master{h in Hs}: ni_vnf1_dot_vm4_1_master[h] <= sum{v in {14}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_1_master{v in {14}, h in Hs}: ni_vnf1_dot_vm4_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm4_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_1_slave{h in Hs}: ni_vnf1_dot_vm4_1_slave[h] <= sum{v in {15}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_1_slave{v in {15}, h in Hs}: ni_vnf1_dot_vm4_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_7_{h in Hs}: ni_vnf1_dot_vm4_1_master[h] + ni_vnf1_dot_vm4_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm5_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_0_master{h in Hs}: ni_vnf1_dot_vm5_0_master[h] <= sum{v in {16}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_0_master{v in {16}, h in Hs}: ni_vnf1_dot_vm5_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm5_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_0_slave{h in Hs}: ni_vnf1_dot_vm5_0_slave[h] <= sum{v in {17}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_0_slave{v in {17}, h in Hs}: ni_vnf1_dot_vm5_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_8_{h in Hs}: ni_vnf1_dot_vm5_0_master[h] + ni_vnf1_dot_vm5_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm5_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_1_master{h in Hs}: ni_vnf1_dot_vm5_1_master[h] <= sum{v in {18}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_1_master{v in {18}, h in Hs}: ni_vnf1_dot_vm5_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm5_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_1_slave{h in Hs}: ni_vnf1_dot_vm5_1_slave[h] <= sum{v in {19}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_1_slave{v in {19}, h in Hs}: ni_vnf1_dot_vm5_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_9_{h in Hs}: ni_vnf1_dot_vm5_1_master[h] + ni_vnf1_dot_vm5_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_master{h in Hs}: ni_vnf1_dot_vm6_0_master[h] <= sum{v in {20}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_master{v in {20}, h in Hs}: ni_vnf1_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_slave{h in Hs}: ni_vnf1_dot_vm6_0_slave[h] <= sum{v in {21}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_slave{v in {21}, h in Hs}: ni_vnf1_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_10_{h in Hs}: ni_vnf1_dot_vm6_0_master[h] + ni_vnf1_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm6_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_1_master{h in Hs}: ni_vnf1_dot_vm6_1_master[h] <= sum{v in {22}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_1_master{v in {22}, h in Hs}: ni_vnf1_dot_vm6_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm6_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_1_slave{h in Hs}: ni_vnf1_dot_vm6_1_slave[h] <= sum{v in {23}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_1_slave{v in {23}, h in Hs}: ni_vnf1_dot_vm6_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_11_{h in Hs}: ni_vnf1_dot_vm6_1_master[h] + ni_vnf1_dot_vm6_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm7_0_master{h in Hs}: ni_vnf1_dot_vm7_0_master[h] <= sum{v in {24}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm7_0_master{v in {24}, h in Hs}: ni_vnf1_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm7_0_slave{h in Hs}: ni_vnf1_dot_vm7_0_slave[h] <= sum{v in {25}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm7_0_slave{v in {25}, h in Hs}: ni_vnf1_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_12_{h in Hs}: ni_vnf1_dot_vm7_0_master[h] + ni_vnf1_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm7_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm7_1_master{h in Hs}: ni_vnf1_dot_vm7_1_master[h] <= sum{v in {26}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm7_1_master{v in {26}, h in Hs}: ni_vnf1_dot_vm7_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm7_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm7_1_slave{h in Hs}: ni_vnf1_dot_vm7_1_slave[h] <= sum{v in {27}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm7_1_slave{v in {27}, h in Hs}: ni_vnf1_dot_vm7_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_13_{h in Hs}: ni_vnf1_dot_vm7_1_master[h] + ni_vnf1_dot_vm7_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm9_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm9_0_master{h in Hs}: ni_vnf2_dot_vm9_0_master[h] <= sum{v in {32}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm9_0_master{v in {32}, h in Hs}: ni_vnf2_dot_vm9_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm9_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm9_0_slave{h in Hs}: ni_vnf2_dot_vm9_0_slave[h] <= sum{v in {33}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm9_0_slave{v in {33}, h in Hs}: ni_vnf2_dot_vm9_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_14_{h in Hs}: ni_vnf2_dot_vm9_0_master[h] + ni_vnf2_dot_vm9_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm9_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm9_1_master{h in Hs}: ni_vnf2_dot_vm9_1_master[h] <= sum{v in {34}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm9_1_master{v in {34}, h in Hs}: ni_vnf2_dot_vm9_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm9_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm9_1_slave{h in Hs}: ni_vnf2_dot_vm9_1_slave[h] <= sum{v in {35}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm9_1_slave{v in {35}, h in Hs}: ni_vnf2_dot_vm9_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_15_{h in Hs}: ni_vnf2_dot_vm9_1_master[h] + ni_vnf2_dot_vm9_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_master{h in Hs}: ni_vnf3_dot_vm1_0_master[h] <= sum{v in {40}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_master{v in {40}, h in Hs}: ni_vnf3_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_slave{h in Hs}: ni_vnf3_dot_vm1_0_slave[h] <= sum{v in {41}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_slave{v in {41}, h in Hs}: ni_vnf3_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_16_{h in Hs}: ni_vnf3_dot_vm1_0_master[h] + ni_vnf3_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_master{h in Hs}: ni_vnf3_dot_vm2_0_master[h] <= sum{v in {42}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_master{v in {42}, h in Hs}: ni_vnf3_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_slave{h in Hs}: ni_vnf3_dot_vm2_0_slave[h] <= sum{v in {43}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_slave{v in {43}, h in Hs}: ni_vnf3_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_17_{h in Hs}: ni_vnf3_dot_vm2_0_master[h] + ni_vnf3_dot_vm2_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm3_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm3_0_master{h in Hs}: ni_vnf3_dot_vm3_0_master[h] <= sum{v in {44}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm3_0_master{v in {44}, h in Hs}: ni_vnf3_dot_vm3_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm3_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm3_0_slave{h in Hs}: ni_vnf3_dot_vm3_0_slave[h] <= sum{v in {45}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm3_0_slave{v in {45}, h in Hs}: ni_vnf3_dot_vm3_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_18_{h in Hs}: ni_vnf3_dot_vm3_0_master[h] + ni_vnf3_dot_vm3_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm4_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm4_0_master{h in Hs}: ni_vnf3_dot_vm4_0_master[h] <= sum{v in {46}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm4_0_master{v in {46}, h in Hs}: ni_vnf3_dot_vm4_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm4_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm4_0_slave{h in Hs}: ni_vnf3_dot_vm4_0_slave[h] <= sum{v in {47}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm4_0_slave{v in {47}, h in Hs}: ni_vnf3_dot_vm4_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_19_{h in Hs}: ni_vnf3_dot_vm4_0_master[h] + ni_vnf3_dot_vm4_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm5_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm5_0_master{h in Hs}: ni_vnf3_dot_vm5_0_master[h] <= sum{v in {48}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm5_0_master{v in {48}, h in Hs}: ni_vnf3_dot_vm5_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm5_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm5_0_slave{h in Hs}: ni_vnf3_dot_vm5_0_slave[h] <= sum{v in {49}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm5_0_slave{v in {49}, h in Hs}: ni_vnf3_dot_vm5_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_20_{h in Hs}: ni_vnf3_dot_vm5_0_master[h] + ni_vnf3_dot_vm5_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm6_0_master{h in Hs}: ni_vnf3_dot_vm6_0_master[h] <= sum{v in {50}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm6_0_master{v in {50}, h in Hs}: ni_vnf3_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm6_0_slave{h in Hs}: ni_vnf3_dot_vm6_0_slave[h] <= sum{v in {51}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm6_0_slave{v in {51}, h in Hs}: ni_vnf3_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_21_{h in Hs}: ni_vnf3_dot_vm6_0_master[h] + ni_vnf3_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm7_0_master{h in Hs}: ni_vnf3_dot_vm7_0_master[h] <= sum{v in {52}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm7_0_master{v in {52}, h in Hs}: ni_vnf3_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm7_0_slave{h in Hs}: ni_vnf3_dot_vm7_0_slave[h] <= sum{v in {53}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm7_0_slave{v in {53}, h in Hs}: ni_vnf3_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_22_{h in Hs}: ni_vnf3_dot_vm7_0_master[h] + ni_vnf3_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf4_dot_vm9_0_master{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm9_0_master{h in Hs}: ni_vnf4_dot_vm9_0_master[h] <= sum{v in {56}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm9_0_master{v in {56}, h in Hs}: ni_vnf4_dot_vm9_0_master[h] >= x[v, h];
var ni_vnf4_dot_vm9_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm9_0_slave{h in Hs}: ni_vnf4_dot_vm9_0_slave[h] <= sum{v in {57}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm9_0_slave{v in {57}, h in Hs}: ni_vnf4_dot_vm9_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_23_{h in Hs}: ni_vnf4_dot_vm9_0_master[h] + ni_vnf4_dot_vm9_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm11_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm11_0_master{h in Hs}: ni_vnf5_dot_vm11_0_master[h] <= sum{v in {60}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm11_0_master{v in {60}, h in Hs}: ni_vnf5_dot_vm11_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm11_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm11_0_slave{h in Hs}: ni_vnf5_dot_vm11_0_slave[h] <= sum{v in {61}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm11_0_slave{v in {61}, h in Hs}: ni_vnf5_dot_vm11_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_24_{h in Hs}: ni_vnf5_dot_vm11_0_master[h] + ni_vnf5_dot_vm11_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm11_1_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm11_1_master{h in Hs}: ni_vnf5_dot_vm11_1_master[h] <= sum{v in {62}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm11_1_master{v in {62}, h in Hs}: ni_vnf5_dot_vm11_1_master[h] >= x[v, h];
var ni_vnf5_dot_vm11_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm11_1_slave{h in Hs}: ni_vnf5_dot_vm11_1_slave[h] <= sum{v in {63}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm11_1_slave{v in {63}, h in Hs}: ni_vnf5_dot_vm11_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_25_{h in Hs}: ni_vnf5_dot_vm11_1_master[h] + ni_vnf5_dot_vm11_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm14_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm14_0_master{h in Hs}: ni_vnf5_dot_vm14_0_master[h] <= sum{v in {76}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm14_0_master{v in {76}, h in Hs}: ni_vnf5_dot_vm14_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm14_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm14_0_slave{h in Hs}: ni_vnf5_dot_vm14_0_slave[h] <= sum{v in {77}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm14_0_slave{v in {77}, h in Hs}: ni_vnf5_dot_vm14_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_26_{h in Hs}: ni_vnf5_dot_vm14_0_master[h] + ni_vnf5_dot_vm14_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm14_1_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm14_1_master{h in Hs}: ni_vnf5_dot_vm14_1_master[h] <= sum{v in {78}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm14_1_master{v in {78}, h in Hs}: ni_vnf5_dot_vm14_1_master[h] >= x[v, h];
var ni_vnf5_dot_vm14_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm14_1_slave{h in Hs}: ni_vnf5_dot_vm14_1_slave[h] <= sum{v in {79}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm14_1_slave{v in {79}, h in Hs}: ni_vnf5_dot_vm14_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_27_{h in Hs}: ni_vnf5_dot_vm14_1_master[h] + ni_vnf5_dot_vm14_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm16_0_master{h in Hs}: ni_vnf5_dot_vm16_0_master[h] <= sum{v in {84}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm16_0_master{v in {84}, h in Hs}: ni_vnf5_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm16_0_slave{h in Hs}: ni_vnf5_dot_vm16_0_slave[h] <= sum{v in {85}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm16_0_slave{v in {85}, h in Hs}: ni_vnf5_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_28_{h in Hs}: ni_vnf5_dot_vm16_0_master[h] + ni_vnf5_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm16_1_master{h in Hs}: ni_vnf5_dot_vm16_1_master[h] <= sum{v in {86}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm16_1_master{v in {86}, h in Hs}: ni_vnf5_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf5_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm16_1_slave{h in Hs}: ni_vnf5_dot_vm16_1_slave[h] <= sum{v in {87}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm16_1_slave{v in {87}, h in Hs}: ni_vnf5_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_29_{h in Hs}: ni_vnf5_dot_vm16_1_master[h] + ni_vnf5_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm18_0_master{h in Hs}: ni_vnf5_dot_vm18_0_master[h] <= sum{v in {92}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm18_0_master{v in {92}, h in Hs}: ni_vnf5_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm18_0_slave{h in Hs}: ni_vnf5_dot_vm18_0_slave[h] <= sum{v in {93}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm18_0_slave{v in {93}, h in Hs}: ni_vnf5_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_30_{h in Hs}: ni_vnf5_dot_vm18_0_master[h] + ni_vnf5_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm18_1_master{h in Hs}: ni_vnf5_dot_vm18_1_master[h] <= sum{v in {94}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm18_1_master{v in {94}, h in Hs}: ni_vnf5_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf5_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm18_1_slave{h in Hs}: ni_vnf5_dot_vm18_1_slave[h] <= sum{v in {95}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm18_1_slave{v in {95}, h in Hs}: ni_vnf5_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_31_{h in Hs}: ni_vnf5_dot_vm18_1_master[h] + ni_vnf5_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm19_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm19_0_master{h in Hs}: ni_vnf5_dot_vm19_0_master[h] <= sum{v in {96}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm19_0_master{v in {96}, h in Hs}: ni_vnf5_dot_vm19_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm19_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm19_0_slave{h in Hs}: ni_vnf5_dot_vm19_0_slave[h] <= sum{v in {97}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm19_0_slave{v in {97}, h in Hs}: ni_vnf5_dot_vm19_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_32_{h in Hs}: ni_vnf5_dot_vm19_0_master[h] + ni_vnf5_dot_vm19_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm19_1_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm19_1_master{h in Hs}: ni_vnf5_dot_vm19_1_master[h] <= sum{v in {98}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm19_1_master{v in {98}, h in Hs}: ni_vnf5_dot_vm19_1_master[h] >= x[v, h];
var ni_vnf5_dot_vm19_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm19_1_slave{h in Hs}: ni_vnf5_dot_vm19_1_slave[h] <= sum{v in {99}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm19_1_slave{v in {99}, h in Hs}: ni_vnf5_dot_vm19_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_33_{h in Hs}: ni_vnf5_dot_vm19_1_master[h] + ni_vnf5_dot_vm19_1_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4
1 4
2 4
3 4
4 2
5 2
6 2
7 2
8 8
9 8
10 8
11 8
12 4
13 4
14 4
15 4
16 8
17 8
18 8
19 8
20 4
21 4
22 4
23 4
24 2
25 2
26 2
27 2
28 9
29 9
30 9
31 9
32 4
33 4
34 4
35 4
36 9
37 9
38 9
39 9
40 4
41 4
42 2
43 2
44 8
45 8
46 4
47 4
48 8
49 8
50 4
51 4
52 2
53 2
54 9
55 9
56 4
57 4
58 9
59 9
60 8
61 8
62 8
63 8
64 4
65 4
66 4
67 4
68 4
69 4
70 4
71 4
72 2
73 2
74 2
75 2
76 2
77 2
78 2
79 2
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 4
89 4
90 4
91 4
92 2
93 2
94 2
95 2
96 2
97 2
98 2
99 2
100 4
101 4
102 4
103 4;
param: MEM := 
0 24
1 24
2 24
3 24
4 8
5 8
6 8
7 8
8 16
9 16
10 16
11 16
12 16
13 16
14 16
15 16
16 32
17 32
18 32
19 32
20 16
21 16
22 16
23 16
24 8
25 8
26 8
27 8
28 16
29 16
30 16
31 16
32 8
33 8
34 8
35 8
36 16
37 16
38 16
39 16
40 24
41 24
42 8
43 8
44 16
45 16
46 16
47 16
48 32
49 32
50 16
51 16
52 8
53 8
54 16
55 16
56 8
57 8
58 16
59 16
60 32
61 32
62 32
63 32
64 12
65 12
66 12
67 12
68 12
69 12
70 12
71 12
72 24
73 24
74 24
75 24
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 16
85 16
86 16
87 16
88 24
89 24
90 24
91 24
92 8
93 8
94 8
95 8
96 10
97 10
98 10
99 10
100 40
101 40
102 40
103 40;
param: NET := 
0 375
1 375
2 375
3 375
4 80
5 80
6 80
7 80
8 1100
9 1100
10 1100
11 1100
12 120
13 120
14 120
15 120
16 235
17 235
18 235
19 235
20 235
21 235
22 235
23 235
24 10
25 10
26 10
27 10
28 3200
29 3200
30 3200
31 3200
32 310
33 310
34 310
35 310
36 800
37 800
38 800
39 800
40 375
41 375
42 80
43 80
44 1100
45 1100
46 120
47 120
48 235
49 235
50 235
51 235
52 10
53 10
54 3200
55 3200
56 310
57 310
58 800
59 800
60 3100
61 3100
62 3100
63 3100
64 1062
65 1062
66 1062
67 1062
68 1062
69 1062
70 1062
71 1062
72 210
73 210
74 210
75 210
76 200
77 200
78 200
79 200
80 720
81 720
82 720
83 720
84 1350
85 1350
86 1350
87 1350
88 200
89 200
90 200
91 200
92 1000
93 1000
94 1000
95 1000
96 600
97 600
98 600
99 600
100 900
101 900
102 900
103 900;


######################
# SOLVE 
######################
end;
