

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..253;
# Set of Hosts
set Hs := 0..30;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {4..6}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {7..9}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {10..12}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm6_0_{h in Hs}: sum{v in {13,14}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm7_0_{h in Hs}: sum{v in {15,16}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm8_0_{h in Hs}: sum{v in {17,18}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm9_0_{h in Hs}: sum{v in {19,20}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm10_0_{h in Hs}: sum{v in {21,22}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm11_0_{h in Hs}: sum{v in {23,24}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm12_0_{h in Hs}: sum{v in {25..27}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm14_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm15_0_{h in Hs}: sum{v in {31,32}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm16_0_{h in Hs}: sum{v in {33,34}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_0_{h in Hs}: sum{v in {35,36}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_0_{h in Hs}: sum{v in {37,38}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_0_{h in Hs}: sum{v in {39,40}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_0_{h in Hs}: sum{v in {41,42}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_0_{h in Hs}: sum{v in {43,44}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_0_{h in Hs}: sum{v in {45,46}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {49..51}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {52..54}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {55..57}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm6_0_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm7_0_{h in Hs}: sum{v in {60,61}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm8_0_{h in Hs}: sum{v in {62,63}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm9_0_{h in Hs}: sum{v in {64,65}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm10_0_{h in Hs}: sum{v in {66,67}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm11_0_{h in Hs}: sum{v in {68,69}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm12_0_{h in Hs}: sum{v in {70..72}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm14_0_{h in Hs}: sum{v in {74,75}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm15_0_{h in Hs}: sum{v in {76,77}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm16_0_{h in Hs}: sum{v in {78,79}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm17_0_{h in Hs}: sum{v in {80,81}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm18_0_{h in Hs}: sum{v in {82,83}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm19_0_{h in Hs}: sum{v in {84,85}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm20_0_{h in Hs}: sum{v in {86,87}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm21_0_{h in Hs}: sum{v in {88,89}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm22_0_{h in Hs}: sum{v in {90,91}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm41_0_{h in Hs}: sum{v in {94..98}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_0_{h in Hs}: sum{v in {99..102}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm42_0_{h in Hs}: sum{v in {103,104}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm43_0_{h in Hs}: sum{v in {105,106}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm44_0_{h in Hs}: sum{v in {107,108}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm41_0_{h in Hs}: sum{v in {109..113}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm45_0_{h in Hs}: sum{v in {114,115}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm46_0_{h in Hs}: sum{v in {116,117}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_0_{h in Hs}: sum{v in {118..121}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm42_0_{h in Hs}: sum{v in {122..124}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm47_0_{h in Hs}: sum{v in {125,126}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm43_0_{h in Hs}: sum{v in {127,128}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm48_0_{h in Hs}: sum{v in {129,130}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm52_0_{h in Hs}: sum{v in {134,135}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm66_0_{h in Hs}: sum{v in {168,169}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm66_1_{h in Hs}: sum{v in {170,171}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm67_0_{h in Hs}: sum{v in {172,173}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm67_1_{h in Hs}: sum{v in {174,175}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm68_0_{h in Hs}: sum{v in {176,177}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm68_1_{h in Hs}: sum{v in {178,179}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm69_0_{h in Hs}: sum{v in {180,181}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm69_1_{h in Hs}: sum{v in {182,183}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm70_0_{h in Hs}: sum{v in {184,185}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm70_1_{h in Hs}: sum{v in {186,187}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm71_0_{h in Hs}: sum{v in {188,189}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm71_1_{h in Hs}: sum{v in {190,191}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm72_0_{h in Hs}: sum{v in {192}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm72_1_{h in Hs}: sum{v in {193}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm73_0_{h in Hs}: sum{v in {194,195}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm75_0_{h in Hs}: sum{v in {199..201}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm83_0_{h in Hs}: sum{v in {229..232}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm84_0_{h in Hs}: sum{v in {233,234}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm86_0_{h in Hs}: sum{v in {237..239}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm87_0_{h in Hs}: sum{v in {240..243}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm92_0_{h in Hs}: sum{v in {250,251}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm92_1_{h in Hs}: sum{v in {252,253}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm77_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm77_0_master{h in Hs}: ni_vnf14_dot_vm77_0_master[h] <= sum{v in {205}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm77_0_master{v in {205}, h in Hs}: ni_vnf14_dot_vm77_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm77_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm77_0_slave{h in Hs}: ni_vnf14_dot_vm77_0_slave[h] <= sum{v in {206}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm77_0_slave{v in {206}, h in Hs}: ni_vnf14_dot_vm77_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf14_dot_vm77_0_master[h] + ni_vnf14_dot_vm77_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm78_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm78_0_master{h in Hs}: ni_vnf14_dot_vm78_0_master[h] <= sum{v in {207}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm78_0_master{v in {207}, h in Hs}: ni_vnf14_dot_vm78_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm78_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm78_0_slave{h in Hs}: ni_vnf14_dot_vm78_0_slave[h] <= sum{v in {208}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm78_0_slave{v in {208}, h in Hs}: ni_vnf14_dot_vm78_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf14_dot_vm78_0_master[h] + ni_vnf14_dot_vm78_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm79_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm79_0_master{h in Hs}: ni_vnf14_dot_vm79_0_master[h] <= sum{v in {209}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm79_0_master{v in {209}, h in Hs}: ni_vnf14_dot_vm79_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm79_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm79_0_slave{h in Hs}: ni_vnf14_dot_vm79_0_slave[h] <= sum{v in {210}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm79_0_slave{v in {210}, h in Hs}: ni_vnf14_dot_vm79_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf14_dot_vm79_0_master[h] + ni_vnf14_dot_vm79_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm80_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm80_0_master{h in Hs}: ni_vnf14_dot_vm80_0_master[h] <= sum{v in {211,212}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm80_0_master{v in {211,212}, h in Hs}: ni_vnf14_dot_vm80_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm80_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm80_0_slave{h in Hs}: ni_vnf14_dot_vm80_0_slave[h] <= sum{v in {213,214}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm80_0_slave{v in {213,214}, h in Hs}: ni_vnf14_dot_vm80_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf14_dot_vm80_0_master[h] + ni_vnf14_dot_vm80_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm81_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm81_0_master{h in Hs}: ni_vnf14_dot_vm81_0_master[h] <= sum{v in {215}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm81_0_master{v in {215}, h in Hs}: ni_vnf14_dot_vm81_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm81_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm81_0_slave{h in Hs}: ni_vnf14_dot_vm81_0_slave[h] <= sum{v in {216}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm81_0_slave{v in {216}, h in Hs}: ni_vnf14_dot_vm81_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf14_dot_vm81_0_master[h] + ni_vnf14_dot_vm81_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm85_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm85_0_master{h in Hs}: ni_vnf14_dot_vm85_0_master[h] <= sum{v in {235}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm85_0_master{v in {235}, h in Hs}: ni_vnf14_dot_vm85_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm85_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm85_0_slave{h in Hs}: ni_vnf14_dot_vm85_0_slave[h] <= sum{v in {236}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm85_0_slave{v in {236}, h in Hs}: ni_vnf14_dot_vm85_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf14_dot_vm85_0_master[h] + ni_vnf14_dot_vm85_0_slave[h] <= 1;

#=== Soft Affinity Variables ===
var vnf13_dot_vm74_0{h in Hs}, binary;
s.t. or_sum_vnf13_dot_vm74_0{h in Hs}: vnf13_dot_vm74_0[h] <= sum{v in {196..198}} x[v, h];
s.t. or_ineq_vnf13_dot_vm74_0{v in {196..198}, h in Hs}: vnf13_dot_vm74_0[h] >= x[v, h];
var vnf13_dot_vm76_0{h in Hs}, binary;
s.t. or_sum_vnf13_dot_vm76_0{h in Hs}: vnf13_dot_vm76_0[h] <= sum{v in {202..204}} x[v, h];
s.t. or_ineq_vnf13_dot_vm76_0{v in {202..204}, h in Hs}: vnf13_dot_vm76_0[h] >= x[v, h];
var vnf16_dot_vm89_0{h in Hs}, binary;
s.t. or_sum_vnf16_dot_vm89_0{h in Hs}: vnf16_dot_vm89_0[h] <= sum{v in {246}} x[v, h];
s.t. or_ineq_vnf16_dot_vm89_0{v in {246}, h in Hs}: vnf16_dot_vm89_0[h] >= x[v, h];
var vnf17_dot_vm89_0{h in Hs}, binary;
s.t. or_sum_vnf17_dot_vm89_0{h in Hs}: vnf17_dot_vm89_0[h] <= sum{v in {247}} x[v, h];
s.t. or_ineq_vnf17_dot_vm89_0{v in {247}, h in Hs}: vnf17_dot_vm89_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf13_dot_vm74_0[h]) - 1 + (sum{h in Hs} vnf13_dot_vm76_0[h]) - 1 + (sum{h in Hs} vnf16_dot_vm89_0[h]) - 1 + (sum{h in Hs} vnf17_dot_vm89_0[h]) - 1) / 8;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 4.0
5 4.0
6 4.0
7 4.0
8 4.0
9 4.0
10 4.0
11 4.0
12 4.0
13 2.0
14 2.0
15 4.0
16 4.0
17 4.0
18 4.0
19 8.0
20 8.0
21 2.0
22 2.0
23 2.0
24 2.0
25 4.0
26 4.0
27 4.0
28 2.0
29 4.0
30 4.0
31 8.0
32 8.0
33 4.0
34 4.0
35 4.0
36 4.0
37 4.0
38 4.0
39 4.0
40 4.0
41 4.0
42 4.0
43 4.0
44 4.0
45 2.0
46 2.0
47 4.0
48 0.0
49 4.0
50 4.0
51 4.0
52 4.0
53 4.0
54 4.0
55 4.0
56 4.0
57 4.0
58 2.0
59 2.0
60 4.0
61 4.0
62 4.0
63 4.0
64 8.0
65 8.0
66 2.0
67 2.0
68 2.0
69 2.0
70 4.0
71 4.0
72 4.0
73 2.0
74 4.0
75 4.0
76 8.0
77 8.0
78 4.0
79 4.0
80 4.0
81 4.0
82 4.0
83 4.0
84 4.0
85 4.0
86 4.0
87 4.0
88 4.0
89 4.0
90 2.0
91 2.0
92 4.0
93 0.0
94 8.0
95 8.0
96 8.0
97 8.0
98 8.0
99 12.0
100 12.0
101 12.0
102 12.0
103 12.0
104 12.0
105 8.0
106 8.0
107 8.0
108 8.0
109 8.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 12.0
117 12.0
118 12.0
119 12.0
120 12.0
121 12.0
122 12.0
123 12.0
124 12.0
125 12.0
126 12.0
127 8.0
128 8.0
129 12.0
130 12.0
131 0.0
132 0.0
133 8.0
134 6.0
135 6.0
136 6.0
137 6.0
138 3.0
139 3.0
140 4.0
141 4.0
142 4.0
143 4.0
144 4.0
145 4.0
146 4.0
147 4.0
148 4.0
149 4.0
150 4.0
151 4.0
152 4.0
153 4.0
154 4.0
155 4.0
156 4.0
157 4.0
158 4.0
159 4.0
160 4.0
161 4.0
162 4.0
163 4.0
164 4.0
165 1.0
166 1.0
167 8.0
168 8.0
169 8.0
170 8.0
171 8.0
172 8.0
173 8.0
174 8.0
175 8.0
176 16.0
177 16.0
178 16.0
179 16.0
180 8.0
181 8.0
182 8.0
183 8.0
184 8.0
185 8.0
186 8.0
187 8.0
188 4.0
189 4.0
190 4.0
191 4.0
192 4.0
193 4.0
194 8.0
195 8.0
196 4.0
197 4.0
198 4.0
199 4.0
200 4.0
201 4.0
202 4.0
203 4.0
204 4.0
205 2.0
206 2.0
207 3.0
208 3.0
209 2.0
210 2.0
211 3.0
212 3.0
213 3.0
214 3.0
215 4.0
216 4.0
217 2.0
218 2.0
219 2.0
220 2.0
221 2.0
222 2.0
223 2.0
224 2.0
225 2.0
226 2.0
227 2.0
228 2.0
229 3.0
230 3.0
231 3.0
232 3.0
233 3.0
234 3.0
235 2.0
236 2.0
237 3.0
238 3.0
239 3.0
240 6.0
241 6.0
242 6.0
243 6.0
244 8.0
245 8.0
246 16.0
247 16.0
248 4.0
249 16.0
250 2.0
251 2.0
252 2.0
253 2.0;
param: MEM := 
0 32.0
1 32.0
2 32.0
3 32.0
4 32.0
5 32.0
6 32.0
7 16.0
8 16.0
9 16.0
10 16.0
11 16.0
12 16.0
13 16.0
14 16.0
15 16.0
16 16.0
17 8.0
18 8.0
19 18.0
20 18.0
21 8.0
22 8.0
23 4.0
24 4.0
25 48.0
26 48.0
27 48.0
28 8.0
29 4.0
30 4.0
31 32.0
32 32.0
33 8.0
34 8.0
35 8.0
36 8.0
37 4.0
38 4.0
39 4.0
40 4.0
41 4.0
42 4.0
43 4.0
44 4.0
45 4.0
46 4.0
47 8.0
48 0.0
49 32.0
50 32.0
51 32.0
52 16.0
53 16.0
54 16.0
55 16.0
56 16.0
57 16.0
58 16.0
59 16.0
60 16.0
61 16.0
62 8.0
63 8.0
64 18.0
65 18.0
66 8.0
67 8.0
68 4.0
69 4.0
70 48.0
71 48.0
72 48.0
73 8.0
74 4.0
75 4.0
76 32.0
77 32.0
78 8.0
79 8.0
80 8.0
81 8.0
82 4.0
83 4.0
84 4.0
85 4.0
86 4.0
87 4.0
88 4.0
89 4.0
90 4.0
91 4.0
92 8.0
93 0.0
94 12.0
95 12.0
96 12.0
97 12.0
98 12.0
99 74.0
100 74.0
101 74.0
102 74.0
103 24.0
104 24.0
105 24.0
106 24.0
107 48.0
108 48.0
109 12.0
110 12.0
111 12.0
112 12.0
113 12.0
114 12.0
115 12.0
116 32.0
117 32.0
118 102.0
119 102.0
120 102.0
121 102.0
122 24.0
123 24.0
124 24.0
125 24.0
126 24.0
127 24.0
128 24.0
129 48.0
130 48.0
131 0.0
132 0.0
133 16.0
134 8.0
135 8.0
136 8.0
137 6.0
138 3.0
139 3.0
140 8.0
141 8.0
142 8.0
143 8.0
144 8.0
145 8.0
146 8.0
147 8.0
148 8.0
149 8.0
150 8.0
151 10.0
152 10.0
153 10.0
154 10.0
155 8.0
156 8.0
157 8.0
158 8.0
159 8.0
160 8.0
161 8.0
162 8.0
163 64.0
164 48.0
165 2.0
166 24.0
167 18.0
168 32.0
169 32.0
170 32.0
171 32.0
172 32.0
173 32.0
174 32.0
175 32.0
176 16.0
177 16.0
178 16.0
179 16.0
180 16.0
181 16.0
182 16.0
183 16.0
184 16.0
185 16.0
186 16.0
187 16.0
188 32.0
189 32.0
190 32.0
191 32.0
192 32.0
193 32.0
194 16.0
195 16.0
196 16.0
197 16.0
198 16.0
199 16.0
200 16.0
201 16.0
202 16.0
203 16.0
204 16.0
205 2.0
206 2.0
207 2.0
208 2.0
209 2.0
210 2.0
211 2.0
212 2.0
213 2.0
214 2.0
215 1.0
216 1.0
217 3.0
218 3.0
219 3.0
220 3.0
221 3.0
222 3.0
223 3.0
224 3.0
225 3.0
226 3.0
227 3.0
228 3.0
229 3.0
230 3.0
231 3.0
232 3.0
233 3.0
234 3.0
235 1.0
236 1.0
237 2.0
238 2.0
239 2.0
240 2.0
241 2.0
242 2.0
243 2.0
244 32.0
245 32.0
246 32.0
247 32.0
248 30.0
249 32.0
250 8.0
251 8.0
252 8.0
253 8.0;
param: NET := 
0 2.0
1 2.0
2 2.0
3 2.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 1.0
110 1.0
111 1.0
112 1.0
113 1.0
114 11.0
115 11.0
116 43.0
117 43.0
118 81.0
119 81.0
120 81.0
121 81.0
122 92.0
123 92.0
124 92.0
125 63.0
126 63.0
127 13.0
128 13.0
129 46.0
130 46.0
131 0.0
132 0.0
133 19.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 9.0
141 9.0
142 1.0
143 1.0
144 1.0
145 1.0
146 1.0
147 5.0
148 5.0
149 18.0
150 18.0
151 38.0
152 38.0
153 38.0
154 38.0
155 28.0
156 28.0
157 27.0
158 27.0
159 6.0
160 6.0
161 21.0
162 21.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 0.0
179 0.0
180 0.0
181 0.0
182 0.0
183 0.0
184 0.0
185 0.0
186 0.0
187 0.0
188 0.0
189 0.0
190 0.0
191 0.0
192 0.0
193 0.0
194 0.0
195 0.0
196 600.0
197 600.0
198 600.0
199 300.0
200 300.0
201 300.0
202 350.0
203 350.0
204 350.0
205 0.0
206 0.0
207 0.0
208 0.0
209 0.0
210 0.0
211 0.0
212 0.0
213 0.0
214 0.0
215 0.0
216 0.0
217 0.0
218 0.0
219 0.0
220 0.0
221 0.0
222 0.0
223 0.0
224 0.0
225 0.0
226 0.0
227 0.0
228 0.0
229 0.0
230 0.0
231 0.0
232 0.0
233 0.0
234 0.0
235 0.0
236 0.0
237 0.0
238 0.0
239 0.0
240 0.0
241 0.0
242 0.0
243 0.0
244 2048.0
245 2048.0
246 2000.0
247 2000.0
248 0.0
249 200.0
250 2.0
251 2.0
252 2.0
253 2.0;


######################
# SOLVE 
######################
end;
