

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..208;
# Set of Hosts
set Hs := 0..26;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {4..6}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {7..9}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {10..12}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm6_0_{h in Hs}: sum{v in {13,14}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm7_0_{h in Hs}: sum{v in {15,16}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm8_0_{h in Hs}: sum{v in {17,18}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm9_0_{h in Hs}: sum{v in {19,20}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm10_0_{h in Hs}: sum{v in {21,22}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm11_0_{h in Hs}: sum{v in {23,24}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm12_0_{h in Hs}: sum{v in {25..27}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm14_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm15_0_{h in Hs}: sum{v in {31,32}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm16_0_{h in Hs}: sum{v in {33,34}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm17_0_{h in Hs}: sum{v in {35,36}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm18_0_{h in Hs}: sum{v in {37,38}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm19_0_{h in Hs}: sum{v in {39,40}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm20_0_{h in Hs}: sum{v in {41,42}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm21_0_{h in Hs}: sum{v in {43,44}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm22_0_{h in Hs}: sum{v in {45,46}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm41_0_{h in Hs}: sum{v in {49..53}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_0_{h in Hs}: sum{v in {54..57}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm42_0_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm43_0_{h in Hs}: sum{v in {60,61}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm44_0_{h in Hs}: sum{v in {62,63}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm41_0_{h in Hs}: sum{v in {64..68}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm45_0_{h in Hs}: sum{v in {69,70}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm46_0_{h in Hs}: sum{v in {71,72}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_0_{h in Hs}: sum{v in {73..76}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm42_0_{h in Hs}: sum{v in {77..79}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm47_0_{h in Hs}: sum{v in {80,81}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm43_0_{h in Hs}: sum{v in {82,83}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm48_0_{h in Hs}: sum{v in {84,85}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm52_0_{h in Hs}: sum{v in {89,90}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm66_0_{h in Hs}: sum{v in {123,124}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm66_1_{h in Hs}: sum{v in {125,126}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm67_0_{h in Hs}: sum{v in {127,128}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm67_1_{h in Hs}: sum{v in {129,130}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm68_0_{h in Hs}: sum{v in {131,132}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm68_1_{h in Hs}: sum{v in {133,134}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm69_0_{h in Hs}: sum{v in {135,136}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm69_1_{h in Hs}: sum{v in {137,138}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm70_0_{h in Hs}: sum{v in {139,140}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm70_1_{h in Hs}: sum{v in {141,142}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm71_0_{h in Hs}: sum{v in {143,144}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm71_1_{h in Hs}: sum{v in {145,146}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm72_0_{h in Hs}: sum{v in {147}} x[v, h]<= 1;
s.t. AAF_vnf11_dot_vm72_1_{h in Hs}: sum{v in {148}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm73_0_{h in Hs}: sum{v in {149,150}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm75_0_{h in Hs}: sum{v in {154..156}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm83_0_{h in Hs}: sum{v in {184..187}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm84_0_{h in Hs}: sum{v in {188,189}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm86_0_{h in Hs}: sum{v in {192..194}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm87_0_{h in Hs}: sum{v in {195..198}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm92_0_{h in Hs}: sum{v in {205}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm92_1_{h in Hs}: sum{v in {206}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm93_0_{h in Hs}: sum{v in {207}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm93_1_{h in Hs}: sum{v in {208}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm77_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm77_0_master{h in Hs}: ni_vnf14_dot_vm77_0_master[h] <= sum{v in {160}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm77_0_master{v in {160}, h in Hs}: ni_vnf14_dot_vm77_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm77_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm77_0_slave{h in Hs}: ni_vnf14_dot_vm77_0_slave[h] <= sum{v in {161}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm77_0_slave{v in {161}, h in Hs}: ni_vnf14_dot_vm77_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf14_dot_vm77_0_master[h] + ni_vnf14_dot_vm77_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm78_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm78_0_master{h in Hs}: ni_vnf14_dot_vm78_0_master[h] <= sum{v in {162}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm78_0_master{v in {162}, h in Hs}: ni_vnf14_dot_vm78_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm78_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm78_0_slave{h in Hs}: ni_vnf14_dot_vm78_0_slave[h] <= sum{v in {163}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm78_0_slave{v in {163}, h in Hs}: ni_vnf14_dot_vm78_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf14_dot_vm78_0_master[h] + ni_vnf14_dot_vm78_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm79_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm79_0_master{h in Hs}: ni_vnf14_dot_vm79_0_master[h] <= sum{v in {164}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm79_0_master{v in {164}, h in Hs}: ni_vnf14_dot_vm79_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm79_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm79_0_slave{h in Hs}: ni_vnf14_dot_vm79_0_slave[h] <= sum{v in {165}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm79_0_slave{v in {165}, h in Hs}: ni_vnf14_dot_vm79_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf14_dot_vm79_0_master[h] + ni_vnf14_dot_vm79_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm80_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm80_0_master{h in Hs}: ni_vnf14_dot_vm80_0_master[h] <= sum{v in {166,167}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm80_0_master{v in {166,167}, h in Hs}: ni_vnf14_dot_vm80_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm80_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm80_0_slave{h in Hs}: ni_vnf14_dot_vm80_0_slave[h] <= sum{v in {168,169}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm80_0_slave{v in {168,169}, h in Hs}: ni_vnf14_dot_vm80_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf14_dot_vm80_0_master[h] + ni_vnf14_dot_vm80_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm81_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm81_0_master{h in Hs}: ni_vnf14_dot_vm81_0_master[h] <= sum{v in {170}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm81_0_master{v in {170}, h in Hs}: ni_vnf14_dot_vm81_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm81_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm81_0_slave{h in Hs}: ni_vnf14_dot_vm81_0_slave[h] <= sum{v in {171}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm81_0_slave{v in {171}, h in Hs}: ni_vnf14_dot_vm81_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf14_dot_vm81_0_master[h] + ni_vnf14_dot_vm81_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm85_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm85_0_master{h in Hs}: ni_vnf14_dot_vm85_0_master[h] <= sum{v in {190}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm85_0_master{v in {190}, h in Hs}: ni_vnf14_dot_vm85_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm85_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm85_0_slave{h in Hs}: ni_vnf14_dot_vm85_0_slave[h] <= sum{v in {191}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm85_0_slave{v in {191}, h in Hs}: ni_vnf14_dot_vm85_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf14_dot_vm85_0_master[h] + ni_vnf14_dot_vm85_0_slave[h] <= 1;

#=== Soft Affinity Variables ===
var vnf13_dot_vm74_0{h in Hs}, binary;
s.t. or_sum_vnf13_dot_vm74_0{h in Hs}: vnf13_dot_vm74_0[h] <= sum{v in {151..153}} x[v, h];
s.t. or_ineq_vnf13_dot_vm74_0{v in {151..153}, h in Hs}: vnf13_dot_vm74_0[h] >= x[v, h];
var vnf13_dot_vm76_0{h in Hs}, binary;
s.t. or_sum_vnf13_dot_vm76_0{h in Hs}: vnf13_dot_vm76_0[h] <= sum{v in {157..159}} x[v, h];
s.t. or_ineq_vnf13_dot_vm76_0{v in {157..159}, h in Hs}: vnf13_dot_vm76_0[h] >= x[v, h];
var vnf16_dot_vm89_0{h in Hs}, binary;
s.t. or_sum_vnf16_dot_vm89_0{h in Hs}: vnf16_dot_vm89_0[h] <= sum{v in {201}} x[v, h];
s.t. or_ineq_vnf16_dot_vm89_0{v in {201}, h in Hs}: vnf16_dot_vm89_0[h] >= x[v, h];
var vnf17_dot_vm89_0{h in Hs}, binary;
s.t. or_sum_vnf17_dot_vm89_0{h in Hs}: vnf17_dot_vm89_0[h] <= sum{v in {202}} x[v, h];
s.t. or_ineq_vnf17_dot_vm89_0{v in {202}, h in Hs}: vnf17_dot_vm89_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf13_dot_vm74_0[h]) - 1 + (sum{h in Hs} vnf13_dot_vm76_0[h]) - 1 + (sum{h in Hs} vnf16_dot_vm89_0[h]) - 1 + (sum{h in Hs} vnf17_dot_vm89_0[h]) - 1) / 8;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 4.0
5 4.0
6 4.0
7 4.0
8 4.0
9 4.0
10 4.0
11 4.0
12 4.0
13 2.0
14 2.0
15 4.0
16 4.0
17 4.0
18 4.0
19 8.0
20 8.0
21 2.0
22 2.0
23 2.0
24 2.0
25 4.0
26 4.0
27 4.0
28 2.0
29 4.0
30 4.0
31 8.0
32 8.0
33 4.0
34 4.0
35 4.0
36 4.0
37 4.0
38 4.0
39 4.0
40 4.0
41 4.0
42 4.0
43 4.0
44 4.0
45 2.0
46 2.0
47 4.0
48 0.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 12.0
55 12.0
56 12.0
57 12.0
58 12.0
59 12.0
60 8.0
61 8.0
62 8.0
63 8.0
64 8.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 12.0
72 12.0
73 12.0
74 12.0
75 12.0
76 12.0
77 12.0
78 12.0
79 12.0
80 12.0
81 12.0
82 8.0
83 8.0
84 12.0
85 12.0
86 0.0
87 0.0
88 8.0
89 6.0
90 6.0
91 6.0
92 6.0
93 3.0
94 3.0
95 4.0
96 4.0
97 4.0
98 4.0
99 4.0
100 4.0
101 4.0
102 4.0
103 4.0
104 4.0
105 4.0
106 4.0
107 4.0
108 4.0
109 4.0
110 4.0
111 4.0
112 4.0
113 4.0
114 4.0
115 4.0
116 4.0
117 4.0
118 4.0
119 4.0
120 1.0
121 1.0
122 8.0
123 8.0
124 8.0
125 8.0
126 8.0
127 8.0
128 8.0
129 8.0
130 8.0
131 16.0
132 16.0
133 16.0
134 16.0
135 8.0
136 8.0
137 8.0
138 8.0
139 8.0
140 8.0
141 8.0
142 8.0
143 4.0
144 4.0
145 4.0
146 4.0
147 4.0
148 4.0
149 8.0
150 8.0
151 4.0
152 4.0
153 4.0
154 4.0
155 4.0
156 4.0
157 4.0
158 4.0
159 4.0
160 2.0
161 2.0
162 3.0
163 3.0
164 2.0
165 2.0
166 3.0
167 3.0
168 3.0
169 3.0
170 4.0
171 4.0
172 2.0
173 2.0
174 2.0
175 2.0
176 2.0
177 2.0
178 2.0
179 2.0
180 2.0
181 2.0
182 2.0
183 2.0
184 3.0
185 3.0
186 3.0
187 3.0
188 3.0
189 3.0
190 2.0
191 2.0
192 3.0
193 3.0
194 3.0
195 6.0
196 6.0
197 6.0
198 6.0
199 8.0
200 8.0
201 16.0
202 16.0
203 4.0
204 16.0
205 2.0
206 2.0
207 2.0
208 2.0;
param: MEM := 
0 32.0
1 32.0
2 32.0
3 32.0
4 32.0
5 32.0
6 32.0
7 16.0
8 16.0
9 16.0
10 16.0
11 16.0
12 16.0
13 16.0
14 16.0
15 16.0
16 16.0
17 8.0
18 8.0
19 18.0
20 18.0
21 8.0
22 8.0
23 4.0
24 4.0
25 48.0
26 48.0
27 48.0
28 8.0
29 4.0
30 4.0
31 32.0
32 32.0
33 8.0
34 8.0
35 8.0
36 8.0
37 4.0
38 4.0
39 4.0
40 4.0
41 4.0
42 4.0
43 4.0
44 4.0
45 4.0
46 4.0
47 8.0
48 0.0
49 12.0
50 12.0
51 12.0
52 12.0
53 12.0
54 74.0
55 74.0
56 74.0
57 74.0
58 24.0
59 24.0
60 24.0
61 24.0
62 48.0
63 48.0
64 12.0
65 12.0
66 12.0
67 12.0
68 12.0
69 12.0
70 12.0
71 32.0
72 32.0
73 102.0
74 102.0
75 102.0
76 102.0
77 24.0
78 24.0
79 24.0
80 24.0
81 24.0
82 24.0
83 24.0
84 48.0
85 48.0
86 0.0
87 0.0
88 16.0
89 8.0
90 8.0
91 8.0
92 6.0
93 3.0
94 3.0
95 8.0
96 8.0
97 8.0
98 8.0
99 8.0
100 8.0
101 8.0
102 8.0
103 8.0
104 8.0
105 8.0
106 10.0
107 10.0
108 10.0
109 10.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 8.0
117 8.0
118 64.0
119 48.0
120 2.0
121 24.0
122 18.0
123 32.0
124 32.0
125 32.0
126 32.0
127 32.0
128 32.0
129 32.0
130 32.0
131 16.0
132 16.0
133 16.0
134 16.0
135 16.0
136 16.0
137 16.0
138 16.0
139 16.0
140 16.0
141 16.0
142 16.0
143 32.0
144 32.0
145 32.0
146 32.0
147 32.0
148 32.0
149 16.0
150 16.0
151 16.0
152 16.0
153 16.0
154 16.0
155 16.0
156 16.0
157 16.0
158 16.0
159 16.0
160 2.0
161 2.0
162 2.0
163 2.0
164 2.0
165 2.0
166 2.0
167 2.0
168 2.0
169 2.0
170 1.0
171 1.0
172 3.0
173 3.0
174 3.0
175 3.0
176 3.0
177 3.0
178 3.0
179 3.0
180 3.0
181 3.0
182 3.0
183 3.0
184 3.0
185 3.0
186 3.0
187 3.0
188 3.0
189 3.0
190 1.0
191 1.0
192 2.0
193 2.0
194 2.0
195 2.0
196 2.0
197 2.0
198 2.0
199 32.0
200 32.0
201 32.0
202 32.0
203 30.0
204 32.0
205 16.0
206 16.0
207 16.0
208 16.0;
param: NET := 
0 2.0
1 2.0
2 2.0
3 2.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 1.0
65 1.0
66 1.0
67 1.0
68 1.0
69 11.0
70 11.0
71 43.0
72 43.0
73 81.0
74 81.0
75 81.0
76 81.0
77 92.0
78 92.0
79 92.0
80 63.0
81 63.0
82 13.0
83 13.0
84 46.0
85 46.0
86 0.0
87 0.0
88 19.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 9.0
96 9.0
97 1.0
98 1.0
99 1.0
100 1.0
101 1.0
102 5.0
103 5.0
104 18.0
105 18.0
106 38.0
107 38.0
108 38.0
109 38.0
110 28.0
111 28.0
112 27.0
113 27.0
114 6.0
115 6.0
116 21.0
117 21.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0
143 0.0
144 0.0
145 0.0
146 0.0
147 0.0
148 0.0
149 0.0
150 0.0
151 600.0
152 600.0
153 600.0
154 300.0
155 300.0
156 300.0
157 350.0
158 350.0
159 350.0
160 0.0
161 0.0
162 0.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 0.0
179 0.0
180 0.0
181 0.0
182 0.0
183 0.0
184 0.0
185 0.0
186 0.0
187 0.0
188 0.0
189 0.0
190 0.0
191 0.0
192 0.0
193 0.0
194 0.0
195 0.0
196 0.0
197 0.0
198 0.0
199 2048.0
200 2048.0
201 2000.0
202 2000.0
203 0.0
204 200.0
205 0.0
206 0.0
207 0.0
208 0.0;


######################
# SOLVE 
######################
end;
