

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..89;
# Set of Hosts
set Hs := 0..7;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..2}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {3..5}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {6..8}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_0_{h in Hs}: sum{v in {9,10}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_0_{h in Hs}: sum{v in {11,12}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm6_0_{h in Hs}: sum{v in {13,14}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm7_0_{h in Hs}: sum{v in {15,16}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm8_0_{h in Hs}: sum{v in {17,18}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm9_0_{h in Hs}: sum{v in {19,20}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm10_0_{h in Hs}: sum{v in {21..23}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm12_0_{h in Hs}: sum{v in {25,26}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm13_0_{h in Hs}: sum{v in {27,28}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm14_0_{h in Hs}: sum{v in {29,30}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm15_0_{h in Hs}: sum{v in {31,32}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm16_0_{h in Hs}: sum{v in {33,34}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm17_0_{h in Hs}: sum{v in {35,36}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm18_0_{h in Hs}: sum{v in {37,38}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm19_0_{h in Hs}: sum{v in {39,40}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm20_0_{h in Hs}: sum{v in {41,42}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm1_0_{h in Hs}: sum{v in {45..47}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {48..50}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {51..53}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {54,55}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {56,57}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {60,61}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm8_0_{h in Hs}: sum{v in {62,63}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_0_{h in Hs}: sum{v in {64,65}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm10_0_{h in Hs}: sum{v in {66..68}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_0_{h in Hs}: sum{v in {70,71}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm13_0_{h in Hs}: sum{v in {72,73}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm14_0_{h in Hs}: sum{v in {74,75}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm15_0_{h in Hs}: sum{v in {76,77}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm16_0_{h in Hs}: sum{v in {78,79}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm17_0_{h in Hs}: sum{v in {80,81}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm18_0_{h in Hs}: sum{v in {82,83}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm19_0_{h in Hs}: sum{v in {84,85}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm20_0_{h in Hs}: sum{v in {86,87}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4.0
1 4.0
2 4.0
3 4.0
4 4.0
5 4.0
6 4.0
7 4.0
8 4.0
9 2.0
10 2.0
11 4.0
12 4.0
13 4.0
14 4.0
15 8.0
16 8.0
17 2.0
18 2.0
19 2.0
20 2.0
21 4.0
22 4.0
23 4.0
24 2.0
25 4.0
26 4.0
27 8.0
28 8.0
29 4.0
30 4.0
31 4.0
32 4.0
33 4.0
34 4.0
35 4.0
36 4.0
37 4.0
38 4.0
39 4.0
40 4.0
41 2.0
42 2.0
43 4.0
44 0.0
45 4.0
46 4.0
47 4.0
48 4.0
49 4.0
50 4.0
51 4.0
52 4.0
53 4.0
54 2.0
55 2.0
56 4.0
57 4.0
58 4.0
59 4.0
60 8.0
61 8.0
62 2.0
63 2.0
64 2.0
65 2.0
66 4.0
67 4.0
68 4.0
69 2.0
70 4.0
71 4.0
72 8.0
73 8.0
74 4.0
75 4.0
76 4.0
77 4.0
78 4.0
79 4.0
80 4.0
81 4.0
82 4.0
83 4.0
84 4.0
85 4.0
86 2.0
87 2.0
88 4.0
89 0.0;
param: MEM := 
0 32.0
1 32.0
2 32.0
3 16.0
4 16.0
5 16.0
6 16.0
7 16.0
8 16.0
9 16.0
10 16.0
11 16.0
12 16.0
13 8.0
14 8.0
15 18.0
16 18.0
17 8.0
18 8.0
19 4.0
20 4.0
21 48.0
22 48.0
23 48.0
24 8.0
25 4.0
26 4.0
27 32.0
28 32.0
29 8.0
30 8.0
31 8.0
32 8.0
33 4.0
34 4.0
35 4.0
36 4.0
37 4.0
38 4.0
39 4.0
40 4.0
41 4.0
42 4.0
43 8.0
44 0.0
45 32.0
46 32.0
47 32.0
48 16.0
49 16.0
50 16.0
51 16.0
52 16.0
53 16.0
54 16.0
55 16.0
56 16.0
57 16.0
58 8.0
59 8.0
60 18.0
61 18.0
62 8.0
63 8.0
64 4.0
65 4.0
66 48.0
67 48.0
68 48.0
69 8.0
70 4.0
71 4.0
72 32.0
73 32.0
74 8.0
75 8.0
76 8.0
77 8.0
78 4.0
79 4.0
80 4.0
81 4.0
82 4.0
83 4.0
84 4.0
85 4.0
86 4.0
87 4.0
88 8.0
89 0.0;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0;


######################
# SOLVE 
######################
end;
