

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..311;
# Set of Hosts
set Hs := 0..30;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {0..2}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {3..5}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm23_0_{h in Hs}: sum{v in {6..8}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm23_1_{h in Hs}: sum{v in {9..11}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm24_0_{h in Hs}: sum{v in {12..14}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm24_1_{h in Hs}: sum{v in {15..17}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm25_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm25_1_{h in Hs}: sum{v in {20,21}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm26_0_{h in Hs}: sum{v in {22,23}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm26_1_{h in Hs}: sum{v in {24,25}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_0_{h in Hs}: sum{v in {26,27}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_1_{h in Hs}: sum{v in {28,29}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm27_0_{h in Hs}: sum{v in {30,31}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm27_1_{h in Hs}: sum{v in {32,33}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm29_0_{h in Hs}: sum{v in {38,39}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm29_1_{h in Hs}: sum{v in {40,41}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm30_0_{h in Hs}: sum{v in {42..44}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm30_1_{h in Hs}: sum{v in {45..47}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm31_0_{h in Hs}: sum{v in {48..53}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm31_1_{h in Hs}: sum{v in {54..59}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm38_0_{h in Hs}: sum{v in {82,83}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm38_1_{h in Hs}: sum{v in {84,85}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm1_0_{h in Hs}: sum{v in {86..88}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {89..91}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {92..94}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {95,96}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {97,98}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm6_0_{h in Hs}: sum{v in {99,100}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {101,102}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm8_0_{h in Hs}: sum{v in {103,104}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_0_{h in Hs}: sum{v in {105,106}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm10_0_{h in Hs}: sum{v in {107..109}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_0_{h in Hs}: sum{v in {111,112}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm13_0_{h in Hs}: sum{v in {113,114}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm14_0_{h in Hs}: sum{v in {115,116}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm15_0_{h in Hs}: sum{v in {117,118}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm16_0_{h in Hs}: sum{v in {119,120}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm17_0_{h in Hs}: sum{v in {121,122}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm18_0_{h in Hs}: sum{v in {123,124}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm19_0_{h in Hs}: sum{v in {125,126}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm20_0_{h in Hs}: sum{v in {127,128}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm39_0_{h in Hs}: sum{v in {131..133}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm40_0_{h in Hs}: sum{v in {134,135}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm41_0_{h in Hs}: sum{v in {136}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm42_0_{h in Hs}: sum{v in {137,138}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm43_0_{h in Hs}: sum{v in {139,140}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm1_0_{h in Hs}: sum{v in {166..168}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm23_0_{h in Hs}: sum{v in {169..171}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm24_0_{h in Hs}: sum{v in {172..174}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm27_0_{h in Hs}: sum{v in {175,176}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm29_0_{h in Hs}: sum{v in {179,180}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm30_0_{h in Hs}: sum{v in {181..183}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm38_0_{h in Hs}: sum{v in {195,196}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm54_0_{h in Hs}: sum{v in {209,210}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm54_1_{h in Hs}: sum{v in {211,212}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm56_0_{h in Hs}: sum{v in {217,218}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm56_1_{h in Hs}: sum{v in {219,220}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm57_0_{h in Hs}: sum{v in {221,222}} x[v, h]<= 1;
s.t. AAF_vnf10_dot_vm57_1_{h in Hs}: sum{v in {223,224}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm63_0_{h in Hs}: sum{v in {247..249}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm68_0_{h in Hs}: sum{v in {258}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm68_1_{h in Hs}: sum{v in {259}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm69_0_{h in Hs}: sum{v in {260}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm69_1_{h in Hs}: sum{v in {261}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm70_0_{h in Hs}: sum{v in {262}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm70_1_{h in Hs}: sum{v in {263}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm71_0_{h in Hs}: sum{v in {264}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm71_1_{h in Hs}: sum{v in {265}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm76_0_{h in Hs}: sum{v in {282}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm76_1_{h in Hs}: sum{v in {283}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm77_0_{h in Hs}: sum{v in {284}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm77_1_{h in Hs}: sum{v in {285}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm62_0_{h in Hs}: sum{v in {286,287}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm62_1_{h in Hs}: sum{v in {288,289}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm63_0_{h in Hs}: sum{v in {290..292}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm63_1_{h in Hs}: sum{v in {293..295}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm64_0_{h in Hs}: sum{v in {296,297}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm64_1_{h in Hs}: sum{v in {298,299}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm65_0_{h in Hs}: sum{v in {300,301}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm65_1_{h in Hs}: sum{v in {302,303}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm66_0_{h in Hs}: sum{v in {304,305}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm66_1_{h in Hs}: sum{v in {306,307}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm67_0_{h in Hs}: sum{v in {308,309}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm67_1_{h in Hs}: sum{v in {310,311}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm28_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm28_0_master{h in Hs}: ni_vnf2_dot_vm28_0_master[h] <= sum{v in {34}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm28_0_master{v in {34}, h in Hs}: ni_vnf2_dot_vm28_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm28_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm28_0_slave{h in Hs}: ni_vnf2_dot_vm28_0_slave[h] <= sum{v in {35}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm28_0_slave{v in {35}, h in Hs}: ni_vnf2_dot_vm28_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf2_dot_vm28_0_master[h] + ni_vnf2_dot_vm28_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm28_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm28_1_master{h in Hs}: ni_vnf2_dot_vm28_1_master[h] <= sum{v in {36}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm28_1_master{v in {36}, h in Hs}: ni_vnf2_dot_vm28_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm28_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm28_1_slave{h in Hs}: ni_vnf2_dot_vm28_1_slave[h] <= sum{v in {37}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm28_1_slave{v in {37}, h in Hs}: ni_vnf2_dot_vm28_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf2_dot_vm28_1_master[h] + ni_vnf2_dot_vm28_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm33_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm33_0_master{h in Hs}: ni_vnf2_dot_vm33_0_master[h] <= sum{v in {62}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm33_0_master{v in {62}, h in Hs}: ni_vnf2_dot_vm33_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm33_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm33_0_slave{h in Hs}: ni_vnf2_dot_vm33_0_slave[h] <= sum{v in {63}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm33_0_slave{v in {63}, h in Hs}: ni_vnf2_dot_vm33_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf2_dot_vm33_0_master[h] + ni_vnf2_dot_vm33_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm33_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm33_1_master{h in Hs}: ni_vnf2_dot_vm33_1_master[h] <= sum{v in {64}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm33_1_master{v in {64}, h in Hs}: ni_vnf2_dot_vm33_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm33_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm33_1_slave{h in Hs}: ni_vnf2_dot_vm33_1_slave[h] <= sum{v in {65}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm33_1_slave{v in {65}, h in Hs}: ni_vnf2_dot_vm33_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf2_dot_vm33_1_master[h] + ni_vnf2_dot_vm33_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm34_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm34_0_master{h in Hs}: ni_vnf2_dot_vm34_0_master[h] <= sum{v in {66}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm34_0_master{v in {66}, h in Hs}: ni_vnf2_dot_vm34_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm34_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm34_0_slave{h in Hs}: ni_vnf2_dot_vm34_0_slave[h] <= sum{v in {67}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm34_0_slave{v in {67}, h in Hs}: ni_vnf2_dot_vm34_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf2_dot_vm34_0_master[h] + ni_vnf2_dot_vm34_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm34_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm34_1_master{h in Hs}: ni_vnf2_dot_vm34_1_master[h] <= sum{v in {68}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm34_1_master{v in {68}, h in Hs}: ni_vnf2_dot_vm34_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm34_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm34_1_slave{h in Hs}: ni_vnf2_dot_vm34_1_slave[h] <= sum{v in {69}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm34_1_slave{v in {69}, h in Hs}: ni_vnf2_dot_vm34_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf2_dot_vm34_1_master[h] + ni_vnf2_dot_vm34_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm35_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm35_0_master{h in Hs}: ni_vnf2_dot_vm35_0_master[h] <= sum{v in {70}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm35_0_master{v in {70}, h in Hs}: ni_vnf2_dot_vm35_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm35_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm35_0_slave{h in Hs}: ni_vnf2_dot_vm35_0_slave[h] <= sum{v in {71}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm35_0_slave{v in {71}, h in Hs}: ni_vnf2_dot_vm35_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_6_{h in Hs}: ni_vnf2_dot_vm35_0_master[h] + ni_vnf2_dot_vm35_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm35_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm35_1_master{h in Hs}: ni_vnf2_dot_vm35_1_master[h] <= sum{v in {72}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm35_1_master{v in {72}, h in Hs}: ni_vnf2_dot_vm35_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm35_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm35_1_slave{h in Hs}: ni_vnf2_dot_vm35_1_slave[h] <= sum{v in {73}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm35_1_slave{v in {73}, h in Hs}: ni_vnf2_dot_vm35_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_7_{h in Hs}: ni_vnf2_dot_vm35_1_master[h] + ni_vnf2_dot_vm35_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm36_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm36_0_master{h in Hs}: ni_vnf2_dot_vm36_0_master[h] <= sum{v in {74}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm36_0_master{v in {74}, h in Hs}: ni_vnf2_dot_vm36_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm36_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm36_0_slave{h in Hs}: ni_vnf2_dot_vm36_0_slave[h] <= sum{v in {75}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm36_0_slave{v in {75}, h in Hs}: ni_vnf2_dot_vm36_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_8_{h in Hs}: ni_vnf2_dot_vm36_0_master[h] + ni_vnf2_dot_vm36_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm36_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm36_1_master{h in Hs}: ni_vnf2_dot_vm36_1_master[h] <= sum{v in {76}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm36_1_master{v in {76}, h in Hs}: ni_vnf2_dot_vm36_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm36_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm36_1_slave{h in Hs}: ni_vnf2_dot_vm36_1_slave[h] <= sum{v in {77}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm36_1_slave{v in {77}, h in Hs}: ni_vnf2_dot_vm36_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_9_{h in Hs}: ni_vnf2_dot_vm36_1_master[h] + ni_vnf2_dot_vm36_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm37_0_master{h in Hs}: ni_vnf2_dot_vm37_0_master[h] <= sum{v in {78}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm37_0_master{v in {78}, h in Hs}: ni_vnf2_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm37_0_slave{h in Hs}: ni_vnf2_dot_vm37_0_slave[h] <= sum{v in {79}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm37_0_slave{v in {79}, h in Hs}: ni_vnf2_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_10_{h in Hs}: ni_vnf2_dot_vm37_0_master[h] + ni_vnf2_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm37_1_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm37_1_master{h in Hs}: ni_vnf2_dot_vm37_1_master[h] <= sum{v in {80}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm37_1_master{v in {80}, h in Hs}: ni_vnf2_dot_vm37_1_master[h] >= x[v, h];
var ni_vnf2_dot_vm37_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm37_1_slave{h in Hs}: ni_vnf2_dot_vm37_1_slave[h] <= sum{v in {81}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm37_1_slave{v in {81}, h in Hs}: ni_vnf2_dot_vm37_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_11_{h in Hs}: ni_vnf2_dot_vm37_1_master[h] + ni_vnf2_dot_vm37_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm28_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm28_0_master{h in Hs}: ni_vnf8_dot_vm28_0_master[h] <= sum{v in {177}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm28_0_master{v in {177}, h in Hs}: ni_vnf8_dot_vm28_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm28_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm28_0_slave{h in Hs}: ni_vnf8_dot_vm28_0_slave[h] <= sum{v in {178}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm28_0_slave{v in {178}, h in Hs}: ni_vnf8_dot_vm28_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_12_{h in Hs}: ni_vnf8_dot_vm28_0_master[h] + ni_vnf8_dot_vm28_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm33_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm33_0_master{h in Hs}: ni_vnf8_dot_vm33_0_master[h] <= sum{v in {185}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm33_0_master{v in {185}, h in Hs}: ni_vnf8_dot_vm33_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm33_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm33_0_slave{h in Hs}: ni_vnf8_dot_vm33_0_slave[h] <= sum{v in {186}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm33_0_slave{v in {186}, h in Hs}: ni_vnf8_dot_vm33_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_13_{h in Hs}: ni_vnf8_dot_vm33_0_master[h] + ni_vnf8_dot_vm33_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm34_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm34_0_master{h in Hs}: ni_vnf8_dot_vm34_0_master[h] <= sum{v in {187}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm34_0_master{v in {187}, h in Hs}: ni_vnf8_dot_vm34_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm34_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm34_0_slave{h in Hs}: ni_vnf8_dot_vm34_0_slave[h] <= sum{v in {188}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm34_0_slave{v in {188}, h in Hs}: ni_vnf8_dot_vm34_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_14_{h in Hs}: ni_vnf8_dot_vm34_0_master[h] + ni_vnf8_dot_vm34_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm35_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm35_0_master{h in Hs}: ni_vnf8_dot_vm35_0_master[h] <= sum{v in {189}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm35_0_master{v in {189}, h in Hs}: ni_vnf8_dot_vm35_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm35_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm35_0_slave{h in Hs}: ni_vnf8_dot_vm35_0_slave[h] <= sum{v in {190}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm35_0_slave{v in {190}, h in Hs}: ni_vnf8_dot_vm35_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_15_{h in Hs}: ni_vnf8_dot_vm35_0_master[h] + ni_vnf8_dot_vm35_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm36_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm36_0_master{h in Hs}: ni_vnf8_dot_vm36_0_master[h] <= sum{v in {191}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm36_0_master{v in {191}, h in Hs}: ni_vnf8_dot_vm36_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm36_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm36_0_slave{h in Hs}: ni_vnf8_dot_vm36_0_slave[h] <= sum{v in {192}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm36_0_slave{v in {192}, h in Hs}: ni_vnf8_dot_vm36_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_16_{h in Hs}: ni_vnf8_dot_vm36_0_master[h] + ni_vnf8_dot_vm36_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_0_master{h in Hs}: ni_vnf8_dot_vm37_0_master[h] <= sum{v in {193}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_0_master{v in {193}, h in Hs}: ni_vnf8_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_0_slave{h in Hs}: ni_vnf8_dot_vm37_0_slave[h] <= sum{v in {194}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_0_slave{v in {194}, h in Hs}: ni_vnf8_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_17_{h in Hs}: ni_vnf8_dot_vm37_0_master[h] + ni_vnf8_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm51_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm51_0_master{h in Hs}: ni_vnf9_dot_vm51_0_master[h] <= sum{v in {197}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm51_0_master{v in {197}, h in Hs}: ni_vnf9_dot_vm51_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm51_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm51_0_slave{h in Hs}: ni_vnf9_dot_vm51_0_slave[h] <= sum{v in {198}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm51_0_slave{v in {198}, h in Hs}: ni_vnf9_dot_vm51_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_18_{h in Hs}: ni_vnf9_dot_vm51_0_master[h] + ni_vnf9_dot_vm51_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm51_1_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm51_1_master{h in Hs}: ni_vnf9_dot_vm51_1_master[h] <= sum{v in {199}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm51_1_master{v in {199}, h in Hs}: ni_vnf9_dot_vm51_1_master[h] >= x[v, h];
var ni_vnf9_dot_vm51_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm51_1_slave{h in Hs}: ni_vnf9_dot_vm51_1_slave[h] <= sum{v in {200}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm51_1_slave{v in {200}, h in Hs}: ni_vnf9_dot_vm51_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_19_{h in Hs}: ni_vnf9_dot_vm51_1_master[h] + ni_vnf9_dot_vm51_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm52_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm52_0_master{h in Hs}: ni_vnf10_dot_vm52_0_master[h] <= sum{v in {201}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm52_0_master{v in {201}, h in Hs}: ni_vnf10_dot_vm52_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm52_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm52_0_slave{h in Hs}: ni_vnf10_dot_vm52_0_slave[h] <= sum{v in {202}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm52_0_slave{v in {202}, h in Hs}: ni_vnf10_dot_vm52_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_20_{h in Hs}: ni_vnf10_dot_vm52_0_master[h] + ni_vnf10_dot_vm52_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm52_1_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm52_1_master{h in Hs}: ni_vnf10_dot_vm52_1_master[h] <= sum{v in {203}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm52_1_master{v in {203}, h in Hs}: ni_vnf10_dot_vm52_1_master[h] >= x[v, h];
var ni_vnf10_dot_vm52_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm52_1_slave{h in Hs}: ni_vnf10_dot_vm52_1_slave[h] <= sum{v in {204}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm52_1_slave{v in {204}, h in Hs}: ni_vnf10_dot_vm52_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_21_{h in Hs}: ni_vnf10_dot_vm52_1_master[h] + ni_vnf10_dot_vm52_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm53_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm53_0_master{h in Hs}: ni_vnf10_dot_vm53_0_master[h] <= sum{v in {205}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm53_0_master{v in {205}, h in Hs}: ni_vnf10_dot_vm53_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm53_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm53_0_slave{h in Hs}: ni_vnf10_dot_vm53_0_slave[h] <= sum{v in {206}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm53_0_slave{v in {206}, h in Hs}: ni_vnf10_dot_vm53_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_22_{h in Hs}: ni_vnf10_dot_vm53_0_master[h] + ni_vnf10_dot_vm53_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm53_1_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm53_1_master{h in Hs}: ni_vnf10_dot_vm53_1_master[h] <= sum{v in {207}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm53_1_master{v in {207}, h in Hs}: ni_vnf10_dot_vm53_1_master[h] >= x[v, h];
var ni_vnf10_dot_vm53_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm53_1_slave{h in Hs}: ni_vnf10_dot_vm53_1_slave[h] <= sum{v in {208}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm53_1_slave{v in {208}, h in Hs}: ni_vnf10_dot_vm53_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_23_{h in Hs}: ni_vnf10_dot_vm53_1_master[h] + ni_vnf10_dot_vm53_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm55_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm55_0_master{h in Hs}: ni_vnf10_dot_vm55_0_master[h] <= sum{v in {213}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm55_0_master{v in {213}, h in Hs}: ni_vnf10_dot_vm55_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm55_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm55_0_slave{h in Hs}: ni_vnf10_dot_vm55_0_slave[h] <= sum{v in {214}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm55_0_slave{v in {214}, h in Hs}: ni_vnf10_dot_vm55_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_24_{h in Hs}: ni_vnf10_dot_vm55_0_master[h] + ni_vnf10_dot_vm55_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm55_1_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm55_1_master{h in Hs}: ni_vnf10_dot_vm55_1_master[h] <= sum{v in {215}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm55_1_master{v in {215}, h in Hs}: ni_vnf10_dot_vm55_1_master[h] >= x[v, h];
var ni_vnf10_dot_vm55_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm55_1_slave{h in Hs}: ni_vnf10_dot_vm55_1_slave[h] <= sum{v in {216}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm55_1_slave{v in {216}, h in Hs}: ni_vnf10_dot_vm55_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_25_{h in Hs}: ni_vnf10_dot_vm55_1_master[h] + ni_vnf10_dot_vm55_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf11_dot_vm58_0_master{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm58_0_master{h in Hs}: ni_vnf11_dot_vm58_0_master[h] <= sum{v in {225}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm58_0_master{v in {225}, h in Hs}: ni_vnf11_dot_vm58_0_master[h] >= x[v, h];
var ni_vnf11_dot_vm58_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm58_0_slave{h in Hs}: ni_vnf11_dot_vm58_0_slave[h] <= sum{v in {226}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm58_0_slave{v in {226}, h in Hs}: ni_vnf11_dot_vm58_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_26_{h in Hs}: ni_vnf11_dot_vm58_0_master[h] + ni_vnf11_dot_vm58_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf11_dot_vm58_1_master{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm58_1_master{h in Hs}: ni_vnf11_dot_vm58_1_master[h] <= sum{v in {227}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm58_1_master{v in {227}, h in Hs}: ni_vnf11_dot_vm58_1_master[h] >= x[v, h];
var ni_vnf11_dot_vm58_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm58_1_slave{h in Hs}: ni_vnf11_dot_vm58_1_slave[h] <= sum{v in {228}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm58_1_slave{v in {228}, h in Hs}: ni_vnf11_dot_vm58_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_27_{h in Hs}: ni_vnf11_dot_vm58_1_master[h] + ni_vnf11_dot_vm58_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm59_0_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm59_0_master{h in Hs}: ni_vnf12_dot_vm59_0_master[h] <= sum{v in {229}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm59_0_master{v in {229}, h in Hs}: ni_vnf12_dot_vm59_0_master[h] >= x[v, h];
var ni_vnf12_dot_vm59_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm59_0_slave{h in Hs}: ni_vnf12_dot_vm59_0_slave[h] <= sum{v in {230}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm59_0_slave{v in {230}, h in Hs}: ni_vnf12_dot_vm59_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_28_{h in Hs}: ni_vnf12_dot_vm59_0_master[h] + ni_vnf12_dot_vm59_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm59_1_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm59_1_master{h in Hs}: ni_vnf12_dot_vm59_1_master[h] <= sum{v in {231}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm59_1_master{v in {231}, h in Hs}: ni_vnf12_dot_vm59_1_master[h] >= x[v, h];
var ni_vnf12_dot_vm59_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm59_1_slave{h in Hs}: ni_vnf12_dot_vm59_1_slave[h] <= sum{v in {232}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm59_1_slave{v in {232}, h in Hs}: ni_vnf12_dot_vm59_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_29_{h in Hs}: ni_vnf12_dot_vm59_1_master[h] + ni_vnf12_dot_vm59_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm60_0_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm60_0_master{h in Hs}: ni_vnf12_dot_vm60_0_master[h] <= sum{v in {233}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm60_0_master{v in {233}, h in Hs}: ni_vnf12_dot_vm60_0_master[h] >= x[v, h];
var ni_vnf12_dot_vm60_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm60_0_slave{h in Hs}: ni_vnf12_dot_vm60_0_slave[h] <= sum{v in {234}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm60_0_slave{v in {234}, h in Hs}: ni_vnf12_dot_vm60_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_30_{h in Hs}: ni_vnf12_dot_vm60_0_master[h] + ni_vnf12_dot_vm60_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm60_1_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm60_1_master{h in Hs}: ni_vnf12_dot_vm60_1_master[h] <= sum{v in {235}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm60_1_master{v in {235}, h in Hs}: ni_vnf12_dot_vm60_1_master[h] >= x[v, h];
var ni_vnf12_dot_vm60_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm60_1_slave{h in Hs}: ni_vnf12_dot_vm60_1_slave[h] <= sum{v in {236}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm60_1_slave{v in {236}, h in Hs}: ni_vnf12_dot_vm60_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_31_{h in Hs}: ni_vnf12_dot_vm60_1_master[h] + ni_vnf12_dot_vm60_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm61_0_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm61_0_master{h in Hs}: ni_vnf12_dot_vm61_0_master[h] <= sum{v in {237}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm61_0_master{v in {237}, h in Hs}: ni_vnf12_dot_vm61_0_master[h] >= x[v, h];
var ni_vnf12_dot_vm61_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm61_0_slave{h in Hs}: ni_vnf12_dot_vm61_0_slave[h] <= sum{v in {238}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm61_0_slave{v in {238}, h in Hs}: ni_vnf12_dot_vm61_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_32_{h in Hs}: ni_vnf12_dot_vm61_0_master[h] + ni_vnf12_dot_vm61_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm61_1_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm61_1_master{h in Hs}: ni_vnf12_dot_vm61_1_master[h] <= sum{v in {239}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm61_1_master{v in {239}, h in Hs}: ni_vnf12_dot_vm61_1_master[h] >= x[v, h];
var ni_vnf12_dot_vm61_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm61_1_slave{h in Hs}: ni_vnf12_dot_vm61_1_slave[h] <= sum{v in {240}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm61_1_slave{v in {240}, h in Hs}: ni_vnf12_dot_vm61_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_33_{h in Hs}: ni_vnf12_dot_vm61_1_master[h] + ni_vnf12_dot_vm61_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm51_0_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm51_0_master{h in Hs}: ni_vnf12_dot_vm51_0_master[h] <= sum{v in {241}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm51_0_master{v in {241}, h in Hs}: ni_vnf12_dot_vm51_0_master[h] >= x[v, h];
var ni_vnf12_dot_vm51_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm51_0_slave{h in Hs}: ni_vnf12_dot_vm51_0_slave[h] <= sum{v in {242}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm51_0_slave{v in {242}, h in Hs}: ni_vnf12_dot_vm51_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_34_{h in Hs}: ni_vnf12_dot_vm51_0_master[h] + ni_vnf12_dot_vm51_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf12_dot_vm51_1_master{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm51_1_master{h in Hs}: ni_vnf12_dot_vm51_1_master[h] <= sum{v in {243}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm51_1_master{v in {243}, h in Hs}: ni_vnf12_dot_vm51_1_master[h] >= x[v, h];
var ni_vnf12_dot_vm51_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf12_dot_vm51_1_slave{h in Hs}: ni_vnf12_dot_vm51_1_slave[h] <= sum{v in {244}} x[v, h];
s.t. or_ineq_ni_vnf12_dot_vm51_1_slave{v in {244}, h in Hs}: ni_vnf12_dot_vm51_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_35_{h in Hs}: ni_vnf12_dot_vm51_1_master[h] + ni_vnf12_dot_vm51_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf13_dot_vm62_0_master{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm62_0_master{h in Hs}: ni_vnf13_dot_vm62_0_master[h] <= sum{v in {245}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm62_0_master{v in {245}, h in Hs}: ni_vnf13_dot_vm62_0_master[h] >= x[v, h];
var ni_vnf13_dot_vm62_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm62_0_slave{h in Hs}: ni_vnf13_dot_vm62_0_slave[h] <= sum{v in {246}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm62_0_slave{v in {246}, h in Hs}: ni_vnf13_dot_vm62_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_36_{h in Hs}: ni_vnf13_dot_vm62_0_master[h] + ni_vnf13_dot_vm62_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf13_dot_vm64_0_master{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm64_0_master{h in Hs}: ni_vnf13_dot_vm64_0_master[h] <= sum{v in {250}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm64_0_master{v in {250}, h in Hs}: ni_vnf13_dot_vm64_0_master[h] >= x[v, h];
var ni_vnf13_dot_vm64_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm64_0_slave{h in Hs}: ni_vnf13_dot_vm64_0_slave[h] <= sum{v in {251}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm64_0_slave{v in {251}, h in Hs}: ni_vnf13_dot_vm64_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_37_{h in Hs}: ni_vnf13_dot_vm64_0_master[h] + ni_vnf13_dot_vm64_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf13_dot_vm65_0_master{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm65_0_master{h in Hs}: ni_vnf13_dot_vm65_0_master[h] <= sum{v in {252}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm65_0_master{v in {252}, h in Hs}: ni_vnf13_dot_vm65_0_master[h] >= x[v, h];
var ni_vnf13_dot_vm65_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm65_0_slave{h in Hs}: ni_vnf13_dot_vm65_0_slave[h] <= sum{v in {253}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm65_0_slave{v in {253}, h in Hs}: ni_vnf13_dot_vm65_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_38_{h in Hs}: ni_vnf13_dot_vm65_0_master[h] + ni_vnf13_dot_vm65_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf13_dot_vm66_0_master{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm66_0_master{h in Hs}: ni_vnf13_dot_vm66_0_master[h] <= sum{v in {254}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm66_0_master{v in {254}, h in Hs}: ni_vnf13_dot_vm66_0_master[h] >= x[v, h];
var ni_vnf13_dot_vm66_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf13_dot_vm66_0_slave{h in Hs}: ni_vnf13_dot_vm66_0_slave[h] <= sum{v in {255}} x[v, h];
s.t. or_ineq_ni_vnf13_dot_vm66_0_slave{v in {255}, h in Hs}: ni_vnf13_dot_vm66_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_39_{h in Hs}: ni_vnf13_dot_vm66_0_master[h] + ni_vnf13_dot_vm66_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf14_dot_vm67_0_master{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm67_0_master{h in Hs}: ni_vnf14_dot_vm67_0_master[h] <= sum{v in {256}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm67_0_master{v in {256}, h in Hs}: ni_vnf14_dot_vm67_0_master[h] >= x[v, h];
var ni_vnf14_dot_vm67_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf14_dot_vm67_0_slave{h in Hs}: ni_vnf14_dot_vm67_0_slave[h] <= sum{v in {257}} x[v, h];
s.t. or_ineq_ni_vnf14_dot_vm67_0_slave{v in {257}, h in Hs}: ni_vnf14_dot_vm67_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_40_{h in Hs}: ni_vnf14_dot_vm67_0_master[h] + ni_vnf14_dot_vm67_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4.0
1 4.0
2 4.0
3 4.0
4 4.0
5 4.0
6 4.0
7 4.0
8 4.0
9 4.0
10 4.0
11 4.0
12 4.0
13 4.0
14 4.0
15 4.0
16 4.0
17 4.0
18 2.0
19 2.0
20 2.0
21 2.0
22 4.0
23 4.0
24 4.0
25 4.0
26 4.0
27 4.0
28 4.0
29 4.0
30 8.0
31 8.0
32 8.0
33 8.0
34 2.0
35 2.0
36 2.0
37 2.0
38 2.0
39 2.0
40 2.0
41 2.0
42 4.0
43 4.0
44 4.0
45 4.0
46 4.0
47 4.0
48 2.0
49 2.0
50 2.0
51 2.0
52 2.0
53 2.0
54 2.0
55 2.0
56 2.0
57 2.0
58 2.0
59 2.0
60 2.0
61 2.0
62 3.0
63 3.0
64 3.0
65 3.0
66 8.0
67 8.0
68 8.0
69 8.0
70 4.0
71 4.0
72 4.0
73 4.0
74 2.0
75 2.0
76 2.0
77 2.0
78 4.0
79 4.0
80 4.0
81 4.0
82 2.0
83 2.0
84 2.0
85 2.0
86 4.0
87 4.0
88 4.0
89 4.0
90 4.0
91 4.0
92 4.0
93 4.0
94 4.0
95 2.0
96 2.0
97 4.0
98 4.0
99 4.0
100 4.0
101 8.0
102 8.0
103 2.0
104 2.0
105 2.0
106 2.0
107 4.0
108 4.0
109 4.0
110 2.0
111 4.0
112 4.0
113 8.0
114 8.0
115 4.0
116 4.0
117 4.0
118 4.0
119 4.0
120 4.0
121 4.0
122 4.0
123 4.0
124 4.0
125 4.0
126 4.0
127 2.0
128 2.0
129 4.0
130 0.0
131 4.0
132 4.0
133 4.0
134 12.0
135 12.0
136 2.0
137 10.0
138 10.0
139 4.0
140 4.0
141 6.0
142 4.0
143 4.0
144 4.0
145 4.0
146 4.0
147 4.0
148 4.0
149 4.0
150 4.0
151 4.0
152 4.0
153 4.0
154 4.0
155 4.0
156 4.0
157 4.0
158 4.0
159 4.0
160 4.0
161 4.0
162 4.0
163 4.0
164 4.0
165 4.0
166 4.0
167 4.0
168 4.0
169 4.0
170 4.0
171 4.0
172 4.0
173 4.0
174 4.0
175 8.0
176 8.0
177 2.0
178 2.0
179 2.0
180 2.0
181 4.0
182 4.0
183 4.0
184 2.0
185 3.0
186 3.0
187 8.0
188 8.0
189 4.0
190 4.0
191 2.0
192 2.0
193 4.0
194 4.0
195 2.0
196 2.0
197 8.0
198 8.0
199 8.0
200 8.0
201 4.0
202 4.0
203 4.0
204 4.0
205 2.0
206 2.0
207 2.0
208 2.0
209 2.0
210 2.0
211 2.0
212 2.0
213 1.0
214 1.0
215 1.0
216 1.0
217 2.0
218 2.0
219 2.0
220 2.0
221 2.0
222 2.0
223 2.0
224 2.0
225 4.0
226 4.0
227 4.0
228 4.0
229 2.0
230 2.0
231 2.0
232 2.0
233 4.0
234 4.0
235 4.0
236 4.0
237 4.0
238 4.0
239 4.0
240 4.0
241 8.0
242 8.0
243 8.0
244 8.0
245 8.0
246 8.0
247 4.0
248 4.0
249 4.0
250 4.0
251 4.0
252 4.0
253 4.0
254 8.0
255 8.0
256 8.0
257 8.0
258 8.0
259 8.0
260 8.0
261 8.0
262 8.0
263 8.0
264 8.0
265 8.0
266 4.0
267 4.0
268 4.0
269 4.0
270 4.0
271 4.0
272 4.0
273 4.0
274 16.0
275 16.0
276 16.0
277 16.0
278 4.0
279 4.0
280 4.0
281 4.0
282 2.0
283 2.0
284 2.0
285 2.0
286 8.0
287 8.0
288 8.0
289 8.0
290 4.0
291 4.0
292 4.0
293 4.0
294 4.0
295 4.0
296 4.0
297 4.0
298 4.0
299 4.0
300 4.0
301 4.0
302 4.0
303 4.0
304 8.0
305 8.0
306 8.0
307 8.0
308 8.0
309 8.0
310 8.0
311 8.0;
param: MEM := 
0 32.0
1 32.0
2 32.0
3 32.0
4 32.0
5 32.0
6 16.0
7 16.0
8 16.0
9 16.0
10 16.0
11 16.0
12 16.0
13 16.0
14 16.0
15 16.0
16 16.0
17 16.0
18 8.0
19 8.0
20 8.0
21 8.0
22 16.0
23 16.0
24 16.0
25 16.0
26 8.0
27 8.0
28 8.0
29 8.0
30 18.0
31 18.0
32 18.0
33 18.0
34 8.0
35 8.0
36 8.0
37 8.0
38 4.0
39 4.0
40 4.0
41 4.0
42 48.0
43 48.0
44 48.0
45 48.0
46 48.0
47 48.0
48 16.0
49 16.0
50 16.0
51 16.0
52 16.0
53 16.0
54 16.0
55 16.0
56 16.0
57 16.0
58 16.0
59 16.0
60 8.0
61 8.0
62 4.0
63 4.0
64 4.0
65 4.0
66 32.0
67 32.0
68 32.0
69 32.0
70 8.0
71 8.0
72 8.0
73 8.0
74 4.0
75 4.0
76 4.0
77 4.0
78 4.0
79 4.0
80 4.0
81 4.0
82 4.0
83 4.0
84 4.0
85 4.0
86 32.0
87 32.0
88 32.0
89 16.0
90 16.0
91 16.0
92 16.0
93 16.0
94 16.0
95 16.0
96 16.0
97 16.0
98 16.0
99 8.0
100 8.0
101 18.0
102 18.0
103 8.0
104 8.0
105 4.0
106 4.0
107 48.0
108 48.0
109 48.0
110 8.0
111 4.0
112 4.0
113 32.0
114 32.0
115 8.0
116 8.0
117 8.0
118 8.0
119 4.0
120 4.0
121 4.0
122 4.0
123 4.0
124 4.0
125 4.0
126 4.0
127 4.0
128 4.0
129 8.0
130 0.0
131 15.0
132 15.0
133 15.0
134 24.0
135 24.0
136 8.0
137 12.0
138 12.0
139 8.0
140 8.0
141 16.0
142 16.0
143 16.0
144 16.0
145 16.0
146 16.0
147 16.0
148 16.0
149 16.0
150 16.0
151 16.0
152 16.0
153 16.0
154 16.0
155 16.0
156 16.0
157 16.0
158 16.0
159 16.0
160 16.0
161 16.0
162 16.0
163 16.0
164 16.0
165 16.0
166 32.0
167 32.0
168 32.0
169 16.0
170 16.0
171 16.0
172 16.0
173 16.0
174 16.0
175 18.0
176 18.0
177 8.0
178 8.0
179 4.0
180 4.0
181 48.0
182 48.0
183 48.0
184 8.0
185 4.0
186 4.0
187 32.0
188 32.0
189 8.0
190 8.0
191 4.0
192 4.0
193 4.0
194 4.0
195 4.0
196 4.0
197 16.0
198 16.0
199 16.0
200 16.0
201 8.0
202 8.0
203 8.0
204 8.0
205 6.0
206 6.0
207 6.0
208 6.0
209 6.0
210 6.0
211 6.0
212 6.0
213 10.0
214 10.0
215 10.0
216 10.0
217 40.0
218 40.0
219 40.0
220 40.0
221 10.0
222 10.0
223 10.0
224 10.0
225 12.0
226 12.0
227 12.0
228 12.0
229 12.0
230 12.0
231 12.0
232 12.0
233 36.0
234 36.0
235 36.0
236 36.0
237 12.0
238 12.0
239 12.0
240 12.0
241 16.0
242 16.0
243 16.0
244 16.0
245 24.0
246 24.0
247 32.0
248 32.0
249 32.0
250 16.0
251 16.0
252 16.0
253 16.0
254 24.0
255 24.0
256 16.0
257 16.0
258 128.0
259 128.0
260 128.0
261 128.0
262 128.0
263 128.0
264 128.0
265 128.0
266 4.0
267 4.0
268 4.0
269 4.0
270 4.0
271 4.0
272 4.0
273 4.0
274 12.0
275 12.0
276 12.0
277 12.0
278 4.0
279 4.0
280 4.0
281 4.0
282 16.0
283 16.0
284 16.0
285 16.0
286 24.0
287 24.0
288 24.0
289 24.0
290 32.0
291 32.0
292 32.0
293 32.0
294 32.0
295 32.0
296 16.0
297 16.0
298 16.0
299 16.0
300 16.0
301 16.0
302 16.0
303 16.0
304 24.0
305 24.0
306 24.0
307 24.0
308 16.0
309 16.0
310 16.0
311 16.0;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 2.0
132 2.0
133 2.0
134 2.0
135 2.0
136 2.0
137 2.0
138 2.0
139 2.0
140 2.0
141 0.0
142 0.0
143 0.0
144 0.0
145 0.0
146 0.0
147 0.0
148 0.0
149 0.0
150 0.0
151 0.0
152 0.0
153 0.0
154 0.0
155 0.0
156 0.0
157 0.0
158 0.0
159 0.0
160 0.0
161 0.0
162 0.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 0.0
179 0.0
180 0.0
181 0.0
182 0.0
183 0.0
184 0.0
185 0.0
186 0.0
187 0.0
188 0.0
189 0.0
190 0.0
191 0.0
192 0.0
193 0.0
194 0.0
195 0.0
196 0.0
197 1350.0
198 1350.0
199 1350.0
200 1350.0
201 1640.0
202 1640.0
203 1640.0
204 1640.0
205 70.0
206 70.0
207 70.0
208 70.0
209 330.0
210 330.0
211 330.0
212 330.0
213 600.0
214 600.0
215 600.0
216 600.0
217 100.0
218 100.0
219 100.0
220 100.0
221 230.0
222 230.0
223 230.0
224 230.0
225 340.0
226 340.0
227 340.0
228 340.0
229 190.0
230 190.0
231 190.0
232 190.0
233 340.0
234 340.0
235 340.0
236 340.0
237 490.0
238 490.0
239 490.0
240 490.0
241 1250.0
242 1250.0
243 1250.0
244 1250.0
245 0.0
246 0.0
247 0.0
248 0.0
249 0.0
250 0.0
251 0.0
252 0.0
253 0.0
254 0.0
255 0.0
256 0.0
257 0.0
258 100.0
259 100.0
260 100.0
261 100.0
262 100.0
263 100.0
264 100.0
265 100.0
266 0.0
267 0.0
268 0.0
269 0.0
270 0.0
271 0.0
272 0.0
273 0.0
274 0.0
275 0.0
276 0.0
277 0.0
278 0.0
279 0.0
280 0.0
281 0.0
282 0.0
283 0.0
284 0.0
285 0.0
286 0.0
287 0.0
288 0.0
289 0.0
290 0.0
291 0.0
292 0.0
293 0.0
294 0.0
295 0.0
296 0.0
297 0.0
298 0.0
299 0.0
300 0.0
301 0.0
302 0.0
303 0.0
304 0.0
305 0.0
306 0.0
307 0.0
308 0.0
309 0.0
310 0.0
311 0.0;


######################
# SOLVE 
######################
end;
