

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..104;
# Set of Hosts
set Hs := 0..30;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {14..29}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm6_0_{h in Hs}: sum{v in {30..45}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm7_0_{h in Hs}: sum{v in {46..49}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm8_0_{h in Hs}: sum{v in {50..53}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm1_0_{h in Hs}: sum{v in {54,55}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {56..67}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_0_{h in Hs}: sum{v in {68..73}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm10_0_{h in Hs}: sum{v in {74..104}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {2..5}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {2..5}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {6..9}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {6..9}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 12
11 12
12 12
13 12
14 12
15 12
16 12
17 12
18 12
19 12
20 12
21 12
22 12
23 12
24 12
25 12
26 12
27 12
28 12
29 12
30 12
31 12
32 12
33 12
34 12
35 12
36 12
37 12
38 12
39 12
40 12
41 12
42 12
43 12
44 12
45 12
46 12
47 12
48 12
49 12
50 12
51 12
52 12
53 12
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 8
73 8
74 16
75 16
76 16
77 16
78 16
79 16
80 16
81 16
82 16
83 16
84 16
85 16
86 16
87 16
88 16
89 16
90 16
91 16
92 16
93 16
94 16
95 16
96 16
97 16
98 16
99 16
100 16
101 16
102 16
103 16
104 16;
param: MEM := 
0 16
1 16
2 48
3 48
4 48
5 48
6 48
7 48
8 48
9 48
10 60
11 60
12 60
13 60
14 60
15 60
16 60
17 60
18 60
19 60
20 60
21 60
22 60
23 60
24 60
25 60
26 60
27 60
28 60
29 60
30 60
31 60
32 60
33 60
34 60
35 60
36 60
37 60
38 60
39 60
40 60
41 60
42 60
43 60
44 60
45 60
46 60
47 60
48 60
49 60
50 60
51 60
52 60
53 60
54 16
55 16
56 42
57 42
58 42
59 42
60 42
61 42
62 42
63 42
64 42
65 42
66 42
67 42
68 16
69 16
70 16
71 16
72 16
73 16
74 8
75 8
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 8
89 8
90 8
91 8
92 8
93 8
94 8
95 8
96 8
97 8
98 8
99 8
100 8
101 8
102 8
103 8
104 8;
param: NET := 
0 675
1 675
2 535
3 535
4 535
5 535
6 535
7 535
8 535
9 535
10 100
11 100
12 100
13 100
14 300
15 300
16 300
17 300
18 300
19 300
20 300
21 300
22 300
23 300
24 300
25 300
26 300
27 300
28 300
29 300
30 300
31 300
32 300
33 300
34 300
35 300
36 300
37 300
38 300
39 300
40 300
41 300
42 300
43 300
44 300
45 300
46 300
47 300
48 300
49 300
50 300
51 300
52 300
53 300
54 171
55 171
56 535
57 535
58 535
59 535
60 535
61 535
62 535
63 535
64 535
65 535
66 535
67 535
68 0
69 0
70 0
71 0
72 0
73 0
74 1600
75 1600
76 1600
77 1600
78 1600
79 1600
80 1600
81 1600
82 1600
83 1600
84 1600
85 1600
86 1600
87 1600
88 1600
89 1600
90 1600
91 1600
92 1600
93 1600
94 1600
95 1600
96 1600
97 1600
98 1600
99 1600
100 1600
101 1600
102 1600
103 1600
104 1600;


######################
# SOLVE 
######################
end;
