

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..13;
# Set of Hosts
set Hs := 0..2;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm1_0_master{h in Hs}: ni_vnf2_dot_vm1_0_master[h] <= sum{v in {2}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm1_0_master{v in {2}, h in Hs}: ni_vnf2_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm1_0_slave{h in Hs}: ni_vnf2_dot_vm1_0_slave[h] <= sum{v in {3}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm1_0_slave{v in {3}, h in Hs}: ni_vnf2_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf2_dot_vm1_0_master[h] + ni_vnf2_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0_master{h in Hs}: ni_vnf2_dot_vm2_0_master[h] <= sum{v in {4..7}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0_master{v in {4..7}, h in Hs}: ni_vnf2_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0_slave{h in Hs}: ni_vnf2_dot_vm2_0_slave[h] <= sum{v in {8..11}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0_slave{v in {8..11}, h in Hs}: ni_vnf2_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf2_dot_vm2_0_master[h] + ni_vnf2_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4
1 4
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8;
param: MEM := 
0 16
1 16
2 54
3 54
4 54
5 54
6 54
7 54
8 54
9 54
10 54
11 54
12 32
13 32;
param: NET := 
0 0
1 0
2 36
3 36
4 87
5 87
6 87
7 87
8 87
9 87
10 87
11 87
12 0
13 0;


######################
# SOLVE 
######################
end;
