

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..11;
# Set of Hosts
set Hs := 0..2;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf3_dot_vm3_0_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm1_0_master{h in Hs}: ni_vnf2_dot_vm1_0_master[h] <= sum{v in {0}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm1_0_master{v in {0}, h in Hs}: ni_vnf2_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm1_0_slave{h in Hs}: ni_vnf2_dot_vm1_0_slave[h] <= sum{v in {1}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm1_0_slave{v in {1}, h in Hs}: ni_vnf2_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf2_dot_vm1_0_master[h] + ni_vnf2_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0_master{h in Hs}: ni_vnf2_dot_vm2_0_master[h] <= sum{v in {2..5}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0_master{v in {2..5}, h in Hs}: ni_vnf2_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0_slave{h in Hs}: ni_vnf2_dot_vm2_0_slave[h] <= sum{v in {6..9}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0_slave{v in {6..9}, h in Hs}: ni_vnf2_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf2_dot_vm2_0_master[h] + ni_vnf2_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0;
param: MEM := 
0 54.0
1 54.0
2 54.0
3 54.0
4 54.0
5 54.0
6 54.0
7 54.0
8 54.0
9 54.0
10 32.0
11 32.0;
param: NET := 
0 36.0
1 36.0
2 87.0
3 87.0
4 87.0
5 87.0
6 87.0
7 87.0
8 87.0
9 87.0
10 0.0
11 0.0;


######################
# SOLVE 
######################
end;
