

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..34;
# Set of Hosts
set Hs := 0..19;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2..10}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {11..30}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm4_0_{h in Hs}: sum{v in {31,32}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {33,34}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 16
1 16
2 16
3 16
4 16
5 16
6 16
7 16
8 16
9 16
10 16
11 16
12 16
13 16
14 16
15 16
16 16
17 16
18 16
19 16
20 16
21 16
22 16
23 16
24 16
25 16
26 16
27 16
28 16
29 16
30 16
31 12
32 12
33 16
34 16;
param: MEM := 
0 96
1 96
2 160
3 160
4 160
5 160
6 160
7 160
8 160
9 160
10 160
11 144
12 144
13 144
14 144
15 144
16 144
17 144
18 144
19 144
20 144
21 144
22 144
23 144
24 144
25 144
26 144
27 144
28 144
29 144
30 144
31 96
32 96
33 96
34 96;
param: NET := 
0 100
1 100
2 400
3 400
4 400
5 400
6 400
7 400
8 400
9 400
10 400
11 4700
12 4700
13 4700
14 4700
15 4700
16 4700
17 4700
18 4700
19 4700
20 4700
21 4700
22 4700
23 4700
24 4700
25 4700
26 4700
27 4700
28 4700
29 4700
30 4700
31 100
32 100
33 100
34 100;


######################
# SOLVE 
######################
end;
