

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..21;
# Set of Hosts
set Hs := 0..3;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {10..12}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {13}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm6_0_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm7_0_{h in Hs}: sum{v in {16,17}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm9_0_{h in Hs}: sum{v in {20,21}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4.0
1 4.0
2 4.0
3 4.0
4 4.0
5 4.0
6 16.0
7 16.0
8 8.0
9 8.0
10 4.0
11 4.0
12 4.0
13 1.0
14 8.0
15 8.0
16 16.0
17 16.0
18 2.0
19 2.0
20 2.0
21 2.0;
param: MEM := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 64.0
7 64.0
8 32.0
9 32.0
10 8.0
11 8.0
12 8.0
13 2.0
14 16.0
15 16.0
16 32.0
17 32.0
18 4.0
19 4.0
20 4.0
21 4.0;
param: NET := 
0 200.0
1 200.0
2 200.0
3 200.0
4 200.0
5 200.0
6 1000.0
7 1000.0
8 1000.0
9 1000.0
10 1000.0
11 1000.0
12 1000.0
13 1000.0
14 20.0
15 20.0
16 100.0
17 100.0
18 70.0
19 70.0
20 70.0
21 70.0;


######################
# SOLVE 
######################
end;
