

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..35;
# Set of Hosts
set Hs := 0..9;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {6,7}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {10..12}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {13}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm6_0_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm7_0_{h in Hs}: sum{v in {16,17}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm8_0_{h in Hs}: sum{v in {18..27}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm9_0_{h in Hs}: sum{v in {28..31}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm10_0_{h in Hs}: sum{v in {32,33}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm11_0_{h in Hs}: sum{v in {34,35}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4
1 4
2 4
3 4
4 4
5 4
6 16
7 16
8 8
9 8
10 4
11 4
12 4
13 1
14 8
15 8
16 16
17 16
18 2
19 2
20 2
21 2
22 2
23 2
24 2
25 2
26 2
27 2
28 2
29 2
30 2
31 2
32 8
33 8
34 2
35 2;
param: MEM := 
0 8
1 8
2 8
3 8
4 8
5 8
6 64
7 64
8 32
9 32
10 8
11 8
12 8
13 2
14 16
15 16
16 32
17 32
18 4
19 4
20 4
21 4
22 4
23 4
24 4
25 4
26 4
27 4
28 4
29 4
30 4
31 4
32 16
33 16
34 2
35 2;
param: NET := 
0 200
1 200
2 200
3 200
4 200
5 200
6 1000
7 1000
8 1000
9 1000
10 1000
11 1000
12 1000
13 1000
14 20
15 20
16 100
17 100
18 70
19 70
20 70
21 70
22 70
23 70
24 70
25 70
26 70
27 70
28 70
29 70
30 70
31 70
32 80
33 80
34 100
35 100;


######################
# SOLVE 
######################
end;
