

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..11;
# Set of Hosts
set Hs := 0..3;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {4..7}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_0_{h in Hs}: sum{v in {8,9}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm5_0_{h in Hs}: sum{v in {10,11}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 16
3 16
4 2
5 2
6 2
7 2
8 2
9 2
10 8
11 8;
param: MEM := 
0 16
1 16
2 32
3 32
4 4
5 4
6 4
7 4
8 4
9 4
10 16
11 16;
param: NET := 
0 20
1 20
2 100
3 100
4 70
5 70
6 70
7 70
8 70
9 70
10 80
11 80;


######################
# SOLVE 
######################
end;
