

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..153;
# Set of Hosts
set Hs := 0..40;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..6}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {7,8}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {9,10}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {11..36}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_1_{h in Hs}: sum{v in {37..62}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {63..65}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_1_{h in Hs}: sum{v in {66..68}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_0_{h in Hs}: sum{v in {69..98}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm5_1_{h in Hs}: sum{v in {99..128}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm6_0_{h in Hs}: sum{v in {129}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm6_1_{h in Hs}: sum{v in {130}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm7_0_{h in Hs}: sum{v in {141,142}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm8_0_{h in Hs}: sum{v in {143..147}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm9_0_{h in Hs}: sum{v in {148,149}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm10_0_{h in Hs}: sum{v in {150..153}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm2_0_master{h in Hs}: ni_vnf5_dot_vm2_0_master[h] <= sum{v in {131}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm2_0_master{v in {131}, h in Hs}: ni_vnf5_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm2_0_slave{h in Hs}: ni_vnf5_dot_vm2_0_slave[h] <= sum{v in {132}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm2_0_slave{v in {132}, h in Hs}: ni_vnf5_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf5_dot_vm2_0_master[h] + ni_vnf5_dot_vm2_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf5_dot_vm3_0_master{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm3_0_master{h in Hs}: ni_vnf5_dot_vm3_0_master[h] <= sum{v in {133..136}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm3_0_master{v in {133..136}, h in Hs}: ni_vnf5_dot_vm3_0_master[h] >= x[v, h];
var ni_vnf5_dot_vm3_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf5_dot_vm3_0_slave{h in Hs}: ni_vnf5_dot_vm3_0_slave[h] <= sum{v in {137..140}} x[v, h];
s.t. or_ineq_ni_vnf5_dot_vm3_0_slave{v in {137..140}, h in Hs}: ni_vnf5_dot_vm3_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf5_dot_vm3_0_master[h] + ni_vnf5_dot_vm3_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9
1 9
2 9
3 9
4 9
5 9
6 9
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8
28 8
29 8
30 8
31 8
32 8
33 8
34 8
35 8
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 8
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 8
64 8
65 8
66 8
67 8
68 8
69 16
70 16
71 16
72 16
73 16
74 16
75 16
76 16
77 16
78 16
79 16
80 16
81 16
82 16
83 16
84 16
85 16
86 16
87 16
88 16
89 16
90 16
91 16
92 16
93 16
94 16
95 16
96 16
97 16
98 16
99 16
100 16
101 16
102 16
103 16
104 16
105 16
106 16
107 16
108 16
109 16
110 16
111 16
112 16
113 16
114 16
115 16
116 16
117 16
118 16
119 16
120 16
121 16
122 16
123 16
124 16
125 16
126 16
127 16
128 16
129 8
130 8
131 8
132 8
133 8
134 8
135 8
136 8
137 8
138 8
139 8
140 8
141 7
142 7
143 2
144 2
145 2
146 2
147 2
148 6
149 6
150 4
151 4
152 4
153 4;
param: MEM := 
0 6
1 6
2 6
3 6
4 6
5 6
6 6
7 16
8 16
9 16
10 16
11 48
12 48
13 48
14 48
15 48
16 48
17 48
18 48
19 48
20 48
21 48
22 48
23 48
24 48
25 48
26 48
27 48
28 48
29 48
30 48
31 48
32 48
33 48
34 48
35 48
36 48
37 48
38 48
39 48
40 48
41 48
42 48
43 48
44 48
45 48
46 48
47 48
48 48
49 48
50 48
51 48
52 48
53 48
54 48
55 48
56 48
57 48
58 48
59 48
60 48
61 48
62 48
63 16
64 16
65 16
66 16
67 16
68 16
69 8
70 8
71 8
72 8
73 8
74 8
75 8
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 8
89 8
90 8
91 8
92 8
93 8
94 8
95 8
96 8
97 8
98 8
99 8
100 8
101 8
102 8
103 8
104 8
105 8
106 8
107 8
108 8
109 8
110 8
111 8
112 8
113 8
114 8
115 8
116 8
117 8
118 8
119 8
120 8
121 8
122 8
123 8
124 8
125 8
126 8
127 8
128 8
129 32
130 32
131 16
132 16
133 48
134 48
135 48
136 48
137 48
138 48
139 48
140 48
141 10
142 10
143 32
144 32
145 32
146 32
147 32
148 4
149 4
150 10
151 10
152 10
153 10;
param: NET := 
0 0
1 0
2 0
3 0
4 0
5 0
6 0
7 675
8 675
9 675
10 675
11 535
12 535
13 535
14 535
15 535
16 535
17 535
18 535
19 535
20 535
21 535
22 535
23 535
24 535
25 535
26 535
27 535
28 535
29 535
30 535
31 535
32 535
33 535
34 535
35 535
36 535
37 535
38 535
39 535
40 535
41 535
42 535
43 535
44 535
45 535
46 535
47 535
48 535
49 535
50 535
51 535
52 535
53 535
54 535
55 535
56 535
57 535
58 535
59 535
60 535
61 535
62 535
63 0
64 0
65 0
66 0
67 0
68 0
69 1600
70 1600
71 1600
72 1600
73 1600
74 1600
75 1600
76 1600
77 1600
78 1600
79 1600
80 1600
81 1600
82 1600
83 1600
84 1600
85 1600
86 1600
87 1600
88 1600
89 1600
90 1600
91 1600
92 1600
93 1600
94 1600
95 1600
96 1600
97 1600
98 1600
99 1600
100 1600
101 1600
102 1600
103 1600
104 1600
105 1600
106 1600
107 1600
108 1600
109 1600
110 1600
111 1600
112 1600
113 1600
114 1600
115 1600
116 1600
117 1600
118 1600
119 1600
120 1600
121 1600
122 1600
123 1600
124 1600
125 1600
126 1600
127 1600
128 1600
129 0
130 0
131 675
132 675
133 535
134 535
135 535
136 535
137 535
138 535
139 535
140 535
141 0
142 0
143 0
144 0
145 0
146 0
147 0
148 0
149 0
150 0
151 0
152 0
153 0;


######################
# SOLVE 
######################
end;
