

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..32;
# Set of Hosts
set Hs := 0..9;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0,1}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0,1}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {2,3}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {2,3}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_master{h in Hs}: ni_vnf1_dot_vm2_0_master[h] <= sum{v in {4..7}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_master{v in {4..7}, h in Hs}: ni_vnf1_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm2_0_slave{h in Hs}: ni_vnf1_dot_vm2_0_slave[h] <= sum{v in {8..11}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm2_0_slave{v in {8..11}, h in Hs}: ni_vnf1_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm2_0_master[h] + ni_vnf1_dot_vm2_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm3_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_master{h in Hs}: ni_vnf1_dot_vm3_0_master[h] <= sum{v in {12,13}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_master{v in {12,13}, h in Hs}: ni_vnf1_dot_vm3_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm3_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_slave{h in Hs}: ni_vnf1_dot_vm3_0_slave[h] <= sum{v in {14,15}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_slave{v in {14,15}, h in Hs}: ni_vnf1_dot_vm3_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf1_dot_vm3_0_master[h] + ni_vnf1_dot_vm3_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_master{h in Hs}: ni_vnf1_dot_vm6_0_master[h] <= sum{v in {18,19}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_master{v in {18,19}, h in Hs}: ni_vnf1_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm6_0_slave{h in Hs}: ni_vnf1_dot_vm6_0_slave[h] <= sum{v in {20,21}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm6_0_slave{v in {20,21}, h in Hs}: ni_vnf1_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf1_dot_vm6_0_master[h] + ni_vnf1_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm7_0_master{h in Hs}: ni_vnf1_dot_vm7_0_master[h] <= sum{v in {22}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm7_0_master{v in {22}, h in Hs}: ni_vnf1_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm7_0_slave{h in Hs}: ni_vnf1_dot_vm7_0_slave[h] <= sum{v in {23}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm7_0_slave{v in {23}, h in Hs}: ni_vnf1_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf1_dot_vm7_0_master[h] + ni_vnf1_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm8_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm8_0_master{h in Hs}: ni_vnf1_dot_vm8_0_master[h] <= sum{v in {24..26}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm8_0_master{v in {24..26}, h in Hs}: ni_vnf1_dot_vm8_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm8_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm8_0_slave{h in Hs}: ni_vnf1_dot_vm8_0_slave[h] <= sum{v in {27..29}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm8_0_slave{v in {27..29}, h in Hs}: ni_vnf1_dot_vm8_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf1_dot_vm8_0_master[h] + ni_vnf1_dot_vm8_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm9_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm9_0_master{h in Hs}: ni_vnf1_dot_vm9_0_master[h] <= sum{v in {30}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm9_0_master{v in {30}, h in Hs}: ni_vnf1_dot_vm9_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm9_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm9_0_slave{h in Hs}: ni_vnf1_dot_vm9_0_slave[h] <= sum{v in {31}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm9_0_slave{v in {31}, h in Hs}: ni_vnf1_dot_vm9_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_6_{h in Hs}: ni_vnf1_dot_vm9_0_master[h] + ni_vnf1_dot_vm9_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 4
1 4
2 4
3 4
4 22
5 22
6 22
7 22
8 22
9 22
10 22
11 22
12 22
13 22
14 22
15 22
16 4
17 2
18 8
19 8
20 8
21 8
22 16
23 16
24 8
25 8
26 8
27 8
28 8
29 8
30 4
31 4
32 4;
param: MEM := 
0 24
1 24
2 24
3 24
4 32
5 32
6 32
7 32
8 32
9 32
10 32
11 32
12 32
13 32
14 32
15 32
16 8
17 6
18 8
19 8
20 8
21 8
22 32
23 32
24 8
25 8
26 8
27 8
28 8
29 8
30 6
31 6
32 14;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0;


######################
# SOLVE 
######################
end;
