

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..152;
# Set of Hosts
set Hs := 0..32;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {4..25}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {26..47}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {50,51}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {52..83}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {84..115}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {126,127}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {128,129}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {130,131}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm6_0_{h in Hs}: sum{v in {132,133}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm11_0_{h in Hs}: sum{v in {134,135}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {136,137}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {138..148}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm1_0_{h in Hs}: sum{v in {149,150}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm2_0_{h in Hs}: sum{v in {151,152}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_master{h in Hs}: ni_vnf3_dot_vm1_0_master[h] <= sum{v in {116}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_master{v in {116}, h in Hs}: ni_vnf3_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm1_0_slave{h in Hs}: ni_vnf3_dot_vm1_0_slave[h] <= sum{v in {117}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm1_0_slave{v in {117}, h in Hs}: ni_vnf3_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf3_dot_vm1_0_master[h] + ni_vnf3_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_master{h in Hs}: ni_vnf3_dot_vm2_0_master[h] <= sum{v in {118..121}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_master{v in {118..121}, h in Hs}: ni_vnf3_dot_vm2_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm2_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0_slave{h in Hs}: ni_vnf3_dot_vm2_0_slave[h] <= sum{v in {122..125}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0_slave{v in {122..125}, h in Hs}: ni_vnf3_dot_vm2_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf3_dot_vm2_0_master[h] + ni_vnf3_dot_vm2_0_slave[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8
28 8
29 8
30 8
31 8
32 8
33 8
34 8
35 8
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 8
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 8
73 8
74 8
75 8
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 8
89 8
90 8
91 8
92 8
93 8
94 8
95 8
96 8
97 8
98 8
99 8
100 8
101 8
102 8
103 8
104 8
105 8
106 8
107 8
108 8
109 8
110 8
111 8
112 8
113 8
114 8
115 8
116 8
117 8
118 8
119 8
120 8
121 8
122 8
123 8
124 8
125 8
126 10
127 10
128 10
129 10
130 10
131 10
132 10
133 10
134 14
135 14
136 8
137 8
138 8
139 8
140 8
141 8
142 8
143 8
144 8
145 8
146 8
147 8
148 8
149 14
150 14
151 14
152 14;
param: MEM := 
0 54
1 54
2 54
3 54
4 54
5 54
6 54
7 54
8 54
9 54
10 54
11 54
12 54
13 54
14 54
15 54
16 54
17 54
18 54
19 54
20 54
21 54
22 54
23 54
24 54
25 54
26 54
27 54
28 54
29 54
30 54
31 54
32 54
33 54
34 54
35 54
36 54
37 54
38 54
39 54
40 54
41 54
42 54
43 54
44 54
45 54
46 54
47 54
48 20
49 20
50 20
51 20
52 54
53 54
54 54
55 54
56 54
57 54
58 54
59 54
60 54
61 54
62 54
63 54
64 54
65 54
66 54
67 54
68 54
69 54
70 54
71 54
72 54
73 54
74 54
75 54
76 54
77 54
78 54
79 54
80 54
81 54
82 54
83 54
84 54
85 54
86 54
87 54
88 54
89 54
90 54
91 54
92 54
93 54
94 54
95 54
96 54
97 54
98 54
99 54
100 54
101 54
102 54
103 54
104 54
105 54
106 54
107 54
108 54
109 54
110 54
111 54
112 54
113 54
114 54
115 54
116 54
117 54
118 54
119 54
120 54
121 54
122 54
123 54
124 54
125 54
126 10
127 10
128 10
129 10
130 10
131 10
132 10
133 10
134 8
135 8
136 20
137 20
138 54
139 54
140 54
141 54
142 54
143 54
144 54
145 54
146 54
147 54
148 54
149 40
150 40
151 8
152 8;
param: NET := 
0 36
1 36
2 36
3 36
4 87
5 87
6 87
7 87
8 87
9 87
10 87
11 87
12 87
13 87
14 87
15 87
16 87
17 87
18 87
19 87
20 87
21 87
22 87
23 87
24 87
25 87
26 87
27 87
28 87
29 87
30 87
31 87
32 87
33 87
34 87
35 87
36 87
37 87
38 87
39 87
40 87
41 87
42 87
43 87
44 87
45 87
46 87
47 87
48 36
49 36
50 36
51 36
52 97
53 97
54 97
55 97
56 97
57 97
58 97
59 97
60 97
61 97
62 97
63 97
64 97
65 97
66 97
67 97
68 97
69 97
70 97
71 97
72 97
73 97
74 97
75 97
76 97
77 97
78 97
79 97
80 97
81 97
82 97
83 97
84 97
85 97
86 97
87 97
88 97
89 97
90 97
91 97
92 97
93 97
94 97
95 97
96 97
97 97
98 97
99 97
100 97
101 97
102 97
103 97
104 97
105 97
106 97
107 97
108 97
109 97
110 97
111 97
112 97
113 97
114 97
115 97
116 36
117 36
118 87
119 87
120 87
121 87
122 87
123 87
124 87
125 87
126 126
127 126
128 126
129 126
130 126
131 126
132 126
133 126
134 40
135 40
136 36
137 36
138 97
139 97
140 97
141 97
142 97
143 97
144 97
145 97
146 97
147 97
148 97
149 350
150 350
151 70
152 70;


######################
# SOLVE 
######################
end;
