

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..656;
# Set of Hosts
set Hs := 0..128;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..4}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {5..8}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {9..11}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_0_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_0_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm6_0_{h in Hs}: sum{v in {16..20}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm7_0_{h in Hs}: sum{v in {21..25}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm8_0_{h in Hs}: sum{v in {26..28}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm9_0_{h in Hs}: sum{v in {29..44}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm10_0_{h in Hs}: sum{v in {45..57}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm11_0_{h in Hs}: sum{v in {58,59}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm12_0_{h in Hs}: sum{v in {60,61}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm13_0_{h in Hs}: sum{v in {62,63}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm14_0_{h in Hs}: sum{v in {64}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_0_{h in Hs}: sum{v in {257..259}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_1_{h in Hs}: sum{v in {260..262}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_2_{h in Hs}: sum{v in {263..265}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_3_{h in Hs}: sum{v in {266..268}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_4_{h in Hs}: sum{v in {269..271}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_5_{h in Hs}: sum{v in {272..274}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_0_{h in Hs}: sum{v in {275..277}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_1_{h in Hs}: sum{v in {278..280}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_2_{h in Hs}: sum{v in {281..283}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_3_{h in Hs}: sum{v in {284..286}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_4_{h in Hs}: sum{v in {287..289}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_5_{h in Hs}: sum{v in {290..292}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_0_{h in Hs}: sum{v in {293..295}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_1_{h in Hs}: sum{v in {296..298}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_2_{h in Hs}: sum{v in {299..301}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_3_{h in Hs}: sum{v in {302..304}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_4_{h in Hs}: sum{v in {305..307}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_5_{h in Hs}: sum{v in {308..310}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_0_{h in Hs}: sum{v in {311..313}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_1_{h in Hs}: sum{v in {314..316}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_2_{h in Hs}: sum{v in {317..319}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_3_{h in Hs}: sum{v in {320..322}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_4_{h in Hs}: sum{v in {323..325}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_5_{h in Hs}: sum{v in {326..328}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_0_{h in Hs}: sum{v in {329..331}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_1_{h in Hs}: sum{v in {332..334}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_2_{h in Hs}: sum{v in {335..337}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_3_{h in Hs}: sum{v in {338..340}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_4_{h in Hs}: sum{v in {341..343}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_5_{h in Hs}: sum{v in {344..346}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_0_{h in Hs}: sum{v in {347..354}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_1_{h in Hs}: sum{v in {355..362}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_2_{h in Hs}: sum{v in {363..370}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_3_{h in Hs}: sum{v in {371..378}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_4_{h in Hs}: sum{v in {379..386}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_5_{h in Hs}: sum{v in {387..394}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm23_0_{h in Hs}: sum{v in {395,396}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm24_0_{h in Hs}: sum{v in {397..399}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_0_{h in Hs}: sum{v in {402..404}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm27_0_{h in Hs}: sum{v in {405..409}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_0_{h in Hs}: sum{v in {413..436}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_1_{h in Hs}: sum{v in {437..460}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_2_{h in Hs}: sum{v in {461..484}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_3_{h in Hs}: sum{v in {485..508}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_0_{h in Hs}: sum{v in {509,510}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_1_{h in Hs}: sum{v in {511,512}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_2_{h in Hs}: sum{v in {513,514}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_3_{h in Hs}: sum{v in {515,516}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_0_{h in Hs}: sum{v in {517,518}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_1_{h in Hs}: sum{v in {519,520}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_2_{h in Hs}: sum{v in {521,522}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_3_{h in Hs}: sum{v in {523,524}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_0_{h in Hs}: sum{v in {525,526}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_1_{h in Hs}: sum{v in {527,528}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_2_{h in Hs}: sum{v in {529,530}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_3_{h in Hs}: sum{v in {531,532}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_0_{h in Hs}: sum{v in {533,534}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_1_{h in Hs}: sum{v in {535,536}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_2_{h in Hs}: sum{v in {537,538}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_3_{h in Hs}: sum{v in {539,540}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_0_{h in Hs}: sum{v in {541,542}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_1_{h in Hs}: sum{v in {543,544}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_2_{h in Hs}: sum{v in {545,546}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_3_{h in Hs}: sum{v in {547,548}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_0_{h in Hs}: sum{v in {549..572}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_1_{h in Hs}: sum{v in {573..596}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_2_{h in Hs}: sum{v in {597..620}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_3_{h in Hs}: sum{v in {621..644}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_0_{h in Hs}: sum{v in {645,646}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_1_{h in Hs}: sum{v in {647,648}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_2_{h in Hs}: sum{v in {649,650}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_3_{h in Hs}: sum{v in {651,652}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm36_0_{h in Hs}: sum{v in {653,654}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm36_1_{h in Hs}: sum{v in {655,656}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9.2
1 9.2
2 9.2
3 9.2
4 9.2
5 12.0
6 12.0
7 12.0
8 12.0
9 13.8
10 13.8
11 13.8
12 9.2
13 9.2
14 9.2
15 9.2
16 9.2
17 9.2
18 9.2
19 9.2
20 9.2
21 4.6
22 4.6
23 4.6
24 4.6
25 4.6
26 13.8
27 13.8
28 13.8
29 21.0
30 21.0
31 21.0
32 21.0
33 21.0
34 21.0
35 21.0
36 21.0
37 21.0
38 21.0
39 21.0
40 21.0
41 21.0
42 21.0
43 21.0
44 21.0
45 25.4769230769231
46 25.4769230769231
47 25.4769230769231
48 25.4769230769231
49 25.4769230769231
50 25.4769230769231
51 25.4769230769231
52 25.4769230769231
53 25.4769230769231
54 25.4769230769231
55 25.4769230769231
56 25.4769230769231
57 25.4769230769231
58 36.8
59 36.8
60 13.8
61 13.8
62 36.8
63 36.8
64 40.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 8.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0
83 8.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 8.0
93 8.0
94 8.0
95 8.0
96 8.0
97 8.0
98 8.0
99 8.0
100 8.0
101 8.0
102 8.0
103 8.0
104 8.0
105 8.0
106 8.0
107 8.0
108 8.0
109 8.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 8.0
117 8.0
118 8.0
119 8.0
120 8.0
121 8.0
122 8.0
123 8.0
124 8.0
125 8.0
126 8.0
127 8.0
128 8.0
129 8.0
130 8.0
131 8.0
132 8.0
133 8.0
134 8.0
135 8.0
136 8.0
137 8.0
138 8.0
139 8.0
140 8.0
141 8.0
142 8.0
143 8.0
144 8.0
145 8.0
146 8.0
147 8.0
148 8.0
149 8.0
150 8.0
151 8.0
152 8.0
153 8.0
154 8.0
155 8.0
156 8.0
157 8.0
158 8.0
159 8.0
160 8.0
161 8.0
162 8.0
163 8.0
164 8.0
165 8.0
166 8.0
167 8.0
168 8.0
169 8.0
170 8.0
171 8.0
172 8.0
173 8.0
174 8.0
175 8.0
176 8.0
177 8.0
178 8.0
179 8.0
180 8.0
181 8.0
182 8.0
183 8.0
184 8.0
185 8.0
186 8.0
187 8.0
188 8.0
189 8.0
190 8.0
191 8.0
192 8.0
193 8.0
194 8.0
195 8.0
196 8.0
197 8.0
198 8.0
199 8.0
200 8.0
201 8.0
202 8.0
203 8.0
204 8.0
205 8.0
206 8.0
207 8.0
208 8.0
209 8.0
210 8.0
211 8.0
212 8.0
213 8.0
214 8.0
215 8.0
216 8.0
217 8.0
218 8.0
219 8.0
220 8.0
221 8.0
222 8.0
223 8.0
224 8.0
225 8.0
226 8.0
227 8.0
228 8.0
229 8.0
230 8.0
231 8.0
232 8.0
233 8.0
234 8.0
235 8.0
236 8.0
237 8.0
238 8.0
239 8.0
240 8.0
241 8.0
242 8.0
243 8.0
244 8.0
245 8.0
246 8.0
247 8.0
248 8.0
249 8.0
250 8.0
251 8.0
252 8.0
253 8.0
254 8.0
255 8.0
256 8.0
257 4.0
258 4.0
259 4.0
260 4.0
261 4.0
262 4.0
263 4.0
264 4.0
265 4.0
266 4.0
267 4.0
268 4.0
269 4.0
270 4.0
271 4.0
272 4.0
273 4.0
274 4.0
275 4.0
276 4.0
277 4.0
278 4.0
279 4.0
280 4.0
281 4.0
282 4.0
283 4.0
284 4.0
285 4.0
286 4.0
287 4.0
288 4.0
289 4.0
290 4.0
291 4.0
292 4.0
293 4.0
294 4.0
295 4.0
296 4.0
297 4.0
298 4.0
299 4.0
300 4.0
301 4.0
302 4.0
303 4.0
304 4.0
305 4.0
306 4.0
307 4.0
308 4.0
309 4.0
310 4.0
311 4.0
312 4.0
313 4.0
314 4.0
315 4.0
316 4.0
317 4.0
318 4.0
319 4.0
320 4.0
321 4.0
322 4.0
323 4.0
324 4.0
325 4.0
326 4.0
327 4.0
328 4.0
329 4.0
330 4.0
331 4.0
332 4.0
333 4.0
334 4.0
335 4.0
336 4.0
337 4.0
338 4.0
339 4.0
340 4.0
341 4.0
342 4.0
343 4.0
344 4.0
345 4.0
346 4.0
347 8.0
348 8.0
349 8.0
350 8.0
351 8.0
352 8.0
353 8.0
354 8.0
355 8.0
356 8.0
357 8.0
358 8.0
359 8.0
360 8.0
361 8.0
362 8.0
363 8.0
364 8.0
365 8.0
366 8.0
367 8.0
368 8.0
369 8.0
370 8.0
371 8.0
372 8.0
373 8.0
374 8.0
375 8.0
376 8.0
377 8.0
378 8.0
379 8.0
380 8.0
381 8.0
382 8.0
383 8.0
384 8.0
385 8.0
386 8.0
387 8.0
388 8.0
389 8.0
390 8.0
391 8.0
392 8.0
393 8.0
394 8.0
395 4.0
396 4.0
397 4.0
398 4.0
399 4.0
400 4.0
401 4.0
402 8.0
403 8.0
404 8.0
405 8.0
406 8.0
407 8.0
408 8.0
409 8.0
410 8.0
411 8.0
412 8.0
413 8.0
414 8.0
415 8.0
416 8.0
417 8.0
418 8.0
419 8.0
420 8.0
421 8.0
422 8.0
423 8.0
424 8.0
425 8.0
426 8.0
427 8.0
428 8.0
429 8.0
430 8.0
431 8.0
432 8.0
433 8.0
434 8.0
435 8.0
436 8.0
437 8.0
438 8.0
439 8.0
440 8.0
441 8.0
442 8.0
443 8.0
444 8.0
445 8.0
446 8.0
447 8.0
448 8.0
449 8.0
450 8.0
451 8.0
452 8.0
453 8.0
454 8.0
455 8.0
456 8.0
457 8.0
458 8.0
459 8.0
460 8.0
461 8.0
462 8.0
463 8.0
464 8.0
465 8.0
466 8.0
467 8.0
468 8.0
469 8.0
470 8.0
471 8.0
472 8.0
473 8.0
474 8.0
475 8.0
476 8.0
477 8.0
478 8.0
479 8.0
480 8.0
481 8.0
482 8.0
483 8.0
484 8.0
485 8.0
486 8.0
487 8.0
488 8.0
489 8.0
490 8.0
491 8.0
492 8.0
493 8.0
494 8.0
495 8.0
496 8.0
497 8.0
498 8.0
499 8.0
500 8.0
501 8.0
502 8.0
503 8.0
504 8.0
505 8.0
506 8.0
507 8.0
508 8.0
509 16.0
510 16.0
511 16.0
512 16.0
513 16.0
514 16.0
515 16.0
516 16.0
517 8.0
518 8.0
519 8.0
520 8.0
521 8.0
522 8.0
523 8.0
524 8.0
525 4.0
526 4.0
527 4.0
528 4.0
529 4.0
530 4.0
531 4.0
532 4.0
533 8.0
534 8.0
535 8.0
536 8.0
537 8.0
538 8.0
539 8.0
540 8.0
541 4.0
542 4.0
543 4.0
544 4.0
545 4.0
546 4.0
547 4.0
548 4.0
549 8.0
550 8.0
551 8.0
552 8.0
553 8.0
554 8.0
555 8.0
556 8.0
557 8.0
558 8.0
559 8.0
560 8.0
561 8.0
562 8.0
563 8.0
564 8.0
565 8.0
566 8.0
567 8.0
568 8.0
569 8.0
570 8.0
571 8.0
572 8.0
573 8.0
574 8.0
575 8.0
576 8.0
577 8.0
578 8.0
579 8.0
580 8.0
581 8.0
582 8.0
583 8.0
584 8.0
585 8.0
586 8.0
587 8.0
588 8.0
589 8.0
590 8.0
591 8.0
592 8.0
593 8.0
594 8.0
595 8.0
596 8.0
597 8.0
598 8.0
599 8.0
600 8.0
601 8.0
602 8.0
603 8.0
604 8.0
605 8.0
606 8.0
607 8.0
608 8.0
609 8.0
610 8.0
611 8.0
612 8.0
613 8.0
614 8.0
615 8.0
616 8.0
617 8.0
618 8.0
619 8.0
620 8.0
621 8.0
622 8.0
623 8.0
624 8.0
625 8.0
626 8.0
627 8.0
628 8.0
629 8.0
630 8.0
631 8.0
632 8.0
633 8.0
634 8.0
635 8.0
636 8.0
637 8.0
638 8.0
639 8.0
640 8.0
641 8.0
642 8.0
643 8.0
644 8.0
645 8.0
646 8.0
647 8.0
648 8.0
649 8.0
650 8.0
651 8.0
652 8.0
653 8.0
654 8.0
655 8.0
656 8.0;
param: MEM := 
0 13.8
1 13.8
2 13.8
3 13.8
4 13.8
5 64.0
6 64.0
7 64.0
8 64.0
9 27.6
10 27.6
11 27.6
12 27.6
13 27.6
14 55.2
15 55.2
16 13.8
17 13.8
18 13.8
19 13.8
20 13.8
21 13.8
22 13.8
23 13.8
24 13.8
25 13.8
26 36.8
27 36.8
28 36.8
29 112.0
30 112.0
31 112.0
32 112.0
33 112.0
34 112.0
35 112.0
36 112.0
37 112.0
38 112.0
39 112.0
40 112.0
41 112.0
42 112.0
43 112.0
44 112.0
45 25.4769230769231
46 25.4769230769231
47 25.4769230769231
48 25.4769230769231
49 25.4769230769231
50 25.4769230769231
51 25.4769230769231
52 25.4769230769231
53 25.4769230769231
54 25.4769230769231
55 25.4769230769231
56 25.4769230769231
57 25.4769230769231
58 27.6
59 27.6
60 27.6
61 27.6
62 55.2
63 55.2
64 80.0
65 32.0
66 32.0
67 32.0
68 32.0
69 32.0
70 32.0
71 32.0
72 32.0
73 32.0
74 32.0
75 32.0
76 32.0
77 32.0
78 32.0
79 32.0
80 32.0
81 32.0
82 32.0
83 32.0
84 32.0
85 32.0
86 32.0
87 32.0
88 32.0
89 32.0
90 32.0
91 32.0
92 32.0
93 32.0
94 32.0
95 32.0
96 32.0
97 32.0
98 32.0
99 32.0
100 32.0
101 32.0
102 32.0
103 32.0
104 32.0
105 32.0
106 32.0
107 32.0
108 32.0
109 32.0
110 32.0
111 32.0
112 32.0
113 32.0
114 32.0
115 32.0
116 32.0
117 32.0
118 32.0
119 32.0
120 32.0
121 32.0
122 32.0
123 32.0
124 32.0
125 32.0
126 32.0
127 32.0
128 32.0
129 32.0
130 32.0
131 32.0
132 32.0
133 32.0
134 32.0
135 32.0
136 32.0
137 32.0
138 32.0
139 32.0
140 32.0
141 32.0
142 32.0
143 32.0
144 32.0
145 32.0
146 32.0
147 32.0
148 32.0
149 32.0
150 32.0
151 32.0
152 32.0
153 32.0
154 32.0
155 32.0
156 32.0
157 32.0
158 32.0
159 32.0
160 32.0
161 32.0
162 32.0
163 32.0
164 32.0
165 32.0
166 32.0
167 32.0
168 32.0
169 32.0
170 32.0
171 32.0
172 32.0
173 32.0
174 32.0
175 32.0
176 32.0
177 32.0
178 32.0
179 32.0
180 32.0
181 32.0
182 32.0
183 32.0
184 32.0
185 32.0
186 32.0
187 32.0
188 32.0
189 32.0
190 32.0
191 32.0
192 32.0
193 32.0
194 32.0
195 32.0
196 32.0
197 32.0
198 32.0
199 32.0
200 32.0
201 32.0
202 32.0
203 32.0
204 32.0
205 32.0
206 32.0
207 32.0
208 32.0
209 32.0
210 32.0
211 32.0
212 32.0
213 32.0
214 32.0
215 32.0
216 32.0
217 32.0
218 32.0
219 32.0
220 32.0
221 32.0
222 32.0
223 32.0
224 32.0
225 32.0
226 32.0
227 32.0
228 32.0
229 32.0
230 32.0
231 32.0
232 32.0
233 32.0
234 32.0
235 32.0
236 32.0
237 32.0
238 32.0
239 32.0
240 32.0
241 32.0
242 32.0
243 32.0
244 32.0
245 32.0
246 32.0
247 32.0
248 32.0
249 32.0
250 32.0
251 32.0
252 32.0
253 32.0
254 32.0
255 32.0
256 32.0
257 16.0
258 16.0
259 16.0
260 16.0
261 16.0
262 16.0
263 16.0
264 16.0
265 16.0
266 16.0
267 16.0
268 16.0
269 16.0
270 16.0
271 16.0
272 16.0
273 16.0
274 16.0
275 13.0
276 13.0
277 13.0
278 13.0
279 13.0
280 13.0
281 13.0
282 13.0
283 13.0
284 13.0
285 13.0
286 13.0
287 13.0
288 13.0
289 13.0
290 13.0
291 13.0
292 13.0
293 16.0
294 16.0
295 16.0
296 16.0
297 16.0
298 16.0
299 16.0
300 16.0
301 16.0
302 16.0
303 16.0
304 16.0
305 16.0
306 16.0
307 16.0
308 16.0
309 16.0
310 16.0
311 16.0
312 16.0
313 16.0
314 16.0
315 16.0
316 16.0
317 16.0
318 16.0
319 16.0
320 16.0
321 16.0
322 16.0
323 16.0
324 16.0
325 16.0
326 16.0
327 16.0
328 16.0
329 16.0
330 16.0
331 16.0
332 16.0
333 16.0
334 16.0
335 16.0
336 16.0
337 16.0
338 16.0
339 16.0
340 16.0
341 16.0
342 16.0
343 16.0
344 16.0
345 16.0
346 16.0
347 16.0
348 16.0
349 16.0
350 16.0
351 16.0
352 16.0
353 16.0
354 16.0
355 16.0
356 16.0
357 16.0
358 16.0
359 16.0
360 16.0
361 16.0
362 16.0
363 16.0
364 16.0
365 16.0
366 16.0
367 16.0
368 16.0
369 16.0
370 16.0
371 16.0
372 16.0
373 16.0
374 16.0
375 16.0
376 16.0
377 16.0
378 16.0
379 16.0
380 16.0
381 16.0
382 16.0
383 16.0
384 16.0
385 16.0
386 16.0
387 16.0
388 16.0
389 16.0
390 16.0
391 16.0
392 16.0
393 16.0
394 16.0
395 8.0
396 8.0
397 16.0
398 16.0
399 16.0
400 8.0
401 8.0
402 32.0
403 32.0
404 32.0
405 32.0
406 32.0
407 32.0
408 32.0
409 32.0
410 64.0
411 64.0
412 64.0
413 32.0
414 32.0
415 32.0
416 32.0
417 32.0
418 32.0
419 32.0
420 32.0
421 32.0
422 32.0
423 32.0
424 32.0
425 32.0
426 32.0
427 32.0
428 32.0
429 32.0
430 32.0
431 32.0
432 32.0
433 32.0
434 32.0
435 32.0
436 32.0
437 32.0
438 32.0
439 32.0
440 32.0
441 32.0
442 32.0
443 32.0
444 32.0
445 32.0
446 32.0
447 32.0
448 32.0
449 32.0
450 32.0
451 32.0
452 32.0
453 32.0
454 32.0
455 32.0
456 32.0
457 32.0
458 32.0
459 32.0
460 32.0
461 32.0
462 32.0
463 32.0
464 32.0
465 32.0
466 32.0
467 32.0
468 32.0
469 32.0
470 32.0
471 32.0
472 32.0
473 32.0
474 32.0
475 32.0
476 32.0
477 32.0
478 32.0
479 32.0
480 32.0
481 32.0
482 32.0
483 32.0
484 32.0
485 32.0
486 32.0
487 32.0
488 32.0
489 32.0
490 32.0
491 32.0
492 32.0
493 32.0
494 32.0
495 32.0
496 32.0
497 32.0
498 32.0
499 32.0
500 32.0
501 32.0
502 32.0
503 32.0
504 32.0
505 32.0
506 32.0
507 32.0
508 32.0
509 16.0
510 16.0
511 16.0
512 16.0
513 16.0
514 16.0
515 16.0
516 16.0
517 16.0
518 16.0
519 16.0
520 16.0
521 16.0
522 16.0
523 16.0
524 16.0
525 32.0
526 32.0
527 32.0
528 32.0
529 32.0
530 32.0
531 32.0
532 32.0
533 16.0
534 16.0
535 16.0
536 16.0
537 16.0
538 16.0
539 16.0
540 16.0
541 32.0
542 32.0
543 32.0
544 32.0
545 32.0
546 32.0
547 32.0
548 32.0
549 32.0
550 32.0
551 32.0
552 32.0
553 32.0
554 32.0
555 32.0
556 32.0
557 32.0
558 32.0
559 32.0
560 32.0
561 32.0
562 32.0
563 32.0
564 32.0
565 32.0
566 32.0
567 32.0
568 32.0
569 32.0
570 32.0
571 32.0
572 32.0
573 32.0
574 32.0
575 32.0
576 32.0
577 32.0
578 32.0
579 32.0
580 32.0
581 32.0
582 32.0
583 32.0
584 32.0
585 32.0
586 32.0
587 32.0
588 32.0
589 32.0
590 32.0
591 32.0
592 32.0
593 32.0
594 32.0
595 32.0
596 32.0
597 32.0
598 32.0
599 32.0
600 32.0
601 32.0
602 32.0
603 32.0
604 32.0
605 32.0
606 32.0
607 32.0
608 32.0
609 32.0
610 32.0
611 32.0
612 32.0
613 32.0
614 32.0
615 32.0
616 32.0
617 32.0
618 32.0
619 32.0
620 32.0
621 32.0
622 32.0
623 32.0
624 32.0
625 32.0
626 32.0
627 32.0
628 32.0
629 32.0
630 32.0
631 32.0
632 32.0
633 32.0
634 32.0
635 32.0
636 32.0
637 32.0
638 32.0
639 32.0
640 32.0
641 32.0
642 32.0
643 32.0
644 32.0
645 16.0
646 16.0
647 16.0
648 16.0
649 16.0
650 16.0
651 16.0
652 16.0
653 16.0
654 16.0
655 16.0
656 16.0;
param: NET := 
0 0.32
1 0.32
2 0.32
3 0.32
4 0.32
5 131.2081380016079
6 131.2081380016079
7 131.2081380016079
8 131.2081380016079
9 59.2603722385641
10 59.2603722385641
11 59.2603722385641
12 30.861643259837518
13 30.861643259837518
14 12.188497675806001
15 12.188497675806001
16 1.0
17 1.0
18 1.0
19 1.0
20 1.0
21 104.03086616343701
22 104.03086616343701
23 104.03086616343701
24 104.03086616343701
25 104.03086616343701
26 293.159713733322
27 293.159713733322
28 293.159713733322
29 1385.368808449405
30 1385.368808449405
31 1385.368808449405
32 1385.368808449405
33 1385.368808449405
34 1385.368808449405
35 1385.368808449405
36 1385.368808449405
37 1385.368808449405
38 1385.368808449405
39 1385.368808449405
40 1385.368808449405
41 1385.368808449405
42 1385.368808449405
43 1385.368808449405
44 1385.368808449405
45 888.905583578461
46 888.905583578461
47 888.905583578461
48 888.905583578461
49 888.905583578461
50 888.905583578461
51 888.905583578461
52 888.905583578461
53 888.905583578461
54 888.905583578461
55 888.905583578461
56 888.905583578461
57 888.905583578461
58 680.863851251586
59 680.863851251586
60 0.02
61 0.02
62 425.63990703224204
63 425.63990703224204
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0
143 0.0
144 0.0
145 0.0
146 0.0
147 0.0
148 0.0
149 0.0
150 0.0
151 0.0
152 0.0
153 0.0
154 0.0
155 0.0
156 0.0
157 0.0
158 0.0
159 0.0
160 0.0
161 0.0
162 0.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 0.0
179 0.0
180 0.0
181 0.0
182 0.0
183 0.0
184 0.0
185 0.0
186 0.0
187 0.0
188 0.0
189 0.0
190 0.0
191 0.0
192 0.0
193 0.0
194 0.0
195 0.0
196 0.0
197 0.0
198 0.0
199 0.0
200 0.0
201 0.0
202 0.0
203 0.0
204 0.0
205 0.0
206 0.0
207 0.0
208 0.0
209 0.0
210 0.0
211 0.0
212 0.0
213 0.0
214 0.0
215 0.0
216 0.0
217 0.0
218 0.0
219 0.0
220 0.0
221 0.0
222 0.0
223 0.0
224 0.0
225 0.0
226 0.0
227 0.0
228 0.0
229 0.0
230 0.0
231 0.0
232 0.0
233 0.0
234 0.0
235 0.0
236 0.0
237 0.0
238 0.0
239 0.0
240 0.0
241 0.0
242 0.0
243 0.0
244 0.0
245 0.0
246 0.0
247 0.0
248 0.0
249 0.0
250 0.0
251 0.0
252 0.0
253 0.0
254 0.0
255 0.0
256 0.0
257 0.0
258 0.0
259 0.0
260 0.0
261 0.0
262 0.0
263 0.0
264 0.0
265 0.0
266 0.0
267 0.0
268 0.0
269 0.0
270 0.0
271 0.0
272 0.0
273 0.0
274 0.0
275 0.0
276 0.0
277 0.0
278 0.0
279 0.0
280 0.0
281 0.0
282 0.0
283 0.0
284 0.0
285 0.0
286 0.0
287 0.0
288 0.0
289 0.0
290 0.0
291 0.0
292 0.0
293 0.0
294 0.0
295 0.0
296 0.0
297 0.0
298 0.0
299 0.0
300 0.0
301 0.0
302 0.0
303 0.0
304 0.0
305 0.0
306 0.0
307 0.0
308 0.0
309 0.0
310 0.0
311 0.0
312 0.0
313 0.0
314 0.0
315 0.0
316 0.0
317 0.0
318 0.0
319 0.0
320 0.0
321 0.0
322 0.0
323 0.0
324 0.0
325 0.0
326 0.0
327 0.0
328 0.0
329 0.0
330 0.0
331 0.0
332 0.0
333 0.0
334 0.0
335 0.0
336 0.0
337 0.0
338 0.0
339 0.0
340 0.0
341 0.0
342 0.0
343 0.0
344 0.0
345 0.0
346 0.0
347 0.0
348 0.0
349 0.0
350 0.0
351 0.0
352 0.0
353 0.0
354 0.0
355 0.0
356 0.0
357 0.0
358 0.0
359 0.0
360 0.0
361 0.0
362 0.0
363 0.0
364 0.0
365 0.0
366 0.0
367 0.0
368 0.0
369 0.0
370 0.0
371 0.0
372 0.0
373 0.0
374 0.0
375 0.0
376 0.0
377 0.0
378 0.0
379 0.0
380 0.0
381 0.0
382 0.0
383 0.0
384 0.0
385 0.0
386 0.0
387 0.0
388 0.0
389 0.0
390 0.0
391 0.0
392 0.0
393 0.0
394 0.0
395 0.0
396 0.0
397 0.0
398 0.0
399 0.0
400 0.0
401 0.0
402 0.0
403 0.0
404 0.0
405 0.0
406 0.0
407 0.0
408 0.0
409 0.0
410 0.0
411 0.0
412 0.0
413 0.0
414 0.0
415 0.0
416 0.0
417 0.0
418 0.0
419 0.0
420 0.0
421 0.0
422 0.0
423 0.0
424 0.0
425 0.0
426 0.0
427 0.0
428 0.0
429 0.0
430 0.0
431 0.0
432 0.0
433 0.0
434 0.0
435 0.0
436 0.0
437 0.0
438 0.0
439 0.0
440 0.0
441 0.0
442 0.0
443 0.0
444 0.0
445 0.0
446 0.0
447 0.0
448 0.0
449 0.0
450 0.0
451 0.0
452 0.0
453 0.0
454 0.0
455 0.0
456 0.0
457 0.0
458 0.0
459 0.0
460 0.0
461 0.0
462 0.0
463 0.0
464 0.0
465 0.0
466 0.0
467 0.0
468 0.0
469 0.0
470 0.0
471 0.0
472 0.0
473 0.0
474 0.0
475 0.0
476 0.0
477 0.0
478 0.0
479 0.0
480 0.0
481 0.0
482 0.0
483 0.0
484 0.0
485 0.0
486 0.0
487 0.0
488 0.0
489 0.0
490 0.0
491 0.0
492 0.0
493 0.0
494 0.0
495 0.0
496 0.0
497 0.0
498 0.0
499 0.0
500 0.0
501 0.0
502 0.0
503 0.0
504 0.0
505 0.0
506 0.0
507 0.0
508 0.0
509 0.0
510 0.0
511 0.0
512 0.0
513 0.0
514 0.0
515 0.0
516 0.0
517 0.0
518 0.0
519 0.0
520 0.0
521 0.0
522 0.0
523 0.0
524 0.0
525 0.0
526 0.0
527 0.0
528 0.0
529 0.0
530 0.0
531 0.0
532 0.0
533 0.0
534 0.0
535 0.0
536 0.0
537 0.0
538 0.0
539 0.0
540 0.0
541 0.0
542 0.0
543 0.0
544 0.0
545 0.0
546 0.0
547 0.0
548 0.0
549 0.0
550 0.0
551 0.0
552 0.0
553 0.0
554 0.0
555 0.0
556 0.0
557 0.0
558 0.0
559 0.0
560 0.0
561 0.0
562 0.0
563 0.0
564 0.0
565 0.0
566 0.0
567 0.0
568 0.0
569 0.0
570 0.0
571 0.0
572 0.0
573 0.0
574 0.0
575 0.0
576 0.0
577 0.0
578 0.0
579 0.0
580 0.0
581 0.0
582 0.0
583 0.0
584 0.0
585 0.0
586 0.0
587 0.0
588 0.0
589 0.0
590 0.0
591 0.0
592 0.0
593 0.0
594 0.0
595 0.0
596 0.0
597 0.0
598 0.0
599 0.0
600 0.0
601 0.0
602 0.0
603 0.0
604 0.0
605 0.0
606 0.0
607 0.0
608 0.0
609 0.0
610 0.0
611 0.0
612 0.0
613 0.0
614 0.0
615 0.0
616 0.0
617 0.0
618 0.0
619 0.0
620 0.0
621 0.0
622 0.0
623 0.0
624 0.0
625 0.0
626 0.0
627 0.0
628 0.0
629 0.0
630 0.0
631 0.0
632 0.0
633 0.0
634 0.0
635 0.0
636 0.0
637 0.0
638 0.0
639 0.0
640 0.0
641 0.0
642 0.0
643 0.0
644 0.0
645 0.0
646 0.0
647 0.0
648 0.0
649 0.0
650 0.0
651 0.0
652 0.0
653 0.0
654 0.0
655 0.0
656 0.0;


######################
# SOLVE 
######################
end;
