

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..779;
# Set of Hosts
set Hs := 0..153;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..4}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {5..8}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {9..11}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm4_0_{h in Hs}: sum{v in {12,13}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm5_0_{h in Hs}: sum{v in {14,15}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm6_0_{h in Hs}: sum{v in {16..20}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm7_0_{h in Hs}: sum{v in {21..26}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm8_0_{h in Hs}: sum{v in {27..30}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm9_0_{h in Hs}: sum{v in {31..48}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm10_0_{h in Hs}: sum{v in {49..64}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm11_0_{h in Hs}: sum{v in {65,66}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm12_0_{h in Hs}: sum{v in {67,68}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm13_0_{h in Hs}: sum{v in {69,70}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm14_0_{h in Hs}: sum{v in {71}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_0_{h in Hs}: sum{v in {360..362}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_1_{h in Hs}: sum{v in {363..365}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_2_{h in Hs}: sum{v in {366..368}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_3_{h in Hs}: sum{v in {369..371}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_4_{h in Hs}: sum{v in {372..374}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm17_5_{h in Hs}: sum{v in {375..377}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_0_{h in Hs}: sum{v in {378..380}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_1_{h in Hs}: sum{v in {381..383}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_2_{h in Hs}: sum{v in {384..386}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_3_{h in Hs}: sum{v in {387..389}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_4_{h in Hs}: sum{v in {390..392}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm18_5_{h in Hs}: sum{v in {393..395}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_0_{h in Hs}: sum{v in {396..398}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_1_{h in Hs}: sum{v in {399..401}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_2_{h in Hs}: sum{v in {402..404}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_3_{h in Hs}: sum{v in {405..407}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_4_{h in Hs}: sum{v in {408..410}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm19_5_{h in Hs}: sum{v in {411..413}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_0_{h in Hs}: sum{v in {414..416}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_1_{h in Hs}: sum{v in {417..419}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_2_{h in Hs}: sum{v in {420..422}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_3_{h in Hs}: sum{v in {423..425}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_4_{h in Hs}: sum{v in {426..428}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm20_5_{h in Hs}: sum{v in {429..431}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_0_{h in Hs}: sum{v in {432..434}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_1_{h in Hs}: sum{v in {435..437}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_2_{h in Hs}: sum{v in {438..440}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_3_{h in Hs}: sum{v in {441..443}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_4_{h in Hs}: sum{v in {444..446}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm21_5_{h in Hs}: sum{v in {447..449}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_0_{h in Hs}: sum{v in {450..457}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_1_{h in Hs}: sum{v in {458..465}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_2_{h in Hs}: sum{v in {466..473}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_3_{h in Hs}: sum{v in {474..481}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_4_{h in Hs}: sum{v in {482..489}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm22_5_{h in Hs}: sum{v in {490..497}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm23_0_{h in Hs}: sum{v in {498,499}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm24_0_{h in Hs}: sum{v in {500..502}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_0_{h in Hs}: sum{v in {505..507}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm27_0_{h in Hs}: sum{v in {508..512}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_0_{h in Hs}: sum{v in {516..539}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_1_{h in Hs}: sum{v in {540..563}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_2_{h in Hs}: sum{v in {564..587}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm29_3_{h in Hs}: sum{v in {588..611}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_0_{h in Hs}: sum{v in {612,613}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_1_{h in Hs}: sum{v in {614,615}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_2_{h in Hs}: sum{v in {616,617}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm30_3_{h in Hs}: sum{v in {618,619}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_0_{h in Hs}: sum{v in {620,621}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_1_{h in Hs}: sum{v in {622,623}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_2_{h in Hs}: sum{v in {624,625}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm31_3_{h in Hs}: sum{v in {626,627}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_0_{h in Hs}: sum{v in {628,629}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_1_{h in Hs}: sum{v in {630,631}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_2_{h in Hs}: sum{v in {632,633}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm32_3_{h in Hs}: sum{v in {634,635}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_0_{h in Hs}: sum{v in {636,637}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_1_{h in Hs}: sum{v in {638,639}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_2_{h in Hs}: sum{v in {640,641}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm31_3_{h in Hs}: sum{v in {642,643}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_0_{h in Hs}: sum{v in {644,645}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_1_{h in Hs}: sum{v in {646,647}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_2_{h in Hs}: sum{v in {648,649}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm32_3_{h in Hs}: sum{v in {650,651}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_0_{h in Hs}: sum{v in {652..678}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_1_{h in Hs}: sum{v in {679..705}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_2_{h in Hs}: sum{v in {706..732}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm33_3_{h in Hs}: sum{v in {733..759}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_0_{h in Hs}: sum{v in {760,761}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_1_{h in Hs}: sum{v in {762,763}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_2_{h in Hs}: sum{v in {764,765}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm34_3_{h in Hs}: sum{v in {766,767}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm35_0_{h in Hs}: sum{v in {768..775}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm36_0_{h in Hs}: sum{v in {776,777}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm36_1_{h in Hs}: sum{v in {778,779}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9.2
1 9.2
2 9.2
3 9.2
4 9.2
5 12.0
6 12.0
7 12.0
8 12.0
9 13.8
10 13.8
11 13.8
12 9.2
13 9.2
14 9.2
15 9.2
16 9.2
17 9.2
18 9.2
19 9.2
20 9.2
21 4.6
22 4.6
23 4.6
24 4.6
25 4.6
26 4.6
27 13.8
28 13.8
29 13.8
30 13.8
31 21.3333333333333
32 21.3333333333333
33 21.3333333333333
34 21.3333333333333
35 21.3333333333333
36 21.3333333333333
37 21.3333333333333
38 21.3333333333333
39 21.3333333333333
40 21.3333333333333
41 21.3333333333333
42 21.3333333333333
43 21.3333333333333
44 21.3333333333333
45 21.3333333333333
46 21.3333333333333
47 21.3333333333333
48 21.3333333333333
49 24.15
50 24.15
51 24.15
52 24.15
53 24.15
54 24.15
55 24.15
56 24.15
57 24.15
58 24.15
59 24.15
60 24.15
61 24.15
62 24.15
63 24.15
64 24.15
65 36.8
66 36.8
67 13.8
68 13.8
69 36.8
70 36.8
71 40.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0
83 8.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 8.0
93 8.0
94 8.0
95 8.0
96 8.0
97 8.0
98 8.0
99 8.0
100 8.0
101 8.0
102 8.0
103 8.0
104 8.0
105 8.0
106 8.0
107 8.0
108 8.0
109 8.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 8.0
117 8.0
118 8.0
119 8.0
120 8.0
121 8.0
122 8.0
123 8.0
124 8.0
125 8.0
126 8.0
127 8.0
128 8.0
129 8.0
130 8.0
131 8.0
132 8.0
133 8.0
134 8.0
135 8.0
136 8.0
137 8.0
138 8.0
139 8.0
140 8.0
141 8.0
142 8.0
143 8.0
144 8.0
145 8.0
146 8.0
147 8.0
148 8.0
149 8.0
150 8.0
151 8.0
152 8.0
153 8.0
154 8.0
155 8.0
156 8.0
157 8.0
158 8.0
159 8.0
160 8.0
161 8.0
162 8.0
163 8.0
164 8.0
165 8.0
166 8.0
167 8.0
168 8.0
169 8.0
170 8.0
171 8.0
172 8.0
173 8.0
174 8.0
175 8.0
176 8.0
177 8.0
178 8.0
179 8.0
180 8.0
181 8.0
182 8.0
183 8.0
184 8.0
185 8.0
186 8.0
187 8.0
188 8.0
189 8.0
190 8.0
191 8.0
192 8.0
193 8.0
194 8.0
195 8.0
196 8.0
197 8.0
198 8.0
199 8.0
200 8.0
201 8.0
202 8.0
203 8.0
204 8.0
205 8.0
206 8.0
207 8.0
208 8.0
209 8.0
210 8.0
211 8.0
212 8.0
213 8.0
214 8.0
215 8.0
216 8.0
217 8.0
218 8.0
219 8.0
220 8.0
221 8.0
222 8.0
223 8.0
224 8.0
225 8.0
226 8.0
227 8.0
228 8.0
229 8.0
230 8.0
231 8.0
232 8.0
233 8.0
234 8.0
235 8.0
236 8.0
237 8.0
238 8.0
239 8.0
240 8.0
241 8.0
242 8.0
243 8.0
244 8.0
245 8.0
246 8.0
247 8.0
248 8.0
249 8.0
250 8.0
251 8.0
252 8.0
253 8.0
254 8.0
255 8.0
256 8.0
257 8.0
258 8.0
259 8.0
260 8.0
261 8.0
262 8.0
263 8.0
264 8.0
265 8.0
266 8.0
267 8.0
268 8.0
269 8.0
270 8.0
271 8.0
272 8.0
273 8.0
274 8.0
275 8.0
276 8.0
277 8.0
278 8.0
279 8.0
280 8.0
281 8.0
282 8.0
283 8.0
284 8.0
285 8.0
286 8.0
287 8.0
288 8.0
289 8.0
290 8.0
291 8.0
292 8.0
293 8.0
294 8.0
295 8.0
296 8.0
297 8.0
298 8.0
299 8.0
300 8.0
301 8.0
302 8.0
303 8.0
304 8.0
305 8.0
306 8.0
307 8.0
308 8.0
309 8.0
310 8.0
311 8.0
312 8.0
313 8.0
314 8.0
315 8.0
316 8.0
317 8.0
318 8.0
319 8.0
320 8.0
321 8.0
322 8.0
323 8.0
324 8.0
325 8.0
326 8.0
327 8.0
328 8.0
329 8.0
330 8.0
331 8.0
332 8.0
333 8.0
334 8.0
335 8.0
336 8.0
337 8.0
338 8.0
339 8.0
340 8.0
341 8.0
342 8.0
343 8.0
344 8.0
345 8.0
346 8.0
347 8.0
348 8.0
349 8.0
350 8.0
351 8.0
352 8.0
353 8.0
354 8.0
355 8.0
356 8.0
357 8.0
358 8.0
359 8.0
360 4.0
361 4.0
362 4.0
363 4.0
364 4.0
365 4.0
366 4.0
367 4.0
368 4.0
369 4.0
370 4.0
371 4.0
372 4.0
373 4.0
374 4.0
375 4.0
376 4.0
377 4.0
378 4.0
379 4.0
380 4.0
381 4.0
382 4.0
383 4.0
384 4.0
385 4.0
386 4.0
387 4.0
388 4.0
389 4.0
390 4.0
391 4.0
392 4.0
393 4.0
394 4.0
395 4.0
396 4.0
397 4.0
398 4.0
399 4.0
400 4.0
401 4.0
402 4.0
403 4.0
404 4.0
405 4.0
406 4.0
407 4.0
408 4.0
409 4.0
410 4.0
411 4.0
412 4.0
413 4.0
414 4.0
415 4.0
416 4.0
417 4.0
418 4.0
419 4.0
420 4.0
421 4.0
422 4.0
423 4.0
424 4.0
425 4.0
426 4.0
427 4.0
428 4.0
429 4.0
430 4.0
431 4.0
432 4.0
433 4.0
434 4.0
435 4.0
436 4.0
437 4.0
438 4.0
439 4.0
440 4.0
441 4.0
442 4.0
443 4.0
444 4.0
445 4.0
446 4.0
447 4.0
448 4.0
449 4.0
450 8.0
451 8.0
452 8.0
453 8.0
454 8.0
455 8.0
456 8.0
457 8.0
458 8.0
459 8.0
460 8.0
461 8.0
462 8.0
463 8.0
464 8.0
465 8.0
466 8.0
467 8.0
468 8.0
469 8.0
470 8.0
471 8.0
472 8.0
473 8.0
474 8.0
475 8.0
476 8.0
477 8.0
478 8.0
479 8.0
480 8.0
481 8.0
482 8.0
483 8.0
484 8.0
485 8.0
486 8.0
487 8.0
488 8.0
489 8.0
490 8.0
491 8.0
492 8.0
493 8.0
494 8.0
495 8.0
496 8.0
497 8.0
498 4.0
499 4.0
500 4.0
501 4.0
502 4.0
503 4.0
504 4.0
505 8.0
506 8.0
507 8.0
508 8.0
509 8.0
510 8.0
511 8.0
512 8.0
513 8.0
514 8.0
515 8.0
516 8.0
517 8.0
518 8.0
519 8.0
520 8.0
521 8.0
522 8.0
523 8.0
524 8.0
525 8.0
526 8.0
527 8.0
528 8.0
529 8.0
530 8.0
531 8.0
532 8.0
533 8.0
534 8.0
535 8.0
536 8.0
537 8.0
538 8.0
539 8.0
540 8.0
541 8.0
542 8.0
543 8.0
544 8.0
545 8.0
546 8.0
547 8.0
548 8.0
549 8.0
550 8.0
551 8.0
552 8.0
553 8.0
554 8.0
555 8.0
556 8.0
557 8.0
558 8.0
559 8.0
560 8.0
561 8.0
562 8.0
563 8.0
564 8.0
565 8.0
566 8.0
567 8.0
568 8.0
569 8.0
570 8.0
571 8.0
572 8.0
573 8.0
574 8.0
575 8.0
576 8.0
577 8.0
578 8.0
579 8.0
580 8.0
581 8.0
582 8.0
583 8.0
584 8.0
585 8.0
586 8.0
587 8.0
588 8.0
589 8.0
590 8.0
591 8.0
592 8.0
593 8.0
594 8.0
595 8.0
596 8.0
597 8.0
598 8.0
599 8.0
600 8.0
601 8.0
602 8.0
603 8.0
604 8.0
605 8.0
606 8.0
607 8.0
608 8.0
609 8.0
610 8.0
611 8.0
612 16.0
613 16.0
614 16.0
615 16.0
616 16.0
617 16.0
618 16.0
619 16.0
620 8.0
621 8.0
622 8.0
623 8.0
624 8.0
625 8.0
626 8.0
627 8.0
628 4.0
629 4.0
630 4.0
631 4.0
632 4.0
633 4.0
634 4.0
635 4.0
636 8.0
637 8.0
638 8.0
639 8.0
640 8.0
641 8.0
642 8.0
643 8.0
644 4.0
645 4.0
646 4.0
647 4.0
648 4.0
649 4.0
650 4.0
651 4.0
652 8.0
653 8.0
654 8.0
655 8.0
656 8.0
657 8.0
658 8.0
659 8.0
660 8.0
661 8.0
662 8.0
663 8.0
664 8.0
665 8.0
666 8.0
667 8.0
668 8.0
669 8.0
670 8.0
671 8.0
672 8.0
673 8.0
674 8.0
675 8.0
676 8.0
677 8.0
678 8.0
679 8.0
680 8.0
681 8.0
682 8.0
683 8.0
684 8.0
685 8.0
686 8.0
687 8.0
688 8.0
689 8.0
690 8.0
691 8.0
692 8.0
693 8.0
694 8.0
695 8.0
696 8.0
697 8.0
698 8.0
699 8.0
700 8.0
701 8.0
702 8.0
703 8.0
704 8.0
705 8.0
706 8.0
707 8.0
708 8.0
709 8.0
710 8.0
711 8.0
712 8.0
713 8.0
714 8.0
715 8.0
716 8.0
717 8.0
718 8.0
719 8.0
720 8.0
721 8.0
722 8.0
723 8.0
724 8.0
725 8.0
726 8.0
727 8.0
728 8.0
729 8.0
730 8.0
731 8.0
732 8.0
733 8.0
734 8.0
735 8.0
736 8.0
737 8.0
738 8.0
739 8.0
740 8.0
741 8.0
742 8.0
743 8.0
744 8.0
745 8.0
746 8.0
747 8.0
748 8.0
749 8.0
750 8.0
751 8.0
752 8.0
753 8.0
754 8.0
755 8.0
756 8.0
757 8.0
758 8.0
759 8.0
760 8.0
761 8.0
762 8.0
763 8.0
764 8.0
765 8.0
766 8.0
767 8.0
768 8.0
769 8.0
770 8.0
771 8.0
772 8.0
773 8.0
774 8.0
775 8.0
776 8.0
777 8.0
778 8.0
779 8.0;
param: MEM := 
0 13.8
1 13.8
2 13.8
3 13.8
4 13.8
5 64.0
6 64.0
7 64.0
8 64.0
9 27.6
10 27.6
11 27.6
12 27.6
13 27.6
14 55.2
15 55.2
16 13.8
17 13.8
18 13.8
19 13.8
20 13.8
21 13.8
22 13.8
23 13.8
24 13.8
25 13.8
26 13.8
27 36.8
28 36.8
29 36.8
30 36.8
31 113.777777777778
32 113.777777777778
33 113.777777777778
34 113.777777777778
35 113.777777777778
36 113.777777777778
37 113.777777777778
38 113.777777777778
39 113.777777777778
40 113.777777777778
41 113.777777777778
42 113.777777777778
43 113.777777777778
44 113.777777777778
45 113.777777777778
46 113.777777777778
47 113.777777777778
48 113.777777777778
49 24.15
50 24.15
51 24.15
52 24.15
53 24.15
54 24.15
55 24.15
56 24.15
57 24.15
58 24.15
59 24.15
60 24.15
61 24.15
62 24.15
63 24.15
64 24.15
65 27.6
66 27.6
67 27.6
68 27.6
69 55.2
70 55.2
71 80.0
72 32.0
73 32.0
74 32.0
75 32.0
76 32.0
77 32.0
78 32.0
79 32.0
80 32.0
81 32.0
82 32.0
83 32.0
84 32.0
85 32.0
86 32.0
87 32.0
88 32.0
89 32.0
90 32.0
91 32.0
92 32.0
93 32.0
94 32.0
95 32.0
96 32.0
97 32.0
98 32.0
99 32.0
100 32.0
101 32.0
102 32.0
103 32.0
104 32.0
105 32.0
106 32.0
107 32.0
108 32.0
109 32.0
110 32.0
111 32.0
112 32.0
113 32.0
114 32.0
115 32.0
116 32.0
117 32.0
118 32.0
119 32.0
120 32.0
121 32.0
122 32.0
123 32.0
124 32.0
125 32.0
126 32.0
127 32.0
128 32.0
129 32.0
130 32.0
131 32.0
132 32.0
133 32.0
134 32.0
135 32.0
136 32.0
137 32.0
138 32.0
139 32.0
140 32.0
141 32.0
142 32.0
143 32.0
144 32.0
145 32.0
146 32.0
147 32.0
148 32.0
149 32.0
150 32.0
151 32.0
152 32.0
153 32.0
154 32.0
155 32.0
156 32.0
157 32.0
158 32.0
159 32.0
160 32.0
161 32.0
162 32.0
163 32.0
164 32.0
165 32.0
166 32.0
167 32.0
168 32.0
169 32.0
170 32.0
171 32.0
172 32.0
173 32.0
174 32.0
175 32.0
176 32.0
177 32.0
178 32.0
179 32.0
180 32.0
181 32.0
182 32.0
183 32.0
184 32.0
185 32.0
186 32.0
187 32.0
188 32.0
189 32.0
190 32.0
191 32.0
192 32.0
193 32.0
194 32.0
195 32.0
196 32.0
197 32.0
198 32.0
199 32.0
200 32.0
201 32.0
202 32.0
203 32.0
204 32.0
205 32.0
206 32.0
207 32.0
208 32.0
209 32.0
210 32.0
211 32.0
212 32.0
213 32.0
214 32.0
215 32.0
216 32.0
217 32.0
218 32.0
219 32.0
220 32.0
221 32.0
222 32.0
223 32.0
224 32.0
225 32.0
226 32.0
227 32.0
228 32.0
229 32.0
230 32.0
231 32.0
232 32.0
233 32.0
234 32.0
235 32.0
236 32.0
237 32.0
238 32.0
239 32.0
240 32.0
241 32.0
242 32.0
243 32.0
244 32.0
245 32.0
246 32.0
247 32.0
248 32.0
249 32.0
250 32.0
251 32.0
252 32.0
253 32.0
254 32.0
255 32.0
256 32.0
257 32.0
258 32.0
259 32.0
260 32.0
261 32.0
262 32.0
263 32.0
264 32.0
265 32.0
266 32.0
267 32.0
268 32.0
269 32.0
270 32.0
271 32.0
272 32.0
273 32.0
274 32.0
275 32.0
276 32.0
277 32.0
278 32.0
279 32.0
280 32.0
281 32.0
282 32.0
283 32.0
284 32.0
285 32.0
286 32.0
287 32.0
288 32.0
289 32.0
290 32.0
291 32.0
292 32.0
293 32.0
294 32.0
295 32.0
296 32.0
297 32.0
298 32.0
299 32.0
300 32.0
301 32.0
302 32.0
303 32.0
304 32.0
305 32.0
306 32.0
307 32.0
308 32.0
309 32.0
310 32.0
311 32.0
312 32.0
313 32.0
314 32.0
315 32.0
316 32.0
317 32.0
318 32.0
319 32.0
320 32.0
321 32.0
322 32.0
323 32.0
324 32.0
325 32.0
326 32.0
327 32.0
328 32.0
329 32.0
330 32.0
331 32.0
332 32.0
333 32.0
334 32.0
335 32.0
336 32.0
337 32.0
338 32.0
339 32.0
340 32.0
341 32.0
342 32.0
343 32.0
344 32.0
345 32.0
346 32.0
347 32.0
348 32.0
349 32.0
350 32.0
351 32.0
352 32.0
353 32.0
354 32.0
355 32.0
356 32.0
357 32.0
358 32.0
359 32.0
360 16.0
361 16.0
362 16.0
363 16.0
364 16.0
365 16.0
366 16.0
367 16.0
368 16.0
369 16.0
370 16.0
371 16.0
372 16.0
373 16.0
374 16.0
375 16.0
376 16.0
377 16.0
378 13.0
379 13.0
380 13.0
381 13.0
382 13.0
383 13.0
384 13.0
385 13.0
386 13.0
387 13.0
388 13.0
389 13.0
390 13.0
391 13.0
392 13.0
393 13.0
394 13.0
395 13.0
396 16.0
397 16.0
398 16.0
399 16.0
400 16.0
401 16.0
402 16.0
403 16.0
404 16.0
405 16.0
406 16.0
407 16.0
408 16.0
409 16.0
410 16.0
411 16.0
412 16.0
413 16.0
414 16.0
415 16.0
416 16.0
417 16.0
418 16.0
419 16.0
420 16.0
421 16.0
422 16.0
423 16.0
424 16.0
425 16.0
426 16.0
427 16.0
428 16.0
429 16.0
430 16.0
431 16.0
432 16.0
433 16.0
434 16.0
435 16.0
436 16.0
437 16.0
438 16.0
439 16.0
440 16.0
441 16.0
442 16.0
443 16.0
444 16.0
445 16.0
446 16.0
447 16.0
448 16.0
449 16.0
450 16.0
451 16.0
452 16.0
453 16.0
454 16.0
455 16.0
456 16.0
457 16.0
458 16.0
459 16.0
460 16.0
461 16.0
462 16.0
463 16.0
464 16.0
465 16.0
466 16.0
467 16.0
468 16.0
469 16.0
470 16.0
471 16.0
472 16.0
473 16.0
474 16.0
475 16.0
476 16.0
477 16.0
478 16.0
479 16.0
480 16.0
481 16.0
482 16.0
483 16.0
484 16.0
485 16.0
486 16.0
487 16.0
488 16.0
489 16.0
490 16.0
491 16.0
492 16.0
493 16.0
494 16.0
495 16.0
496 16.0
497 16.0
498 8.0
499 8.0
500 16.0
501 16.0
502 16.0
503 8.0
504 8.0
505 32.0
506 32.0
507 32.0
508 32.0
509 32.0
510 32.0
511 32.0
512 32.0
513 64.0
514 64.0
515 64.0
516 32.0
517 32.0
518 32.0
519 32.0
520 32.0
521 32.0
522 32.0
523 32.0
524 32.0
525 32.0
526 32.0
527 32.0
528 32.0
529 32.0
530 32.0
531 32.0
532 32.0
533 32.0
534 32.0
535 32.0
536 32.0
537 32.0
538 32.0
539 32.0
540 32.0
541 32.0
542 32.0
543 32.0
544 32.0
545 32.0
546 32.0
547 32.0
548 32.0
549 32.0
550 32.0
551 32.0
552 32.0
553 32.0
554 32.0
555 32.0
556 32.0
557 32.0
558 32.0
559 32.0
560 32.0
561 32.0
562 32.0
563 32.0
564 32.0
565 32.0
566 32.0
567 32.0
568 32.0
569 32.0
570 32.0
571 32.0
572 32.0
573 32.0
574 32.0
575 32.0
576 32.0
577 32.0
578 32.0
579 32.0
580 32.0
581 32.0
582 32.0
583 32.0
584 32.0
585 32.0
586 32.0
587 32.0
588 32.0
589 32.0
590 32.0
591 32.0
592 32.0
593 32.0
594 32.0
595 32.0
596 32.0
597 32.0
598 32.0
599 32.0
600 32.0
601 32.0
602 32.0
603 32.0
604 32.0
605 32.0
606 32.0
607 32.0
608 32.0
609 32.0
610 32.0
611 32.0
612 16.0
613 16.0
614 16.0
615 16.0
616 16.0
617 16.0
618 16.0
619 16.0
620 16.0
621 16.0
622 16.0
623 16.0
624 16.0
625 16.0
626 16.0
627 16.0
628 32.0
629 32.0
630 32.0
631 32.0
632 32.0
633 32.0
634 32.0
635 32.0
636 16.0
637 16.0
638 16.0
639 16.0
640 16.0
641 16.0
642 16.0
643 16.0
644 32.0
645 32.0
646 32.0
647 32.0
648 32.0
649 32.0
650 32.0
651 32.0
652 32.0
653 32.0
654 32.0
655 32.0
656 32.0
657 32.0
658 32.0
659 32.0
660 32.0
661 32.0
662 32.0
663 32.0
664 32.0
665 32.0
666 32.0
667 32.0
668 32.0
669 32.0
670 32.0
671 32.0
672 32.0
673 32.0
674 32.0
675 32.0
676 32.0
677 32.0
678 32.0
679 32.0
680 32.0
681 32.0
682 32.0
683 32.0
684 32.0
685 32.0
686 32.0
687 32.0
688 32.0
689 32.0
690 32.0
691 32.0
692 32.0
693 32.0
694 32.0
695 32.0
696 32.0
697 32.0
698 32.0
699 32.0
700 32.0
701 32.0
702 32.0
703 32.0
704 32.0
705 32.0
706 32.0
707 32.0
708 32.0
709 32.0
710 32.0
711 32.0
712 32.0
713 32.0
714 32.0
715 32.0
716 32.0
717 32.0
718 32.0
719 32.0
720 32.0
721 32.0
722 32.0
723 32.0
724 32.0
725 32.0
726 32.0
727 32.0
728 32.0
729 32.0
730 32.0
731 32.0
732 32.0
733 32.0
734 32.0
735 32.0
736 32.0
737 32.0
738 32.0
739 32.0
740 32.0
741 32.0
742 32.0
743 32.0
744 32.0
745 32.0
746 32.0
747 32.0
748 32.0
749 32.0
750 32.0
751 32.0
752 32.0
753 32.0
754 32.0
755 32.0
756 32.0
757 32.0
758 32.0
759 32.0
760 16.0
761 16.0
762 16.0
763 16.0
764 16.0
765 16.0
766 16.0
767 16.0
768 16.0
769 16.0
770 16.0
771 16.0
772 16.0
773 16.0
774 16.0
775 16.0
776 16.0
777 16.0
778 16.0
779 16.0;
param: NET := 
0 0.32
1 0.32
2 0.32
3 0.32
4 0.32
5 152.9269018223415
6 152.9269018223415
7 152.9269018223415
8 152.9269018223415
9 69.0696877900305
10 69.0696877900305
11 69.0696877900305
12 35.96848772579381
13 35.96848772579381
14 13.9494785261358
15 13.9494785261358
16 1.0
17 1.0
18 1.0
19 1.0
20 1.0
21 121.249345588883
22 121.249345588883
23 121.249345588883
24 121.249345588883
25 121.249345588883
26 121.249345588883
27 341.68451938685297
28 341.68451938685297
29 341.68451938685297
30 341.68451938685297
31 1614.68764807013
32 1614.68764807013
33 1614.68764807013
34 1614.68764807013
35 1614.68764807013
36 1614.68764807013
37 1614.68764807013
38 1614.68764807013
39 1614.68764807013
40 1614.68764807013
41 1614.68764807013
42 1614.68764807013
43 1614.68764807013
44 1614.68764807013
45 1614.68764807013
46 1614.68764807013
47 1614.68764807013
48 1614.68764807013
49 1036.04531685046
50 1036.04531685046
51 1036.04531685046
52 1036.04531685046
53 1036.04531685046
54 1036.04531685046
55 1036.04531685046
56 1036.04531685046
57 1036.04531685046
58 1036.04531685046
59 1036.04531685046
60 1036.04531685046
61 1036.04531685046
62 1036.04531685046
63 1036.04531685046
64 1036.04531685046
65 793.56662567269
66 793.56662567269
67 0.02
68 0.02
69 496.07914104543204
70 496.07914104543204
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0
143 0.0
144 0.0
145 0.0
146 0.0
147 0.0
148 0.0
149 0.0
150 0.0
151 0.0
152 0.0
153 0.0
154 0.0
155 0.0
156 0.0
157 0.0
158 0.0
159 0.0
160 0.0
161 0.0
162 0.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 0.0
179 0.0
180 0.0
181 0.0
182 0.0
183 0.0
184 0.0
185 0.0
186 0.0
187 0.0
188 0.0
189 0.0
190 0.0
191 0.0
192 0.0
193 0.0
194 0.0
195 0.0
196 0.0
197 0.0
198 0.0
199 0.0
200 0.0
201 0.0
202 0.0
203 0.0
204 0.0
205 0.0
206 0.0
207 0.0
208 0.0
209 0.0
210 0.0
211 0.0
212 0.0
213 0.0
214 0.0
215 0.0
216 0.0
217 0.0
218 0.0
219 0.0
220 0.0
221 0.0
222 0.0
223 0.0
224 0.0
225 0.0
226 0.0
227 0.0
228 0.0
229 0.0
230 0.0
231 0.0
232 0.0
233 0.0
234 0.0
235 0.0
236 0.0
237 0.0
238 0.0
239 0.0
240 0.0
241 0.0
242 0.0
243 0.0
244 0.0
245 0.0
246 0.0
247 0.0
248 0.0
249 0.0
250 0.0
251 0.0
252 0.0
253 0.0
254 0.0
255 0.0
256 0.0
257 0.0
258 0.0
259 0.0
260 0.0
261 0.0
262 0.0
263 0.0
264 0.0
265 0.0
266 0.0
267 0.0
268 0.0
269 0.0
270 0.0
271 0.0
272 0.0
273 0.0
274 0.0
275 0.0
276 0.0
277 0.0
278 0.0
279 0.0
280 0.0
281 0.0
282 0.0
283 0.0
284 0.0
285 0.0
286 0.0
287 0.0
288 0.0
289 0.0
290 0.0
291 0.0
292 0.0
293 0.0
294 0.0
295 0.0
296 0.0
297 0.0
298 0.0
299 0.0
300 0.0
301 0.0
302 0.0
303 0.0
304 0.0
305 0.0
306 0.0
307 0.0
308 0.0
309 0.0
310 0.0
311 0.0
312 0.0
313 0.0
314 0.0
315 0.0
316 0.0
317 0.0
318 0.0
319 0.0
320 0.0
321 0.0
322 0.0
323 0.0
324 0.0
325 0.0
326 0.0
327 0.0
328 0.0
329 0.0
330 0.0
331 0.0
332 0.0
333 0.0
334 0.0
335 0.0
336 0.0
337 0.0
338 0.0
339 0.0
340 0.0
341 0.0
342 0.0
343 0.0
344 0.0
345 0.0
346 0.0
347 0.0
348 0.0
349 0.0
350 0.0
351 0.0
352 0.0
353 0.0
354 0.0
355 0.0
356 0.0
357 0.0
358 0.0
359 0.0
360 0.0
361 0.0
362 0.0
363 0.0
364 0.0
365 0.0
366 0.0
367 0.0
368 0.0
369 0.0
370 0.0
371 0.0
372 0.0
373 0.0
374 0.0
375 0.0
376 0.0
377 0.0
378 0.0
379 0.0
380 0.0
381 0.0
382 0.0
383 0.0
384 0.0
385 0.0
386 0.0
387 0.0
388 0.0
389 0.0
390 0.0
391 0.0
392 0.0
393 0.0
394 0.0
395 0.0
396 0.0
397 0.0
398 0.0
399 0.0
400 0.0
401 0.0
402 0.0
403 0.0
404 0.0
405 0.0
406 0.0
407 0.0
408 0.0
409 0.0
410 0.0
411 0.0
412 0.0
413 0.0
414 0.0
415 0.0
416 0.0
417 0.0
418 0.0
419 0.0
420 0.0
421 0.0
422 0.0
423 0.0
424 0.0
425 0.0
426 0.0
427 0.0
428 0.0
429 0.0
430 0.0
431 0.0
432 0.0
433 0.0
434 0.0
435 0.0
436 0.0
437 0.0
438 0.0
439 0.0
440 0.0
441 0.0
442 0.0
443 0.0
444 0.0
445 0.0
446 0.0
447 0.0
448 0.0
449 0.0
450 0.0
451 0.0
452 0.0
453 0.0
454 0.0
455 0.0
456 0.0
457 0.0
458 0.0
459 0.0
460 0.0
461 0.0
462 0.0
463 0.0
464 0.0
465 0.0
466 0.0
467 0.0
468 0.0
469 0.0
470 0.0
471 0.0
472 0.0
473 0.0
474 0.0
475 0.0
476 0.0
477 0.0
478 0.0
479 0.0
480 0.0
481 0.0
482 0.0
483 0.0
484 0.0
485 0.0
486 0.0
487 0.0
488 0.0
489 0.0
490 0.0
491 0.0
492 0.0
493 0.0
494 0.0
495 0.0
496 0.0
497 0.0
498 0.0
499 0.0
500 0.0
501 0.0
502 0.0
503 0.0
504 0.0
505 0.0
506 0.0
507 0.0
508 0.0
509 0.0
510 0.0
511 0.0
512 0.0
513 0.0
514 0.0
515 0.0
516 0.0
517 0.0
518 0.0
519 0.0
520 0.0
521 0.0
522 0.0
523 0.0
524 0.0
525 0.0
526 0.0
527 0.0
528 0.0
529 0.0
530 0.0
531 0.0
532 0.0
533 0.0
534 0.0
535 0.0
536 0.0
537 0.0
538 0.0
539 0.0
540 0.0
541 0.0
542 0.0
543 0.0
544 0.0
545 0.0
546 0.0
547 0.0
548 0.0
549 0.0
550 0.0
551 0.0
552 0.0
553 0.0
554 0.0
555 0.0
556 0.0
557 0.0
558 0.0
559 0.0
560 0.0
561 0.0
562 0.0
563 0.0
564 0.0
565 0.0
566 0.0
567 0.0
568 0.0
569 0.0
570 0.0
571 0.0
572 0.0
573 0.0
574 0.0
575 0.0
576 0.0
577 0.0
578 0.0
579 0.0
580 0.0
581 0.0
582 0.0
583 0.0
584 0.0
585 0.0
586 0.0
587 0.0
588 0.0
589 0.0
590 0.0
591 0.0
592 0.0
593 0.0
594 0.0
595 0.0
596 0.0
597 0.0
598 0.0
599 0.0
600 0.0
601 0.0
602 0.0
603 0.0
604 0.0
605 0.0
606 0.0
607 0.0
608 0.0
609 0.0
610 0.0
611 0.0
612 0.0
613 0.0
614 0.0
615 0.0
616 0.0
617 0.0
618 0.0
619 0.0
620 0.0
621 0.0
622 0.0
623 0.0
624 0.0
625 0.0
626 0.0
627 0.0
628 0.0
629 0.0
630 0.0
631 0.0
632 0.0
633 0.0
634 0.0
635 0.0
636 0.0
637 0.0
638 0.0
639 0.0
640 0.0
641 0.0
642 0.0
643 0.0
644 0.0
645 0.0
646 0.0
647 0.0
648 0.0
649 0.0
650 0.0
651 0.0
652 0.0
653 0.0
654 0.0
655 0.0
656 0.0
657 0.0
658 0.0
659 0.0
660 0.0
661 0.0
662 0.0
663 0.0
664 0.0
665 0.0
666 0.0
667 0.0
668 0.0
669 0.0
670 0.0
671 0.0
672 0.0
673 0.0
674 0.0
675 0.0
676 0.0
677 0.0
678 0.0
679 0.0
680 0.0
681 0.0
682 0.0
683 0.0
684 0.0
685 0.0
686 0.0
687 0.0
688 0.0
689 0.0
690 0.0
691 0.0
692 0.0
693 0.0
694 0.0
695 0.0
696 0.0
697 0.0
698 0.0
699 0.0
700 0.0
701 0.0
702 0.0
703 0.0
704 0.0
705 0.0
706 0.0
707 0.0
708 0.0
709 0.0
710 0.0
711 0.0
712 0.0
713 0.0
714 0.0
715 0.0
716 0.0
717 0.0
718 0.0
719 0.0
720 0.0
721 0.0
722 0.0
723 0.0
724 0.0
725 0.0
726 0.0
727 0.0
728 0.0
729 0.0
730 0.0
731 0.0
732 0.0
733 0.0
734 0.0
735 0.0
736 0.0
737 0.0
738 0.0
739 0.0
740 0.0
741 0.0
742 0.0
743 0.0
744 0.0
745 0.0
746 0.0
747 0.0
748 0.0
749 0.0
750 0.0
751 0.0
752 0.0
753 0.0
754 0.0
755 0.0
756 0.0
757 0.0
758 0.0
759 0.0
760 0.0
761 0.0
762 0.0
763 0.0
764 0.0
765 0.0
766 0.0
767 0.0
768 0.0
769 0.0
770 0.0
771 0.0
772 0.0
773 0.0
774 0.0
775 0.0
776 0.0
777 0.0
778 0.0
779 0.0;


######################
# SOLVE 
######################
end;
