

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..27;
# Set of Hosts
set Hs := 0..16;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {2..7}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm3_0_{h in Hs}: sum{v in {8..10}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {11..27}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 16
12 16
13 16
14 16
15 16
16 16
17 16
18 16
19 16
20 16
21 16
22 16
23 16
24 16
25 16
26 16
27 16;
param: MEM := 
0 16
1 16
2 48
3 48
4 48
5 48
6 48
7 48
8 16
9 16
10 16
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8;
param: NET := 
0 171.0
1 171.0
2 535.0
3 535.0
4 535.0
5 535.0
6 535.0
7 535.0
8 0.0
9 0.0
10 0.0
11 1600.0
12 1600.0
13 1600.0
14 1600.0
15 1600.0
16 1600.0
17 1600.0
18 1600.0
19 1600.0
20 1600.0
21 1600.0
22 1600.0
23 1600.0
24 1600.0
25 1600.0
26 1600.0
27 1600.0;


######################
# SOLVE 
######################
end;
