

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..9;
# Set of Hosts
set Hs := 0..2;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..2}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {3,4}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm3_0_{h in Hs}: sum{v in {5,6}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm4_0_{h in Hs}: sum{v in {7..9}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 16
1 16
2 16
3 8
4 8
5 8
6 8
7 8
8 8
9 8;
param: MEM := 
0 8
1 8
2 8
3 16
4 16
5 48
6 48
7 16
8 16
9 16;
param: NET := 
0 1600.0
1 1600.0
2 1600.0
3 171.0
4 171.0
5 535.0
6 535.0
7 0.0
8 0.0
9 0.0;


######################
# SOLVE 
######################
end;
