

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..104;
# Set of Hosts
set Hs := 0..31;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..2}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {3..13}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {14..24}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {25..45}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm3_0_{h in Hs}: sum{v in {46,47}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm3_1_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm4_0_{h in Hs}: sum{v in {50..67}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm4_1_{h in Hs}: sum{v in {68..85}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_0_{h in Hs}: sum{v in {86..88}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_1_{h in Hs}: sum{v in {89..91}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm3_0_{h in Hs}: sum{v in {92,93}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm4_0_{h in Hs}: sum{v in {94..101}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm5_0_{h in Hs}: sum{v in {102..104}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0{h in Hs}: ni_vnf3_dot_vm2_0[h] <= sum{v in {25..45}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0{v in {25..45}, h in Hs}: ni_vnf3_dot_vm2_0[h] >= x[v, h];
var ni_vnf2_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0{h in Hs}: ni_vnf2_dot_vm2_0[h] <= sum{v in {3..13}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0{v in {3..13}, h in Hs}: ni_vnf2_dot_vm2_0[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf2_dot_vm2_0[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm2_1{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_1{h in Hs}: ni_vnf2_dot_vm2_1[h] <= sum{v in {14..24}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_1{v in {14..24}, h in Hs}: ni_vnf2_dot_vm2_1[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf2_dot_vm2_1[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9.0
1 9.0
2 9.0
3 16.0
4 16.0
5 16.0
6 16.0
7 16.0
8 16.0
9 16.0
10 16.0
11 16.0
12 16.0
13 16.0
14 16.0
15 16.0
16 16.0
17 16.0
18 16.0
19 16.0
20 16.0
21 16.0
22 16.0
23 16.0
24 16.0
25 16.0
26 16.0
27 16.0
28 16.0
29 16.0
30 16.0
31 16.0
32 16.0
33 16.0
34 16.0
35 16.0
36 16.0
37 16.0
38 16.0
39 16.0
40 16.0
41 16.0
42 16.0
43 16.0
44 16.0
45 16.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 8.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 8.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0
83 8.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 8.0
93 8.0
94 8.0
95 8.0
96 8.0
97 8.0
98 8.0
99 8.0
100 8.0
101 8.0
102 8.0
103 8.0
104 8.0;
param: MEM := 
0 6.0
1 6.0
2 6.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 8.0
21 8.0
22 8.0
23 8.0
24 8.0
25 8.0
26 8.0
27 8.0
28 8.0
29 8.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 8.0
39 8.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 16.0
47 16.0
48 16.0
49 16.0
50 48.0
51 48.0
52 48.0
53 48.0
54 48.0
55 48.0
56 48.0
57 48.0
58 48.0
59 48.0
60 48.0
61 48.0
62 48.0
63 48.0
64 48.0
65 48.0
66 48.0
67 48.0
68 48.0
69 48.0
70 48.0
71 48.0
72 48.0
73 48.0
74 48.0
75 48.0
76 48.0
77 48.0
78 48.0
79 48.0
80 48.0
81 48.0
82 48.0
83 48.0
84 48.0
85 48.0
86 16.0
87 16.0
88 16.0
89 16.0
90 16.0
91 16.0
92 16.0
93 16.0
94 48.0
95 48.0
96 48.0
97 48.0
98 48.0
99 48.0
100 48.0
101 48.0
102 16.0
103 16.0
104 16.0;
param: NET := 
0 0.0
1 0.0
2 0.0
3 1600.0
4 1600.0
5 1600.0
6 1600.0
7 1600.0
8 1600.0
9 1600.0
10 1600.0
11 1600.0
12 1600.0
13 1600.0
14 1600.0
15 1600.0
16 1600.0
17 1600.0
18 1600.0
19 1600.0
20 1600.0
21 1600.0
22 1600.0
23 1600.0
24 1600.0
25 1600.0
26 1600.0
27 1600.0
28 1600.0
29 1600.0
30 1600.0
31 1600.0
32 1600.0
33 1600.0
34 1600.0
35 1600.0
36 1600.0
37 1600.0
38 1600.0
39 1600.0
40 1600.0
41 1600.0
42 1600.0
43 1600.0
44 1600.0
45 1600.0
46 675.0
47 675.0
48 675.0
49 675.0
50 535.0
51 535.0
52 535.0
53 535.0
54 535.0
55 535.0
56 535.0
57 535.0
58 535.0
59 535.0
60 535.0
61 535.0
62 535.0
63 535.0
64 535.0
65 535.0
66 535.0
67 535.0
68 535.0
69 535.0
70 535.0
71 535.0
72 535.0
73 535.0
74 535.0
75 535.0
76 535.0
77 535.0
78 535.0
79 535.0
80 535.0
81 535.0
82 535.0
83 535.0
84 535.0
85 535.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 171.0
93 171.0
94 535.0
95 535.0
96 535.0
97 535.0
98 535.0
99 535.0
100 535.0
101 535.0
102 0.0
103 0.0
104 0.0;


######################
# SOLVE 
######################
end;
