

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..142;
# Set of Hosts
set Hs := 0..46;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..2}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {3..15}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {16..28}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {29..45}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {46..62}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm3_0_{h in Hs}: sum{v in {63,64}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm3_1_{h in Hs}: sum{v in {65,66}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm4_0_{h in Hs}: sum{v in {67..90}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm4_1_{h in Hs}: sum{v in {91..114}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_0_{h in Hs}: sum{v in {115..117}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_1_{h in Hs}: sum{v in {118..120}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm3_0_{h in Hs}: sum{v in {121,122}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm4_0_{h in Hs}: sum{v in {123..128}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm5_0_{h in Hs}: sum{v in {129..131}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm3_0_{h in Hs}: sum{v in {132,133}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm4_0_{h in Hs}: sum{v in {134..139}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm5_0_{h in Hs}: sum{v in {140..142}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0{h in Hs}: ni_vnf3_dot_vm2_0[h] <= sum{v in {29..45}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0{v in {29..45}, h in Hs}: ni_vnf3_dot_vm2_0[h] >= x[v, h];
var ni_vnf2_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0{h in Hs}: ni_vnf2_dot_vm2_0[h] <= sum{v in {3..15}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0{v in {3..15}, h in Hs}: ni_vnf2_dot_vm2_0[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf2_dot_vm2_0[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm2_1{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_1{h in Hs}: ni_vnf2_dot_vm2_1[h] <= sum{v in {16..28}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_1{v in {16..28}, h in Hs}: ni_vnf2_dot_vm2_1[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf2_dot_vm2_1[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf4_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf4_dot_vm2_0{h in Hs}: ni_vnf4_dot_vm2_0[h] <= sum{v in {46..62}} x[v, h];
s.t. or_ineq_ni_vnf4_dot_vm2_0{v in {46..62}, h in Hs}: ni_vnf4_dot_vm2_0[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf4_dot_vm2_0[h] <= 1;
# --- OR variables for CAAF ----
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf4_dot_vm2_0[h] + ni_vnf2_dot_vm2_0[h] <= 1;
# --- OR variables for CAAF ----
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf4_dot_vm2_0[h] + ni_vnf2_dot_vm2_1[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9
1 9
2 9
3 16
4 16
5 16
6 16
7 16
8 16
9 16
10 16
11 16
12 16
13 16
14 16
15 16
16 16
17 16
18 16
19 16
20 16
21 16
22 16
23 16
24 16
25 16
26 16
27 16
28 16
29 16
30 16
31 16
32 16
33 16
34 16
35 16
36 16
37 16
38 16
39 16
40 16
41 16
42 16
43 16
44 16
45 16
46 16
47 16
48 16
49 16
50 16
51 16
52 16
53 16
54 16
55 16
56 16
57 16
58 16
59 16
60 16
61 16
62 16
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 8
73 8
74 8
75 8
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 8
89 8
90 8
91 8
92 8
93 8
94 8
95 8
96 8
97 8
98 8
99 8
100 8
101 8
102 8
103 8
104 8
105 8
106 8
107 8
108 8
109 8
110 8
111 8
112 8
113 8
114 8
115 8
116 8
117 8
118 8
119 8
120 8
121 8
122 8
123 8
124 8
125 8
126 8
127 8
128 8
129 8
130 8
131 8
132 8
133 8
134 8
135 8
136 8
137 8
138 8
139 8
140 8
141 8
142 8;
param: MEM := 
0 6
1 6
2 6
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8
28 8
29 8
30 8
31 8
32 8
33 8
34 8
35 8
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 8
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 16
64 16
65 16
66 16
67 48
68 48
69 48
70 48
71 48
72 48
73 48
74 48
75 48
76 48
77 48
78 48
79 48
80 48
81 48
82 48
83 48
84 48
85 48
86 48
87 48
88 48
89 48
90 48
91 48
92 48
93 48
94 48
95 48
96 48
97 48
98 48
99 48
100 48
101 48
102 48
103 48
104 48
105 48
106 48
107 48
108 48
109 48
110 48
111 48
112 48
113 48
114 48
115 16
116 16
117 16
118 16
119 16
120 16
121 16
122 16
123 48
124 48
125 48
126 48
127 48
128 48
129 16
130 16
131 16
132 16
133 16
134 48
135 48
136 48
137 48
138 48
139 48
140 16
141 16
142 16;
param: NET := 
0 0.0
1 0.0
2 0.0
3 1600.0
4 1600.0
5 1600.0
6 1600.0
7 1600.0
8 1600.0
9 1600.0
10 1600.0
11 1600.0
12 1600.0
13 1600.0
14 1600.0
15 1600.0
16 1600.0
17 1600.0
18 1600.0
19 1600.0
20 1600.0
21 1600.0
22 1600.0
23 1600.0
24 1600.0
25 1600.0
26 1600.0
27 1600.0
28 1600.0
29 1600.0
30 1600.0
31 1600.0
32 1600.0
33 1600.0
34 1600.0
35 1600.0
36 1600.0
37 1600.0
38 1600.0
39 1600.0
40 1600.0
41 1600.0
42 1600.0
43 1600.0
44 1600.0
45 1600.0
46 1600.0
47 1600.0
48 1600.0
49 1600.0
50 1600.0
51 1600.0
52 1600.0
53 1600.0
54 1600.0
55 1600.0
56 1600.0
57 1600.0
58 1600.0
59 1600.0
60 1600.0
61 1600.0
62 1600.0
63 675.0
64 675.0
65 675.0
66 675.0
67 535.0
68 535.0
69 535.0
70 535.0
71 535.0
72 535.0
73 535.0
74 535.0
75 535.0
76 535.0
77 535.0
78 535.0
79 535.0
80 535.0
81 535.0
82 535.0
83 535.0
84 535.0
85 535.0
86 535.0
87 535.0
88 535.0
89 535.0
90 535.0
91 535.0
92 535.0
93 535.0
94 535.0
95 535.0
96 535.0
97 535.0
98 535.0
99 535.0
100 535.0
101 535.0
102 535.0
103 535.0
104 535.0
105 535.0
106 535.0
107 535.0
108 535.0
109 535.0
110 535.0
111 535.0
112 535.0
113 535.0
114 535.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 171.0
122 171.0
123 535.0
124 535.0
125 535.0
126 535.0
127 535.0
128 535.0
129 0.0
130 0.0
131 0.0
132 171.0
133 171.0
134 535.0
135 535.0
136 535.0
137 535.0
138 535.0
139 535.0
140 0.0
141 0.0
142 0.0;


######################
# SOLVE 
######################
end;
