

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..104;
# Set of Hosts
set Hs := 0..31;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..2}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {3..13}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {14..24}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {25..45}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {46,47}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_1_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {50..67}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_1_{h in Hs}: sum{v in {68..85}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {86..88}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_1_{h in Hs}: sum{v in {89..91}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm3_0_{h in Hs}: sum{v in {92,93}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm4_0_{h in Hs}: sum{v in {94..101}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_0_{h in Hs}: sum{v in {102..104}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0{h in Hs}: ni_vnf3_dot_vm2_0[h] <= sum{v in {25..45}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0{v in {25..45}, h in Hs}: ni_vnf3_dot_vm2_0[h] >= x[v, h];
var ni_vnf2_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0{h in Hs}: ni_vnf2_dot_vm2_0[h] <= sum{v in {3..13}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0{v in {3..13}, h in Hs}: ni_vnf2_dot_vm2_0[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf2_dot_vm2_0[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm2_1{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_1{h in Hs}: ni_vnf2_dot_vm2_1[h] <= sum{v in {14..24}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_1{v in {14..24}, h in Hs}: ni_vnf2_dot_vm2_1[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf2_dot_vm2_1[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9
1 9
2 9
3 16
4 16
5 16
6 16
7 16
8 16
9 16
10 16
11 16
12 16
13 16
14 16
15 16
16 16
17 16
18 16
19 16
20 16
21 16
22 16
23 16
24 16
25 16
26 16
27 16
28 16
29 16
30 16
31 16
32 16
33 16
34 16
35 16
36 16
37 16
38 16
39 16
40 16
41 16
42 16
43 16
44 16
45 16
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 8
73 8
74 8
75 8
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 8
89 8
90 8
91 8
92 8
93 8
94 8
95 8
96 8
97 8
98 8
99 8
100 8
101 8
102 8
103 8
104 8;
param: MEM := 
0 6
1 6
2 6
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8
28 8
29 8
30 8
31 8
32 8
33 8
34 8
35 8
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 8
46 16
47 16
48 16
49 16
50 48
51 48
52 48
53 48
54 48
55 48
56 48
57 48
58 48
59 48
60 48
61 48
62 48
63 48
64 48
65 48
66 48
67 48
68 48
69 48
70 48
71 48
72 48
73 48
74 48
75 48
76 48
77 48
78 48
79 48
80 48
81 48
82 48
83 48
84 48
85 48
86 16
87 16
88 16
89 16
90 16
91 16
92 16
93 16
94 48
95 48
96 48
97 48
98 48
99 48
100 48
101 48
102 16
103 16
104 16;
param: NET := 
0 0.0
1 0.0
2 0.0
3 1600.0
4 1600.0
5 1600.0
6 1600.0
7 1600.0
8 1600.0
9 1600.0
10 1600.0
11 1600.0
12 1600.0
13 1600.0
14 1600.0
15 1600.0
16 1600.0
17 1600.0
18 1600.0
19 1600.0
20 1600.0
21 1600.0
22 1600.0
23 1600.0
24 1600.0
25 1600.0
26 1600.0
27 1600.0
28 1600.0
29 1600.0
30 1600.0
31 1600.0
32 1600.0
33 1600.0
34 1600.0
35 1600.0
36 1600.0
37 1600.0
38 1600.0
39 1600.0
40 1600.0
41 1600.0
42 1600.0
43 1600.0
44 1600.0
45 1600.0
46 675.0
47 675.0
48 675.0
49 675.0
50 535.0
51 535.0
52 535.0
53 535.0
54 535.0
55 535.0
56 535.0
57 535.0
58 535.0
59 535.0
60 535.0
61 535.0
62 535.0
63 535.0
64 535.0
65 535.0
66 535.0
67 535.0
68 535.0
69 535.0
70 535.0
71 535.0
72 535.0
73 535.0
74 535.0
75 535.0
76 535.0
77 535.0
78 535.0
79 535.0
80 535.0
81 535.0
82 535.0
83 535.0
84 535.0
85 535.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 171.0
93 171.0
94 535.0
95 535.0
96 535.0
97 535.0
98 535.0
99 535.0
100 535.0
101 535.0
102 0.0
103 0.0
104 0.0;


######################
# SOLVE 
######################
end;
