

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..22;
# Set of Hosts
set Hs := 0..5;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0..2}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {3..5}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {6..8}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm3_0_{h in Hs}: sum{v in {9,10}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm4_0_{h in Hs}: sum{v in {11,12}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm5_0_{h in Hs}: sum{v in {13..15}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm3_0_{h in Hs}: sum{v in {16,17}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm4_0_{h in Hs}: sum{v in {18,19}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm5_0_{h in Hs}: sum{v in {20..22}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm2_0{h in Hs}: ni_vnf3_dot_vm2_0[h] <= sum{v in {6..8}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm2_0{v in {6..8}, h in Hs}: ni_vnf3_dot_vm2_0[h] >= x[v, h];
var ni_vnf2_dot_vm2_0{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm2_0{h in Hs}: ni_vnf2_dot_vm2_0[h] <= sum{v in {3..5}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm2_0{v in {3..5}, h in Hs}: ni_vnf2_dot_vm2_0[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf3_dot_vm2_0[h] + ni_vnf2_dot_vm2_0[h] <= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 9
1 9
2 9
3 16
4 16
5 16
6 16
7 16
8 16
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8;
param: MEM := 
0 6
1 6
2 6
3 8
4 8
5 8
6 8
7 8
8 8
9 16
10 16
11 48
12 48
13 16
14 16
15 16
16 16
17 16
18 48
19 48
20 16
21 16
22 16;
param: NET := 
0 0.0
1 0.0
2 0.0
3 1600.0
4 1600.0
5 1600.0
6 1600.0
7 1600.0
8 1600.0
9 675.0
10 675.0
11 535.0
12 535.0
13 0.0
14 0.0
15 0.0
16 171.0
17 171.0
18 535.0
19 535.0
20 0.0
21 0.0
22 0.0;


######################
# SOLVE 
######################
end;
