

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..228;
# Set of Hosts
set Hs := 0..52;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {4..25}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {26..47}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {48,49}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {50,51}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {52..75}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {76..99}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm1_0_{h in Hs}: sum{v in {100,101}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {102..109}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm1_0_{h in Hs}: sum{v in {110,111}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {112..119}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {120,121}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {122..148}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm3_0_{h in Hs}: sum{v in {149,150}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm4_0_{h in Hs}: sum{v in {151..154}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm5_0_{h in Hs}: sum{v in {155,156}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm6_0_{h in Hs}: sum{v in {157..160}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm7_0_{h in Hs}: sum{v in {161,162}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm8_0_{h in Hs}: sum{v in {163,164}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_0_{h in Hs}: sum{v in {165,166}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_1_{h in Hs}: sum{v in {167,168}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_2_{h in Hs}: sum{v in {169,170}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_3_{h in Hs}: sum{v in {171,172}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_0_{h in Hs}: sum{v in {173..186}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_1_{h in Hs}: sum{v in {187..200}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_2_{h in Hs}: sum{v in {201..214}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_3_{h in Hs}: sum{v in {215..228}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 8.0
21 8.0
22 8.0
23 8.0
24 8.0
25 8.0
26 8.0
27 8.0
28 8.0
29 8.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 8.0
39 8.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 8.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 8.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0
83 8.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 8.0
93 8.0
94 8.0
95 8.0
96 8.0
97 8.0
98 8.0
99 8.0
100 8.0
101 8.0
102 8.0
103 8.0
104 8.0
105 8.0
106 8.0
107 8.0
108 8.0
109 8.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 8.0
117 8.0
118 8.0
119 8.0
120 4.0
121 4.0
122 8.0
123 8.0
124 8.0
125 8.0
126 8.0
127 8.0
128 8.0
129 8.0
130 8.0
131 8.0
132 8.0
133 8.0
134 8.0
135 8.0
136 8.0
137 8.0
138 8.0
139 8.0
140 8.0
141 8.0
142 8.0
143 8.0
144 8.0
145 8.0
146 8.0
147 8.0
148 8.0
149 7.0
150 7.0
151 2.0
152 2.0
153 2.0
154 2.0
155 6.0
156 6.0
157 4.0
158 4.0
159 4.0
160 4.0
161 2.0
162 2.0
163 4.0
164 4.0
165 4.0
166 4.0
167 4.0
168 4.0
169 4.0
170 4.0
171 4.0
172 4.0
173 16.0
174 16.0
175 16.0
176 16.0
177 16.0
178 16.0
179 16.0
180 16.0
181 16.0
182 16.0
183 16.0
184 16.0
185 16.0
186 16.0
187 16.0
188 16.0
189 16.0
190 16.0
191 16.0
192 16.0
193 16.0
194 16.0
195 16.0
196 16.0
197 16.0
198 16.0
199 16.0
200 16.0
201 16.0
202 16.0
203 16.0
204 16.0
205 16.0
206 16.0
207 16.0
208 16.0
209 16.0
210 16.0
211 16.0
212 16.0
213 16.0
214 16.0
215 16.0
216 16.0
217 16.0
218 16.0
219 16.0
220 16.0
221 16.0
222 16.0
223 16.0
224 16.0
225 16.0
226 16.0
227 16.0
228 16.0;
param: MEM := 
0 54.0
1 54.0
2 54.0
3 54.0
4 54.0
5 54.0
6 54.0
7 54.0
8 54.0
9 54.0
10 54.0
11 54.0
12 54.0
13 54.0
14 54.0
15 54.0
16 54.0
17 54.0
18 54.0
19 54.0
20 54.0
21 54.0
22 54.0
23 54.0
24 54.0
25 54.0
26 54.0
27 54.0
28 54.0
29 54.0
30 54.0
31 54.0
32 54.0
33 54.0
34 54.0
35 54.0
36 54.0
37 54.0
38 54.0
39 54.0
40 54.0
41 54.0
42 54.0
43 54.0
44 54.0
45 54.0
46 54.0
47 54.0
48 20.0
49 20.0
50 20.0
51 20.0
52 54.0
53 54.0
54 54.0
55 54.0
56 54.0
57 54.0
58 54.0
59 54.0
60 54.0
61 54.0
62 54.0
63 54.0
64 54.0
65 54.0
66 54.0
67 54.0
68 54.0
69 54.0
70 54.0
71 54.0
72 54.0
73 54.0
74 54.0
75 54.0
76 54.0
77 54.0
78 54.0
79 54.0
80 54.0
81 54.0
82 54.0
83 54.0
84 54.0
85 54.0
86 54.0
87 54.0
88 54.0
89 54.0
90 54.0
91 54.0
92 54.0
93 54.0
94 54.0
95 54.0
96 54.0
97 54.0
98 54.0
99 54.0
100 20.0
101 20.0
102 54.0
103 54.0
104 54.0
105 54.0
106 54.0
107 54.0
108 54.0
109 54.0
110 20.0
111 20.0
112 54.0
113 54.0
114 54.0
115 54.0
116 54.0
117 54.0
118 54.0
119 54.0
120 12.0
121 12.0
122 32.0
123 32.0
124 32.0
125 32.0
126 32.0
127 32.0
128 32.0
129 32.0
130 32.0
131 32.0
132 32.0
133 32.0
134 32.0
135 32.0
136 32.0
137 32.0
138 32.0
139 32.0
140 32.0
141 32.0
142 32.0
143 32.0
144 32.0
145 32.0
146 32.0
147 32.0
148 32.0
149 10.0
150 10.0
151 32.0
152 32.0
153 32.0
154 32.0
155 4.0
156 4.0
157 10.0
158 10.0
159 10.0
160 10.0
161 6.0
162 6.0
163 10.0
164 10.0
165 6.0
166 6.0
167 6.0
168 6.0
169 6.0
170 6.0
171 6.0
172 6.0
173 12.0
174 12.0
175 12.0
176 12.0
177 12.0
178 12.0
179 12.0
180 12.0
181 12.0
182 12.0
183 12.0
184 12.0
185 12.0
186 12.0
187 12.0
188 12.0
189 12.0
190 12.0
191 12.0
192 12.0
193 12.0
194 12.0
195 12.0
196 12.0
197 12.0
198 12.0
199 12.0
200 12.0
201 12.0
202 12.0
203 12.0
204 12.0
205 12.0
206 12.0
207 12.0
208 12.0
209 12.0
210 12.0
211 12.0
212 12.0
213 12.0
214 12.0
215 12.0
216 12.0
217 12.0
218 12.0
219 12.0
220 12.0
221 12.0
222 12.0
223 12.0
224 12.0
225 12.0
226 12.0
227 12.0
228 12.0;
param: NET := 
0 36.0
1 36.0
2 36.0
3 36.0
4 87.0
5 87.0
6 87.0
7 87.0
8 87.0
9 87.0
10 87.0
11 87.0
12 87.0
13 87.0
14 87.0
15 87.0
16 87.0
17 87.0
18 87.0
19 87.0
20 87.0
21 87.0
22 87.0
23 87.0
24 87.0
25 87.0
26 87.0
27 87.0
28 87.0
29 87.0
30 87.0
31 87.0
32 87.0
33 87.0
34 87.0
35 87.0
36 87.0
37 87.0
38 87.0
39 87.0
40 87.0
41 87.0
42 87.0
43 87.0
44 87.0
45 87.0
46 87.0
47 87.0
48 36.0
49 36.0
50 36.0
51 36.0
52 97.0
53 97.0
54 97.0
55 97.0
56 97.0
57 97.0
58 97.0
59 97.0
60 97.0
61 97.0
62 97.0
63 97.0
64 97.0
65 97.0
66 97.0
67 97.0
68 97.0
69 97.0
70 97.0
71 97.0
72 97.0
73 97.0
74 97.0
75 97.0
76 97.0
77 97.0
78 97.0
79 97.0
80 97.0
81 97.0
82 97.0
83 97.0
84 97.0
85 97.0
86 97.0
87 97.0
88 97.0
89 97.0
90 97.0
91 97.0
92 97.0
93 97.0
94 97.0
95 97.0
96 97.0
97 97.0
98 97.0
99 97.0
100 36.0
101 36.0
102 97.0
103 97.0
104 97.0
105 97.0
106 97.0
107 97.0
108 97.0
109 97.0
110 36.0
111 36.0
112 97.0
113 97.0
114 97.0
115 97.0
116 97.0
117 97.0
118 97.0
119 97.0
120 40.0
121 40.0
122 370.0
123 370.0
124 370.0
125 370.0
126 370.0
127 370.0
128 370.0
129 370.0
130 370.0
131 370.0
132 370.0
133 370.0
134 370.0
135 370.0
136 370.0
137 370.0
138 370.0
139 370.0
140 370.0
141 370.0
142 370.0
143 370.0
144 370.0
145 370.0
146 370.0
147 370.0
148 370.0
149 0.0
150 0.0
151 0.0
152 0.0
153 0.0
154 0.0
155 0.0
156 0.0
157 0.0
158 0.0
159 0.0
160 0.0
161 0.0
162 0.0
163 0.0
164 0.0
165 155.0
166 155.0
167 155.0
168 155.0
169 155.0
170 155.0
171 155.0
172 155.0
173 95.0
174 95.0
175 95.0
176 95.0
177 95.0
178 95.0
179 95.0
180 95.0
181 95.0
182 95.0
183 95.0
184 95.0
185 95.0
186 95.0
187 95.0
188 95.0
189 95.0
190 95.0
191 95.0
192 95.0
193 95.0
194 95.0
195 95.0
196 95.0
197 95.0
198 95.0
199 95.0
200 95.0
201 95.0
202 95.0
203 95.0
204 95.0
205 95.0
206 95.0
207 95.0
208 95.0
209 95.0
210 95.0
211 95.0
212 95.0
213 95.0
214 95.0
215 95.0
216 95.0
217 95.0
218 95.0
219 95.0
220 95.0
221 95.0
222 95.0
223 95.0
224 95.0
225 95.0
226 95.0
227 95.0
228 95.0;


######################
# SOLVE 
######################
end;
