

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..256;
# Set of Hosts
set Hs := 0..56;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm1_0_{h in Hs}: sum{v in {0,1}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_1_{h in Hs}: sum{v in {2,3}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm1_2_{h in Hs}: sum{v in {4,5}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {6..21}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {22..37}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_2_{h in Hs}: sum{v in {38..53}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_0_{h in Hs}: sum{v in {54,55}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm1_1_{h in Hs}: sum{v in {56,57}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_0_{h in Hs}: sum{v in {58..83}} x[v, h]<= 1;
s.t. AAF_vnf2_dot_vm2_1_{h in Hs}: sum{v in {84..109}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm1_0_{h in Hs}: sum{v in {110,111}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm2_0_{h in Hs}: sum{v in {112..119}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm1_0_{h in Hs}: sum{v in {120,121}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm2_0_{h in Hs}: sum{v in {122..129}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm1_0_{h in Hs}: sum{v in {130,131}} x[v, h]<= 1;
s.t. AAF_vnf5_dot_vm2_0_{h in Hs}: sum{v in {132..161}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm3_0_{h in Hs}: sum{v in {162,163}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm4_0_{h in Hs}: sum{v in {164..167}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm5_0_{h in Hs}: sum{v in {168,169}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm6_0_{h in Hs}: sum{v in {170..173}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm7_0_{h in Hs}: sum{v in {174,175}} x[v, h]<= 1;
s.t. AAF_vnf7_dot_vm8_0_{h in Hs}: sum{v in {176,177}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm3_0_{h in Hs}: sum{v in {178,179}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm9_0_{h in Hs}: sum{v in {180..186}} x[v, h]<= 1;
s.t. AAF_vnf8_dot_vm6_0_{h in Hs}: sum{v in {187,188}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_0_{h in Hs}: sum{v in {189,190}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_1_{h in Hs}: sum{v in {191,192}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_2_{h in Hs}: sum{v in {193,194}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm1_3_{h in Hs}: sum{v in {195,196}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_0_{h in Hs}: sum{v in {197..211}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_1_{h in Hs}: sum{v in {212..226}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_2_{h in Hs}: sum{v in {227..241}} x[v, h]<= 1;
s.t. AAF_vnf9_dot_vm2_3_{h in Hs}: sum{v in {242..256}} x[v, h]<= 1;


######################
# COST 
######################
minimize obj:  + 100 * used_hosts;

######################
# DATA 
######################
data;
param: CPU := 
0 8
1 8
2 8
3 8
4 8
5 8
6 8
7 8
8 8
9 8
10 8
11 8
12 8
13 8
14 8
15 8
16 8
17 8
18 8
19 8
20 8
21 8
22 8
23 8
24 8
25 8
26 8
27 8
28 8
29 8
30 8
31 8
32 8
33 8
34 8
35 8
36 8
37 8
38 8
39 8
40 8
41 8
42 8
43 8
44 8
45 8
46 8
47 8
48 8
49 8
50 8
51 8
52 8
53 8
54 8
55 8
56 8
57 8
58 8
59 8
60 8
61 8
62 8
63 8
64 8
65 8
66 8
67 8
68 8
69 8
70 8
71 8
72 8
73 8
74 8
75 8
76 8
77 8
78 8
79 8
80 8
81 8
82 8
83 8
84 8
85 8
86 8
87 8
88 8
89 8
90 8
91 8
92 8
93 8
94 8
95 8
96 8
97 8
98 8
99 8
100 8
101 8
102 8
103 8
104 8
105 8
106 8
107 8
108 8
109 8
110 8
111 8
112 8
113 8
114 8
115 8
116 8
117 8
118 8
119 8
120 8
121 8
122 8
123 8
124 8
125 8
126 8
127 8
128 8
129 8
130 4
131 4
132 8
133 8
134 8
135 8
136 8
137 8
138 8
139 8
140 8
141 8
142 8
143 8
144 8
145 8
146 8
147 8
148 8
149 8
150 8
151 8
152 8
153 8
154 8
155 8
156 8
157 8
158 8
159 8
160 8
161 8
162 7
163 7
164 2
165 2
166 2
167 2
168 6
169 6
170 4
171 4
172 4
173 4
174 2
175 2
176 4
177 4
178 4
179 4
180 4
181 4
182 4
183 4
184 4
185 4
186 4
187 3
188 3
189 4
190 4
191 4
192 4
193 4
194 4
195 4
196 4
197 16
198 16
199 16
200 16
201 16
202 16
203 16
204 16
205 16
206 16
207 16
208 16
209 16
210 16
211 16
212 16
213 16
214 16
215 16
216 16
217 16
218 16
219 16
220 16
221 16
222 16
223 16
224 16
225 16
226 16
227 16
228 16
229 16
230 16
231 16
232 16
233 16
234 16
235 16
236 16
237 16
238 16
239 16
240 16
241 16
242 16
243 16
244 16
245 16
246 16
247 16
248 16
249 16
250 16
251 16
252 16
253 16
254 16
255 16
256 16;
param: MEM := 
0 54
1 54
2 54
3 54
4 54
5 54
6 54
7 54
8 54
9 54
10 54
11 54
12 54
13 54
14 54
15 54
16 54
17 54
18 54
19 54
20 54
21 54
22 54
23 54
24 54
25 54
26 54
27 54
28 54
29 54
30 54
31 54
32 54
33 54
34 54
35 54
36 54
37 54
38 54
39 54
40 54
41 54
42 54
43 54
44 54
45 54
46 54
47 54
48 54
49 54
50 54
51 54
52 54
53 54
54 20
55 20
56 20
57 20
58 54
59 54
60 54
61 54
62 54
63 54
64 54
65 54
66 54
67 54
68 54
69 54
70 54
71 54
72 54
73 54
74 54
75 54
76 54
77 54
78 54
79 54
80 54
81 54
82 54
83 54
84 54
85 54
86 54
87 54
88 54
89 54
90 54
91 54
92 54
93 54
94 54
95 54
96 54
97 54
98 54
99 54
100 54
101 54
102 54
103 54
104 54
105 54
106 54
107 54
108 54
109 54
110 20
111 20
112 54
113 54
114 54
115 54
116 54
117 54
118 54
119 54
120 20
121 20
122 54
123 54
124 54
125 54
126 54
127 54
128 54
129 54
130 12
131 12
132 32
133 32
134 32
135 32
136 32
137 32
138 32
139 32
140 32
141 32
142 32
143 32
144 32
145 32
146 32
147 32
148 32
149 32
150 32
151 32
152 32
153 32
154 32
155 32
156 32
157 32
158 32
159 32
160 32
161 32
162 10
163 10
164 32
165 32
166 32
167 32
168 4
169 4
170 10
171 10
172 10
173 10
174 6
175 6
176 10
177 10
178 10
179 10
180 16
181 16
182 16
183 16
184 16
185 16
186 16
187 16
188 16
189 6
190 6
191 6
192 6
193 6
194 6
195 6
196 6
197 12
198 12
199 12
200 12
201 12
202 12
203 12
204 12
205 12
206 12
207 12
208 12
209 12
210 12
211 12
212 12
213 12
214 12
215 12
216 12
217 12
218 12
219 12
220 12
221 12
222 12
223 12
224 12
225 12
226 12
227 12
228 12
229 12
230 12
231 12
232 12
233 12
234 12
235 12
236 12
237 12
238 12
239 12
240 12
241 12
242 12
243 12
244 12
245 12
246 12
247 12
248 12
249 12
250 12
251 12
252 12
253 12
254 12
255 12
256 12;
param: NET := 
0 36.0
1 36.0
2 36.0
3 36.0
4 36.0
5 36.0
6 87.0
7 87.0
8 87.0
9 87.0
10 87.0
11 87.0
12 87.0
13 87.0
14 87.0
15 87.0
16 87.0
17 87.0
18 87.0
19 87.0
20 87.0
21 87.0
22 87.0
23 87.0
24 87.0
25 87.0
26 87.0
27 87.0
28 87.0
29 87.0
30 87.0
31 87.0
32 87.0
33 87.0
34 87.0
35 87.0
36 87.0
37 87.0
38 87.0
39 87.0
40 87.0
41 87.0
42 87.0
43 87.0
44 87.0
45 87.0
46 87.0
47 87.0
48 87.0
49 87.0
50 87.0
51 87.0
52 87.0
53 87.0
54 36.0
55 36.0
56 36.0
57 36.0
58 97.0
59 97.0
60 97.0
61 97.0
62 97.0
63 97.0
64 97.0
65 97.0
66 97.0
67 97.0
68 97.0
69 97.0
70 97.0
71 97.0
72 97.0
73 97.0
74 97.0
75 97.0
76 97.0
77 97.0
78 97.0
79 97.0
80 97.0
81 97.0
82 97.0
83 97.0
84 97.0
85 97.0
86 97.0
87 97.0
88 97.0
89 97.0
90 97.0
91 97.0
92 97.0
93 97.0
94 97.0
95 97.0
96 97.0
97 97.0
98 97.0
99 97.0
100 97.0
101 97.0
102 97.0
103 97.0
104 97.0
105 97.0
106 97.0
107 97.0
108 97.0
109 97.0
110 36.0
111 36.0
112 97.0
113 97.0
114 97.0
115 97.0
116 97.0
117 97.0
118 97.0
119 97.0
120 36.0
121 36.0
122 97.0
123 97.0
124 97.0
125 97.0
126 97.0
127 97.0
128 97.0
129 97.0
130 40.0
131 40.0
132 370.0
133 370.0
134 370.0
135 370.0
136 370.0
137 370.0
138 370.0
139 370.0
140 370.0
141 370.0
142 370.0
143 370.0
144 370.0
145 370.0
146 370.0
147 370.0
148 370.0
149 370.0
150 370.0
151 370.0
152 370.0
153 370.0
154 370.0
155 370.0
156 370.0
157 370.0
158 370.0
159 370.0
160 370.0
161 370.0
162 0.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 18.0
179 18.0
180 23.0
181 23.0
182 23.0
183 23.0
184 23.0
185 23.0
186 23.0
187 8.0
188 8.0
189 155.0
190 155.0
191 155.0
192 155.0
193 155.0
194 155.0
195 155.0
196 155.0
197 95.0
198 95.0
199 95.0
200 95.0
201 95.0
202 95.0
203 95.0
204 95.0
205 95.0
206 95.0
207 95.0
208 95.0
209 95.0
210 95.0
211 95.0
212 95.0
213 95.0
214 95.0
215 95.0
216 95.0
217 95.0
218 95.0
219 95.0
220 95.0
221 95.0
222 95.0
223 95.0
224 95.0
225 95.0
226 95.0
227 95.0
228 95.0
229 95.0
230 95.0
231 95.0
232 95.0
233 95.0
234 95.0
235 95.0
236 95.0
237 95.0
238 95.0
239 95.0
240 95.0
241 95.0
242 95.0
243 95.0
244 95.0
245 95.0
246 95.0
247 95.0
248 95.0
249 95.0
250 95.0
251 95.0
252 95.0
253 95.0
254 95.0
255 95.0
256 95.0;


######################
# SOLVE 
######################
end;
