

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..1521;
# Set of Hosts
set Hs := 0..169;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {80..82}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {83..85}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {108..110}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_1_{h in Hs}: sum{v in {111..113}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm8_0_{h in Hs}: sum{v in {114..116}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm8_1_{h in Hs}: sum{v in {117..119}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_0_{h in Hs}: sum{v in {120..122}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_1_{h in Hs}: sum{v in {123..125}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm10_0_{h in Hs}: sum{v in {126..128}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm10_1_{h in Hs}: sum{v in {129..131}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_0_{h in Hs}: sum{v in {200,201}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_1_{h in Hs}: sum{v in {202,203}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm13_0_{h in Hs}: sum{v in {204,205}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm13_1_{h in Hs}: sum{v in {206,207}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm14_0_{h in Hs}: sum{v in {208..214}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm14_1_{h in Hs}: sum{v in {215..221}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm23_0_{h in Hs}: sum{v in {262}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm23_1_{h in Hs}: sum{v in {263}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm24_0_{h in Hs}: sum{v in {264}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm24_1_{h in Hs}: sum{v in {265}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm25_0_{h in Hs}: sum{v in {266}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm25_1_{h in Hs}: sum{v in {267}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm25_2_{h in Hs}: sum{v in {268}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_0_{h in Hs}: sum{v in {269}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_1_{h in Hs}: sum{v in {270}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_2_{h in Hs}: sum{v in {271}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm27_0_{h in Hs}: sum{v in {272}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm28_0_{h in Hs}: sum{v in {273}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm29_0_{h in Hs}: sum{v in {274}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm36_0_{h in Hs}: sum{v in {292}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm41_0_{h in Hs}: sum{v in {369}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm42_0_{h in Hs}: sum{v in {370}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm42_1_{h in Hs}: sum{v in {371}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm42_2_{h in Hs}: sum{v in {372}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm43_0_{h in Hs}: sum{v in {373}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm43_1_{h in Hs}: sum{v in {374}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm43_2_{h in Hs}: sum{v in {375}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_0_{h in Hs}: sum{v in {376}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_1_{h in Hs}: sum{v in {377}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_2_{h in Hs}: sum{v in {378}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_3_{h in Hs}: sum{v in {379}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_0_{h in Hs}: sum{v in {380}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_1_{h in Hs}: sum{v in {381}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_2_{h in Hs}: sum{v in {382}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_3_{h in Hs}: sum{v in {383}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_0_{h in Hs}: sum{v in {384}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_1_{h in Hs}: sum{v in {385}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_2_{h in Hs}: sum{v in {386}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_3_{h in Hs}: sum{v in {387}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_0_{h in Hs}: sum{v in {388}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_1_{h in Hs}: sum{v in {389}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_2_{h in Hs}: sum{v in {390}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_3_{h in Hs}: sum{v in {391}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_0_{h in Hs}: sum{v in {392}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_1_{h in Hs}: sum{v in {393}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_2_{h in Hs}: sum{v in {394}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_3_{h in Hs}: sum{v in {395}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_4_{h in Hs}: sum{v in {396}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_5_{h in Hs}: sum{v in {397}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm49_0_{h in Hs}: sum{v in {398}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm51_0_{h in Hs}: sum{v in {477..479}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm51_1_{h in Hs}: sum{v in {480..482}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm51_2_{h in Hs}: sum{v in {483..485}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm7_0_{h in Hs}: sum{v in {527..529}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm7_1_{h in Hs}: sum{v in {530..532}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm7_2_{h in Hs}: sum{v in {533..535}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm8_0_{h in Hs}: sum{v in {536..538}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm8_1_{h in Hs}: sum{v in {539..541}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm8_2_{h in Hs}: sum{v in {542..544}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm9_0_{h in Hs}: sum{v in {545..547}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm9_1_{h in Hs}: sum{v in {548..550}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm9_2_{h in Hs}: sum{v in {551..553}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm12_0_{h in Hs}: sum{v in {656..658}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm12_1_{h in Hs}: sum{v in {659..661}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm12_2_{h in Hs}: sum{v in {662..664}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm13_0_{h in Hs}: sum{v in {665,666}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm13_1_{h in Hs}: sum{v in {667,668}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm13_2_{h in Hs}: sum{v in {669,670}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm14_0_{h in Hs}: sum{v in {671..677}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm14_1_{h in Hs}: sum{v in {678..684}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm14_2_{h in Hs}: sum{v in {685..691}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm57_0_{h in Hs}: sum{v in {752,753}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm57_1_{h in Hs}: sum{v in {754,755}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm10_0_{h in Hs}: sum{v in {756,757}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm10_1_{h in Hs}: sum{v in {758,759}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_0_{h in Hs}: sum{v in {763,764}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_1_{h in Hs}: sum{v in {765,766}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_2_{h in Hs}: sum{v in {767,768}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_3_{h in Hs}: sum{v in {769,770}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_0_{h in Hs}: sum{v in {771,772}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_1_{h in Hs}: sum{v in {773,774}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_2_{h in Hs}: sum{v in {775,776}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_3_{h in Hs}: sum{v in {777,778}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_0_{h in Hs}: sum{v in {779,780}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_1_{h in Hs}: sum{v in {781,782}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_2_{h in Hs}: sum{v in {783,784}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_3_{h in Hs}: sum{v in {785,786}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_0_{h in Hs}: sum{v in {787..794}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_1_{h in Hs}: sum{v in {795..802}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_2_{h in Hs}: sum{v in {803..810}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_3_{h in Hs}: sum{v in {811..818}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_0_{h in Hs}: sum{v in {819,820}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_1_{h in Hs}: sum{v in {821,822}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_2_{h in Hs}: sum{v in {823,824}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_3_{h in Hs}: sum{v in {825,826}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_0_{h in Hs}: sum{v in {827..850}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_1_{h in Hs}: sum{v in {851..874}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_2_{h in Hs}: sum{v in {875..898}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_3_{h in Hs}: sum{v in {899..922}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_0_{h in Hs}: sum{v in {1051,1052}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_1_{h in Hs}: sum{v in {1053,1054}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_2_{h in Hs}: sum{v in {1055,1056}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_3_{h in Hs}: sum{v in {1057,1058}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm59_0_{h in Hs}: sum{v in {1164,1165}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm60_0_{h in Hs}: sum{v in {1166,1167}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm61_0_{h in Hs}: sum{v in {1168,1169}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm62_0_{h in Hs}: sum{v in {1170..1173}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm63_0_{h in Hs}: sum{v in {1174,1175}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm65_0_{h in Hs}: sum{v in {1176..1198}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm7_0_{h in Hs}: sum{v in {1226..1228}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm7_1_{h in Hs}: sum{v in {1229..1231}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm8_0_{h in Hs}: sum{v in {1232..1234}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm8_1_{h in Hs}: sum{v in {1235..1237}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm9_0_{h in Hs}: sum{v in {1238..1240}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm9_1_{h in Hs}: sum{v in {1241..1243}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm12_0_{h in Hs}: sum{v in {1280,1281}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm12_1_{h in Hs}: sum{v in {1282,1283}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm13_0_{h in Hs}: sum{v in {1284,1285}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm13_1_{h in Hs}: sum{v in {1286,1287}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm14_0_{h in Hs}: sum{v in {1288..1290}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm14_1_{h in Hs}: sum{v in {1291..1293}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm57_0_{h in Hs}: sum{v in {1330,1331}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm57_1_{h in Hs}: sum{v in {1332,1333}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm10_0_{h in Hs}: sum{v in {1334,1335}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm10_1_{h in Hs}: sum{v in {1336,1337}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm77_0_{h in Hs}: sum{v in {1338,1339}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm77_1_{h in Hs}: sum{v in {1340,1341}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm78_0_{h in Hs}: sum{v in {1342,1343}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm78_1_{h in Hs}: sum{v in {1344,1345}} x[v, h]<= 1;
s.t. AAF_vnf25_dot_vm80_0_{h in Hs}: sum{v in {1469}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm83_0_{h in Hs}: sum{v in {1475}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm84_0_{h in Hs}: sum{v in {1477}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm85_0_{h in Hs}: sum{v in {1478}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm90_0_{h in Hs}: sum{v in {1483}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm93_0_{h in Hs}: sum{v in {1486}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm94_0_{h in Hs}: sum{v in {1487}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm95_0_{h in Hs}: sum{v in {1488}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm96_0_{h in Hs}: sum{v in {1489}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm97_0_{h in Hs}: sum{v in {1490}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm98_0_{h in Hs}: sum{v in {1491}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm99_0_{h in Hs}: sum{v in {1492}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm100_0_{h in Hs}: sum{v in {1493}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm101_0_{h in Hs}: sum{v in {1494}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm102_0_{h in Hs}: sum{v in {1495}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm103_0_{h in Hs}: sum{v in {1496}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm104_0_{h in Hs}: sum{v in {1497}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm105_0_{h in Hs}: sum{v in {1498}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm106_0_{h in Hs}: sum{v in {1499}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm107_0_{h in Hs}: sum{v in {1500}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm108_0_{h in Hs}: sum{v in {1501}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm118_0_{h in Hs}: sum{v in {1511}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm119_0_{h in Hs}: sum{v in {1512}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm120_0_{h in Hs}: sum{v in {1513}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm121_0_{h in Hs}: sum{v in {1514}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm122_0_{h in Hs}: sum{v in {1515}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm123_0_{h in Hs}: sum{v in {1516}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm124_0_{h in Hs}: sum{v in {1517}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm125_0_{h in Hs}: sum{v in {1518}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0..19}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0..19}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {20..39}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {20..39}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_1_master{h in Hs}: ni_vnf1_dot_vm1_1_master[h] <= sum{v in {40..59}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_1_master{v in {40..59}, h in Hs}: ni_vnf1_dot_vm1_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_1_slave{h in Hs}: ni_vnf1_dot_vm1_1_slave[h] <= sum{v in {60..79}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_1_slave{v in {60..79}, h in Hs}: ni_vnf1_dot_vm1_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm1_1_master[h] + ni_vnf1_dot_vm1_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm3_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_master{h in Hs}: ni_vnf1_dot_vm3_0_master[h] <= sum{v in {86}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_master{v in {86}, h in Hs}: ni_vnf1_dot_vm3_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm3_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_slave{h in Hs}: ni_vnf1_dot_vm3_0_slave[h] <= sum{v in {87}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_slave{v in {87}, h in Hs}: ni_vnf1_dot_vm3_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf1_dot_vm3_0_master[h] + ni_vnf1_dot_vm3_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm3_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_1_master{h in Hs}: ni_vnf1_dot_vm3_1_master[h] <= sum{v in {88}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_1_master{v in {88}, h in Hs}: ni_vnf1_dot_vm3_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm3_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_1_slave{h in Hs}: ni_vnf1_dot_vm3_1_slave[h] <= sum{v in {89}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_1_slave{v in {89}, h in Hs}: ni_vnf1_dot_vm3_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf1_dot_vm3_1_master[h] + ni_vnf1_dot_vm3_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm4_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_0_master{h in Hs}: ni_vnf1_dot_vm4_0_master[h] <= sum{v in {90..92}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_0_master{v in {90..92}, h in Hs}: ni_vnf1_dot_vm4_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm4_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_0_slave{h in Hs}: ni_vnf1_dot_vm4_0_slave[h] <= sum{v in {93..95}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_0_slave{v in {93..95}, h in Hs}: ni_vnf1_dot_vm4_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf1_dot_vm4_0_master[h] + ni_vnf1_dot_vm4_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm4_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_1_master{h in Hs}: ni_vnf1_dot_vm4_1_master[h] <= sum{v in {96..98}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_1_master{v in {96..98}, h in Hs}: ni_vnf1_dot_vm4_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm4_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_1_slave{h in Hs}: ni_vnf1_dot_vm4_1_slave[h] <= sum{v in {99..101}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_1_slave{v in {99..101}, h in Hs}: ni_vnf1_dot_vm4_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf1_dot_vm4_1_master[h] + ni_vnf1_dot_vm4_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm5_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_0_master{h in Hs}: ni_vnf1_dot_vm5_0_master[h] <= sum{v in {102}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_0_master{v in {102}, h in Hs}: ni_vnf1_dot_vm5_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm5_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_0_slave{h in Hs}: ni_vnf1_dot_vm5_0_slave[h] <= sum{v in {103}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_0_slave{v in {103}, h in Hs}: ni_vnf1_dot_vm5_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_6_{h in Hs}: ni_vnf1_dot_vm5_0_master[h] + ni_vnf1_dot_vm5_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm5_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_1_master{h in Hs}: ni_vnf1_dot_vm5_1_master[h] <= sum{v in {104}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_1_master{v in {104}, h in Hs}: ni_vnf1_dot_vm5_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm5_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_1_slave{h in Hs}: ni_vnf1_dot_vm5_1_slave[h] <= sum{v in {105}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_1_slave{v in {105}, h in Hs}: ni_vnf1_dot_vm5_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_7_{h in Hs}: ni_vnf1_dot_vm5_1_master[h] + ni_vnf1_dot_vm5_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm6_0_master{h in Hs}: ni_vnf2_dot_vm6_0_master[h] <= sum{v in {106}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm6_0_master{v in {106}, h in Hs}: ni_vnf2_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm6_0_slave{h in Hs}: ni_vnf2_dot_vm6_0_slave[h] <= sum{v in {107}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm6_0_slave{v in {107}, h in Hs}: ni_vnf2_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_8_{h in Hs}: ni_vnf2_dot_vm6_0_master[h] + ni_vnf2_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm11_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_0_master{h in Hs}: ni_vnf3_dot_vm11_0_master[h] <= sum{v in {132..148}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_0_master{v in {132..148}, h in Hs}: ni_vnf3_dot_vm11_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm11_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_0_slave{h in Hs}: ni_vnf3_dot_vm11_0_slave[h] <= sum{v in {149..165}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_0_slave{v in {149..165}, h in Hs}: ni_vnf3_dot_vm11_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_9_{h in Hs}: ni_vnf3_dot_vm11_0_master[h] + ni_vnf3_dot_vm11_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm11_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_1_master{h in Hs}: ni_vnf3_dot_vm11_1_master[h] <= sum{v in {166..182}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_1_master{v in {166..182}, h in Hs}: ni_vnf3_dot_vm11_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm11_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_1_slave{h in Hs}: ni_vnf3_dot_vm11_1_slave[h] <= sum{v in {183..199}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_1_slave{v in {183..199}, h in Hs}: ni_vnf3_dot_vm11_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_10_{h in Hs}: ni_vnf3_dot_vm11_1_master[h] + ni_vnf3_dot_vm11_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_0_master{h in Hs}: ni_vnf3_dot_vm15_0_master[h] <= sum{v in {222}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_0_master{v in {222}, h in Hs}: ni_vnf3_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_0_slave{h in Hs}: ni_vnf3_dot_vm15_0_slave[h] <= sum{v in {223}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_0_slave{v in {223}, h in Hs}: ni_vnf3_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_11_{h in Hs}: ni_vnf3_dot_vm15_0_master[h] + ni_vnf3_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_1_master{h in Hs}: ni_vnf3_dot_vm15_1_master[h] <= sum{v in {224}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_1_master{v in {224}, h in Hs}: ni_vnf3_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_1_slave{h in Hs}: ni_vnf3_dot_vm15_1_slave[h] <= sum{v in {225}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_1_slave{v in {225}, h in Hs}: ni_vnf3_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_12_{h in Hs}: ni_vnf3_dot_vm15_1_master[h] + ni_vnf3_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_0_master{h in Hs}: ni_vnf3_dot_vm16_0_master[h] <= sum{v in {226}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_0_master{v in {226}, h in Hs}: ni_vnf3_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_0_slave{h in Hs}: ni_vnf3_dot_vm16_0_slave[h] <= sum{v in {227}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_0_slave{v in {227}, h in Hs}: ni_vnf3_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_13_{h in Hs}: ni_vnf3_dot_vm16_0_master[h] + ni_vnf3_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_1_master{h in Hs}: ni_vnf3_dot_vm16_1_master[h] <= sum{v in {228}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_1_master{v in {228}, h in Hs}: ni_vnf3_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_1_slave{h in Hs}: ni_vnf3_dot_vm16_1_slave[h] <= sum{v in {229}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_1_slave{v in {229}, h in Hs}: ni_vnf3_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_14_{h in Hs}: ni_vnf3_dot_vm16_1_master[h] + ni_vnf3_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_0_master{h in Hs}: ni_vnf3_dot_vm17_0_master[h] <= sum{v in {230..232}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_0_master{v in {230..232}, h in Hs}: ni_vnf3_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_0_slave{h in Hs}: ni_vnf3_dot_vm17_0_slave[h] <= sum{v in {233..235}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_0_slave{v in {233..235}, h in Hs}: ni_vnf3_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_15_{h in Hs}: ni_vnf3_dot_vm17_0_master[h] + ni_vnf3_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_1_master{h in Hs}: ni_vnf3_dot_vm17_1_master[h] <= sum{v in {236..238}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_1_master{v in {236..238}, h in Hs}: ni_vnf3_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_1_slave{h in Hs}: ni_vnf3_dot_vm17_1_slave[h] <= sum{v in {239..241}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_1_slave{v in {239..241}, h in Hs}: ni_vnf3_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_16_{h in Hs}: ni_vnf3_dot_vm17_1_master[h] + ni_vnf3_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_0_master{h in Hs}: ni_vnf3_dot_vm18_0_master[h] <= sum{v in {242}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_0_master{v in {242}, h in Hs}: ni_vnf3_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_0_slave{h in Hs}: ni_vnf3_dot_vm18_0_slave[h] <= sum{v in {243}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_0_slave{v in {243}, h in Hs}: ni_vnf3_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_17_{h in Hs}: ni_vnf3_dot_vm18_0_master[h] + ni_vnf3_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_1_master{h in Hs}: ni_vnf3_dot_vm18_1_master[h] <= sum{v in {244}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_1_master{v in {244}, h in Hs}: ni_vnf3_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_1_slave{h in Hs}: ni_vnf3_dot_vm18_1_slave[h] <= sum{v in {245}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_1_slave{v in {245}, h in Hs}: ni_vnf3_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_18_{h in Hs}: ni_vnf3_dot_vm18_1_master[h] + ni_vnf3_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm19_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_0_master{h in Hs}: ni_vnf3_dot_vm19_0_master[h] <= sum{v in {246}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_0_master{v in {246}, h in Hs}: ni_vnf3_dot_vm19_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm19_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_0_slave{h in Hs}: ni_vnf3_dot_vm19_0_slave[h] <= sum{v in {247}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_0_slave{v in {247}, h in Hs}: ni_vnf3_dot_vm19_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_19_{h in Hs}: ni_vnf3_dot_vm19_0_master[h] + ni_vnf3_dot_vm19_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm19_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_1_master{h in Hs}: ni_vnf3_dot_vm19_1_master[h] <= sum{v in {248}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_1_master{v in {248}, h in Hs}: ni_vnf3_dot_vm19_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm19_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_1_slave{h in Hs}: ni_vnf3_dot_vm19_1_slave[h] <= sum{v in {249}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_1_slave{v in {249}, h in Hs}: ni_vnf3_dot_vm19_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_20_{h in Hs}: ni_vnf3_dot_vm19_1_master[h] + ni_vnf3_dot_vm19_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm20_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_0_master{h in Hs}: ni_vnf3_dot_vm20_0_master[h] <= sum{v in {250}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_0_master{v in {250}, h in Hs}: ni_vnf3_dot_vm20_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm20_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_0_slave{h in Hs}: ni_vnf3_dot_vm20_0_slave[h] <= sum{v in {251}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_0_slave{v in {251}, h in Hs}: ni_vnf3_dot_vm20_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_21_{h in Hs}: ni_vnf3_dot_vm20_0_master[h] + ni_vnf3_dot_vm20_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm20_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_1_master{h in Hs}: ni_vnf3_dot_vm20_1_master[h] <= sum{v in {252}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_1_master{v in {252}, h in Hs}: ni_vnf3_dot_vm20_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm20_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_1_slave{h in Hs}: ni_vnf3_dot_vm20_1_slave[h] <= sum{v in {253}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_1_slave{v in {253}, h in Hs}: ni_vnf3_dot_vm20_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_22_{h in Hs}: ni_vnf3_dot_vm20_1_master[h] + ni_vnf3_dot_vm20_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm21_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_0_master{h in Hs}: ni_vnf3_dot_vm21_0_master[h] <= sum{v in {254}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_0_master{v in {254}, h in Hs}: ni_vnf3_dot_vm21_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm21_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_0_slave{h in Hs}: ni_vnf3_dot_vm21_0_slave[h] <= sum{v in {255}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_0_slave{v in {255}, h in Hs}: ni_vnf3_dot_vm21_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_23_{h in Hs}: ni_vnf3_dot_vm21_0_master[h] + ni_vnf3_dot_vm21_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm21_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_1_master{h in Hs}: ni_vnf3_dot_vm21_1_master[h] <= sum{v in {256}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_1_master{v in {256}, h in Hs}: ni_vnf3_dot_vm21_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm21_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_1_slave{h in Hs}: ni_vnf3_dot_vm21_1_slave[h] <= sum{v in {257}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_1_slave{v in {257}, h in Hs}: ni_vnf3_dot_vm21_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_24_{h in Hs}: ni_vnf3_dot_vm21_1_master[h] + ni_vnf3_dot_vm21_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm22_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_0_master{h in Hs}: ni_vnf3_dot_vm22_0_master[h] <= sum{v in {258}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_0_master{v in {258}, h in Hs}: ni_vnf3_dot_vm22_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm22_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_0_slave{h in Hs}: ni_vnf3_dot_vm22_0_slave[h] <= sum{v in {259}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_0_slave{v in {259}, h in Hs}: ni_vnf3_dot_vm22_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_25_{h in Hs}: ni_vnf3_dot_vm22_0_master[h] + ni_vnf3_dot_vm22_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm22_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_1_master{h in Hs}: ni_vnf3_dot_vm22_1_master[h] <= sum{v in {260}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_1_master{v in {260}, h in Hs}: ni_vnf3_dot_vm22_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm22_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_1_slave{h in Hs}: ni_vnf3_dot_vm22_1_slave[h] <= sum{v in {261}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_1_slave{v in {261}, h in Hs}: ni_vnf3_dot_vm22_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_26_{h in Hs}: ni_vnf3_dot_vm22_1_master[h] + ni_vnf3_dot_vm22_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_0_master{h in Hs}: ni_vnf7_dot_vm37_0_master[h] <= sum{v in {293..295}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_0_master{v in {293..295}, h in Hs}: ni_vnf7_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_0_slave{h in Hs}: ni_vnf7_dot_vm37_0_slave[h] <= sum{v in {296..298}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_0_slave{v in {296..298}, h in Hs}: ni_vnf7_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_27_{h in Hs}: ni_vnf7_dot_vm37_0_master[h] + ni_vnf7_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm37_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_1_master{h in Hs}: ni_vnf7_dot_vm37_1_master[h] <= sum{v in {299..301}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_1_master{v in {299..301}, h in Hs}: ni_vnf7_dot_vm37_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm37_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_1_slave{h in Hs}: ni_vnf7_dot_vm37_1_slave[h] <= sum{v in {302..304}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_1_slave{v in {302..304}, h in Hs}: ni_vnf7_dot_vm37_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_28_{h in Hs}: ni_vnf7_dot_vm37_1_master[h] + ni_vnf7_dot_vm37_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_0_master{h in Hs}: ni_vnf7_dot_vm38_0_master[h] <= sum{v in {305}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_0_master{v in {305}, h in Hs}: ni_vnf7_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_0_slave{h in Hs}: ni_vnf7_dot_vm38_0_slave[h] <= sum{v in {306}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_0_slave{v in {306}, h in Hs}: ni_vnf7_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_29_{h in Hs}: ni_vnf7_dot_vm38_0_master[h] + ni_vnf7_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm38_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_1_master{h in Hs}: ni_vnf7_dot_vm38_1_master[h] <= sum{v in {307}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_1_master{v in {307}, h in Hs}: ni_vnf7_dot_vm38_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm38_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_1_slave{h in Hs}: ni_vnf7_dot_vm38_1_slave[h] <= sum{v in {308}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_1_slave{v in {308}, h in Hs}: ni_vnf7_dot_vm38_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_30_{h in Hs}: ni_vnf7_dot_vm38_1_master[h] + ni_vnf7_dot_vm38_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_0_master{h in Hs}: ni_vnf7_dot_vm39_0_master[h] <= sum{v in {309}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_0_master{v in {309}, h in Hs}: ni_vnf7_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_0_slave{h in Hs}: ni_vnf7_dot_vm39_0_slave[h] <= sum{v in {310}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_0_slave{v in {310}, h in Hs}: ni_vnf7_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_31_{h in Hs}: ni_vnf7_dot_vm39_0_master[h] + ni_vnf7_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm39_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_1_master{h in Hs}: ni_vnf7_dot_vm39_1_master[h] <= sum{v in {311}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_1_master{v in {311}, h in Hs}: ni_vnf7_dot_vm39_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm39_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_1_slave{h in Hs}: ni_vnf7_dot_vm39_1_slave[h] <= sum{v in {312}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_1_slave{v in {312}, h in Hs}: ni_vnf7_dot_vm39_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_32_{h in Hs}: ni_vnf7_dot_vm39_1_master[h] + ni_vnf7_dot_vm39_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_0_master{h in Hs}: ni_vnf7_dot_vm40_0_master[h] <= sum{v in {313}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_0_master{v in {313}, h in Hs}: ni_vnf7_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_0_slave{h in Hs}: ni_vnf7_dot_vm40_0_slave[h] <= sum{v in {314}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_0_slave{v in {314}, h in Hs}: ni_vnf7_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_33_{h in Hs}: ni_vnf7_dot_vm40_0_master[h] + ni_vnf7_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm40_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_1_master{h in Hs}: ni_vnf7_dot_vm40_1_master[h] <= sum{v in {315}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_1_master{v in {315}, h in Hs}: ni_vnf7_dot_vm40_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm40_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_1_slave{h in Hs}: ni_vnf7_dot_vm40_1_slave[h] <= sum{v in {316}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_1_slave{v in {316}, h in Hs}: ni_vnf7_dot_vm40_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_34_{h in Hs}: ni_vnf7_dot_vm40_1_master[h] + ni_vnf7_dot_vm40_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_0_master{h in Hs}: ni_vnf7_dot_vm7_0_master[h] <= sum{v in {317}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_0_master{v in {317}, h in Hs}: ni_vnf7_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_0_slave{h in Hs}: ni_vnf7_dot_vm7_0_slave[h] <= sum{v in {318}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_0_slave{v in {318}, h in Hs}: ni_vnf7_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_35_{h in Hs}: ni_vnf7_dot_vm7_0_master[h] + ni_vnf7_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm7_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_1_master{h in Hs}: ni_vnf7_dot_vm7_1_master[h] <= sum{v in {319}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_1_master{v in {319}, h in Hs}: ni_vnf7_dot_vm7_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm7_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_1_slave{h in Hs}: ni_vnf7_dot_vm7_1_slave[h] <= sum{v in {320}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_1_slave{v in {320}, h in Hs}: ni_vnf7_dot_vm7_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_36_{h in Hs}: ni_vnf7_dot_vm7_1_master[h] + ni_vnf7_dot_vm7_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_0_master{h in Hs}: ni_vnf8_dot_vm37_0_master[h] <= sum{v in {321..325}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_0_master{v in {321..325}, h in Hs}: ni_vnf8_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_0_slave{h in Hs}: ni_vnf8_dot_vm37_0_slave[h] <= sum{v in {326..330}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_0_slave{v in {326..330}, h in Hs}: ni_vnf8_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_37_{h in Hs}: ni_vnf8_dot_vm37_0_master[h] + ni_vnf8_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm37_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_1_master{h in Hs}: ni_vnf8_dot_vm37_1_master[h] <= sum{v in {331..335}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_1_master{v in {331..335}, h in Hs}: ni_vnf8_dot_vm37_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm37_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_1_slave{h in Hs}: ni_vnf8_dot_vm37_1_slave[h] <= sum{v in {336..340}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_1_slave{v in {336..340}, h in Hs}: ni_vnf8_dot_vm37_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_38_{h in Hs}: ni_vnf8_dot_vm37_1_master[h] + ni_vnf8_dot_vm37_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_0_master{h in Hs}: ni_vnf8_dot_vm38_0_master[h] <= sum{v in {341}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_0_master{v in {341}, h in Hs}: ni_vnf8_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_0_slave{h in Hs}: ni_vnf8_dot_vm38_0_slave[h] <= sum{v in {342}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_0_slave{v in {342}, h in Hs}: ni_vnf8_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_39_{h in Hs}: ni_vnf8_dot_vm38_0_master[h] + ni_vnf8_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm38_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_1_master{h in Hs}: ni_vnf8_dot_vm38_1_master[h] <= sum{v in {343}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_1_master{v in {343}, h in Hs}: ni_vnf8_dot_vm38_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm38_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_1_slave{h in Hs}: ni_vnf8_dot_vm38_1_slave[h] <= sum{v in {344}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_1_slave{v in {344}, h in Hs}: ni_vnf8_dot_vm38_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_40_{h in Hs}: ni_vnf8_dot_vm38_1_master[h] + ni_vnf8_dot_vm38_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_0_master{h in Hs}: ni_vnf8_dot_vm39_0_master[h] <= sum{v in {345}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_0_master{v in {345}, h in Hs}: ni_vnf8_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_0_slave{h in Hs}: ni_vnf8_dot_vm39_0_slave[h] <= sum{v in {346}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_0_slave{v in {346}, h in Hs}: ni_vnf8_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_41_{h in Hs}: ni_vnf8_dot_vm39_0_master[h] + ni_vnf8_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm39_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_1_master{h in Hs}: ni_vnf8_dot_vm39_1_master[h] <= sum{v in {347}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_1_master{v in {347}, h in Hs}: ni_vnf8_dot_vm39_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm39_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_1_slave{h in Hs}: ni_vnf8_dot_vm39_1_slave[h] <= sum{v in {348}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_1_slave{v in {348}, h in Hs}: ni_vnf8_dot_vm39_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_42_{h in Hs}: ni_vnf8_dot_vm39_1_master[h] + ni_vnf8_dot_vm39_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_0_master{h in Hs}: ni_vnf8_dot_vm40_0_master[h] <= sum{v in {349}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_0_master{v in {349}, h in Hs}: ni_vnf8_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_0_slave{h in Hs}: ni_vnf8_dot_vm40_0_slave[h] <= sum{v in {350}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_0_slave{v in {350}, h in Hs}: ni_vnf8_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_43_{h in Hs}: ni_vnf8_dot_vm40_0_master[h] + ni_vnf8_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm40_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_1_master{h in Hs}: ni_vnf8_dot_vm40_1_master[h] <= sum{v in {351}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_1_master{v in {351}, h in Hs}: ni_vnf8_dot_vm40_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm40_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_1_slave{h in Hs}: ni_vnf8_dot_vm40_1_slave[h] <= sum{v in {352}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_1_slave{v in {352}, h in Hs}: ni_vnf8_dot_vm40_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_44_{h in Hs}: ni_vnf8_dot_vm40_1_master[h] + ni_vnf8_dot_vm40_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_0_master{h in Hs}: ni_vnf8_dot_vm7_0_master[h] <= sum{v in {353}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_0_master{v in {353}, h in Hs}: ni_vnf8_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_0_slave{h in Hs}: ni_vnf8_dot_vm7_0_slave[h] <= sum{v in {354}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_0_slave{v in {354}, h in Hs}: ni_vnf8_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_45_{h in Hs}: ni_vnf8_dot_vm7_0_master[h] + ni_vnf8_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm7_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_1_master{h in Hs}: ni_vnf8_dot_vm7_1_master[h] <= sum{v in {355}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_1_master{v in {355}, h in Hs}: ni_vnf8_dot_vm7_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm7_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_1_slave{h in Hs}: ni_vnf8_dot_vm7_1_slave[h] <= sum{v in {356}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_1_slave{v in {356}, h in Hs}: ni_vnf8_dot_vm7_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_46_{h in Hs}: ni_vnf8_dot_vm7_1_master[h] + ni_vnf8_dot_vm7_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm37_0_master{h in Hs}: ni_vnf9_dot_vm37_0_master[h] <= sum{v in {357,358}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm37_0_master{v in {357,358}, h in Hs}: ni_vnf9_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm37_0_slave{h in Hs}: ni_vnf9_dot_vm37_0_slave[h] <= sum{v in {359,360}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm37_0_slave{v in {359,360}, h in Hs}: ni_vnf9_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_47_{h in Hs}: ni_vnf9_dot_vm37_0_master[h] + ni_vnf9_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm38_0_master{h in Hs}: ni_vnf9_dot_vm38_0_master[h] <= sum{v in {361}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm38_0_master{v in {361}, h in Hs}: ni_vnf9_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm38_0_slave{h in Hs}: ni_vnf9_dot_vm38_0_slave[h] <= sum{v in {362}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm38_0_slave{v in {362}, h in Hs}: ni_vnf9_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_48_{h in Hs}: ni_vnf9_dot_vm38_0_master[h] + ni_vnf9_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm39_0_master{h in Hs}: ni_vnf9_dot_vm39_0_master[h] <= sum{v in {363}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm39_0_master{v in {363}, h in Hs}: ni_vnf9_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm39_0_slave{h in Hs}: ni_vnf9_dot_vm39_0_slave[h] <= sum{v in {364}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm39_0_slave{v in {364}, h in Hs}: ni_vnf9_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_49_{h in Hs}: ni_vnf9_dot_vm39_0_master[h] + ni_vnf9_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm40_0_master{h in Hs}: ni_vnf9_dot_vm40_0_master[h] <= sum{v in {365}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm40_0_master{v in {365}, h in Hs}: ni_vnf9_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm40_0_slave{h in Hs}: ni_vnf9_dot_vm40_0_slave[h] <= sum{v in {366}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm40_0_slave{v in {366}, h in Hs}: ni_vnf9_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_50_{h in Hs}: ni_vnf9_dot_vm40_0_master[h] + ni_vnf9_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm7_0_master{h in Hs}: ni_vnf9_dot_vm7_0_master[h] <= sum{v in {367}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm7_0_master{v in {367}, h in Hs}: ni_vnf9_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm7_0_slave{h in Hs}: ni_vnf9_dot_vm7_0_slave[h] <= sum{v in {368}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm7_0_slave{v in {368}, h in Hs}: ni_vnf9_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_51_{h in Hs}: ni_vnf9_dot_vm7_0_master[h] + ni_vnf9_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm50_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_0_master{h in Hs}: ni_vnf16_dot_vm50_0_master[h] <= sum{v in {399..411}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_0_master{v in {399..411}, h in Hs}: ni_vnf16_dot_vm50_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm50_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_0_slave{h in Hs}: ni_vnf16_dot_vm50_0_slave[h] <= sum{v in {412..424}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_0_slave{v in {412..424}, h in Hs}: ni_vnf16_dot_vm50_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_52_{h in Hs}: ni_vnf16_dot_vm50_0_master[h] + ni_vnf16_dot_vm50_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm50_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_1_master{h in Hs}: ni_vnf16_dot_vm50_1_master[h] <= sum{v in {425..437}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_1_master{v in {425..437}, h in Hs}: ni_vnf16_dot_vm50_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm50_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_1_slave{h in Hs}: ni_vnf16_dot_vm50_1_slave[h] <= sum{v in {438..450}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_1_slave{v in {438..450}, h in Hs}: ni_vnf16_dot_vm50_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_53_{h in Hs}: ni_vnf16_dot_vm50_1_master[h] + ni_vnf16_dot_vm50_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm50_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_2_master{h in Hs}: ni_vnf16_dot_vm50_2_master[h] <= sum{v in {451..463}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_2_master{v in {451..463}, h in Hs}: ni_vnf16_dot_vm50_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm50_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_2_slave{h in Hs}: ni_vnf16_dot_vm50_2_slave[h] <= sum{v in {464..476}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_2_slave{v in {464..476}, h in Hs}: ni_vnf16_dot_vm50_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_54_{h in Hs}: ni_vnf16_dot_vm50_2_master[h] + ni_vnf16_dot_vm50_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm52_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_0_master{h in Hs}: ni_vnf16_dot_vm52_0_master[h] <= sum{v in {486}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_0_master{v in {486}, h in Hs}: ni_vnf16_dot_vm52_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm52_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_0_slave{h in Hs}: ni_vnf16_dot_vm52_0_slave[h] <= sum{v in {487}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_0_slave{v in {487}, h in Hs}: ni_vnf16_dot_vm52_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_55_{h in Hs}: ni_vnf16_dot_vm52_0_master[h] + ni_vnf16_dot_vm52_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm52_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_1_master{h in Hs}: ni_vnf16_dot_vm52_1_master[h] <= sum{v in {488}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_1_master{v in {488}, h in Hs}: ni_vnf16_dot_vm52_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm52_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_1_slave{h in Hs}: ni_vnf16_dot_vm52_1_slave[h] <= sum{v in {489}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_1_slave{v in {489}, h in Hs}: ni_vnf16_dot_vm52_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_56_{h in Hs}: ni_vnf16_dot_vm52_1_master[h] + ni_vnf16_dot_vm52_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm52_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_2_master{h in Hs}: ni_vnf16_dot_vm52_2_master[h] <= sum{v in {490}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_2_master{v in {490}, h in Hs}: ni_vnf16_dot_vm52_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm52_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_2_slave{h in Hs}: ni_vnf16_dot_vm52_2_slave[h] <= sum{v in {491}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_2_slave{v in {491}, h in Hs}: ni_vnf16_dot_vm52_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_57_{h in Hs}: ni_vnf16_dot_vm52_2_master[h] + ni_vnf16_dot_vm52_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm53_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_0_master{h in Hs}: ni_vnf16_dot_vm53_0_master[h] <= sum{v in {492..495}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_0_master{v in {492..495}, h in Hs}: ni_vnf16_dot_vm53_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm53_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_0_slave{h in Hs}: ni_vnf16_dot_vm53_0_slave[h] <= sum{v in {496..499}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_0_slave{v in {496..499}, h in Hs}: ni_vnf16_dot_vm53_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_58_{h in Hs}: ni_vnf16_dot_vm53_0_master[h] + ni_vnf16_dot_vm53_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm53_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_1_master{h in Hs}: ni_vnf16_dot_vm53_1_master[h] <= sum{v in {500..503}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_1_master{v in {500..503}, h in Hs}: ni_vnf16_dot_vm53_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm53_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_1_slave{h in Hs}: ni_vnf16_dot_vm53_1_slave[h] <= sum{v in {504..507}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_1_slave{v in {504..507}, h in Hs}: ni_vnf16_dot_vm53_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_59_{h in Hs}: ni_vnf16_dot_vm53_1_master[h] + ni_vnf16_dot_vm53_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm53_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_2_master{h in Hs}: ni_vnf16_dot_vm53_2_master[h] <= sum{v in {508..511}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_2_master{v in {508..511}, h in Hs}: ni_vnf16_dot_vm53_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm53_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_2_slave{h in Hs}: ni_vnf16_dot_vm53_2_slave[h] <= sum{v in {512..515}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_2_slave{v in {512..515}, h in Hs}: ni_vnf16_dot_vm53_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_60_{h in Hs}: ni_vnf16_dot_vm53_2_master[h] + ni_vnf16_dot_vm53_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm54_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_0_master{h in Hs}: ni_vnf16_dot_vm54_0_master[h] <= sum{v in {516}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_0_master{v in {516}, h in Hs}: ni_vnf16_dot_vm54_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm54_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_0_slave{h in Hs}: ni_vnf16_dot_vm54_0_slave[h] <= sum{v in {517}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_0_slave{v in {517}, h in Hs}: ni_vnf16_dot_vm54_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_61_{h in Hs}: ni_vnf16_dot_vm54_0_master[h] + ni_vnf16_dot_vm54_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm54_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_1_master{h in Hs}: ni_vnf16_dot_vm54_1_master[h] <= sum{v in {518}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_1_master{v in {518}, h in Hs}: ni_vnf16_dot_vm54_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm54_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_1_slave{h in Hs}: ni_vnf16_dot_vm54_1_slave[h] <= sum{v in {519}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_1_slave{v in {519}, h in Hs}: ni_vnf16_dot_vm54_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_62_{h in Hs}: ni_vnf16_dot_vm54_1_master[h] + ni_vnf16_dot_vm54_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm54_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_2_master{h in Hs}: ni_vnf16_dot_vm54_2_master[h] <= sum{v in {520}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_2_master{v in {520}, h in Hs}: ni_vnf16_dot_vm54_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm54_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_2_slave{h in Hs}: ni_vnf16_dot_vm54_2_slave[h] <= sum{v in {521}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_2_slave{v in {521}, h in Hs}: ni_vnf16_dot_vm54_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_63_{h in Hs}: ni_vnf16_dot_vm54_2_master[h] + ni_vnf16_dot_vm54_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm55_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm55_0_master{h in Hs}: ni_vnf16_dot_vm55_0_master[h] <= sum{v in {522}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm55_0_master{v in {522}, h in Hs}: ni_vnf16_dot_vm55_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm55_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm55_0_slave{h in Hs}: ni_vnf16_dot_vm55_0_slave[h] <= sum{v in {523}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm55_0_slave{v in {523}, h in Hs}: ni_vnf16_dot_vm55_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_64_{h in Hs}: ni_vnf16_dot_vm55_0_master[h] + ni_vnf16_dot_vm55_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm11_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_0_master{h in Hs}: ni_vnf17_dot_vm11_0_master[h] <= sum{v in {554..570}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_0_master{v in {554..570}, h in Hs}: ni_vnf17_dot_vm11_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm11_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_0_slave{h in Hs}: ni_vnf17_dot_vm11_0_slave[h] <= sum{v in {571..587}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_0_slave{v in {571..587}, h in Hs}: ni_vnf17_dot_vm11_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_65_{h in Hs}: ni_vnf17_dot_vm11_0_master[h] + ni_vnf17_dot_vm11_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm11_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_1_master{h in Hs}: ni_vnf17_dot_vm11_1_master[h] <= sum{v in {588..604}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_1_master{v in {588..604}, h in Hs}: ni_vnf17_dot_vm11_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm11_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_1_slave{h in Hs}: ni_vnf17_dot_vm11_1_slave[h] <= sum{v in {605..621}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_1_slave{v in {605..621}, h in Hs}: ni_vnf17_dot_vm11_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_66_{h in Hs}: ni_vnf17_dot_vm11_1_master[h] + ni_vnf17_dot_vm11_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm11_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_2_master{h in Hs}: ni_vnf17_dot_vm11_2_master[h] <= sum{v in {622..638}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_2_master{v in {622..638}, h in Hs}: ni_vnf17_dot_vm11_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm11_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_2_slave{h in Hs}: ni_vnf17_dot_vm11_2_slave[h] <= sum{v in {639..655}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_2_slave{v in {639..655}, h in Hs}: ni_vnf17_dot_vm11_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_67_{h in Hs}: ni_vnf17_dot_vm11_2_master[h] + ni_vnf17_dot_vm11_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_0_master{h in Hs}: ni_vnf17_dot_vm15_0_master[h] <= sum{v in {692}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_0_master{v in {692}, h in Hs}: ni_vnf17_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_0_slave{h in Hs}: ni_vnf17_dot_vm15_0_slave[h] <= sum{v in {693}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_0_slave{v in {693}, h in Hs}: ni_vnf17_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_68_{h in Hs}: ni_vnf17_dot_vm15_0_master[h] + ni_vnf17_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_1_master{h in Hs}: ni_vnf17_dot_vm15_1_master[h] <= sum{v in {694}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_1_master{v in {694}, h in Hs}: ni_vnf17_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_1_slave{h in Hs}: ni_vnf17_dot_vm15_1_slave[h] <= sum{v in {695}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_1_slave{v in {695}, h in Hs}: ni_vnf17_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_69_{h in Hs}: ni_vnf17_dot_vm15_1_master[h] + ni_vnf17_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm15_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_2_master{h in Hs}: ni_vnf17_dot_vm15_2_master[h] <= sum{v in {696}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_2_master{v in {696}, h in Hs}: ni_vnf17_dot_vm15_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm15_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_2_slave{h in Hs}: ni_vnf17_dot_vm15_2_slave[h] <= sum{v in {697}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_2_slave{v in {697}, h in Hs}: ni_vnf17_dot_vm15_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_70_{h in Hs}: ni_vnf17_dot_vm15_2_master[h] + ni_vnf17_dot_vm15_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_0_master{h in Hs}: ni_vnf17_dot_vm16_0_master[h] <= sum{v in {698}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_0_master{v in {698}, h in Hs}: ni_vnf17_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_0_slave{h in Hs}: ni_vnf17_dot_vm16_0_slave[h] <= sum{v in {699}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_0_slave{v in {699}, h in Hs}: ni_vnf17_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_71_{h in Hs}: ni_vnf17_dot_vm16_0_master[h] + ni_vnf17_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_1_master{h in Hs}: ni_vnf17_dot_vm16_1_master[h] <= sum{v in {700}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_1_master{v in {700}, h in Hs}: ni_vnf17_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_1_slave{h in Hs}: ni_vnf17_dot_vm16_1_slave[h] <= sum{v in {701}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_1_slave{v in {701}, h in Hs}: ni_vnf17_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_72_{h in Hs}: ni_vnf17_dot_vm16_1_master[h] + ni_vnf17_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm16_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_2_master{h in Hs}: ni_vnf17_dot_vm16_2_master[h] <= sum{v in {702}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_2_master{v in {702}, h in Hs}: ni_vnf17_dot_vm16_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm16_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_2_slave{h in Hs}: ni_vnf17_dot_vm16_2_slave[h] <= sum{v in {703}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_2_slave{v in {703}, h in Hs}: ni_vnf17_dot_vm16_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_73_{h in Hs}: ni_vnf17_dot_vm16_2_master[h] + ni_vnf17_dot_vm16_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_0_master{h in Hs}: ni_vnf17_dot_vm17_0_master[h] <= sum{v in {704..706}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_0_master{v in {704..706}, h in Hs}: ni_vnf17_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_0_slave{h in Hs}: ni_vnf17_dot_vm17_0_slave[h] <= sum{v in {707..709}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_0_slave{v in {707..709}, h in Hs}: ni_vnf17_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_74_{h in Hs}: ni_vnf17_dot_vm17_0_master[h] + ni_vnf17_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_1_master{h in Hs}: ni_vnf17_dot_vm17_1_master[h] <= sum{v in {710..712}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_1_master{v in {710..712}, h in Hs}: ni_vnf17_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_1_slave{h in Hs}: ni_vnf17_dot_vm17_1_slave[h] <= sum{v in {713..715}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_1_slave{v in {713..715}, h in Hs}: ni_vnf17_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_75_{h in Hs}: ni_vnf17_dot_vm17_1_master[h] + ni_vnf17_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm17_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_2_master{h in Hs}: ni_vnf17_dot_vm17_2_master[h] <= sum{v in {716..718}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_2_master{v in {716..718}, h in Hs}: ni_vnf17_dot_vm17_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm17_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_2_slave{h in Hs}: ni_vnf17_dot_vm17_2_slave[h] <= sum{v in {719..721}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_2_slave{v in {719..721}, h in Hs}: ni_vnf17_dot_vm17_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_76_{h in Hs}: ni_vnf17_dot_vm17_2_master[h] + ni_vnf17_dot_vm17_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_0_master{h in Hs}: ni_vnf17_dot_vm18_0_master[h] <= sum{v in {722}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_0_master{v in {722}, h in Hs}: ni_vnf17_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_0_slave{h in Hs}: ni_vnf17_dot_vm18_0_slave[h] <= sum{v in {723}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_0_slave{v in {723}, h in Hs}: ni_vnf17_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_77_{h in Hs}: ni_vnf17_dot_vm18_0_master[h] + ni_vnf17_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_1_master{h in Hs}: ni_vnf17_dot_vm18_1_master[h] <= sum{v in {724}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_1_master{v in {724}, h in Hs}: ni_vnf17_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_1_slave{h in Hs}: ni_vnf17_dot_vm18_1_slave[h] <= sum{v in {725}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_1_slave{v in {725}, h in Hs}: ni_vnf17_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_78_{h in Hs}: ni_vnf17_dot_vm18_1_master[h] + ni_vnf17_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm18_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_2_master{h in Hs}: ni_vnf17_dot_vm18_2_master[h] <= sum{v in {726}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_2_master{v in {726}, h in Hs}: ni_vnf17_dot_vm18_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm18_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_2_slave{h in Hs}: ni_vnf17_dot_vm18_2_slave[h] <= sum{v in {727}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_2_slave{v in {727}, h in Hs}: ni_vnf17_dot_vm18_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_79_{h in Hs}: ni_vnf17_dot_vm18_2_master[h] + ni_vnf17_dot_vm18_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm20_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_0_master{h in Hs}: ni_vnf17_dot_vm20_0_master[h] <= sum{v in {728}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_0_master{v in {728}, h in Hs}: ni_vnf17_dot_vm20_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm20_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_0_slave{h in Hs}: ni_vnf17_dot_vm20_0_slave[h] <= sum{v in {729}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_0_slave{v in {729}, h in Hs}: ni_vnf17_dot_vm20_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_80_{h in Hs}: ni_vnf17_dot_vm20_0_master[h] + ni_vnf17_dot_vm20_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm20_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_1_master{h in Hs}: ni_vnf17_dot_vm20_1_master[h] <= sum{v in {730}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_1_master{v in {730}, h in Hs}: ni_vnf17_dot_vm20_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm20_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_1_slave{h in Hs}: ni_vnf17_dot_vm20_1_slave[h] <= sum{v in {731}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_1_slave{v in {731}, h in Hs}: ni_vnf17_dot_vm20_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_81_{h in Hs}: ni_vnf17_dot_vm20_1_master[h] + ni_vnf17_dot_vm20_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm20_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_2_master{h in Hs}: ni_vnf17_dot_vm20_2_master[h] <= sum{v in {732}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_2_master{v in {732}, h in Hs}: ni_vnf17_dot_vm20_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm20_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_2_slave{h in Hs}: ni_vnf17_dot_vm20_2_slave[h] <= sum{v in {733}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_2_slave{v in {733}, h in Hs}: ni_vnf17_dot_vm20_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_82_{h in Hs}: ni_vnf17_dot_vm20_2_master[h] + ni_vnf17_dot_vm20_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm21_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_0_master{h in Hs}: ni_vnf17_dot_vm21_0_master[h] <= sum{v in {734,735}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_0_master{v in {734,735}, h in Hs}: ni_vnf17_dot_vm21_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm21_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_0_slave{h in Hs}: ni_vnf17_dot_vm21_0_slave[h] <= sum{v in {736,737}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_0_slave{v in {736,737}, h in Hs}: ni_vnf17_dot_vm21_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_83_{h in Hs}: ni_vnf17_dot_vm21_0_master[h] + ni_vnf17_dot_vm21_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm21_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_1_master{h in Hs}: ni_vnf17_dot_vm21_1_master[h] <= sum{v in {738,739}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_1_master{v in {738,739}, h in Hs}: ni_vnf17_dot_vm21_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm21_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_1_slave{h in Hs}: ni_vnf17_dot_vm21_1_slave[h] <= sum{v in {740,741}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_1_slave{v in {740,741}, h in Hs}: ni_vnf17_dot_vm21_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_84_{h in Hs}: ni_vnf17_dot_vm21_1_master[h] + ni_vnf17_dot_vm21_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm21_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_2_master{h in Hs}: ni_vnf17_dot_vm21_2_master[h] <= sum{v in {742,743}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_2_master{v in {742,743}, h in Hs}: ni_vnf17_dot_vm21_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm21_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_2_slave{h in Hs}: ni_vnf17_dot_vm21_2_slave[h] <= sum{v in {744,745}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_2_slave{v in {744,745}, h in Hs}: ni_vnf17_dot_vm21_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_85_{h in Hs}: ni_vnf17_dot_vm21_2_master[h] + ni_vnf17_dot_vm21_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm22_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_0_master{h in Hs}: ni_vnf17_dot_vm22_0_master[h] <= sum{v in {746}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_0_master{v in {746}, h in Hs}: ni_vnf17_dot_vm22_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm22_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_0_slave{h in Hs}: ni_vnf17_dot_vm22_0_slave[h] <= sum{v in {747}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_0_slave{v in {747}, h in Hs}: ni_vnf17_dot_vm22_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_86_{h in Hs}: ni_vnf17_dot_vm22_0_master[h] + ni_vnf17_dot_vm22_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm22_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_1_master{h in Hs}: ni_vnf17_dot_vm22_1_master[h] <= sum{v in {748}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_1_master{v in {748}, h in Hs}: ni_vnf17_dot_vm22_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm22_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_1_slave{h in Hs}: ni_vnf17_dot_vm22_1_slave[h] <= sum{v in {749}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_1_slave{v in {749}, h in Hs}: ni_vnf17_dot_vm22_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_87_{h in Hs}: ni_vnf17_dot_vm22_1_master[h] + ni_vnf17_dot_vm22_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm22_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_2_master{h in Hs}: ni_vnf17_dot_vm22_2_master[h] <= sum{v in {750}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_2_master{v in {750}, h in Hs}: ni_vnf17_dot_vm22_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm22_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_2_slave{h in Hs}: ni_vnf17_dot_vm22_2_slave[h] <= sum{v in {751}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_2_slave{v in {751}, h in Hs}: ni_vnf17_dot_vm22_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_88_{h in Hs}: ni_vnf17_dot_vm22_2_master[h] + ni_vnf17_dot_vm22_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_0_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_0_master{h in Hs}: ni_vnf18_dot_vm65_0_master[h] <= sum{v in {923..938}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_0_master{v in {923..938}, h in Hs}: ni_vnf18_dot_vm65_0_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_0_slave{h in Hs}: ni_vnf18_dot_vm65_0_slave[h] <= sum{v in {939..954}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_0_slave{v in {939..954}, h in Hs}: ni_vnf18_dot_vm65_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_89_{h in Hs}: ni_vnf18_dot_vm65_0_master[h] + ni_vnf18_dot_vm65_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_1_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_1_master{h in Hs}: ni_vnf18_dot_vm65_1_master[h] <= sum{v in {955..970}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_1_master{v in {955..970}, h in Hs}: ni_vnf18_dot_vm65_1_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_1_slave{h in Hs}: ni_vnf18_dot_vm65_1_slave[h] <= sum{v in {971..986}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_1_slave{v in {971..986}, h in Hs}: ni_vnf18_dot_vm65_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_90_{h in Hs}: ni_vnf18_dot_vm65_1_master[h] + ni_vnf18_dot_vm65_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_2_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_2_master{h in Hs}: ni_vnf18_dot_vm65_2_master[h] <= sum{v in {987..1002}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_2_master{v in {987..1002}, h in Hs}: ni_vnf18_dot_vm65_2_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_2_slave{h in Hs}: ni_vnf18_dot_vm65_2_slave[h] <= sum{v in {1003..1018}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_2_slave{v in {1003..1018}, h in Hs}: ni_vnf18_dot_vm65_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_91_{h in Hs}: ni_vnf18_dot_vm65_2_master[h] + ni_vnf18_dot_vm65_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_3_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_3_master{h in Hs}: ni_vnf18_dot_vm65_3_master[h] <= sum{v in {1019..1034}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_3_master{v in {1019..1034}, h in Hs}: ni_vnf18_dot_vm65_3_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_3_slave{h in Hs}: ni_vnf18_dot_vm65_3_slave[h] <= sum{v in {1035..1050}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_3_slave{v in {1035..1050}, h in Hs}: ni_vnf18_dot_vm65_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_92_{h in Hs}: ni_vnf18_dot_vm65_3_master[h] + ni_vnf18_dot_vm65_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_0_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_0_master{h in Hs}: ni_vnf18_dot_vm67_0_master[h] <= sum{v in {1059,1060}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_0_master{v in {1059,1060}, h in Hs}: ni_vnf18_dot_vm67_0_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_0_slave{h in Hs}: ni_vnf18_dot_vm67_0_slave[h] <= sum{v in {1061,1062}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_0_slave{v in {1061,1062}, h in Hs}: ni_vnf18_dot_vm67_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_93_{h in Hs}: ni_vnf18_dot_vm67_0_master[h] + ni_vnf18_dot_vm67_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_1_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_1_master{h in Hs}: ni_vnf18_dot_vm67_1_master[h] <= sum{v in {1063,1064}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_1_master{v in {1063,1064}, h in Hs}: ni_vnf18_dot_vm67_1_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_1_slave{h in Hs}: ni_vnf18_dot_vm67_1_slave[h] <= sum{v in {1065,1066}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_1_slave{v in {1065,1066}, h in Hs}: ni_vnf18_dot_vm67_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_94_{h in Hs}: ni_vnf18_dot_vm67_1_master[h] + ni_vnf18_dot_vm67_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_2_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_2_master{h in Hs}: ni_vnf18_dot_vm67_2_master[h] <= sum{v in {1067,1068}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_2_master{v in {1067,1068}, h in Hs}: ni_vnf18_dot_vm67_2_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_2_slave{h in Hs}: ni_vnf18_dot_vm67_2_slave[h] <= sum{v in {1069,1070}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_2_slave{v in {1069,1070}, h in Hs}: ni_vnf18_dot_vm67_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_95_{h in Hs}: ni_vnf18_dot_vm67_2_master[h] + ni_vnf18_dot_vm67_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_3_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_3_master{h in Hs}: ni_vnf18_dot_vm67_3_master[h] <= sum{v in {1071,1072}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_3_master{v in {1071,1072}, h in Hs}: ni_vnf18_dot_vm67_3_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_3_slave{h in Hs}: ni_vnf18_dot_vm67_3_slave[h] <= sum{v in {1073,1074}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_3_slave{v in {1073,1074}, h in Hs}: ni_vnf18_dot_vm67_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_96_{h in Hs}: ni_vnf18_dot_vm67_3_master[h] + ni_vnf18_dot_vm67_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_0_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_0_master{h in Hs}: ni_vnf18_dot_vm68_0_master[h] <= sum{v in {1075..1077}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_0_master{v in {1075..1077}, h in Hs}: ni_vnf18_dot_vm68_0_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_0_slave{h in Hs}: ni_vnf18_dot_vm68_0_slave[h] <= sum{v in {1078..1080}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_0_slave{v in {1078..1080}, h in Hs}: ni_vnf18_dot_vm68_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_97_{h in Hs}: ni_vnf18_dot_vm68_0_master[h] + ni_vnf18_dot_vm68_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_1_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_1_master{h in Hs}: ni_vnf18_dot_vm68_1_master[h] <= sum{v in {1081..1083}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_1_master{v in {1081..1083}, h in Hs}: ni_vnf18_dot_vm68_1_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_1_slave{h in Hs}: ni_vnf18_dot_vm68_1_slave[h] <= sum{v in {1084..1086}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_1_slave{v in {1084..1086}, h in Hs}: ni_vnf18_dot_vm68_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_98_{h in Hs}: ni_vnf18_dot_vm68_1_master[h] + ni_vnf18_dot_vm68_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_2_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_2_master{h in Hs}: ni_vnf18_dot_vm68_2_master[h] <= sum{v in {1087..1089}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_2_master{v in {1087..1089}, h in Hs}: ni_vnf18_dot_vm68_2_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_2_slave{h in Hs}: ni_vnf18_dot_vm68_2_slave[h] <= sum{v in {1090..1092}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_2_slave{v in {1090..1092}, h in Hs}: ni_vnf18_dot_vm68_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_99_{h in Hs}: ni_vnf18_dot_vm68_2_master[h] + ni_vnf18_dot_vm68_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_3_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_3_master{h in Hs}: ni_vnf18_dot_vm68_3_master[h] <= sum{v in {1093..1095}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_3_master{v in {1093..1095}, h in Hs}: ni_vnf18_dot_vm68_3_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_3_slave{h in Hs}: ni_vnf18_dot_vm68_3_slave[h] <= sum{v in {1096..1098}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_3_slave{v in {1096..1098}, h in Hs}: ni_vnf18_dot_vm68_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_100_{h in Hs}: ni_vnf18_dot_vm68_3_master[h] + ni_vnf18_dot_vm68_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf19_dot_vm66_0_master{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm66_0_master{h in Hs}: ni_vnf19_dot_vm66_0_master[h] <= sum{v in {1199}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm66_0_master{v in {1199}, h in Hs}: ni_vnf19_dot_vm66_0_master[h] >= x[v, h];
var ni_vnf19_dot_vm66_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm66_0_slave{h in Hs}: ni_vnf19_dot_vm66_0_slave[h] <= sum{v in {1200}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm66_0_slave{v in {1200}, h in Hs}: ni_vnf19_dot_vm66_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_101_{h in Hs}: ni_vnf19_dot_vm66_0_master[h] + ni_vnf19_dot_vm66_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf19_dot_vm67_0_master{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm67_0_master{h in Hs}: ni_vnf19_dot_vm67_0_master[h] <= sum{v in {1201,1202}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm67_0_master{v in {1201,1202}, h in Hs}: ni_vnf19_dot_vm67_0_master[h] >= x[v, h];
var ni_vnf19_dot_vm67_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm67_0_slave{h in Hs}: ni_vnf19_dot_vm67_0_slave[h] <= sum{v in {1203,1204}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm67_0_slave{v in {1203,1204}, h in Hs}: ni_vnf19_dot_vm67_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_102_{h in Hs}: ni_vnf19_dot_vm67_0_master[h] + ni_vnf19_dot_vm67_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf19_dot_vm68_0_master{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm68_0_master{h in Hs}: ni_vnf19_dot_vm68_0_master[h] <= sum{v in {1205,1206}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm68_0_master{v in {1205,1206}, h in Hs}: ni_vnf19_dot_vm68_0_master[h] >= x[v, h];
var ni_vnf19_dot_vm68_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm68_0_slave{h in Hs}: ni_vnf19_dot_vm68_0_slave[h] <= sum{v in {1207,1208}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm68_0_slave{v in {1207,1208}, h in Hs}: ni_vnf19_dot_vm68_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_103_{h in Hs}: ni_vnf19_dot_vm68_0_master[h] + ni_vnf19_dot_vm68_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm11_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_0_master{h in Hs}: ni_vnf20_dot_vm11_0_master[h] <= sum{v in {1244..1252}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_0_master{v in {1244..1252}, h in Hs}: ni_vnf20_dot_vm11_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm11_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_0_slave{h in Hs}: ni_vnf20_dot_vm11_0_slave[h] <= sum{v in {1253..1261}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_0_slave{v in {1253..1261}, h in Hs}: ni_vnf20_dot_vm11_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_104_{h in Hs}: ni_vnf20_dot_vm11_0_master[h] + ni_vnf20_dot_vm11_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm11_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_1_master{h in Hs}: ni_vnf20_dot_vm11_1_master[h] <= sum{v in {1262..1270}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_1_master{v in {1262..1270}, h in Hs}: ni_vnf20_dot_vm11_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm11_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_1_slave{h in Hs}: ni_vnf20_dot_vm11_1_slave[h] <= sum{v in {1271..1279}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_1_slave{v in {1271..1279}, h in Hs}: ni_vnf20_dot_vm11_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_105_{h in Hs}: ni_vnf20_dot_vm11_1_master[h] + ni_vnf20_dot_vm11_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_0_master{h in Hs}: ni_vnf20_dot_vm15_0_master[h] <= sum{v in {1294}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_0_master{v in {1294}, h in Hs}: ni_vnf20_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_0_slave{h in Hs}: ni_vnf20_dot_vm15_0_slave[h] <= sum{v in {1295}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_0_slave{v in {1295}, h in Hs}: ni_vnf20_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_106_{h in Hs}: ni_vnf20_dot_vm15_0_master[h] + ni_vnf20_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_1_master{h in Hs}: ni_vnf20_dot_vm15_1_master[h] <= sum{v in {1296}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_1_master{v in {1296}, h in Hs}: ni_vnf20_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_1_slave{h in Hs}: ni_vnf20_dot_vm15_1_slave[h] <= sum{v in {1297}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_1_slave{v in {1297}, h in Hs}: ni_vnf20_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_107_{h in Hs}: ni_vnf20_dot_vm15_1_master[h] + ni_vnf20_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_0_master{h in Hs}: ni_vnf20_dot_vm16_0_master[h] <= sum{v in {1298}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_0_master{v in {1298}, h in Hs}: ni_vnf20_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_0_slave{h in Hs}: ni_vnf20_dot_vm16_0_slave[h] <= sum{v in {1299}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_0_slave{v in {1299}, h in Hs}: ni_vnf20_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_108_{h in Hs}: ni_vnf20_dot_vm16_0_master[h] + ni_vnf20_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_1_master{h in Hs}: ni_vnf20_dot_vm16_1_master[h] <= sum{v in {1300}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_1_master{v in {1300}, h in Hs}: ni_vnf20_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_1_slave{h in Hs}: ni_vnf20_dot_vm16_1_slave[h] <= sum{v in {1301}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_1_slave{v in {1301}, h in Hs}: ni_vnf20_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_109_{h in Hs}: ni_vnf20_dot_vm16_1_master[h] + ni_vnf20_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_0_master{h in Hs}: ni_vnf20_dot_vm17_0_master[h] <= sum{v in {1302,1303}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_0_master{v in {1302,1303}, h in Hs}: ni_vnf20_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_0_slave{h in Hs}: ni_vnf20_dot_vm17_0_slave[h] <= sum{v in {1304,1305}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_0_slave{v in {1304,1305}, h in Hs}: ni_vnf20_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_110_{h in Hs}: ni_vnf20_dot_vm17_0_master[h] + ni_vnf20_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_1_master{h in Hs}: ni_vnf20_dot_vm17_1_master[h] <= sum{v in {1306,1307}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_1_master{v in {1306,1307}, h in Hs}: ni_vnf20_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_1_slave{h in Hs}: ni_vnf20_dot_vm17_1_slave[h] <= sum{v in {1308,1309}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_1_slave{v in {1308,1309}, h in Hs}: ni_vnf20_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_111_{h in Hs}: ni_vnf20_dot_vm17_1_master[h] + ni_vnf20_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_0_master{h in Hs}: ni_vnf20_dot_vm18_0_master[h] <= sum{v in {1310}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_0_master{v in {1310}, h in Hs}: ni_vnf20_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_0_slave{h in Hs}: ni_vnf20_dot_vm18_0_slave[h] <= sum{v in {1311}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_0_slave{v in {1311}, h in Hs}: ni_vnf20_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_112_{h in Hs}: ni_vnf20_dot_vm18_0_master[h] + ni_vnf20_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_1_master{h in Hs}: ni_vnf20_dot_vm18_1_master[h] <= sum{v in {1312}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_1_master{v in {1312}, h in Hs}: ni_vnf20_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_1_slave{h in Hs}: ni_vnf20_dot_vm18_1_slave[h] <= sum{v in {1313}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_1_slave{v in {1313}, h in Hs}: ni_vnf20_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_113_{h in Hs}: ni_vnf20_dot_vm18_1_master[h] + ni_vnf20_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm20_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_0_master{h in Hs}: ni_vnf20_dot_vm20_0_master[h] <= sum{v in {1314}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_0_master{v in {1314}, h in Hs}: ni_vnf20_dot_vm20_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm20_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_0_slave{h in Hs}: ni_vnf20_dot_vm20_0_slave[h] <= sum{v in {1315}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_0_slave{v in {1315}, h in Hs}: ni_vnf20_dot_vm20_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_114_{h in Hs}: ni_vnf20_dot_vm20_0_master[h] + ni_vnf20_dot_vm20_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm20_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_1_master{h in Hs}: ni_vnf20_dot_vm20_1_master[h] <= sum{v in {1316}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_1_master{v in {1316}, h in Hs}: ni_vnf20_dot_vm20_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm20_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_1_slave{h in Hs}: ni_vnf20_dot_vm20_1_slave[h] <= sum{v in {1317}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_1_slave{v in {1317}, h in Hs}: ni_vnf20_dot_vm20_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_115_{h in Hs}: ni_vnf20_dot_vm20_1_master[h] + ni_vnf20_dot_vm20_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm21_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_0_master{h in Hs}: ni_vnf20_dot_vm21_0_master[h] <= sum{v in {1318}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_0_master{v in {1318}, h in Hs}: ni_vnf20_dot_vm21_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm21_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_0_slave{h in Hs}: ni_vnf20_dot_vm21_0_slave[h] <= sum{v in {1319}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_0_slave{v in {1319}, h in Hs}: ni_vnf20_dot_vm21_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_116_{h in Hs}: ni_vnf20_dot_vm21_0_master[h] + ni_vnf20_dot_vm21_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm21_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_1_master{h in Hs}: ni_vnf20_dot_vm21_1_master[h] <= sum{v in {1320}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_1_master{v in {1320}, h in Hs}: ni_vnf20_dot_vm21_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm21_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_1_slave{h in Hs}: ni_vnf20_dot_vm21_1_slave[h] <= sum{v in {1321}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_1_slave{v in {1321}, h in Hs}: ni_vnf20_dot_vm21_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_117_{h in Hs}: ni_vnf20_dot_vm21_1_master[h] + ni_vnf20_dot_vm21_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm22_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_0_master{h in Hs}: ni_vnf20_dot_vm22_0_master[h] <= sum{v in {1322}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_0_master{v in {1322}, h in Hs}: ni_vnf20_dot_vm22_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm22_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_0_slave{h in Hs}: ni_vnf20_dot_vm22_0_slave[h] <= sum{v in {1323}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_0_slave{v in {1323}, h in Hs}: ni_vnf20_dot_vm22_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_118_{h in Hs}: ni_vnf20_dot_vm22_0_master[h] + ni_vnf20_dot_vm22_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm22_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_1_master{h in Hs}: ni_vnf20_dot_vm22_1_master[h] <= sum{v in {1324}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_1_master{v in {1324}, h in Hs}: ni_vnf20_dot_vm22_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm22_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_1_slave{h in Hs}: ni_vnf20_dot_vm22_1_slave[h] <= sum{v in {1325}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_1_slave{v in {1325}, h in Hs}: ni_vnf20_dot_vm22_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_119_{h in Hs}: ni_vnf20_dot_vm22_1_master[h] + ni_vnf20_dot_vm22_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm76_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_0_master{h in Hs}: ni_vnf20_dot_vm76_0_master[h] <= sum{v in {1326}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_0_master{v in {1326}, h in Hs}: ni_vnf20_dot_vm76_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm76_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_0_slave{h in Hs}: ni_vnf20_dot_vm76_0_slave[h] <= sum{v in {1327}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_0_slave{v in {1327}, h in Hs}: ni_vnf20_dot_vm76_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_120_{h in Hs}: ni_vnf20_dot_vm76_0_master[h] + ni_vnf20_dot_vm76_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm76_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_1_master{h in Hs}: ni_vnf20_dot_vm76_1_master[h] <= sum{v in {1328}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_1_master{v in {1328}, h in Hs}: ni_vnf20_dot_vm76_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm76_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_1_slave{h in Hs}: ni_vnf20_dot_vm76_1_slave[h] <= sum{v in {1329}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_1_slave{v in {1329}, h in Hs}: ni_vnf20_dot_vm76_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_121_{h in Hs}: ni_vnf20_dot_vm76_1_master[h] + ni_vnf20_dot_vm76_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_0_master{h in Hs}: ni_vnf21_dot_vm37_0_master[h] <= sum{v in {1348..1352}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_0_master{v in {1348..1352}, h in Hs}: ni_vnf21_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_0_slave{h in Hs}: ni_vnf21_dot_vm37_0_slave[h] <= sum{v in {1353..1357}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_0_slave{v in {1353..1357}, h in Hs}: ni_vnf21_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_122_{h in Hs}: ni_vnf21_dot_vm37_0_master[h] + ni_vnf21_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_1_master{h in Hs}: ni_vnf21_dot_vm37_1_master[h] <= sum{v in {1358..1362}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_1_master{v in {1358..1362}, h in Hs}: ni_vnf21_dot_vm37_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_1_slave{h in Hs}: ni_vnf21_dot_vm37_1_slave[h] <= sum{v in {1363..1367}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_1_slave{v in {1363..1367}, h in Hs}: ni_vnf21_dot_vm37_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_123_{h in Hs}: ni_vnf21_dot_vm37_1_master[h] + ni_vnf21_dot_vm37_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_2_master{h in Hs}: ni_vnf21_dot_vm37_2_master[h] <= sum{v in {1368..1372}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_2_master{v in {1368..1372}, h in Hs}: ni_vnf21_dot_vm37_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_2_slave{h in Hs}: ni_vnf21_dot_vm37_2_slave[h] <= sum{v in {1373..1377}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_2_slave{v in {1373..1377}, h in Hs}: ni_vnf21_dot_vm37_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_124_{h in Hs}: ni_vnf21_dot_vm37_2_master[h] + ni_vnf21_dot_vm37_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_3_master{h in Hs}: ni_vnf21_dot_vm37_3_master[h] <= sum{v in {1378..1382}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_3_master{v in {1378..1382}, h in Hs}: ni_vnf21_dot_vm37_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_3_slave{h in Hs}: ni_vnf21_dot_vm37_3_slave[h] <= sum{v in {1383..1387}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_3_slave{v in {1383..1387}, h in Hs}: ni_vnf21_dot_vm37_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_125_{h in Hs}: ni_vnf21_dot_vm37_3_master[h] + ni_vnf21_dot_vm37_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_0_master{h in Hs}: ni_vnf21_dot_vm38_0_master[h] <= sum{v in {1388}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_0_master{v in {1388}, h in Hs}: ni_vnf21_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_0_slave{h in Hs}: ni_vnf21_dot_vm38_0_slave[h] <= sum{v in {1389}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_0_slave{v in {1389}, h in Hs}: ni_vnf21_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_126_{h in Hs}: ni_vnf21_dot_vm38_0_master[h] + ni_vnf21_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_1_master{h in Hs}: ni_vnf21_dot_vm38_1_master[h] <= sum{v in {1390}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_1_master{v in {1390}, h in Hs}: ni_vnf21_dot_vm38_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_1_slave{h in Hs}: ni_vnf21_dot_vm38_1_slave[h] <= sum{v in {1391}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_1_slave{v in {1391}, h in Hs}: ni_vnf21_dot_vm38_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_127_{h in Hs}: ni_vnf21_dot_vm38_1_master[h] + ni_vnf21_dot_vm38_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_2_master{h in Hs}: ni_vnf21_dot_vm38_2_master[h] <= sum{v in {1392}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_2_master{v in {1392}, h in Hs}: ni_vnf21_dot_vm38_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_2_slave{h in Hs}: ni_vnf21_dot_vm38_2_slave[h] <= sum{v in {1393}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_2_slave{v in {1393}, h in Hs}: ni_vnf21_dot_vm38_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_128_{h in Hs}: ni_vnf21_dot_vm38_2_master[h] + ni_vnf21_dot_vm38_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_3_master{h in Hs}: ni_vnf21_dot_vm38_3_master[h] <= sum{v in {1394}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_3_master{v in {1394}, h in Hs}: ni_vnf21_dot_vm38_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_3_slave{h in Hs}: ni_vnf21_dot_vm38_3_slave[h] <= sum{v in {1395}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_3_slave{v in {1395}, h in Hs}: ni_vnf21_dot_vm38_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_129_{h in Hs}: ni_vnf21_dot_vm38_3_master[h] + ni_vnf21_dot_vm38_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_0_master{h in Hs}: ni_vnf21_dot_vm39_0_master[h] <= sum{v in {1396}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_0_master{v in {1396}, h in Hs}: ni_vnf21_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_0_slave{h in Hs}: ni_vnf21_dot_vm39_0_slave[h] <= sum{v in {1397}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_0_slave{v in {1397}, h in Hs}: ni_vnf21_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_130_{h in Hs}: ni_vnf21_dot_vm39_0_master[h] + ni_vnf21_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_1_master{h in Hs}: ni_vnf21_dot_vm39_1_master[h] <= sum{v in {1398}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_1_master{v in {1398}, h in Hs}: ni_vnf21_dot_vm39_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_1_slave{h in Hs}: ni_vnf21_dot_vm39_1_slave[h] <= sum{v in {1399}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_1_slave{v in {1399}, h in Hs}: ni_vnf21_dot_vm39_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_131_{h in Hs}: ni_vnf21_dot_vm39_1_master[h] + ni_vnf21_dot_vm39_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_2_master{h in Hs}: ni_vnf21_dot_vm39_2_master[h] <= sum{v in {1400}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_2_master{v in {1400}, h in Hs}: ni_vnf21_dot_vm39_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_2_slave{h in Hs}: ni_vnf21_dot_vm39_2_slave[h] <= sum{v in {1401}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_2_slave{v in {1401}, h in Hs}: ni_vnf21_dot_vm39_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_132_{h in Hs}: ni_vnf21_dot_vm39_2_master[h] + ni_vnf21_dot_vm39_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_3_master{h in Hs}: ni_vnf21_dot_vm39_3_master[h] <= sum{v in {1402}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_3_master{v in {1402}, h in Hs}: ni_vnf21_dot_vm39_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_3_slave{h in Hs}: ni_vnf21_dot_vm39_3_slave[h] <= sum{v in {1403}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_3_slave{v in {1403}, h in Hs}: ni_vnf21_dot_vm39_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_133_{h in Hs}: ni_vnf21_dot_vm39_3_master[h] + ni_vnf21_dot_vm39_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_0_master{h in Hs}: ni_vnf21_dot_vm40_0_master[h] <= sum{v in {1404}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_0_master{v in {1404}, h in Hs}: ni_vnf21_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_0_slave{h in Hs}: ni_vnf21_dot_vm40_0_slave[h] <= sum{v in {1405}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_0_slave{v in {1405}, h in Hs}: ni_vnf21_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_134_{h in Hs}: ni_vnf21_dot_vm40_0_master[h] + ni_vnf21_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_1_master{h in Hs}: ni_vnf21_dot_vm40_1_master[h] <= sum{v in {1406}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_1_master{v in {1406}, h in Hs}: ni_vnf21_dot_vm40_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_1_slave{h in Hs}: ni_vnf21_dot_vm40_1_slave[h] <= sum{v in {1407}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_1_slave{v in {1407}, h in Hs}: ni_vnf21_dot_vm40_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_135_{h in Hs}: ni_vnf21_dot_vm40_1_master[h] + ni_vnf21_dot_vm40_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_2_master{h in Hs}: ni_vnf21_dot_vm40_2_master[h] <= sum{v in {1408}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_2_master{v in {1408}, h in Hs}: ni_vnf21_dot_vm40_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_2_slave{h in Hs}: ni_vnf21_dot_vm40_2_slave[h] <= sum{v in {1409}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_2_slave{v in {1409}, h in Hs}: ni_vnf21_dot_vm40_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_136_{h in Hs}: ni_vnf21_dot_vm40_2_master[h] + ni_vnf21_dot_vm40_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_3_master{h in Hs}: ni_vnf21_dot_vm40_3_master[h] <= sum{v in {1410}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_3_master{v in {1410}, h in Hs}: ni_vnf21_dot_vm40_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_3_slave{h in Hs}: ni_vnf21_dot_vm40_3_slave[h] <= sum{v in {1411}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_3_slave{v in {1411}, h in Hs}: ni_vnf21_dot_vm40_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_137_{h in Hs}: ni_vnf21_dot_vm40_3_master[h] + ni_vnf21_dot_vm40_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_0_master{h in Hs}: ni_vnf21_dot_vm7_0_master[h] <= sum{v in {1412}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_0_master{v in {1412}, h in Hs}: ni_vnf21_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_0_slave{h in Hs}: ni_vnf21_dot_vm7_0_slave[h] <= sum{v in {1413}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_0_slave{v in {1413}, h in Hs}: ni_vnf21_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_138_{h in Hs}: ni_vnf21_dot_vm7_0_master[h] + ni_vnf21_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_1_master{h in Hs}: ni_vnf21_dot_vm7_1_master[h] <= sum{v in {1414}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_1_master{v in {1414}, h in Hs}: ni_vnf21_dot_vm7_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_1_slave{h in Hs}: ni_vnf21_dot_vm7_1_slave[h] <= sum{v in {1415}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_1_slave{v in {1415}, h in Hs}: ni_vnf21_dot_vm7_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_139_{h in Hs}: ni_vnf21_dot_vm7_1_master[h] + ni_vnf21_dot_vm7_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_2_master{h in Hs}: ni_vnf21_dot_vm7_2_master[h] <= sum{v in {1416}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_2_master{v in {1416}, h in Hs}: ni_vnf21_dot_vm7_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_2_slave{h in Hs}: ni_vnf21_dot_vm7_2_slave[h] <= sum{v in {1417}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_2_slave{v in {1417}, h in Hs}: ni_vnf21_dot_vm7_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_140_{h in Hs}: ni_vnf21_dot_vm7_2_master[h] + ni_vnf21_dot_vm7_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_3_master{h in Hs}: ni_vnf21_dot_vm7_3_master[h] <= sum{v in {1418}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_3_master{v in {1418}, h in Hs}: ni_vnf21_dot_vm7_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_3_slave{h in Hs}: ni_vnf21_dot_vm7_3_slave[h] <= sum{v in {1419}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_3_slave{v in {1419}, h in Hs}: ni_vnf21_dot_vm7_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_141_{h in Hs}: ni_vnf21_dot_vm7_3_master[h] + ni_vnf21_dot_vm7_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm37_0_master{h in Hs}: ni_vnf22_dot_vm37_0_master[h] <= sum{v in {1420..1424}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm37_0_master{v in {1420..1424}, h in Hs}: ni_vnf22_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm37_0_slave{h in Hs}: ni_vnf22_dot_vm37_0_slave[h] <= sum{v in {1425..1429}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm37_0_slave{v in {1425..1429}, h in Hs}: ni_vnf22_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_142_{h in Hs}: ni_vnf22_dot_vm37_0_master[h] + ni_vnf22_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm38_0_master{h in Hs}: ni_vnf22_dot_vm38_0_master[h] <= sum{v in {1430,1431}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm38_0_master{v in {1430,1431}, h in Hs}: ni_vnf22_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm38_0_slave{h in Hs}: ni_vnf22_dot_vm38_0_slave[h] <= sum{v in {1432,1433}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm38_0_slave{v in {1432,1433}, h in Hs}: ni_vnf22_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_143_{h in Hs}: ni_vnf22_dot_vm38_0_master[h] + ni_vnf22_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm39_0_master{h in Hs}: ni_vnf22_dot_vm39_0_master[h] <= sum{v in {1434}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm39_0_master{v in {1434}, h in Hs}: ni_vnf22_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm39_0_slave{h in Hs}: ni_vnf22_dot_vm39_0_slave[h] <= sum{v in {1435}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm39_0_slave{v in {1435}, h in Hs}: ni_vnf22_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_144_{h in Hs}: ni_vnf22_dot_vm39_0_master[h] + ni_vnf22_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm40_0_master{h in Hs}: ni_vnf22_dot_vm40_0_master[h] <= sum{v in {1436}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm40_0_master{v in {1436}, h in Hs}: ni_vnf22_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm40_0_slave{h in Hs}: ni_vnf22_dot_vm40_0_slave[h] <= sum{v in {1437}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm40_0_slave{v in {1437}, h in Hs}: ni_vnf22_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_145_{h in Hs}: ni_vnf22_dot_vm40_0_master[h] + ni_vnf22_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm79_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm79_0_master{h in Hs}: ni_vnf22_dot_vm79_0_master[h] <= sum{v in {1438}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm79_0_master{v in {1438}, h in Hs}: ni_vnf22_dot_vm79_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm79_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm79_0_slave{h in Hs}: ni_vnf22_dot_vm79_0_slave[h] <= sum{v in {1439}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm79_0_slave{v in {1439}, h in Hs}: ni_vnf22_dot_vm79_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_146_{h in Hs}: ni_vnf22_dot_vm79_0_master[h] + ni_vnf22_dot_vm79_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm7_0_master{h in Hs}: ni_vnf22_dot_vm7_0_master[h] <= sum{v in {1440}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm7_0_master{v in {1440}, h in Hs}: ni_vnf22_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm7_0_slave{h in Hs}: ni_vnf22_dot_vm7_0_slave[h] <= sum{v in {1441}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm7_0_slave{v in {1441}, h in Hs}: ni_vnf22_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_147_{h in Hs}: ni_vnf22_dot_vm7_0_master[h] + ni_vnf22_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm37_0_master{h in Hs}: ni_vnf24_dot_vm37_0_master[h] <= sum{v in {1447..1451}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm37_0_master{v in {1447..1451}, h in Hs}: ni_vnf24_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm37_0_slave{h in Hs}: ni_vnf24_dot_vm37_0_slave[h] <= sum{v in {1452..1456}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm37_0_slave{v in {1452..1456}, h in Hs}: ni_vnf24_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_148_{h in Hs}: ni_vnf24_dot_vm37_0_master[h] + ni_vnf24_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm38_0_master{h in Hs}: ni_vnf24_dot_vm38_0_master[h] <= sum{v in {1457,1458}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm38_0_master{v in {1457,1458}, h in Hs}: ni_vnf24_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm38_0_slave{h in Hs}: ni_vnf24_dot_vm38_0_slave[h] <= sum{v in {1459,1460}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm38_0_slave{v in {1459,1460}, h in Hs}: ni_vnf24_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_149_{h in Hs}: ni_vnf24_dot_vm38_0_master[h] + ni_vnf24_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm39_0_master{h in Hs}: ni_vnf24_dot_vm39_0_master[h] <= sum{v in {1461}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm39_0_master{v in {1461}, h in Hs}: ni_vnf24_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm39_0_slave{h in Hs}: ni_vnf24_dot_vm39_0_slave[h] <= sum{v in {1462}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm39_0_slave{v in {1462}, h in Hs}: ni_vnf24_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_150_{h in Hs}: ni_vnf24_dot_vm39_0_master[h] + ni_vnf24_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm40_0_master{h in Hs}: ni_vnf24_dot_vm40_0_master[h] <= sum{v in {1463}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm40_0_master{v in {1463}, h in Hs}: ni_vnf24_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm40_0_slave{h in Hs}: ni_vnf24_dot_vm40_0_slave[h] <= sum{v in {1464}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm40_0_slave{v in {1464}, h in Hs}: ni_vnf24_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_151_{h in Hs}: ni_vnf24_dot_vm40_0_master[h] + ni_vnf24_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm79_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm79_0_master{h in Hs}: ni_vnf24_dot_vm79_0_master[h] <= sum{v in {1465}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm79_0_master{v in {1465}, h in Hs}: ni_vnf24_dot_vm79_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm79_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm79_0_slave{h in Hs}: ni_vnf24_dot_vm79_0_slave[h] <= sum{v in {1466}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm79_0_slave{v in {1466}, h in Hs}: ni_vnf24_dot_vm79_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_152_{h in Hs}: ni_vnf24_dot_vm79_0_master[h] + ni_vnf24_dot_vm79_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm7_0_master{h in Hs}: ni_vnf24_dot_vm7_0_master[h] <= sum{v in {1467}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm7_0_master{v in {1467}, h in Hs}: ni_vnf24_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm7_0_slave{h in Hs}: ni_vnf24_dot_vm7_0_slave[h] <= sum{v in {1468}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm7_0_slave{v in {1468}, h in Hs}: ni_vnf24_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_153_{h in Hs}: ni_vnf24_dot_vm7_0_master[h] + ni_vnf24_dot_vm7_0_slave[h] <= 1;

#=== Soft Affinity Variables ===
var vnf5_dot_vm30_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm30_0{h in Hs}: vnf5_dot_vm30_0[h] <= sum{v in {275,276}} x[v, h];
s.t. or_ineq_vnf5_dot_vm30_0{v in {275,276}, h in Hs}: vnf5_dot_vm30_0[h] >= x[v, h];
var vnf5_dot_vm31_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm31_0{h in Hs}: vnf5_dot_vm31_0[h] <= sum{v in {277,278}} x[v, h];
s.t. or_ineq_vnf5_dot_vm31_0{v in {277,278}, h in Hs}: vnf5_dot_vm31_0[h] >= x[v, h];
var vnf5_dot_vm32_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm32_0{h in Hs}: vnf5_dot_vm32_0[h] <= sum{v in {279..285}} x[v, h];
s.t. or_ineq_vnf5_dot_vm32_0{v in {279..285}, h in Hs}: vnf5_dot_vm32_0[h] >= x[v, h];
var vnf5_dot_vm33_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm33_0{h in Hs}: vnf5_dot_vm33_0[h] <= sum{v in {286,287}} x[v, h];
s.t. or_ineq_vnf5_dot_vm33_0{v in {286,287}, h in Hs}: vnf5_dot_vm33_0[h] >= x[v, h];
var vnf5_dot_vm34_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm34_0{h in Hs}: vnf5_dot_vm34_0[h] <= sum{v in {288,289}} x[v, h];
s.t. or_ineq_vnf5_dot_vm34_0{v in {288,289}, h in Hs}: vnf5_dot_vm34_0[h] >= x[v, h];
var vnf5_dot_vm35_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm35_0{h in Hs}: vnf5_dot_vm35_0[h] <= sum{v in {290,291}} x[v, h];
s.t. or_ineq_vnf5_dot_vm35_0{v in {290,291}, h in Hs}: vnf5_dot_vm35_0[h] >= x[v, h];
var vnf26_dot_vm82_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm82_0{h in Hs}: vnf26_dot_vm82_0[h] <= sum{v in {1474}} x[v, h];
s.t. or_ineq_vnf26_dot_vm82_0{v in {1474}, h in Hs}: vnf26_dot_vm82_0[h] >= x[v, h];
var vnf26_dot_vm14_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm14_0{h in Hs}: vnf26_dot_vm14_0[h] <= sum{v in {1476}} x[v, h];
s.t. or_ineq_vnf26_dot_vm14_0{v in {1476}, h in Hs}: vnf26_dot_vm14_0[h] >= x[v, h];
var vnf26_dot_vm86_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm86_0{h in Hs}: vnf26_dot_vm86_0[h] <= sum{v in {1479}} x[v, h];
s.t. or_ineq_vnf26_dot_vm86_0{v in {1479}, h in Hs}: vnf26_dot_vm86_0[h] >= x[v, h];
var vnf26_dot_vm87_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm87_0{h in Hs}: vnf26_dot_vm87_0[h] <= sum{v in {1480}} x[v, h];
s.t. or_ineq_vnf26_dot_vm87_0{v in {1480}, h in Hs}: vnf26_dot_vm87_0[h] >= x[v, h];
var vnf26_dot_vm88_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm88_0{h in Hs}: vnf26_dot_vm88_0[h] <= sum{v in {1481}} x[v, h];
s.t. or_ineq_vnf26_dot_vm88_0{v in {1481}, h in Hs}: vnf26_dot_vm88_0[h] >= x[v, h];
var vnf26_dot_vm89_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm89_0{h in Hs}: vnf26_dot_vm89_0[h] <= sum{v in {1482}} x[v, h];
s.t. or_ineq_vnf26_dot_vm89_0{v in {1482}, h in Hs}: vnf26_dot_vm89_0[h] >= x[v, h];
var vnf26_dot_vm91_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm91_0{h in Hs}: vnf26_dot_vm91_0[h] <= sum{v in {1484}} x[v, h];
s.t. or_ineq_vnf26_dot_vm91_0{v in {1484}, h in Hs}: vnf26_dot_vm91_0[h] >= x[v, h];
var vnf26_dot_vm92_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm92_0{h in Hs}: vnf26_dot_vm92_0[h] <= sum{v in {1485}} x[v, h];
s.t. or_ineq_vnf26_dot_vm92_0{v in {1485}, h in Hs}: vnf26_dot_vm92_0[h] >= x[v, h];
var vnf26_dot_vm109_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm109_0{h in Hs}: vnf26_dot_vm109_0[h] <= sum{v in {1502}} x[v, h];
s.t. or_ineq_vnf26_dot_vm109_0{v in {1502}, h in Hs}: vnf26_dot_vm109_0[h] >= x[v, h];
var vnf26_dot_vm110_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm110_0{h in Hs}: vnf26_dot_vm110_0[h] <= sum{v in {1503}} x[v, h];
s.t. or_ineq_vnf26_dot_vm110_0{v in {1503}, h in Hs}: vnf26_dot_vm110_0[h] >= x[v, h];
var vnf26_dot_vm111_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm111_0{h in Hs}: vnf26_dot_vm111_0[h] <= sum{v in {1504}} x[v, h];
s.t. or_ineq_vnf26_dot_vm111_0{v in {1504}, h in Hs}: vnf26_dot_vm111_0[h] >= x[v, h];
var vnf26_dot_vm112_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm112_0{h in Hs}: vnf26_dot_vm112_0[h] <= sum{v in {1505}} x[v, h];
s.t. or_ineq_vnf26_dot_vm112_0{v in {1505}, h in Hs}: vnf26_dot_vm112_0[h] >= x[v, h];
var vnf26_dot_vm113_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm113_0{h in Hs}: vnf26_dot_vm113_0[h] <= sum{v in {1506}} x[v, h];
s.t. or_ineq_vnf26_dot_vm113_0{v in {1506}, h in Hs}: vnf26_dot_vm113_0[h] >= x[v, h];
var vnf26_dot_vm114_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm114_0{h in Hs}: vnf26_dot_vm114_0[h] <= sum{v in {1507}} x[v, h];
s.t. or_ineq_vnf26_dot_vm114_0{v in {1507}, h in Hs}: vnf26_dot_vm114_0[h] >= x[v, h];
var vnf26_dot_vm115_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm115_0{h in Hs}: vnf26_dot_vm115_0[h] <= sum{v in {1508}} x[v, h];
s.t. or_ineq_vnf26_dot_vm115_0{v in {1508}, h in Hs}: vnf26_dot_vm115_0[h] >= x[v, h];
var vnf26_dot_vm116_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm116_0{h in Hs}: vnf26_dot_vm116_0[h] <= sum{v in {1509}} x[v, h];
s.t. or_ineq_vnf26_dot_vm116_0{v in {1509}, h in Hs}: vnf26_dot_vm116_0[h] >= x[v, h];
var vnf26_dot_vm117_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm117_0{h in Hs}: vnf26_dot_vm117_0[h] <= sum{v in {1510}} x[v, h];
s.t. or_ineq_vnf26_dot_vm117_0{v in {1510}, h in Hs}: vnf26_dot_vm117_0[h] >= x[v, h];
var vnf26_dot_vm126_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm126_0{h in Hs}: vnf26_dot_vm126_0[h] <= sum{v in {1519}} x[v, h];
s.t. or_ineq_vnf26_dot_vm126_0{v in {1519}, h in Hs}: vnf26_dot_vm126_0[h] >= x[v, h];
var vnf26_dot_vm127_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm127_0{h in Hs}: vnf26_dot_vm127_0[h] <= sum{v in {1520}} x[v, h];
s.t. or_ineq_vnf26_dot_vm127_0{v in {1520}, h in Hs}: vnf26_dot_vm127_0[h] >= x[v, h];
var vnf26_dot_vm128_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm128_0{h in Hs}: vnf26_dot_vm128_0[h] <= sum{v in {1521}} x[v, h];
s.t. or_ineq_vnf26_dot_vm128_0{v in {1521}, h in Hs}: vnf26_dot_vm128_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf5_dot_vm30_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm31_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm32_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm33_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm34_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm35_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm82_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm14_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm86_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm87_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm88_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm89_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm91_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm92_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm109_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm110_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm111_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm112_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm113_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm114_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm115_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm116_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm117_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm126_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm127_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm128_0[h]) - 1) / 37;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 8.0
21 8.0
22 8.0
23 8.0
24 8.0
25 8.0
26 8.0
27 8.0
28 8.0
29 8.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 8.0
39 8.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 8.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 8.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 4.0
81 4.0
82 4.0
83 4.0
84 4.0
85 4.0
86 4.0
87 4.0
88 4.0
89 4.0
90 4.0
91 4.0
92 4.0
93 4.0
94 4.0
95 4.0
96 4.0
97 4.0
98 4.0
99 4.0
100 4.0
101 4.0
102 8.0
103 8.0
104 8.0
105 8.0
106 8.0
107 8.0
108 4.0
109 4.0
110 4.0
111 4.0
112 4.0
113 4.0
114 4.0
115 4.0
116 4.0
117 4.0
118 4.0
119 4.0
120 4.0
121 4.0
122 4.0
123 4.0
124 4.0
125 4.0
126 4.0
127 4.0
128 4.0
129 4.0
130 4.0
131 4.0
132 8.0
133 8.0
134 8.0
135 8.0
136 8.0
137 8.0
138 8.0
139 8.0
140 8.0
141 8.0
142 8.0
143 8.0
144 8.0
145 8.0
146 8.0
147 8.0
148 8.0
149 8.0
150 8.0
151 8.0
152 8.0
153 8.0
154 8.0
155 8.0
156 8.0
157 8.0
158 8.0
159 8.0
160 8.0
161 8.0
162 8.0
163 8.0
164 8.0
165 8.0
166 8.0
167 8.0
168 8.0
169 8.0
170 8.0
171 8.0
172 8.0
173 8.0
174 8.0
175 8.0
176 8.0
177 8.0
178 8.0
179 8.0
180 8.0
181 8.0
182 8.0
183 8.0
184 8.0
185 8.0
186 8.0
187 8.0
188 8.0
189 8.0
190 8.0
191 8.0
192 8.0
193 8.0
194 8.0
195 8.0
196 8.0
197 8.0
198 8.0
199 8.0
200 2.0
201 2.0
202 2.0
203 2.0
204 2.0
205 2.0
206 2.0
207 2.0
208 4.0
209 4.0
210 4.0
211 4.0
212 4.0
213 4.0
214 4.0
215 4.0
216 4.0
217 4.0
218 4.0
219 4.0
220 4.0
221 4.0
222 2.0
223 2.0
224 2.0
225 2.0
226 3.0
227 3.0
228 3.0
229 3.0
230 8.0
231 8.0
232 8.0
233 8.0
234 8.0
235 8.0
236 8.0
237 8.0
238 8.0
239 8.0
240 8.0
241 8.0
242 4.0
243 4.0
244 4.0
245 4.0
246 4.0
247 4.0
248 4.0
249 4.0
250 2.0
251 2.0
252 2.0
253 2.0
254 4.0
255 4.0
256 4.0
257 4.0
258 2.0
259 2.0
260 2.0
261 2.0
262 8.0
263 8.0
264 8.0
265 8.0
266 8.0
267 8.0
268 8.0
269 16.0
270 16.0
271 16.0
272 8.0
273 8.0
274 4.0
275 6.0
276 6.0
277 6.0
278 6.0
279 7.0
280 7.0
281 7.0
282 7.0
283 7.0
284 7.0
285 7.0
286 8.0
287 8.0
288 8.0
289 8.0
290 6.0
291 6.0
292 8.0
293 4.0
294 4.0
295 4.0
296 4.0
297 4.0
298 4.0
299 4.0
300 4.0
301 4.0
302 4.0
303 4.0
304 4.0
305 2.0
306 2.0
307 2.0
308 2.0
309 4.0
310 4.0
311 4.0
312 4.0
313 8.0
314 8.0
315 8.0
316 8.0
317 2.0
318 2.0
319 2.0
320 2.0
321 4.0
322 4.0
323 4.0
324 4.0
325 4.0
326 4.0
327 4.0
328 4.0
329 4.0
330 4.0
331 4.0
332 4.0
333 4.0
334 4.0
335 4.0
336 4.0
337 4.0
338 4.0
339 4.0
340 4.0
341 2.0
342 2.0
343 2.0
344 2.0
345 4.0
346 4.0
347 4.0
348 4.0
349 8.0
350 8.0
351 8.0
352 8.0
353 2.0
354 2.0
355 2.0
356 2.0
357 4.0
358 4.0
359 4.0
360 4.0
361 2.0
362 2.0
363 4.0
364 4.0
365 8.0
366 8.0
367 2.0
368 2.0
369 16.0
370 8.0
371 8.0
372 8.0
373 8.0
374 8.0
375 8.0
376 8.0
377 8.0
378 8.0
379 8.0
380 2.0
381 2.0
382 2.0
383 2.0
384 4.0
385 4.0
386 4.0
387 4.0
388 4.0
389 4.0
390 4.0
391 4.0
392 8.0
393 8.0
394 8.0
395 8.0
396 8.0
397 8.0
398 4.0
399 8.0
400 8.0
401 8.0
402 8.0
403 8.0
404 8.0
405 8.0
406 8.0
407 8.0
408 8.0
409 8.0
410 8.0
411 8.0
412 8.0
413 8.0
414 8.0
415 8.0
416 8.0
417 8.0
418 8.0
419 8.0
420 8.0
421 8.0
422 8.0
423 8.0
424 8.0
425 8.0
426 8.0
427 8.0
428 8.0
429 8.0
430 8.0
431 8.0
432 8.0
433 8.0
434 8.0
435 8.0
436 8.0
437 8.0
438 8.0
439 8.0
440 8.0
441 8.0
442 8.0
443 8.0
444 8.0
445 8.0
446 8.0
447 8.0
448 8.0
449 8.0
450 8.0
451 8.0
452 8.0
453 8.0
454 8.0
455 8.0
456 8.0
457 8.0
458 8.0
459 8.0
460 8.0
461 8.0
462 8.0
463 8.0
464 8.0
465 8.0
466 8.0
467 8.0
468 8.0
469 8.0
470 8.0
471 8.0
472 8.0
473 8.0
474 8.0
475 8.0
476 8.0
477 4.0
478 4.0
479 4.0
480 4.0
481 4.0
482 4.0
483 4.0
484 4.0
485 4.0
486 4.0
487 4.0
488 4.0
489 4.0
490 4.0
491 4.0
492 4.0
493 4.0
494 4.0
495 4.0
496 4.0
497 4.0
498 4.0
499 4.0
500 4.0
501 4.0
502 4.0
503 4.0
504 4.0
505 4.0
506 4.0
507 4.0
508 4.0
509 4.0
510 4.0
511 4.0
512 4.0
513 4.0
514 4.0
515 4.0
516 8.0
517 8.0
518 8.0
519 8.0
520 8.0
521 8.0
522 8.0
523 8.0
524 0.0
525 0.0
526 0.0
527 4.0
528 4.0
529 4.0
530 4.0
531 4.0
532 4.0
533 4.0
534 4.0
535 4.0
536 4.0
537 4.0
538 4.0
539 4.0
540 4.0
541 4.0
542 4.0
543 4.0
544 4.0
545 4.0
546 4.0
547 4.0
548 4.0
549 4.0
550 4.0
551 4.0
552 4.0
553 4.0
554 8.0
555 8.0
556 8.0
557 8.0
558 8.0
559 8.0
560 8.0
561 8.0
562 8.0
563 8.0
564 8.0
565 8.0
566 8.0
567 8.0
568 8.0
569 8.0
570 8.0
571 8.0
572 8.0
573 8.0
574 8.0
575 8.0
576 8.0
577 8.0
578 8.0
579 8.0
580 8.0
581 8.0
582 8.0
583 8.0
584 8.0
585 8.0
586 8.0
587 8.0
588 8.0
589 8.0
590 8.0
591 8.0
592 8.0
593 8.0
594 8.0
595 8.0
596 8.0
597 8.0
598 8.0
599 8.0
600 8.0
601 8.0
602 8.0
603 8.0
604 8.0
605 8.0
606 8.0
607 8.0
608 8.0
609 8.0
610 8.0
611 8.0
612 8.0
613 8.0
614 8.0
615 8.0
616 8.0
617 8.0
618 8.0
619 8.0
620 8.0
621 8.0
622 8.0
623 8.0
624 8.0
625 8.0
626 8.0
627 8.0
628 8.0
629 8.0
630 8.0
631 8.0
632 8.0
633 8.0
634 8.0
635 8.0
636 8.0
637 8.0
638 8.0
639 8.0
640 8.0
641 8.0
642 8.0
643 8.0
644 8.0
645 8.0
646 8.0
647 8.0
648 8.0
649 8.0
650 8.0
651 8.0
652 8.0
653 8.0
654 8.0
655 8.0
656 2.0
657 2.0
658 2.0
659 2.0
660 2.0
661 2.0
662 2.0
663 2.0
664 2.0
665 2.0
666 2.0
667 2.0
668 2.0
669 2.0
670 2.0
671 4.0
672 4.0
673 4.0
674 4.0
675 4.0
676 4.0
677 4.0
678 4.0
679 4.0
680 4.0
681 4.0
682 4.0
683 4.0
684 4.0
685 4.0
686 4.0
687 4.0
688 4.0
689 4.0
690 4.0
691 4.0
692 2.0
693 2.0
694 2.0
695 2.0
696 2.0
697 2.0
698 4.0
699 4.0
700 4.0
701 4.0
702 4.0
703 4.0
704 8.0
705 8.0
706 8.0
707 8.0
708 8.0
709 8.0
710 8.0
711 8.0
712 8.0
713 8.0
714 8.0
715 8.0
716 8.0
717 8.0
718 8.0
719 8.0
720 8.0
721 8.0
722 4.0
723 4.0
724 4.0
725 4.0
726 4.0
727 4.0
728 4.0
729 4.0
730 4.0
731 4.0
732 4.0
733 4.0
734 4.0
735 4.0
736 4.0
737 4.0
738 4.0
739 4.0
740 4.0
741 4.0
742 4.0
743 4.0
744 4.0
745 4.0
746 2.0
747 2.0
748 2.0
749 2.0
750 2.0
751 2.0
752 4.0
753 4.0
754 4.0
755 4.0
756 4.0
757 4.0
758 4.0
759 4.0
760 0.0
761 0.0
762 0.0
763 2.0
764 2.0
765 2.0
766 2.0
767 2.0
768 2.0
769 2.0
770 2.0
771 3.0
772 3.0
773 3.0
774 3.0
775 3.0
776 3.0
777 3.0
778 3.0
779 2.0
780 2.0
781 2.0
782 2.0
783 2.0
784 2.0
785 2.0
786 2.0
787 3.0
788 3.0
789 3.0
790 3.0
791 3.0
792 3.0
793 3.0
794 3.0
795 3.0
796 3.0
797 3.0
798 3.0
799 3.0
800 3.0
801 3.0
802 3.0
803 3.0
804 3.0
805 3.0
806 3.0
807 3.0
808 3.0
809 3.0
810 3.0
811 3.0
812 3.0
813 3.0
814 3.0
815 3.0
816 3.0
817 3.0
818 3.0
819 4.0
820 4.0
821 4.0
822 4.0
823 4.0
824 4.0
825 4.0
826 4.0
827 2.0
828 2.0
829 2.0
830 2.0
831 2.0
832 2.0
833 2.0
834 2.0
835 2.0
836 2.0
837 2.0
838 2.0
839 2.0
840 2.0
841 2.0
842 2.0
843 2.0
844 2.0
845 2.0
846 2.0
847 2.0
848 2.0
849 2.0
850 2.0
851 2.0
852 2.0
853 2.0
854 2.0
855 2.0
856 2.0
857 2.0
858 2.0
859 2.0
860 2.0
861 2.0
862 2.0
863 2.0
864 2.0
865 2.0
866 2.0
867 2.0
868 2.0
869 2.0
870 2.0
871 2.0
872 2.0
873 2.0
874 2.0
875 2.0
876 2.0
877 2.0
878 2.0
879 2.0
880 2.0
881 2.0
882 2.0
883 2.0
884 2.0
885 2.0
886 2.0
887 2.0
888 2.0
889 2.0
890 2.0
891 2.0
892 2.0
893 2.0
894 2.0
895 2.0
896 2.0
897 2.0
898 2.0
899 2.0
900 2.0
901 2.0
902 2.0
903 2.0
904 2.0
905 2.0
906 2.0
907 2.0
908 2.0
909 2.0
910 2.0
911 2.0
912 2.0
913 2.0
914 2.0
915 2.0
916 2.0
917 2.0
918 2.0
919 2.0
920 2.0
921 2.0
922 2.0
923 3.0
924 3.0
925 3.0
926 3.0
927 3.0
928 3.0
929 3.0
930 3.0
931 3.0
932 3.0
933 3.0
934 3.0
935 3.0
936 3.0
937 3.0
938 3.0
939 3.0
940 3.0
941 3.0
942 3.0
943 3.0
944 3.0
945 3.0
946 3.0
947 3.0
948 3.0
949 3.0
950 3.0
951 3.0
952 3.0
953 3.0
954 3.0
955 3.0
956 3.0
957 3.0
958 3.0
959 3.0
960 3.0
961 3.0
962 3.0
963 3.0
964 3.0
965 3.0
966 3.0
967 3.0
968 3.0
969 3.0
970 3.0
971 3.0
972 3.0
973 3.0
974 3.0
975 3.0
976 3.0
977 3.0
978 3.0
979 3.0
980 3.0
981 3.0
982 3.0
983 3.0
984 3.0
985 3.0
986 3.0
987 3.0
988 3.0
989 3.0
990 3.0
991 3.0
992 3.0
993 3.0
994 3.0
995 3.0
996 3.0
997 3.0
998 3.0
999 3.0
1000 3.0
1001 3.0
1002 3.0
1003 3.0
1004 3.0
1005 3.0
1006 3.0
1007 3.0
1008 3.0
1009 3.0
1010 3.0
1011 3.0
1012 3.0
1013 3.0
1014 3.0
1015 3.0
1016 3.0
1017 3.0
1018 3.0
1019 3.0
1020 3.0
1021 3.0
1022 3.0
1023 3.0
1024 3.0
1025 3.0
1026 3.0
1027 3.0
1028 3.0
1029 3.0
1030 3.0
1031 3.0
1032 3.0
1033 3.0
1034 3.0
1035 3.0
1036 3.0
1037 3.0
1038 3.0
1039 3.0
1040 3.0
1041 3.0
1042 3.0
1043 3.0
1044 3.0
1045 3.0
1046 3.0
1047 3.0
1048 3.0
1049 3.0
1050 3.0
1051 2.0
1052 2.0
1053 2.0
1054 2.0
1055 2.0
1056 2.0
1057 2.0
1058 2.0
1059 3.0
1060 3.0
1061 3.0
1062 3.0
1063 3.0
1064 3.0
1065 3.0
1066 3.0
1067 3.0
1068 3.0
1069 3.0
1070 3.0
1071 3.0
1072 3.0
1073 3.0
1074 3.0
1075 6.0
1076 6.0
1077 6.0
1078 6.0
1079 6.0
1080 6.0
1081 6.0
1082 6.0
1083 6.0
1084 6.0
1085 6.0
1086 6.0
1087 6.0
1088 6.0
1089 6.0
1090 6.0
1091 6.0
1092 6.0
1093 6.0
1094 6.0
1095 6.0
1096 6.0
1097 6.0
1098 6.0
1099 1.0
1100 1.0
1101 1.0
1102 1.0
1103 1.0
1104 1.0
1105 1.0
1106 1.0
1107 1.0
1108 1.0
1109 1.0
1110 1.0
1111 1.0
1112 1.0
1113 1.0
1114 1.0
1115 2.0
1116 2.0
1117 2.0
1118 2.0
1119 2.0
1120 2.0
1121 2.0
1122 2.0
1123 2.0
1124 2.0
1125 2.0
1126 2.0
1127 2.0
1128 2.0
1129 2.0
1130 2.0
1131 4.0
1132 4.0
1133 4.0
1134 4.0
1135 4.0
1136 4.0
1137 4.0
1138 4.0
1139 4.0
1140 4.0
1141 4.0
1142 4.0
1143 4.0
1144 4.0
1145 4.0
1146 4.0
1147 4.0
1148 4.0
1149 4.0
1150 4.0
1151 4.0
1152 4.0
1153 4.0
1154 4.0
1155 0.0
1156 0.0
1157 0.0
1158 0.0
1159 0.0
1160 0.0
1161 0.0
1162 0.0
1163 0.0
1164 2.0
1165 2.0
1166 3.0
1167 3.0
1168 2.0
1169 2.0
1170 3.0
1171 3.0
1172 3.0
1173 3.0
1174 4.0
1175 4.0
1176 3.0
1177 3.0
1178 3.0
1179 3.0
1180 3.0
1181 3.0
1182 3.0
1183 3.0
1184 3.0
1185 3.0
1186 3.0
1187 3.0
1188 3.0
1189 3.0
1190 3.0
1191 3.0
1192 3.0
1193 3.0
1194 3.0
1195 3.0
1196 3.0
1197 3.0
1198 3.0
1199 2.0
1200 2.0
1201 3.0
1202 3.0
1203 3.0
1204 3.0
1205 6.0
1206 6.0
1207 6.0
1208 6.0
1209 1.0
1210 1.0
1211 1.0
1212 1.0
1213 1.0
1214 1.0
1215 2.0
1216 2.0
1217 4.0
1218 4.0
1219 4.0
1220 4.0
1221 4.0
1222 4.0
1223 0.0
1224 0.0
1225 0.0
1226 4.0
1227 4.0
1228 4.0
1229 4.0
1230 4.0
1231 4.0
1232 4.0
1233 4.0
1234 4.0
1235 4.0
1236 4.0
1237 4.0
1238 4.0
1239 4.0
1240 4.0
1241 4.0
1242 4.0
1243 4.0
1244 8.0
1245 8.0
1246 8.0
1247 8.0
1248 8.0
1249 8.0
1250 8.0
1251 8.0
1252 8.0
1253 8.0
1254 8.0
1255 8.0
1256 8.0
1257 8.0
1258 8.0
1259 8.0
1260 8.0
1261 8.0
1262 8.0
1263 8.0
1264 8.0
1265 8.0
1266 8.0
1267 8.0
1268 8.0
1269 8.0
1270 8.0
1271 8.0
1272 8.0
1273 8.0
1274 8.0
1275 8.0
1276 8.0
1277 8.0
1278 8.0
1279 8.0
1280 2.0
1281 2.0
1282 2.0
1283 2.0
1284 2.0
1285 2.0
1286 2.0
1287 2.0
1288 4.0
1289 4.0
1290 4.0
1291 4.0
1292 4.0
1293 4.0
1294 2.0
1295 2.0
1296 2.0
1297 2.0
1298 3.0
1299 3.0
1300 3.0
1301 3.0
1302 8.0
1303 8.0
1304 8.0
1305 8.0
1306 8.0
1307 8.0
1308 8.0
1309 8.0
1310 4.0
1311 4.0
1312 4.0
1313 4.0
1314 2.0
1315 2.0
1316 2.0
1317 2.0
1318 4.0
1319 4.0
1320 4.0
1321 4.0
1322 2.0
1323 2.0
1324 2.0
1325 2.0
1326 4.0
1327 4.0
1328 4.0
1329 4.0
1330 4.0
1331 4.0
1332 4.0
1333 4.0
1334 4.0
1335 4.0
1336 4.0
1337 4.0
1338 2.0
1339 2.0
1340 2.0
1341 2.0
1342 8.0
1343 8.0
1344 8.0
1345 8.0
1346 0.0
1347 0.0
1348 4.0
1349 4.0
1350 4.0
1351 4.0
1352 4.0
1353 4.0
1354 4.0
1355 4.0
1356 4.0
1357 4.0
1358 4.0
1359 4.0
1360 4.0
1361 4.0
1362 4.0
1363 4.0
1364 4.0
1365 4.0
1366 4.0
1367 4.0
1368 4.0
1369 4.0
1370 4.0
1371 4.0
1372 4.0
1373 4.0
1374 4.0
1375 4.0
1376 4.0
1377 4.0
1378 4.0
1379 4.0
1380 4.0
1381 4.0
1382 4.0
1383 4.0
1384 4.0
1385 4.0
1386 4.0
1387 4.0
1388 2.0
1389 2.0
1390 2.0
1391 2.0
1392 2.0
1393 2.0
1394 2.0
1395 2.0
1396 4.0
1397 4.0
1398 4.0
1399 4.0
1400 4.0
1401 4.0
1402 4.0
1403 4.0
1404 8.0
1405 8.0
1406 8.0
1407 8.0
1408 8.0
1409 8.0
1410 8.0
1411 8.0
1412 2.0
1413 2.0
1414 2.0
1415 2.0
1416 2.0
1417 2.0
1418 2.0
1419 2.0
1420 4.0
1421 4.0
1422 4.0
1423 4.0
1424 4.0
1425 4.0
1426 4.0
1427 4.0
1428 4.0
1429 4.0
1430 2.0
1431 2.0
1432 2.0
1433 2.0
1434 4.0
1435 4.0
1436 8.0
1437 8.0
1438 4.0
1439 4.0
1440 2.0
1441 2.0
1442 8.0
1443 8.0
1444 8.0
1445 4.0
1446 16.0
1447 4.0
1448 4.0
1449 4.0
1450 4.0
1451 4.0
1452 4.0
1453 4.0
1454 4.0
1455 4.0
1456 4.0
1457 2.0
1458 2.0
1459 2.0
1460 2.0
1461 4.0
1462 4.0
1463 8.0
1464 8.0
1465 4.0
1466 4.0
1467 2.0
1468 2.0
1469 8.0
1470 8.0
1471 8.0
1472 4.0
1473 16.0
1474 4.0
1475 6.0
1476 16.0
1477 8.0
1478 2.0
1479 2.0
1480 3.0
1481 5.0
1482 4.0
1483 4.0
1484 8.0
1485 6.0
1486 6.0
1487 6.0
1488 6.0
1489 6.0
1490 5.0
1491 5.0
1492 8.0
1493 8.0
1494 8.0
1495 8.0
1496 8.0
1497 8.0
1498 8.0
1499 8.0
1500 4.0
1501 4.0
1502 12.0
1503 12.0
1504 12.0
1505 12.0
1506 8.0
1507 8.0
1508 8.0
1509 8.0
1510 8.0
1511 4.0
1512 4.0
1513 9.0
1514 9.0
1515 9.0
1516 9.0
1517 4.0
1518 4.0
1519 8.0
1520 8.0
1521 8.0;
param: MEM := 
0 24.0
1 24.0
2 24.0
3 24.0
4 24.0
5 24.0
6 24.0
7 24.0
8 24.0
9 24.0
10 24.0
11 24.0
12 24.0
13 24.0
14 24.0
15 24.0
16 24.0
17 24.0
18 24.0
19 24.0
20 24.0
21 24.0
22 24.0
23 24.0
24 24.0
25 24.0
26 24.0
27 24.0
28 24.0
29 24.0
30 24.0
31 24.0
32 24.0
33 24.0
34 24.0
35 24.0
36 24.0
37 24.0
38 24.0
39 24.0
40 24.0
41 24.0
42 24.0
43 24.0
44 24.0
45 24.0
46 24.0
47 24.0
48 24.0
49 24.0
50 24.0
51 24.0
52 24.0
53 24.0
54 24.0
55 24.0
56 24.0
57 24.0
58 24.0
59 24.0
60 24.0
61 24.0
62 24.0
63 24.0
64 24.0
65 24.0
66 24.0
67 24.0
68 24.0
69 24.0
70 24.0
71 24.0
72 24.0
73 24.0
74 24.0
75 24.0
76 24.0
77 24.0
78 24.0
79 24.0
80 32.0
81 32.0
82 32.0
83 32.0
84 32.0
85 32.0
86 16.0
87 16.0
88 16.0
89 16.0
90 16.0
91 16.0
92 16.0
93 16.0
94 16.0
95 16.0
96 16.0
97 16.0
98 16.0
99 16.0
100 16.0
101 16.0
102 24.0
103 24.0
104 24.0
105 24.0
106 16.0
107 16.0
108 32.0
109 32.0
110 32.0
111 32.0
112 32.0
113 32.0
114 16.0
115 16.0
116 16.0
117 16.0
118 16.0
119 16.0
120 16.0
121 16.0
122 16.0
123 16.0
124 16.0
125 16.0
126 8.0
127 8.0
128 8.0
129 8.0
130 8.0
131 8.0
132 18.0
133 18.0
134 18.0
135 18.0
136 18.0
137 18.0
138 18.0
139 18.0
140 18.0
141 18.0
142 18.0
143 18.0
144 18.0
145 18.0
146 18.0
147 18.0
148 18.0
149 18.0
150 18.0
151 18.0
152 18.0
153 18.0
154 18.0
155 18.0
156 18.0
157 18.0
158 18.0
159 18.0
160 18.0
161 18.0
162 18.0
163 18.0
164 18.0
165 18.0
166 18.0
167 18.0
168 18.0
169 18.0
170 18.0
171 18.0
172 18.0
173 18.0
174 18.0
175 18.0
176 18.0
177 18.0
178 18.0
179 18.0
180 18.0
181 18.0
182 18.0
183 18.0
184 18.0
185 18.0
186 18.0
187 18.0
188 18.0
189 18.0
190 18.0
191 18.0
192 18.0
193 18.0
194 18.0
195 18.0
196 18.0
197 18.0
198 18.0
199 18.0
200 8.0
201 8.0
202 8.0
203 8.0
204 4.0
205 4.0
206 4.0
207 4.0
208 48.0
209 48.0
210 48.0
211 48.0
212 48.0
213 48.0
214 48.0
215 48.0
216 48.0
217 48.0
218 48.0
219 48.0
220 48.0
221 48.0
222 8.0
223 8.0
224 8.0
225 8.0
226 4.0
227 4.0
228 4.0
229 4.0
230 32.0
231 32.0
232 32.0
233 32.0
234 32.0
235 32.0
236 32.0
237 32.0
238 32.0
239 32.0
240 32.0
241 32.0
242 8.0
243 8.0
244 8.0
245 8.0
246 8.0
247 8.0
248 8.0
249 8.0
250 4.0
251 4.0
252 4.0
253 4.0
254 4.0
255 4.0
256 4.0
257 4.0
258 4.0
259 4.0
260 4.0
261 4.0
262 128.0
263 128.0
264 128.0
265 128.0
266 128.0
267 128.0
268 128.0
269 32.0
270 32.0
271 32.0
272 4.0
273 8.0
274 4.0
275 8.0
276 8.0
277 8.0
278 8.0
279 6.0
280 6.0
281 6.0
282 6.0
283 6.0
284 6.0
285 6.0
286 32.0
287 32.0
288 8.0
289 8.0
290 16.0
291 16.0
292 32.0
293 24.0
294 24.0
295 24.0
296 24.0
297 24.0
298 24.0
299 24.0
300 24.0
301 24.0
302 24.0
303 24.0
304 24.0
305 8.0
306 8.0
307 8.0
308 8.0
309 16.0
310 16.0
311 16.0
312 16.0
313 32.0
314 32.0
315 32.0
316 32.0
317 8.0
318 8.0
319 8.0
320 8.0
321 24.0
322 24.0
323 24.0
324 24.0
325 24.0
326 24.0
327 24.0
328 24.0
329 24.0
330 24.0
331 24.0
332 24.0
333 24.0
334 24.0
335 24.0
336 24.0
337 24.0
338 24.0
339 24.0
340 24.0
341 8.0
342 8.0
343 8.0
344 8.0
345 16.0
346 16.0
347 16.0
348 16.0
349 32.0
350 32.0
351 32.0
352 32.0
353 8.0
354 8.0
355 8.0
356 8.0
357 24.0
358 24.0
359 24.0
360 24.0
361 8.0
362 8.0
363 16.0
364 16.0
365 32.0
366 32.0
367 8.0
368 8.0
369 32.0
370 32.0
371 32.0
372 32.0
373 32.0
374 32.0
375 32.0
376 32.0
377 32.0
378 32.0
379 32.0
380 4.0
381 4.0
382 4.0
383 4.0
384 8.0
385 8.0
386 8.0
387 8.0
388 8.0
389 8.0
390 8.0
391 8.0
392 64.0
393 64.0
394 64.0
395 64.0
396 64.0
397 64.0
398 16.0
399 24.0
400 24.0
401 24.0
402 24.0
403 24.0
404 24.0
405 24.0
406 24.0
407 24.0
408 24.0
409 24.0
410 24.0
411 24.0
412 24.0
413 24.0
414 24.0
415 24.0
416 24.0
417 24.0
418 24.0
419 24.0
420 24.0
421 24.0
422 24.0
423 24.0
424 24.0
425 24.0
426 24.0
427 24.0
428 24.0
429 24.0
430 24.0
431 24.0
432 24.0
433 24.0
434 24.0
435 24.0
436 24.0
437 24.0
438 24.0
439 24.0
440 24.0
441 24.0
442 24.0
443 24.0
444 24.0
445 24.0
446 24.0
447 24.0
448 24.0
449 24.0
450 24.0
451 24.0
452 24.0
453 24.0
454 24.0
455 24.0
456 24.0
457 24.0
458 24.0
459 24.0
460 24.0
461 24.0
462 24.0
463 24.0
464 24.0
465 24.0
466 24.0
467 24.0
468 24.0
469 24.0
470 24.0
471 24.0
472 24.0
473 24.0
474 24.0
475 24.0
476 24.0
477 32.0
478 32.0
479 32.0
480 32.0
481 32.0
482 32.0
483 32.0
484 32.0
485 32.0
486 16.0
487 16.0
488 16.0
489 16.0
490 16.0
491 16.0
492 16.0
493 16.0
494 16.0
495 16.0
496 16.0
497 16.0
498 16.0
499 16.0
500 16.0
501 16.0
502 16.0
503 16.0
504 16.0
505 16.0
506 16.0
507 16.0
508 16.0
509 16.0
510 16.0
511 16.0
512 16.0
513 16.0
514 16.0
515 16.0
516 24.0
517 24.0
518 24.0
519 24.0
520 24.0
521 24.0
522 16.0
523 16.0
524 0.0
525 0.0
526 0.0
527 32.0
528 32.0
529 32.0
530 32.0
531 32.0
532 32.0
533 32.0
534 32.0
535 32.0
536 16.0
537 16.0
538 16.0
539 16.0
540 16.0
541 16.0
542 16.0
543 16.0
544 16.0
545 16.0
546 16.0
547 16.0
548 16.0
549 16.0
550 16.0
551 16.0
552 16.0
553 16.0
554 18.0
555 18.0
556 18.0
557 18.0
558 18.0
559 18.0
560 18.0
561 18.0
562 18.0
563 18.0
564 18.0
565 18.0
566 18.0
567 18.0
568 18.0
569 18.0
570 18.0
571 18.0
572 18.0
573 18.0
574 18.0
575 18.0
576 18.0
577 18.0
578 18.0
579 18.0
580 18.0
581 18.0
582 18.0
583 18.0
584 18.0
585 18.0
586 18.0
587 18.0
588 18.0
589 18.0
590 18.0
591 18.0
592 18.0
593 18.0
594 18.0
595 18.0
596 18.0
597 18.0
598 18.0
599 18.0
600 18.0
601 18.0
602 18.0
603 18.0
604 18.0
605 18.0
606 18.0
607 18.0
608 18.0
609 18.0
610 18.0
611 18.0
612 18.0
613 18.0
614 18.0
615 18.0
616 18.0
617 18.0
618 18.0
619 18.0
620 18.0
621 18.0
622 18.0
623 18.0
624 18.0
625 18.0
626 18.0
627 18.0
628 18.0
629 18.0
630 18.0
631 18.0
632 18.0
633 18.0
634 18.0
635 18.0
636 18.0
637 18.0
638 18.0
639 18.0
640 18.0
641 18.0
642 18.0
643 18.0
644 18.0
645 18.0
646 18.0
647 18.0
648 18.0
649 18.0
650 18.0
651 18.0
652 18.0
653 18.0
654 18.0
655 18.0
656 8.0
657 8.0
658 8.0
659 8.0
660 8.0
661 8.0
662 8.0
663 8.0
664 8.0
665 4.0
666 4.0
667 4.0
668 4.0
669 4.0
670 4.0
671 48.0
672 48.0
673 48.0
674 48.0
675 48.0
676 48.0
677 48.0
678 48.0
679 48.0
680 48.0
681 48.0
682 48.0
683 48.0
684 48.0
685 48.0
686 48.0
687 48.0
688 48.0
689 48.0
690 48.0
691 48.0
692 8.0
693 8.0
694 8.0
695 8.0
696 8.0
697 8.0
698 4.0
699 4.0
700 4.0
701 4.0
702 4.0
703 4.0
704 32.0
705 32.0
706 32.0
707 32.0
708 32.0
709 32.0
710 32.0
711 32.0
712 32.0
713 32.0
714 32.0
715 32.0
716 32.0
717 32.0
718 32.0
719 32.0
720 32.0
721 32.0
722 8.0
723 8.0
724 8.0
725 8.0
726 8.0
727 8.0
728 4.0
729 4.0
730 4.0
731 4.0
732 4.0
733 4.0
734 4.0
735 4.0
736 4.0
737 4.0
738 4.0
739 4.0
740 4.0
741 4.0
742 4.0
743 4.0
744 4.0
745 4.0
746 4.0
747 4.0
748 4.0
749 4.0
750 4.0
751 4.0
752 8.0
753 8.0
754 8.0
755 8.0
756 8.0
757 8.0
758 8.0
759 8.0
760 0.0
761 0.0
762 0.0
763 2.0
764 2.0
765 2.0
766 2.0
767 2.0
768 2.0
769 2.0
770 2.0
771 2.0
772 2.0
773 2.0
774 2.0
775 2.0
776 2.0
777 2.0
778 2.0
779 2.0
780 2.0
781 2.0
782 2.0
783 2.0
784 2.0
785 2.0
786 2.0
787 2.0
788 2.0
789 2.0
790 2.0
791 2.0
792 2.0
793 2.0
794 2.0
795 2.0
796 2.0
797 2.0
798 2.0
799 2.0
800 2.0
801 2.0
802 2.0
803 2.0
804 2.0
805 2.0
806 2.0
807 2.0
808 2.0
809 2.0
810 2.0
811 2.0
812 2.0
813 2.0
814 2.0
815 2.0
816 2.0
817 2.0
818 2.0
819 1.0
820 1.0
821 1.0
822 1.0
823 1.0
824 1.0
825 1.0
826 1.0
827 3.0
828 3.0
829 3.0
830 3.0
831 3.0
832 3.0
833 3.0
834 3.0
835 3.0
836 3.0
837 3.0
838 3.0
839 3.0
840 3.0
841 3.0
842 3.0
843 3.0
844 3.0
845 3.0
846 3.0
847 3.0
848 3.0
849 3.0
850 3.0
851 3.0
852 3.0
853 3.0
854 3.0
855 3.0
856 3.0
857 3.0
858 3.0
859 3.0
860 3.0
861 3.0
862 3.0
863 3.0
864 3.0
865 3.0
866 3.0
867 3.0
868 3.0
869 3.0
870 3.0
871 3.0
872 3.0
873 3.0
874 3.0
875 3.0
876 3.0
877 3.0
878 3.0
879 3.0
880 3.0
881 3.0
882 3.0
883 3.0
884 3.0
885 3.0
886 3.0
887 3.0
888 3.0
889 3.0
890 3.0
891 3.0
892 3.0
893 3.0
894 3.0
895 3.0
896 3.0
897 3.0
898 3.0
899 3.0
900 3.0
901 3.0
902 3.0
903 3.0
904 3.0
905 3.0
906 3.0
907 3.0
908 3.0
909 3.0
910 3.0
911 3.0
912 3.0
913 3.0
914 3.0
915 3.0
916 3.0
917 3.0
918 3.0
919 3.0
920 3.0
921 3.0
922 3.0
923 3.0
924 3.0
925 3.0
926 3.0
927 3.0
928 3.0
929 3.0
930 3.0
931 3.0
932 3.0
933 3.0
934 3.0
935 3.0
936 3.0
937 3.0
938 3.0
939 3.0
940 3.0
941 3.0
942 3.0
943 3.0
944 3.0
945 3.0
946 3.0
947 3.0
948 3.0
949 3.0
950 3.0
951 3.0
952 3.0
953 3.0
954 3.0
955 3.0
956 3.0
957 3.0
958 3.0
959 3.0
960 3.0
961 3.0
962 3.0
963 3.0
964 3.0
965 3.0
966 3.0
967 3.0
968 3.0
969 3.0
970 3.0
971 3.0
972 3.0
973 3.0
974 3.0
975 3.0
976 3.0
977 3.0
978 3.0
979 3.0
980 3.0
981 3.0
982 3.0
983 3.0
984 3.0
985 3.0
986 3.0
987 3.0
988 3.0
989 3.0
990 3.0
991 3.0
992 3.0
993 3.0
994 3.0
995 3.0
996 3.0
997 3.0
998 3.0
999 3.0
1000 3.0
1001 3.0
1002 3.0
1003 3.0
1004 3.0
1005 3.0
1006 3.0
1007 3.0
1008 3.0
1009 3.0
1010 3.0
1011 3.0
1012 3.0
1013 3.0
1014 3.0
1015 3.0
1016 3.0
1017 3.0
1018 3.0
1019 3.0
1020 3.0
1021 3.0
1022 3.0
1023 3.0
1024 3.0
1025 3.0
1026 3.0
1027 3.0
1028 3.0
1029 3.0
1030 3.0
1031 3.0
1032 3.0
1033 3.0
1034 3.0
1035 3.0
1036 3.0
1037 3.0
1038 3.0
1039 3.0
1040 3.0
1041 3.0
1042 3.0
1043 3.0
1044 3.0
1045 3.0
1046 3.0
1047 3.0
1048 3.0
1049 3.0
1050 3.0
1051 1.0
1052 1.0
1053 1.0
1054 1.0
1055 1.0
1056 1.0
1057 1.0
1058 1.0
1059 2.0
1060 2.0
1061 2.0
1062 2.0
1063 2.0
1064 2.0
1065 2.0
1066 2.0
1067 2.0
1068 2.0
1069 2.0
1070 2.0
1071 2.0
1072 2.0
1073 2.0
1074 2.0
1075 2.0
1076 2.0
1077 2.0
1078 2.0
1079 2.0
1080 2.0
1081 2.0
1082 2.0
1083 2.0
1084 2.0
1085 2.0
1086 2.0
1087 2.0
1088 2.0
1089 2.0
1090 2.0
1091 2.0
1092 2.0
1093 2.0
1094 2.0
1095 2.0
1096 2.0
1097 2.0
1098 2.0
1099 0.5
1100 0.5
1101 0.5
1102 0.5
1103 0.5
1104 0.5
1105 0.5
1106 0.5
1107 0.5
1108 0.5
1109 0.5
1110 0.5
1111 0.5
1112 0.5
1113 0.5
1114 0.5
1115 1.0
1116 1.0
1117 1.0
1118 1.0
1119 1.0
1120 1.0
1121 1.0
1122 1.0
1123 1.0
1124 1.0
1125 1.0
1126 1.0
1127 1.0
1128 1.0
1129 1.0
1130 1.0
1131 1.0
1132 1.0
1133 1.0
1134 1.0
1135 1.0
1136 1.0
1137 1.0
1138 1.0
1139 1.0
1140 1.0
1141 1.0
1142 1.0
1143 1.0
1144 1.0
1145 1.0
1146 1.0
1147 1.0
1148 1.0
1149 1.0
1150 1.0
1151 1.0
1152 1.0
1153 1.0
1154 1.0
1155 0.0
1156 0.0
1157 0.0
1158 0.0
1159 0.0
1160 0.0
1161 0.0
1162 0.0
1163 0.0
1164 2.0
1165 2.0
1166 2.0
1167 2.0
1168 2.0
1169 2.0
1170 2.0
1171 2.0
1172 2.0
1173 2.0
1174 1.0
1175 1.0
1176 3.0
1177 3.0
1178 3.0
1179 3.0
1180 3.0
1181 3.0
1182 3.0
1183 3.0
1184 3.0
1185 3.0
1186 3.0
1187 3.0
1188 3.0
1189 3.0
1190 3.0
1191 3.0
1192 3.0
1193 3.0
1194 3.0
1195 3.0
1196 3.0
1197 3.0
1198 3.0
1199 1.0
1200 1.0
1201 2.0
1202 2.0
1203 2.0
1204 2.0
1205 2.0
1206 2.0
1207 2.0
1208 2.0
1209 0.5
1210 0.5
1211 0.5
1212 0.5
1213 0.5
1214 0.5
1215 1.0
1216 1.0
1217 1.0
1218 1.0
1219 1.0
1220 1.0
1221 1.0
1222 1.0
1223 0.0
1224 0.0
1225 0.0
1226 32.0
1227 32.0
1228 32.0
1229 32.0
1230 32.0
1231 32.0
1232 16.0
1233 16.0
1234 16.0
1235 16.0
1236 16.0
1237 16.0
1238 16.0
1239 16.0
1240 16.0
1241 16.0
1242 16.0
1243 16.0
1244 18.0
1245 18.0
1246 18.0
1247 18.0
1248 18.0
1249 18.0
1250 18.0
1251 18.0
1252 18.0
1253 18.0
1254 18.0
1255 18.0
1256 18.0
1257 18.0
1258 18.0
1259 18.0
1260 18.0
1261 18.0
1262 18.0
1263 18.0
1264 18.0
1265 18.0
1266 18.0
1267 18.0
1268 18.0
1269 18.0
1270 18.0
1271 18.0
1272 18.0
1273 18.0
1274 18.0
1275 18.0
1276 18.0
1277 18.0
1278 18.0
1279 18.0
1280 8.0
1281 8.0
1282 8.0
1283 8.0
1284 4.0
1285 4.0
1286 4.0
1287 4.0
1288 48.0
1289 48.0
1290 48.0
1291 48.0
1292 48.0
1293 48.0
1294 8.0
1295 8.0
1296 8.0
1297 8.0
1298 4.0
1299 4.0
1300 4.0
1301 4.0
1302 32.0
1303 32.0
1304 32.0
1305 32.0
1306 32.0
1307 32.0
1308 32.0
1309 32.0
1310 8.0
1311 8.0
1312 8.0
1313 8.0
1314 4.0
1315 4.0
1316 4.0
1317 4.0
1318 4.0
1319 4.0
1320 4.0
1321 4.0
1322 4.0
1323 4.0
1324 4.0
1325 4.0
1326 4.0
1327 4.0
1328 4.0
1329 4.0
1330 8.0
1331 8.0
1332 8.0
1333 8.0
1334 8.0
1335 8.0
1336 8.0
1337 8.0
1338 8.0
1339 8.0
1340 8.0
1341 8.0
1342 32.0
1343 32.0
1344 32.0
1345 32.0
1346 0.0
1347 0.0
1348 24.0
1349 24.0
1350 24.0
1351 24.0
1352 24.0
1353 24.0
1354 24.0
1355 24.0
1356 24.0
1357 24.0
1358 24.0
1359 24.0
1360 24.0
1361 24.0
1362 24.0
1363 24.0
1364 24.0
1365 24.0
1366 24.0
1367 24.0
1368 24.0
1369 24.0
1370 24.0
1371 24.0
1372 24.0
1373 24.0
1374 24.0
1375 24.0
1376 24.0
1377 24.0
1378 24.0
1379 24.0
1380 24.0
1381 24.0
1382 24.0
1383 24.0
1384 24.0
1385 24.0
1386 24.0
1387 24.0
1388 8.0
1389 8.0
1390 8.0
1391 8.0
1392 8.0
1393 8.0
1394 8.0
1395 8.0
1396 16.0
1397 16.0
1398 16.0
1399 16.0
1400 16.0
1401 16.0
1402 16.0
1403 16.0
1404 32.0
1405 32.0
1406 32.0
1407 32.0
1408 32.0
1409 32.0
1410 32.0
1411 32.0
1412 8.0
1413 8.0
1414 8.0
1415 8.0
1416 8.0
1417 8.0
1418 8.0
1419 8.0
1420 24.0
1421 24.0
1422 24.0
1423 24.0
1424 24.0
1425 24.0
1426 24.0
1427 24.0
1428 24.0
1429 24.0
1430 8.0
1431 8.0
1432 8.0
1433 8.0
1434 16.0
1435 16.0
1436 32.0
1437 32.0
1438 16.0
1439 16.0
1440 8.0
1441 8.0
1442 128.0
1443 4.0
1444 8.0
1445 4.0
1446 32.0
1447 24.0
1448 24.0
1449 24.0
1450 24.0
1451 24.0
1452 24.0
1453 24.0
1454 24.0
1455 24.0
1456 24.0
1457 8.0
1458 8.0
1459 8.0
1460 8.0
1461 16.0
1462 16.0
1463 32.0
1464 32.0
1465 16.0
1466 16.0
1467 8.0
1468 8.0
1469 128.0
1470 4.0
1471 8.0
1472 4.0
1473 32.0
1474 4.0
1475 6.0
1476 90.0
1477 6.0
1478 2.0
1479 5.0
1480 6.0
1481 7.0
1482 6.0
1483 4.0
1484 10.0
1485 6.0
1486 7.0
1487 7.0
1488 13.0
1489 13.0
1490 7.0
1491 7.0
1492 34.0
1493 34.0
1494 34.0
1495 34.0
1496 34.0
1497 34.0
1498 34.0
1499 34.0
1500 4.0
1501 4.0
1502 134.0
1503 134.0
1504 134.0
1505 134.0
1506 32.0
1507 32.0
1508 32.0
1509 32.0
1510 32.0
1511 14.0
1512 14.0
1513 12.0
1514 12.0
1515 12.0
1516 12.0
1517 4.0
1518 4.0
1519 32.0
1520 32.0
1521 32.0;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0
143 0.0
144 0.0
145 0.0
146 0.0
147 0.0
148 0.0
149 0.0
150 0.0
151 0.0
152 0.0
153 0.0
154 0.0
155 0.0
156 0.0
157 0.0
158 0.0
159 0.0
160 0.0
161 0.0
162 0.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 0.0
179 0.0
180 0.0
181 0.0
182 0.0
183 0.0
184 0.0
185 0.0
186 0.0
187 0.0
188 0.0
189 0.0
190 0.0
191 0.0
192 0.0
193 0.0
194 0.0
195 0.0
196 0.0
197 0.0
198 0.0
199 0.0
200 0.0
201 0.0
202 0.0
203 0.0
204 0.0
205 0.0
206 0.0
207 0.0
208 0.0
209 0.0
210 0.0
211 0.0
212 0.0
213 0.0
214 0.0
215 0.0
216 0.0
217 0.0
218 0.0
219 0.0
220 0.0
221 0.0
222 0.0
223 0.0
224 0.0
225 0.0
226 0.0
227 0.0
228 0.0
229 0.0
230 0.0
231 0.0
232 0.0
233 0.0
234 0.0
235 0.0
236 0.0
237 0.0
238 0.0
239 0.0
240 0.0
241 0.0
242 0.0
243 0.0
244 0.0
245 0.0
246 0.0
247 0.0
248 0.0
249 0.0
250 0.0
251 0.0
252 0.0
253 0.0
254 0.0
255 0.0
256 0.0
257 0.0
258 0.0
259 0.0
260 0.0
261 0.0
262 100.0
263 100.0
264 100.0
265 100.0
266 100.0
267 100.0
268 100.0
269 0.0
270 0.0
271 0.0
272 0.0
273 0.0
274 0.0
275 0.0
276 0.0
277 0.0
278 0.0
279 0.0
280 0.0
281 0.0
282 0.0
283 0.0
284 0.0
285 0.0
286 0.0
287 0.0
288 0.0
289 0.0
290 0.0
291 0.0
292 0.0
293 375.0
294 375.0
295 375.0
296 375.0
297 375.0
298 375.0
299 375.0
300 375.0
301 375.0
302 375.0
303 375.0
304 375.0
305 80.0
306 80.0
307 80.0
308 80.0
309 120.0
310 120.0
311 120.0
312 120.0
313 235.0
314 235.0
315 235.0
316 235.0
317 10.0
318 10.0
319 10.0
320 10.0
321 375.0
322 375.0
323 375.0
324 375.0
325 375.0
326 375.0
327 375.0
328 375.0
329 375.0
330 375.0
331 375.0
332 375.0
333 375.0
334 375.0
335 375.0
336 375.0
337 375.0
338 375.0
339 375.0
340 375.0
341 80.0
342 80.0
343 80.0
344 80.0
345 120.0
346 120.0
347 120.0
348 120.0
349 235.0
350 235.0
351 235.0
352 235.0
353 10.0
354 10.0
355 10.0
356 10.0
357 375.0
358 375.0
359 375.0
360 375.0
361 80.0
362 80.0
363 120.0
364 120.0
365 235.0
366 235.0
367 10.0
368 10.0
369 0.0
370 100.0
371 100.0
372 100.0
373 100.0
374 100.0
375 100.0
376 100.0
377 100.0
378 100.0
379 100.0
380 0.0
381 0.0
382 0.0
383 0.0
384 0.0
385 0.0
386 0.0
387 0.0
388 0.0
389 0.0
390 0.0
391 0.0
392 100.0
393 100.0
394 100.0
395 100.0
396 100.0
397 100.0
398 0.0
399 0.0
400 0.0
401 0.0
402 0.0
403 0.0
404 0.0
405 0.0
406 0.0
407 0.0
408 0.0
409 0.0
410 0.0
411 0.0
412 0.0
413 0.0
414 0.0
415 0.0
416 0.0
417 0.0
418 0.0
419 0.0
420 0.0
421 0.0
422 0.0
423 0.0
424 0.0
425 0.0
426 0.0
427 0.0
428 0.0
429 0.0
430 0.0
431 0.0
432 0.0
433 0.0
434 0.0
435 0.0
436 0.0
437 0.0
438 0.0
439 0.0
440 0.0
441 0.0
442 0.0
443 0.0
444 0.0
445 0.0
446 0.0
447 0.0
448 0.0
449 0.0
450 0.0
451 0.0
452 0.0
453 0.0
454 0.0
455 0.0
456 0.0
457 0.0
458 0.0
459 0.0
460 0.0
461 0.0
462 0.0
463 0.0
464 0.0
465 0.0
466 0.0
467 0.0
468 0.0
469 0.0
470 0.0
471 0.0
472 0.0
473 0.0
474 0.0
475 0.0
476 0.0
477 0.0
478 0.0
479 0.0
480 0.0
481 0.0
482 0.0
483 0.0
484 0.0
485 0.0
486 0.0
487 0.0
488 0.0
489 0.0
490 0.0
491 0.0
492 0.0
493 0.0
494 0.0
495 0.0
496 0.0
497 0.0
498 0.0
499 0.0
500 0.0
501 0.0
502 0.0
503 0.0
504 0.0
505 0.0
506 0.0
507 0.0
508 0.0
509 0.0
510 0.0
511 0.0
512 0.0
513 0.0
514 0.0
515 0.0
516 0.0
517 0.0
518 0.0
519 0.0
520 0.0
521 0.0
522 0.0
523 0.0
524 0.0
525 0.0
526 0.0
527 0.0
528 0.0
529 0.0
530 0.0
531 0.0
532 0.0
533 0.0
534 0.0
535 0.0
536 0.0
537 0.0
538 0.0
539 0.0
540 0.0
541 0.0
542 0.0
543 0.0
544 0.0
545 0.0
546 0.0
547 0.0
548 0.0
549 0.0
550 0.0
551 0.0
552 0.0
553 0.0
554 0.0
555 0.0
556 0.0
557 0.0
558 0.0
559 0.0
560 0.0
561 0.0
562 0.0
563 0.0
564 0.0
565 0.0
566 0.0
567 0.0
568 0.0
569 0.0
570 0.0
571 0.0
572 0.0
573 0.0
574 0.0
575 0.0
576 0.0
577 0.0
578 0.0
579 0.0
580 0.0
581 0.0
582 0.0
583 0.0
584 0.0
585 0.0
586 0.0
587 0.0
588 0.0
589 0.0
590 0.0
591 0.0
592 0.0
593 0.0
594 0.0
595 0.0
596 0.0
597 0.0
598 0.0
599 0.0
600 0.0
601 0.0
602 0.0
603 0.0
604 0.0
605 0.0
606 0.0
607 0.0
608 0.0
609 0.0
610 0.0
611 0.0
612 0.0
613 0.0
614 0.0
615 0.0
616 0.0
617 0.0
618 0.0
619 0.0
620 0.0
621 0.0
622 0.0
623 0.0
624 0.0
625 0.0
626 0.0
627 0.0
628 0.0
629 0.0
630 0.0
631 0.0
632 0.0
633 0.0
634 0.0
635 0.0
636 0.0
637 0.0
638 0.0
639 0.0
640 0.0
641 0.0
642 0.0
643 0.0
644 0.0
645 0.0
646 0.0
647 0.0
648 0.0
649 0.0
650 0.0
651 0.0
652 0.0
653 0.0
654 0.0
655 0.0
656 0.0
657 0.0
658 0.0
659 0.0
660 0.0
661 0.0
662 0.0
663 0.0
664 0.0
665 0.0
666 0.0
667 0.0
668 0.0
669 0.0
670 0.0
671 0.0
672 0.0
673 0.0
674 0.0
675 0.0
676 0.0
677 0.0
678 0.0
679 0.0
680 0.0
681 0.0
682 0.0
683 0.0
684 0.0
685 0.0
686 0.0
687 0.0
688 0.0
689 0.0
690 0.0
691 0.0
692 0.0
693 0.0
694 0.0
695 0.0
696 0.0
697 0.0
698 0.0
699 0.0
700 0.0
701 0.0
702 0.0
703 0.0
704 0.0
705 0.0
706 0.0
707 0.0
708 0.0
709 0.0
710 0.0
711 0.0
712 0.0
713 0.0
714 0.0
715 0.0
716 0.0
717 0.0
718 0.0
719 0.0
720 0.0
721 0.0
722 0.0
723 0.0
724 0.0
725 0.0
726 0.0
727 0.0
728 0.0
729 0.0
730 0.0
731 0.0
732 0.0
733 0.0
734 0.0
735 0.0
736 0.0
737 0.0
738 0.0
739 0.0
740 0.0
741 0.0
742 0.0
743 0.0
744 0.0
745 0.0
746 0.0
747 0.0
748 0.0
749 0.0
750 0.0
751 0.0
752 0.0
753 0.0
754 0.0
755 0.0
756 0.0
757 0.0
758 0.0
759 0.0
760 0.0
761 0.0
762 0.0
763 0.0
764 0.0
765 0.0
766 0.0
767 0.0
768 0.0
769 0.0
770 0.0
771 0.0
772 0.0
773 0.0
774 0.0
775 0.0
776 0.0
777 0.0
778 0.0
779 0.0
780 0.0
781 0.0
782 0.0
783 0.0
784 0.0
785 0.0
786 0.0
787 0.0
788 0.0
789 0.0
790 0.0
791 0.0
792 0.0
793 0.0
794 0.0
795 0.0
796 0.0
797 0.0
798 0.0
799 0.0
800 0.0
801 0.0
802 0.0
803 0.0
804 0.0
805 0.0
806 0.0
807 0.0
808 0.0
809 0.0
810 0.0
811 0.0
812 0.0
813 0.0
814 0.0
815 0.0
816 0.0
817 0.0
818 0.0
819 0.0
820 0.0
821 0.0
822 0.0
823 0.0
824 0.0
825 0.0
826 0.0
827 0.0
828 0.0
829 0.0
830 0.0
831 0.0
832 0.0
833 0.0
834 0.0
835 0.0
836 0.0
837 0.0
838 0.0
839 0.0
840 0.0
841 0.0
842 0.0
843 0.0
844 0.0
845 0.0
846 0.0
847 0.0
848 0.0
849 0.0
850 0.0
851 0.0
852 0.0
853 0.0
854 0.0
855 0.0
856 0.0
857 0.0
858 0.0
859 0.0
860 0.0
861 0.0
862 0.0
863 0.0
864 0.0
865 0.0
866 0.0
867 0.0
868 0.0
869 0.0
870 0.0
871 0.0
872 0.0
873 0.0
874 0.0
875 0.0
876 0.0
877 0.0
878 0.0
879 0.0
880 0.0
881 0.0
882 0.0
883 0.0
884 0.0
885 0.0
886 0.0
887 0.0
888 0.0
889 0.0
890 0.0
891 0.0
892 0.0
893 0.0
894 0.0
895 0.0
896 0.0
897 0.0
898 0.0
899 0.0
900 0.0
901 0.0
902 0.0
903 0.0
904 0.0
905 0.0
906 0.0
907 0.0
908 0.0
909 0.0
910 0.0
911 0.0
912 0.0
913 0.0
914 0.0
915 0.0
916 0.0
917 0.0
918 0.0
919 0.0
920 0.0
921 0.0
922 0.0
923 0.0
924 0.0
925 0.0
926 0.0
927 0.0
928 0.0
929 0.0
930 0.0
931 0.0
932 0.0
933 0.0
934 0.0
935 0.0
936 0.0
937 0.0
938 0.0
939 0.0
940 0.0
941 0.0
942 0.0
943 0.0
944 0.0
945 0.0
946 0.0
947 0.0
948 0.0
949 0.0
950 0.0
951 0.0
952 0.0
953 0.0
954 0.0
955 0.0
956 0.0
957 0.0
958 0.0
959 0.0
960 0.0
961 0.0
962 0.0
963 0.0
964 0.0
965 0.0
966 0.0
967 0.0
968 0.0
969 0.0
970 0.0
971 0.0
972 0.0
973 0.0
974 0.0
975 0.0
976 0.0
977 0.0
978 0.0
979 0.0
980 0.0
981 0.0
982 0.0
983 0.0
984 0.0
985 0.0
986 0.0
987 0.0
988 0.0
989 0.0
990 0.0
991 0.0
992 0.0
993 0.0
994 0.0
995 0.0
996 0.0
997 0.0
998 0.0
999 0.0
1000 0.0
1001 0.0
1002 0.0
1003 0.0
1004 0.0
1005 0.0
1006 0.0
1007 0.0
1008 0.0
1009 0.0
1010 0.0
1011 0.0
1012 0.0
1013 0.0
1014 0.0
1015 0.0
1016 0.0
1017 0.0
1018 0.0
1019 0.0
1020 0.0
1021 0.0
1022 0.0
1023 0.0
1024 0.0
1025 0.0
1026 0.0
1027 0.0
1028 0.0
1029 0.0
1030 0.0
1031 0.0
1032 0.0
1033 0.0
1034 0.0
1035 0.0
1036 0.0
1037 0.0
1038 0.0
1039 0.0
1040 0.0
1041 0.0
1042 0.0
1043 0.0
1044 0.0
1045 0.0
1046 0.0
1047 0.0
1048 0.0
1049 0.0
1050 0.0
1051 0.0
1052 0.0
1053 0.0
1054 0.0
1055 0.0
1056 0.0
1057 0.0
1058 0.0
1059 0.0
1060 0.0
1061 0.0
1062 0.0
1063 0.0
1064 0.0
1065 0.0
1066 0.0
1067 0.0
1068 0.0
1069 0.0
1070 0.0
1071 0.0
1072 0.0
1073 0.0
1074 0.0
1075 0.0
1076 0.0
1077 0.0
1078 0.0
1079 0.0
1080 0.0
1081 0.0
1082 0.0
1083 0.0
1084 0.0
1085 0.0
1086 0.0
1087 0.0
1088 0.0
1089 0.0
1090 0.0
1091 0.0
1092 0.0
1093 0.0
1094 0.0
1095 0.0
1096 0.0
1097 0.0
1098 0.0
1099 0.0
1100 0.0
1101 0.0
1102 0.0
1103 0.0
1104 0.0
1105 0.0
1106 0.0
1107 0.0
1108 0.0
1109 0.0
1110 0.0
1111 0.0
1112 0.0
1113 0.0
1114 0.0
1115 0.0
1116 0.0
1117 0.0
1118 0.0
1119 0.0
1120 0.0
1121 0.0
1122 0.0
1123 0.0
1124 0.0
1125 0.0
1126 0.0
1127 0.0
1128 0.0
1129 0.0
1130 0.0
1131 0.0
1132 0.0
1133 0.0
1134 0.0
1135 0.0
1136 0.0
1137 0.0
1138 0.0
1139 0.0
1140 0.0
1141 0.0
1142 0.0
1143 0.0
1144 0.0
1145 0.0
1146 0.0
1147 0.0
1148 0.0
1149 0.0
1150 0.0
1151 0.0
1152 0.0
1153 0.0
1154 0.0
1155 0.0
1156 0.0
1157 0.0
1158 0.0
1159 0.0
1160 0.0
1161 0.0
1162 0.0
1163 0.0
1164 200.0
1165 200.0
1166 200.0
1167 200.0
1168 200.0
1169 200.0
1170 200.0
1171 200.0
1172 200.0
1173 200.0
1174 150.0
1175 150.0
1176 100.0
1177 100.0
1178 100.0
1179 100.0
1180 100.0
1181 100.0
1182 100.0
1183 100.0
1184 100.0
1185 100.0
1186 100.0
1187 100.0
1188 100.0
1189 100.0
1190 100.0
1191 100.0
1192 100.0
1193 100.0
1194 100.0
1195 100.0
1196 100.0
1197 100.0
1198 100.0
1199 200.0
1200 200.0
1201 100.0
1202 100.0
1203 100.0
1204 100.0
1205 300.0
1206 300.0
1207 300.0
1208 300.0
1209 0.0
1210 0.0
1211 0.0
1212 0.0
1213 0.0
1214 0.0
1215 0.0
1216 0.0
1217 0.0
1218 0.0
1219 0.0
1220 0.0
1221 0.0
1222 0.0
1223 0.0
1224 0.0
1225 0.0
1226 0.0
1227 0.0
1228 0.0
1229 0.0
1230 0.0
1231 0.0
1232 0.0
1233 0.0
1234 0.0
1235 0.0
1236 0.0
1237 0.0
1238 0.0
1239 0.0
1240 0.0
1241 0.0
1242 0.0
1243 0.0
1244 0.0
1245 0.0
1246 0.0
1247 0.0
1248 0.0
1249 0.0
1250 0.0
1251 0.0
1252 0.0
1253 0.0
1254 0.0
1255 0.0
1256 0.0
1257 0.0
1258 0.0
1259 0.0
1260 0.0
1261 0.0
1262 0.0
1263 0.0
1264 0.0
1265 0.0
1266 0.0
1267 0.0
1268 0.0
1269 0.0
1270 0.0
1271 0.0
1272 0.0
1273 0.0
1274 0.0
1275 0.0
1276 0.0
1277 0.0
1278 0.0
1279 0.0
1280 0.0
1281 0.0
1282 0.0
1283 0.0
1284 0.0
1285 0.0
1286 0.0
1287 0.0
1288 0.0
1289 0.0
1290 0.0
1291 0.0
1292 0.0
1293 0.0
1294 0.0
1295 0.0
1296 0.0
1297 0.0
1298 0.0
1299 0.0
1300 0.0
1301 0.0
1302 0.0
1303 0.0
1304 0.0
1305 0.0
1306 0.0
1307 0.0
1308 0.0
1309 0.0
1310 0.0
1311 0.0
1312 0.0
1313 0.0
1314 0.0
1315 0.0
1316 0.0
1317 0.0
1318 0.0
1319 0.0
1320 0.0
1321 0.0
1322 0.0
1323 0.0
1324 0.0
1325 0.0
1326 0.0
1327 0.0
1328 0.0
1329 0.0
1330 0.0
1331 0.0
1332 0.0
1333 0.0
1334 0.0
1335 0.0
1336 0.0
1337 0.0
1338 0.0
1339 0.0
1340 0.0
1341 0.0
1342 0.0
1343 0.0
1344 0.0
1345 0.0
1346 0.0
1347 0.0
1348 375.0
1349 375.0
1350 375.0
1351 375.0
1352 375.0
1353 375.0
1354 375.0
1355 375.0
1356 375.0
1357 375.0
1358 375.0
1359 375.0
1360 375.0
1361 375.0
1362 375.0
1363 375.0
1364 375.0
1365 375.0
1366 375.0
1367 375.0
1368 375.0
1369 375.0
1370 375.0
1371 375.0
1372 375.0
1373 375.0
1374 375.0
1375 375.0
1376 375.0
1377 375.0
1378 375.0
1379 375.0
1380 375.0
1381 375.0
1382 375.0
1383 375.0
1384 375.0
1385 375.0
1386 375.0
1387 375.0
1388 80.0
1389 80.0
1390 80.0
1391 80.0
1392 80.0
1393 80.0
1394 80.0
1395 80.0
1396 120.0
1397 120.0
1398 120.0
1399 120.0
1400 120.0
1401 120.0
1402 120.0
1403 120.0
1404 235.0
1405 235.0
1406 235.0
1407 235.0
1408 235.0
1409 235.0
1410 235.0
1411 235.0
1412 10.0
1413 10.0
1414 10.0
1415 10.0
1416 10.0
1417 10.0
1418 10.0
1419 10.0
1420 375.0
1421 375.0
1422 375.0
1423 375.0
1424 375.0
1425 375.0
1426 375.0
1427 375.0
1428 375.0
1429 375.0
1430 80.0
1431 80.0
1432 80.0
1433 80.0
1434 120.0
1435 120.0
1436 235.0
1437 235.0
1438 235.0
1439 235.0
1440 10.0
1441 10.0
1442 100.0
1443 0.0
1444 0.0
1445 0.0
1446 0.0
1447 375.0
1448 375.0
1449 375.0
1450 375.0
1451 375.0
1452 375.0
1453 375.0
1454 375.0
1455 375.0
1456 375.0
1457 80.0
1458 80.0
1459 80.0
1460 80.0
1461 120.0
1462 120.0
1463 235.0
1464 235.0
1465 235.0
1466 235.0
1467 10.0
1468 10.0
1469 100.0
1470 0.0
1471 0.0
1472 0.0
1473 0.0
1474 0.0
1475 0.0
1476 0.0
1477 0.0
1478 0.0
1479 0.0
1480 0.0
1481 0.0
1482 0.0
1483 0.0
1484 0.0
1485 0.0
1486 0.0
1487 0.0
1488 0.0
1489 0.0
1490 0.0
1491 0.0
1492 0.0
1493 0.0
1494 0.0
1495 0.0
1496 0.0
1497 0.0
1498 0.0
1499 0.0
1500 0.0
1501 0.0
1502 0.0
1503 0.0
1504 0.0
1505 0.0
1506 0.0
1507 0.0
1508 0.0
1509 0.0
1510 0.0
1511 0.0
1512 0.0
1513 0.0
1514 0.0
1515 0.0
1516 0.0
1517 0.0
1518 0.0
1519 0.0
1520 0.0
1521 0.0;


######################
# SOLVE 
######################
end;
