

######################
# PARAMETERS 
######################
# SETS
# Set of all Virtual Machines
set Vs := 0..1628;
# Set of Hosts
set Hs := 0..186;

# PARAMETERS
param CPU {Vs};
param MEM {Vs};
param NET {Vs};
# VARIABLES
# VirtualMachine on Host Variables
var x{v in Vs, h in Hs}, binary;


######################
# CONSTRAINTS 
######################

#=== Utilized Resources ===
var uh{h in Hs}, binary;
s.t. or_sum_uh{h in Hs}: uh[h] <= sum{v in Vs} x[v, h];
s.t. or_ineq_uh{v in Vs, h in Hs}: uh[h] >= x[v, h];
var used_hosts, integer;
s.t. used_hosts_def: used_hosts = sum{h in Hs} uh[h];

#=== Injiective Allocation Constraints ===
s.t. inject_cnstr_{v in Vs} : sum {h in Hs} x[v, h] = 1;

#=== Resource constraints ===
s.t. res_cpu_cnstr_{h in Hs} : sum {v in Vs} CPU[v] * x[v, h] <= 44.0;
s.t. res_mem_cnstr_{h in Hs} : sum {v in Vs} MEM[v] * x[v, h] <= 420.0;
s.t. res_net_cnstr_{h in Hs} : sum {v in Vs} NET[v] * x[v, h] <= 15000.0;

#=== Auto Anti-Affinity constraints ===
s.t. AAF_vnf1_dot_vm2_0_{h in Hs}: sum{v in {120..122}} x[v, h]<= 1;
s.t. AAF_vnf1_dot_vm2_1_{h in Hs}: sum{v in {123..125}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_0_{h in Hs}: sum{v in {152..154}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm7_1_{h in Hs}: sum{v in {155..157}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm8_0_{h in Hs}: sum{v in {158..160}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm8_1_{h in Hs}: sum{v in {161..163}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_0_{h in Hs}: sum{v in {164..166}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm9_1_{h in Hs}: sum{v in {167..169}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm10_0_{h in Hs}: sum{v in {170..172}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm10_1_{h in Hs}: sum{v in {173..175}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_0_{h in Hs}: sum{v in {248,249}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm12_1_{h in Hs}: sum{v in {250,251}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm13_0_{h in Hs}: sum{v in {252,253}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm13_1_{h in Hs}: sum{v in {254,255}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm14_0_{h in Hs}: sum{v in {256..262}} x[v, h]<= 1;
s.t. AAF_vnf3_dot_vm14_1_{h in Hs}: sum{v in {263..269}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm23_0_{h in Hs}: sum{v in {310}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm23_1_{h in Hs}: sum{v in {311}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm24_0_{h in Hs}: sum{v in {312}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm24_1_{h in Hs}: sum{v in {313}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm25_0_{h in Hs}: sum{v in {314}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm25_1_{h in Hs}: sum{v in {315}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm25_2_{h in Hs}: sum{v in {316}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_0_{h in Hs}: sum{v in {317}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_1_{h in Hs}: sum{v in {318}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm26_2_{h in Hs}: sum{v in {319}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm27_0_{h in Hs}: sum{v in {320}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm28_0_{h in Hs}: sum{v in {321}} x[v, h]<= 1;
s.t. AAF_vnf4_dot_vm29_0_{h in Hs}: sum{v in {322}} x[v, h]<= 1;
s.t. AAF_vnf6_dot_vm36_0_{h in Hs}: sum{v in {340}} x[v, h]<= 1;
s.t. AAF_vnf12_dot_vm41_0_{h in Hs}: sum{v in {443}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm42_0_{h in Hs}: sum{v in {444}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm42_1_{h in Hs}: sum{v in {445}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm42_2_{h in Hs}: sum{v in {446}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm43_0_{h in Hs}: sum{v in {447}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm43_1_{h in Hs}: sum{v in {448}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm43_2_{h in Hs}: sum{v in {449}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_0_{h in Hs}: sum{v in {450}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_1_{h in Hs}: sum{v in {451}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_2_{h in Hs}: sum{v in {452}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm44_3_{h in Hs}: sum{v in {453}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_0_{h in Hs}: sum{v in {454}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_1_{h in Hs}: sum{v in {455}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_2_{h in Hs}: sum{v in {456}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm45_3_{h in Hs}: sum{v in {457}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_0_{h in Hs}: sum{v in {458}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_1_{h in Hs}: sum{v in {459}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_2_{h in Hs}: sum{v in {460}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm46_3_{h in Hs}: sum{v in {461}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_0_{h in Hs}: sum{v in {462}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_1_{h in Hs}: sum{v in {463}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_2_{h in Hs}: sum{v in {464}} x[v, h]<= 1;
s.t. AAF_vnf13_dot_vm47_3_{h in Hs}: sum{v in {465}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_0_{h in Hs}: sum{v in {466}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_1_{h in Hs}: sum{v in {467}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_2_{h in Hs}: sum{v in {468}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_3_{h in Hs}: sum{v in {469}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_4_{h in Hs}: sum{v in {470}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_5_{h in Hs}: sum{v in {471}} x[v, h]<= 1;
s.t. AAF_vnf14_dot_vm48_6_{h in Hs}: sum{v in {472}} x[v, h]<= 1;
s.t. AAF_vnf15_dot_vm49_0_{h in Hs}: sum{v in {473}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm51_0_{h in Hs}: sum{v in {552..554}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm51_1_{h in Hs}: sum{v in {555..557}} x[v, h]<= 1;
s.t. AAF_vnf16_dot_vm51_2_{h in Hs}: sum{v in {558..560}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm7_0_{h in Hs}: sum{v in {602..604}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm7_1_{h in Hs}: sum{v in {605..607}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm7_2_{h in Hs}: sum{v in {608..610}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm8_0_{h in Hs}: sum{v in {611..613}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm8_1_{h in Hs}: sum{v in {614..616}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm8_2_{h in Hs}: sum{v in {617..619}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm9_0_{h in Hs}: sum{v in {620..622}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm9_1_{h in Hs}: sum{v in {623..625}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm9_2_{h in Hs}: sum{v in {626..628}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm12_0_{h in Hs}: sum{v in {731..733}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm12_1_{h in Hs}: sum{v in {734..736}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm12_2_{h in Hs}: sum{v in {737..739}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm13_0_{h in Hs}: sum{v in {740,741}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm13_1_{h in Hs}: sum{v in {742,743}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm13_2_{h in Hs}: sum{v in {744,745}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm14_0_{h in Hs}: sum{v in {746..752}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm14_1_{h in Hs}: sum{v in {753..759}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm14_2_{h in Hs}: sum{v in {760..766}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm57_0_{h in Hs}: sum{v in {827,828}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm57_1_{h in Hs}: sum{v in {829,830}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm10_0_{h in Hs}: sum{v in {831,832}} x[v, h]<= 1;
s.t. AAF_vnf17_dot_vm10_1_{h in Hs}: sum{v in {833,834}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_0_{h in Hs}: sum{v in {838,839}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_1_{h in Hs}: sum{v in {840,841}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_2_{h in Hs}: sum{v in {842,843}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm59_3_{h in Hs}: sum{v in {844,845}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_0_{h in Hs}: sum{v in {846,847}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_1_{h in Hs}: sum{v in {848,849}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_2_{h in Hs}: sum{v in {850,851}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm60_3_{h in Hs}: sum{v in {852,853}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_0_{h in Hs}: sum{v in {854,855}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_1_{h in Hs}: sum{v in {856,857}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_2_{h in Hs}: sum{v in {858,859}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm61_3_{h in Hs}: sum{v in {860,861}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_0_{h in Hs}: sum{v in {862..869}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_1_{h in Hs}: sum{v in {870..877}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_2_{h in Hs}: sum{v in {878..885}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm62_3_{h in Hs}: sum{v in {886..893}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_0_{h in Hs}: sum{v in {894,895}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_1_{h in Hs}: sum{v in {896,897}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_2_{h in Hs}: sum{v in {898,899}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm63_3_{h in Hs}: sum{v in {900,901}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_0_{h in Hs}: sum{v in {902..925}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_1_{h in Hs}: sum{v in {926..949}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_2_{h in Hs}: sum{v in {950..973}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm64_3_{h in Hs}: sum{v in {974..997}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_0_{h in Hs}: sum{v in {1126,1127}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_1_{h in Hs}: sum{v in {1128,1129}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_2_{h in Hs}: sum{v in {1130,1131}} x[v, h]<= 1;
s.t. AAF_vnf18_dot_vm66_3_{h in Hs}: sum{v in {1132,1133}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm59_0_{h in Hs}: sum{v in {1239,1240}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm60_0_{h in Hs}: sum{v in {1241,1242}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm61_0_{h in Hs}: sum{v in {1243,1244}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm62_0_{h in Hs}: sum{v in {1245..1248}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm63_0_{h in Hs}: sum{v in {1249,1250}} x[v, h]<= 1;
s.t. AAF_vnf19_dot_vm65_0_{h in Hs}: sum{v in {1251..1273}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm7_0_{h in Hs}: sum{v in {1301..1303}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm7_1_{h in Hs}: sum{v in {1304..1306}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm8_0_{h in Hs}: sum{v in {1307..1309}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm8_1_{h in Hs}: sum{v in {1310..1312}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm9_0_{h in Hs}: sum{v in {1313..1315}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm9_1_{h in Hs}: sum{v in {1316..1318}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm12_0_{h in Hs}: sum{v in {1375,1376}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm12_1_{h in Hs}: sum{v in {1377,1378}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm13_0_{h in Hs}: sum{v in {1379,1380}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm13_1_{h in Hs}: sum{v in {1381,1382}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm14_0_{h in Hs}: sum{v in {1383..1391}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm14_1_{h in Hs}: sum{v in {1392..1400}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm57_0_{h in Hs}: sum{v in {1441,1442}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm57_1_{h in Hs}: sum{v in {1443,1444}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm10_0_{h in Hs}: sum{v in {1445,1446}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm10_1_{h in Hs}: sum{v in {1447,1448}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm77_0_{h in Hs}: sum{v in {1449,1450}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm77_1_{h in Hs}: sum{v in {1451,1452}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm78_0_{h in Hs}: sum{v in {1453,1454}} x[v, h]<= 1;
s.t. AAF_vnf20_dot_vm78_1_{h in Hs}: sum{v in {1455,1456}} x[v, h]<= 1;
s.t. AAF_vnf25_dot_vm80_0_{h in Hs}: sum{v in {1576}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm83_0_{h in Hs}: sum{v in {1582}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm84_0_{h in Hs}: sum{v in {1584}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm85_0_{h in Hs}: sum{v in {1585}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm90_0_{h in Hs}: sum{v in {1590}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm93_0_{h in Hs}: sum{v in {1593}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm94_0_{h in Hs}: sum{v in {1594}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm95_0_{h in Hs}: sum{v in {1595}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm96_0_{h in Hs}: sum{v in {1596}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm97_0_{h in Hs}: sum{v in {1597}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm98_0_{h in Hs}: sum{v in {1598}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm99_0_{h in Hs}: sum{v in {1599}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm100_0_{h in Hs}: sum{v in {1600}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm101_0_{h in Hs}: sum{v in {1601}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm102_0_{h in Hs}: sum{v in {1602}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm103_0_{h in Hs}: sum{v in {1603}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm104_0_{h in Hs}: sum{v in {1604}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm105_0_{h in Hs}: sum{v in {1605}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm106_0_{h in Hs}: sum{v in {1606}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm107_0_{h in Hs}: sum{v in {1607}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm108_0_{h in Hs}: sum{v in {1608}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm118_0_{h in Hs}: sum{v in {1618}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm119_0_{h in Hs}: sum{v in {1619}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm120_0_{h in Hs}: sum{v in {1620}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm121_0_{h in Hs}: sum{v in {1621}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm122_0_{h in Hs}: sum{v in {1622}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm123_0_{h in Hs}: sum{v in {1623}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm124_0_{h in Hs}: sum{v in {1624}} x[v, h]<= 1;
s.t. AAF_vnf26_dot_vm125_0_{h in Hs}: sum{v in {1625}} x[v, h]<= 1;

#=== Cross Anti-Affinity constraints ===
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_master{h in Hs}: ni_vnf1_dot_vm1_0_master[h] <= sum{v in {0..29}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_master{v in {0..29}, h in Hs}: ni_vnf1_dot_vm1_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_0_slave{h in Hs}: ni_vnf1_dot_vm1_0_slave[h] <= sum{v in {30..59}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_0_slave{v in {30..59}, h in Hs}: ni_vnf1_dot_vm1_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_0_{h in Hs}: ni_vnf1_dot_vm1_0_master[h] + ni_vnf1_dot_vm1_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm1_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_1_master{h in Hs}: ni_vnf1_dot_vm1_1_master[h] <= sum{v in {60..89}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_1_master{v in {60..89}, h in Hs}: ni_vnf1_dot_vm1_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm1_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm1_1_slave{h in Hs}: ni_vnf1_dot_vm1_1_slave[h] <= sum{v in {90..119}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm1_1_slave{v in {90..119}, h in Hs}: ni_vnf1_dot_vm1_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_1_{h in Hs}: ni_vnf1_dot_vm1_1_master[h] + ni_vnf1_dot_vm1_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm3_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_master{h in Hs}: ni_vnf1_dot_vm3_0_master[h] <= sum{v in {126}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_master{v in {126}, h in Hs}: ni_vnf1_dot_vm3_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm3_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_0_slave{h in Hs}: ni_vnf1_dot_vm3_0_slave[h] <= sum{v in {127}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_0_slave{v in {127}, h in Hs}: ni_vnf1_dot_vm3_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_2_{h in Hs}: ni_vnf1_dot_vm3_0_master[h] + ni_vnf1_dot_vm3_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm3_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_1_master{h in Hs}: ni_vnf1_dot_vm3_1_master[h] <= sum{v in {128}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_1_master{v in {128}, h in Hs}: ni_vnf1_dot_vm3_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm3_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm3_1_slave{h in Hs}: ni_vnf1_dot_vm3_1_slave[h] <= sum{v in {129}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm3_1_slave{v in {129}, h in Hs}: ni_vnf1_dot_vm3_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_3_{h in Hs}: ni_vnf1_dot_vm3_1_master[h] + ni_vnf1_dot_vm3_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm4_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_0_master{h in Hs}: ni_vnf1_dot_vm4_0_master[h] <= sum{v in {130..133}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_0_master{v in {130..133}, h in Hs}: ni_vnf1_dot_vm4_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm4_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_0_slave{h in Hs}: ni_vnf1_dot_vm4_0_slave[h] <= sum{v in {134..137}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_0_slave{v in {134..137}, h in Hs}: ni_vnf1_dot_vm4_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_4_{h in Hs}: ni_vnf1_dot_vm4_0_master[h] + ni_vnf1_dot_vm4_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm4_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_1_master{h in Hs}: ni_vnf1_dot_vm4_1_master[h] <= sum{v in {138..141}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_1_master{v in {138..141}, h in Hs}: ni_vnf1_dot_vm4_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm4_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm4_1_slave{h in Hs}: ni_vnf1_dot_vm4_1_slave[h] <= sum{v in {142..145}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm4_1_slave{v in {142..145}, h in Hs}: ni_vnf1_dot_vm4_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_5_{h in Hs}: ni_vnf1_dot_vm4_1_master[h] + ni_vnf1_dot_vm4_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm5_0_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_0_master{h in Hs}: ni_vnf1_dot_vm5_0_master[h] <= sum{v in {146}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_0_master{v in {146}, h in Hs}: ni_vnf1_dot_vm5_0_master[h] >= x[v, h];
var ni_vnf1_dot_vm5_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_0_slave{h in Hs}: ni_vnf1_dot_vm5_0_slave[h] <= sum{v in {147}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_0_slave{v in {147}, h in Hs}: ni_vnf1_dot_vm5_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_6_{h in Hs}: ni_vnf1_dot_vm5_0_master[h] + ni_vnf1_dot_vm5_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf1_dot_vm5_1_master{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_1_master{h in Hs}: ni_vnf1_dot_vm5_1_master[h] <= sum{v in {148}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_1_master{v in {148}, h in Hs}: ni_vnf1_dot_vm5_1_master[h] >= x[v, h];
var ni_vnf1_dot_vm5_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf1_dot_vm5_1_slave{h in Hs}: ni_vnf1_dot_vm5_1_slave[h] <= sum{v in {149}} x[v, h];
s.t. or_ineq_ni_vnf1_dot_vm5_1_slave{v in {149}, h in Hs}: ni_vnf1_dot_vm5_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_7_{h in Hs}: ni_vnf1_dot_vm5_1_master[h] + ni_vnf1_dot_vm5_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf2_dot_vm6_0_master{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm6_0_master{h in Hs}: ni_vnf2_dot_vm6_0_master[h] <= sum{v in {150}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm6_0_master{v in {150}, h in Hs}: ni_vnf2_dot_vm6_0_master[h] >= x[v, h];
var ni_vnf2_dot_vm6_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf2_dot_vm6_0_slave{h in Hs}: ni_vnf2_dot_vm6_0_slave[h] <= sum{v in {151}} x[v, h];
s.t. or_ineq_ni_vnf2_dot_vm6_0_slave{v in {151}, h in Hs}: ni_vnf2_dot_vm6_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_8_{h in Hs}: ni_vnf2_dot_vm6_0_master[h] + ni_vnf2_dot_vm6_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm11_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_0_master{h in Hs}: ni_vnf3_dot_vm11_0_master[h] <= sum{v in {176..193}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_0_master{v in {176..193}, h in Hs}: ni_vnf3_dot_vm11_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm11_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_0_slave{h in Hs}: ni_vnf3_dot_vm11_0_slave[h] <= sum{v in {194..211}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_0_slave{v in {194..211}, h in Hs}: ni_vnf3_dot_vm11_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_9_{h in Hs}: ni_vnf3_dot_vm11_0_master[h] + ni_vnf3_dot_vm11_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm11_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_1_master{h in Hs}: ni_vnf3_dot_vm11_1_master[h] <= sum{v in {212..229}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_1_master{v in {212..229}, h in Hs}: ni_vnf3_dot_vm11_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm11_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm11_1_slave{h in Hs}: ni_vnf3_dot_vm11_1_slave[h] <= sum{v in {230..247}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm11_1_slave{v in {230..247}, h in Hs}: ni_vnf3_dot_vm11_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_10_{h in Hs}: ni_vnf3_dot_vm11_1_master[h] + ni_vnf3_dot_vm11_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_0_master{h in Hs}: ni_vnf3_dot_vm15_0_master[h] <= sum{v in {270}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_0_master{v in {270}, h in Hs}: ni_vnf3_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_0_slave{h in Hs}: ni_vnf3_dot_vm15_0_slave[h] <= sum{v in {271}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_0_slave{v in {271}, h in Hs}: ni_vnf3_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_11_{h in Hs}: ni_vnf3_dot_vm15_0_master[h] + ni_vnf3_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_1_master{h in Hs}: ni_vnf3_dot_vm15_1_master[h] <= sum{v in {272}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_1_master{v in {272}, h in Hs}: ni_vnf3_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm15_1_slave{h in Hs}: ni_vnf3_dot_vm15_1_slave[h] <= sum{v in {273}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm15_1_slave{v in {273}, h in Hs}: ni_vnf3_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_12_{h in Hs}: ni_vnf3_dot_vm15_1_master[h] + ni_vnf3_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_0_master{h in Hs}: ni_vnf3_dot_vm16_0_master[h] <= sum{v in {274}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_0_master{v in {274}, h in Hs}: ni_vnf3_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_0_slave{h in Hs}: ni_vnf3_dot_vm16_0_slave[h] <= sum{v in {275}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_0_slave{v in {275}, h in Hs}: ni_vnf3_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_13_{h in Hs}: ni_vnf3_dot_vm16_0_master[h] + ni_vnf3_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_1_master{h in Hs}: ni_vnf3_dot_vm16_1_master[h] <= sum{v in {276}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_1_master{v in {276}, h in Hs}: ni_vnf3_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm16_1_slave{h in Hs}: ni_vnf3_dot_vm16_1_slave[h] <= sum{v in {277}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm16_1_slave{v in {277}, h in Hs}: ni_vnf3_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_14_{h in Hs}: ni_vnf3_dot_vm16_1_master[h] + ni_vnf3_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_0_master{h in Hs}: ni_vnf3_dot_vm17_0_master[h] <= sum{v in {278..280}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_0_master{v in {278..280}, h in Hs}: ni_vnf3_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_0_slave{h in Hs}: ni_vnf3_dot_vm17_0_slave[h] <= sum{v in {281..283}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_0_slave{v in {281..283}, h in Hs}: ni_vnf3_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_15_{h in Hs}: ni_vnf3_dot_vm17_0_master[h] + ni_vnf3_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_1_master{h in Hs}: ni_vnf3_dot_vm17_1_master[h] <= sum{v in {284..286}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_1_master{v in {284..286}, h in Hs}: ni_vnf3_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm17_1_slave{h in Hs}: ni_vnf3_dot_vm17_1_slave[h] <= sum{v in {287..289}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm17_1_slave{v in {287..289}, h in Hs}: ni_vnf3_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_16_{h in Hs}: ni_vnf3_dot_vm17_1_master[h] + ni_vnf3_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_0_master{h in Hs}: ni_vnf3_dot_vm18_0_master[h] <= sum{v in {290}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_0_master{v in {290}, h in Hs}: ni_vnf3_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_0_slave{h in Hs}: ni_vnf3_dot_vm18_0_slave[h] <= sum{v in {291}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_0_slave{v in {291}, h in Hs}: ni_vnf3_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_17_{h in Hs}: ni_vnf3_dot_vm18_0_master[h] + ni_vnf3_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_1_master{h in Hs}: ni_vnf3_dot_vm18_1_master[h] <= sum{v in {292}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_1_master{v in {292}, h in Hs}: ni_vnf3_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm18_1_slave{h in Hs}: ni_vnf3_dot_vm18_1_slave[h] <= sum{v in {293}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm18_1_slave{v in {293}, h in Hs}: ni_vnf3_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_18_{h in Hs}: ni_vnf3_dot_vm18_1_master[h] + ni_vnf3_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm19_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_0_master{h in Hs}: ni_vnf3_dot_vm19_0_master[h] <= sum{v in {294}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_0_master{v in {294}, h in Hs}: ni_vnf3_dot_vm19_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm19_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_0_slave{h in Hs}: ni_vnf3_dot_vm19_0_slave[h] <= sum{v in {295}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_0_slave{v in {295}, h in Hs}: ni_vnf3_dot_vm19_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_19_{h in Hs}: ni_vnf3_dot_vm19_0_master[h] + ni_vnf3_dot_vm19_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm19_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_1_master{h in Hs}: ni_vnf3_dot_vm19_1_master[h] <= sum{v in {296}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_1_master{v in {296}, h in Hs}: ni_vnf3_dot_vm19_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm19_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm19_1_slave{h in Hs}: ni_vnf3_dot_vm19_1_slave[h] <= sum{v in {297}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm19_1_slave{v in {297}, h in Hs}: ni_vnf3_dot_vm19_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_20_{h in Hs}: ni_vnf3_dot_vm19_1_master[h] + ni_vnf3_dot_vm19_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm20_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_0_master{h in Hs}: ni_vnf3_dot_vm20_0_master[h] <= sum{v in {298}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_0_master{v in {298}, h in Hs}: ni_vnf3_dot_vm20_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm20_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_0_slave{h in Hs}: ni_vnf3_dot_vm20_0_slave[h] <= sum{v in {299}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_0_slave{v in {299}, h in Hs}: ni_vnf3_dot_vm20_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_21_{h in Hs}: ni_vnf3_dot_vm20_0_master[h] + ni_vnf3_dot_vm20_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm20_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_1_master{h in Hs}: ni_vnf3_dot_vm20_1_master[h] <= sum{v in {300}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_1_master{v in {300}, h in Hs}: ni_vnf3_dot_vm20_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm20_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm20_1_slave{h in Hs}: ni_vnf3_dot_vm20_1_slave[h] <= sum{v in {301}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm20_1_slave{v in {301}, h in Hs}: ni_vnf3_dot_vm20_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_22_{h in Hs}: ni_vnf3_dot_vm20_1_master[h] + ni_vnf3_dot_vm20_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm21_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_0_master{h in Hs}: ni_vnf3_dot_vm21_0_master[h] <= sum{v in {302}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_0_master{v in {302}, h in Hs}: ni_vnf3_dot_vm21_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm21_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_0_slave{h in Hs}: ni_vnf3_dot_vm21_0_slave[h] <= sum{v in {303}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_0_slave{v in {303}, h in Hs}: ni_vnf3_dot_vm21_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_23_{h in Hs}: ni_vnf3_dot_vm21_0_master[h] + ni_vnf3_dot_vm21_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm21_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_1_master{h in Hs}: ni_vnf3_dot_vm21_1_master[h] <= sum{v in {304}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_1_master{v in {304}, h in Hs}: ni_vnf3_dot_vm21_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm21_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm21_1_slave{h in Hs}: ni_vnf3_dot_vm21_1_slave[h] <= sum{v in {305}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm21_1_slave{v in {305}, h in Hs}: ni_vnf3_dot_vm21_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_24_{h in Hs}: ni_vnf3_dot_vm21_1_master[h] + ni_vnf3_dot_vm21_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm22_0_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_0_master{h in Hs}: ni_vnf3_dot_vm22_0_master[h] <= sum{v in {306}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_0_master{v in {306}, h in Hs}: ni_vnf3_dot_vm22_0_master[h] >= x[v, h];
var ni_vnf3_dot_vm22_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_0_slave{h in Hs}: ni_vnf3_dot_vm22_0_slave[h] <= sum{v in {307}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_0_slave{v in {307}, h in Hs}: ni_vnf3_dot_vm22_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_25_{h in Hs}: ni_vnf3_dot_vm22_0_master[h] + ni_vnf3_dot_vm22_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf3_dot_vm22_1_master{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_1_master{h in Hs}: ni_vnf3_dot_vm22_1_master[h] <= sum{v in {308}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_1_master{v in {308}, h in Hs}: ni_vnf3_dot_vm22_1_master[h] >= x[v, h];
var ni_vnf3_dot_vm22_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf3_dot_vm22_1_slave{h in Hs}: ni_vnf3_dot_vm22_1_slave[h] <= sum{v in {309}} x[v, h];
s.t. or_ineq_ni_vnf3_dot_vm22_1_slave{v in {309}, h in Hs}: ni_vnf3_dot_vm22_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_26_{h in Hs}: ni_vnf3_dot_vm22_1_master[h] + ni_vnf3_dot_vm22_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_0_master{h in Hs}: ni_vnf7_dot_vm37_0_master[h] <= sum{v in {341..344}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_0_master{v in {341..344}, h in Hs}: ni_vnf7_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_0_slave{h in Hs}: ni_vnf7_dot_vm37_0_slave[h] <= sum{v in {345..348}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_0_slave{v in {345..348}, h in Hs}: ni_vnf7_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_27_{h in Hs}: ni_vnf7_dot_vm37_0_master[h] + ni_vnf7_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm37_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_1_master{h in Hs}: ni_vnf7_dot_vm37_1_master[h] <= sum{v in {349..352}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_1_master{v in {349..352}, h in Hs}: ni_vnf7_dot_vm37_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm37_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm37_1_slave{h in Hs}: ni_vnf7_dot_vm37_1_slave[h] <= sum{v in {353..356}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm37_1_slave{v in {353..356}, h in Hs}: ni_vnf7_dot_vm37_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_28_{h in Hs}: ni_vnf7_dot_vm37_1_master[h] + ni_vnf7_dot_vm37_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_0_master{h in Hs}: ni_vnf7_dot_vm38_0_master[h] <= sum{v in {357}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_0_master{v in {357}, h in Hs}: ni_vnf7_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_0_slave{h in Hs}: ni_vnf7_dot_vm38_0_slave[h] <= sum{v in {358}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_0_slave{v in {358}, h in Hs}: ni_vnf7_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_29_{h in Hs}: ni_vnf7_dot_vm38_0_master[h] + ni_vnf7_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm38_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_1_master{h in Hs}: ni_vnf7_dot_vm38_1_master[h] <= sum{v in {359}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_1_master{v in {359}, h in Hs}: ni_vnf7_dot_vm38_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm38_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm38_1_slave{h in Hs}: ni_vnf7_dot_vm38_1_slave[h] <= sum{v in {360}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm38_1_slave{v in {360}, h in Hs}: ni_vnf7_dot_vm38_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_30_{h in Hs}: ni_vnf7_dot_vm38_1_master[h] + ni_vnf7_dot_vm38_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_0_master{h in Hs}: ni_vnf7_dot_vm39_0_master[h] <= sum{v in {361}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_0_master{v in {361}, h in Hs}: ni_vnf7_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_0_slave{h in Hs}: ni_vnf7_dot_vm39_0_slave[h] <= sum{v in {362}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_0_slave{v in {362}, h in Hs}: ni_vnf7_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_31_{h in Hs}: ni_vnf7_dot_vm39_0_master[h] + ni_vnf7_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm39_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_1_master{h in Hs}: ni_vnf7_dot_vm39_1_master[h] <= sum{v in {363}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_1_master{v in {363}, h in Hs}: ni_vnf7_dot_vm39_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm39_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm39_1_slave{h in Hs}: ni_vnf7_dot_vm39_1_slave[h] <= sum{v in {364}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm39_1_slave{v in {364}, h in Hs}: ni_vnf7_dot_vm39_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_32_{h in Hs}: ni_vnf7_dot_vm39_1_master[h] + ni_vnf7_dot_vm39_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_0_master{h in Hs}: ni_vnf7_dot_vm40_0_master[h] <= sum{v in {365}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_0_master{v in {365}, h in Hs}: ni_vnf7_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_0_slave{h in Hs}: ni_vnf7_dot_vm40_0_slave[h] <= sum{v in {366}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_0_slave{v in {366}, h in Hs}: ni_vnf7_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_33_{h in Hs}: ni_vnf7_dot_vm40_0_master[h] + ni_vnf7_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm40_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_1_master{h in Hs}: ni_vnf7_dot_vm40_1_master[h] <= sum{v in {367}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_1_master{v in {367}, h in Hs}: ni_vnf7_dot_vm40_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm40_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm40_1_slave{h in Hs}: ni_vnf7_dot_vm40_1_slave[h] <= sum{v in {368}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm40_1_slave{v in {368}, h in Hs}: ni_vnf7_dot_vm40_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_34_{h in Hs}: ni_vnf7_dot_vm40_1_master[h] + ni_vnf7_dot_vm40_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_0_master{h in Hs}: ni_vnf7_dot_vm7_0_master[h] <= sum{v in {369}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_0_master{v in {369}, h in Hs}: ni_vnf7_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf7_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_0_slave{h in Hs}: ni_vnf7_dot_vm7_0_slave[h] <= sum{v in {370}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_0_slave{v in {370}, h in Hs}: ni_vnf7_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_35_{h in Hs}: ni_vnf7_dot_vm7_0_master[h] + ni_vnf7_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf7_dot_vm7_1_master{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_1_master{h in Hs}: ni_vnf7_dot_vm7_1_master[h] <= sum{v in {371}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_1_master{v in {371}, h in Hs}: ni_vnf7_dot_vm7_1_master[h] >= x[v, h];
var ni_vnf7_dot_vm7_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf7_dot_vm7_1_slave{h in Hs}: ni_vnf7_dot_vm7_1_slave[h] <= sum{v in {372}} x[v, h];
s.t. or_ineq_ni_vnf7_dot_vm7_1_slave{v in {372}, h in Hs}: ni_vnf7_dot_vm7_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_36_{h in Hs}: ni_vnf7_dot_vm7_1_master[h] + ni_vnf7_dot_vm7_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_0_master{h in Hs}: ni_vnf8_dot_vm37_0_master[h] <= sum{v in {373..377}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_0_master{v in {373..377}, h in Hs}: ni_vnf8_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_0_slave{h in Hs}: ni_vnf8_dot_vm37_0_slave[h] <= sum{v in {378..382}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_0_slave{v in {378..382}, h in Hs}: ni_vnf8_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_37_{h in Hs}: ni_vnf8_dot_vm37_0_master[h] + ni_vnf8_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm37_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_1_master{h in Hs}: ni_vnf8_dot_vm37_1_master[h] <= sum{v in {383..387}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_1_master{v in {383..387}, h in Hs}: ni_vnf8_dot_vm37_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm37_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm37_1_slave{h in Hs}: ni_vnf8_dot_vm37_1_slave[h] <= sum{v in {388..392}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm37_1_slave{v in {388..392}, h in Hs}: ni_vnf8_dot_vm37_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_38_{h in Hs}: ni_vnf8_dot_vm37_1_master[h] + ni_vnf8_dot_vm37_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_0_master{h in Hs}: ni_vnf8_dot_vm38_0_master[h] <= sum{v in {393}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_0_master{v in {393}, h in Hs}: ni_vnf8_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_0_slave{h in Hs}: ni_vnf8_dot_vm38_0_slave[h] <= sum{v in {394}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_0_slave{v in {394}, h in Hs}: ni_vnf8_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_39_{h in Hs}: ni_vnf8_dot_vm38_0_master[h] + ni_vnf8_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm38_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_1_master{h in Hs}: ni_vnf8_dot_vm38_1_master[h] <= sum{v in {395}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_1_master{v in {395}, h in Hs}: ni_vnf8_dot_vm38_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm38_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm38_1_slave{h in Hs}: ni_vnf8_dot_vm38_1_slave[h] <= sum{v in {396}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm38_1_slave{v in {396}, h in Hs}: ni_vnf8_dot_vm38_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_40_{h in Hs}: ni_vnf8_dot_vm38_1_master[h] + ni_vnf8_dot_vm38_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_0_master{h in Hs}: ni_vnf8_dot_vm39_0_master[h] <= sum{v in {397}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_0_master{v in {397}, h in Hs}: ni_vnf8_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_0_slave{h in Hs}: ni_vnf8_dot_vm39_0_slave[h] <= sum{v in {398}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_0_slave{v in {398}, h in Hs}: ni_vnf8_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_41_{h in Hs}: ni_vnf8_dot_vm39_0_master[h] + ni_vnf8_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm39_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_1_master{h in Hs}: ni_vnf8_dot_vm39_1_master[h] <= sum{v in {399}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_1_master{v in {399}, h in Hs}: ni_vnf8_dot_vm39_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm39_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm39_1_slave{h in Hs}: ni_vnf8_dot_vm39_1_slave[h] <= sum{v in {400}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm39_1_slave{v in {400}, h in Hs}: ni_vnf8_dot_vm39_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_42_{h in Hs}: ni_vnf8_dot_vm39_1_master[h] + ni_vnf8_dot_vm39_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_0_master{h in Hs}: ni_vnf8_dot_vm40_0_master[h] <= sum{v in {401}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_0_master{v in {401}, h in Hs}: ni_vnf8_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_0_slave{h in Hs}: ni_vnf8_dot_vm40_0_slave[h] <= sum{v in {402}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_0_slave{v in {402}, h in Hs}: ni_vnf8_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_43_{h in Hs}: ni_vnf8_dot_vm40_0_master[h] + ni_vnf8_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm40_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_1_master{h in Hs}: ni_vnf8_dot_vm40_1_master[h] <= sum{v in {403}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_1_master{v in {403}, h in Hs}: ni_vnf8_dot_vm40_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm40_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm40_1_slave{h in Hs}: ni_vnf8_dot_vm40_1_slave[h] <= sum{v in {404}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm40_1_slave{v in {404}, h in Hs}: ni_vnf8_dot_vm40_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_44_{h in Hs}: ni_vnf8_dot_vm40_1_master[h] + ni_vnf8_dot_vm40_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_0_master{h in Hs}: ni_vnf8_dot_vm7_0_master[h] <= sum{v in {405}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_0_master{v in {405}, h in Hs}: ni_vnf8_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf8_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_0_slave{h in Hs}: ni_vnf8_dot_vm7_0_slave[h] <= sum{v in {406}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_0_slave{v in {406}, h in Hs}: ni_vnf8_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_45_{h in Hs}: ni_vnf8_dot_vm7_0_master[h] + ni_vnf8_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf8_dot_vm7_1_master{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_1_master{h in Hs}: ni_vnf8_dot_vm7_1_master[h] <= sum{v in {407}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_1_master{v in {407}, h in Hs}: ni_vnf8_dot_vm7_1_master[h] >= x[v, h];
var ni_vnf8_dot_vm7_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf8_dot_vm7_1_slave{h in Hs}: ni_vnf8_dot_vm7_1_slave[h] <= sum{v in {408}} x[v, h];
s.t. or_ineq_ni_vnf8_dot_vm7_1_slave{v in {408}, h in Hs}: ni_vnf8_dot_vm7_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_46_{h in Hs}: ni_vnf8_dot_vm7_1_master[h] + ni_vnf8_dot_vm7_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm37_0_master{h in Hs}: ni_vnf9_dot_vm37_0_master[h] <= sum{v in {409,410}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm37_0_master{v in {409,410}, h in Hs}: ni_vnf9_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm37_0_slave{h in Hs}: ni_vnf9_dot_vm37_0_slave[h] <= sum{v in {411,412}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm37_0_slave{v in {411,412}, h in Hs}: ni_vnf9_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_47_{h in Hs}: ni_vnf9_dot_vm37_0_master[h] + ni_vnf9_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm38_0_master{h in Hs}: ni_vnf9_dot_vm38_0_master[h] <= sum{v in {413}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm38_0_master{v in {413}, h in Hs}: ni_vnf9_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm38_0_slave{h in Hs}: ni_vnf9_dot_vm38_0_slave[h] <= sum{v in {414}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm38_0_slave{v in {414}, h in Hs}: ni_vnf9_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_48_{h in Hs}: ni_vnf9_dot_vm38_0_master[h] + ni_vnf9_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm39_0_master{h in Hs}: ni_vnf9_dot_vm39_0_master[h] <= sum{v in {415}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm39_0_master{v in {415}, h in Hs}: ni_vnf9_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm39_0_slave{h in Hs}: ni_vnf9_dot_vm39_0_slave[h] <= sum{v in {416}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm39_0_slave{v in {416}, h in Hs}: ni_vnf9_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_49_{h in Hs}: ni_vnf9_dot_vm39_0_master[h] + ni_vnf9_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm40_0_master{h in Hs}: ni_vnf9_dot_vm40_0_master[h] <= sum{v in {417}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm40_0_master{v in {417}, h in Hs}: ni_vnf9_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm40_0_slave{h in Hs}: ni_vnf9_dot_vm40_0_slave[h] <= sum{v in {418}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm40_0_slave{v in {418}, h in Hs}: ni_vnf9_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_50_{h in Hs}: ni_vnf9_dot_vm40_0_master[h] + ni_vnf9_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf9_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm7_0_master{h in Hs}: ni_vnf9_dot_vm7_0_master[h] <= sum{v in {419}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm7_0_master{v in {419}, h in Hs}: ni_vnf9_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf9_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf9_dot_vm7_0_slave{h in Hs}: ni_vnf9_dot_vm7_0_slave[h] <= sum{v in {420}} x[v, h];
s.t. or_ineq_ni_vnf9_dot_vm7_0_slave{v in {420}, h in Hs}: ni_vnf9_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_51_{h in Hs}: ni_vnf9_dot_vm7_0_master[h] + ni_vnf9_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm37_0_master{h in Hs}: ni_vnf10_dot_vm37_0_master[h] <= sum{v in {421,422}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm37_0_master{v in {421,422}, h in Hs}: ni_vnf10_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm37_0_slave{h in Hs}: ni_vnf10_dot_vm37_0_slave[h] <= sum{v in {423,424}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm37_0_slave{v in {423,424}, h in Hs}: ni_vnf10_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_52_{h in Hs}: ni_vnf10_dot_vm37_0_master[h] + ni_vnf10_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm38_0_master{h in Hs}: ni_vnf10_dot_vm38_0_master[h] <= sum{v in {425}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm38_0_master{v in {425}, h in Hs}: ni_vnf10_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm38_0_slave{h in Hs}: ni_vnf10_dot_vm38_0_slave[h] <= sum{v in {426}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm38_0_slave{v in {426}, h in Hs}: ni_vnf10_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_53_{h in Hs}: ni_vnf10_dot_vm38_0_master[h] + ni_vnf10_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm39_0_master{h in Hs}: ni_vnf10_dot_vm39_0_master[h] <= sum{v in {427}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm39_0_master{v in {427}, h in Hs}: ni_vnf10_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm39_0_slave{h in Hs}: ni_vnf10_dot_vm39_0_slave[h] <= sum{v in {428}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm39_0_slave{v in {428}, h in Hs}: ni_vnf10_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_54_{h in Hs}: ni_vnf10_dot_vm39_0_master[h] + ni_vnf10_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm40_0_master{h in Hs}: ni_vnf10_dot_vm40_0_master[h] <= sum{v in {429}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm40_0_master{v in {429}, h in Hs}: ni_vnf10_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm40_0_slave{h in Hs}: ni_vnf10_dot_vm40_0_slave[h] <= sum{v in {430}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm40_0_slave{v in {430}, h in Hs}: ni_vnf10_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_55_{h in Hs}: ni_vnf10_dot_vm40_0_master[h] + ni_vnf10_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf10_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm7_0_master{h in Hs}: ni_vnf10_dot_vm7_0_master[h] <= sum{v in {431}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm7_0_master{v in {431}, h in Hs}: ni_vnf10_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf10_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf10_dot_vm7_0_slave{h in Hs}: ni_vnf10_dot_vm7_0_slave[h] <= sum{v in {432}} x[v, h];
s.t. or_ineq_ni_vnf10_dot_vm7_0_slave{v in {432}, h in Hs}: ni_vnf10_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_56_{h in Hs}: ni_vnf10_dot_vm7_0_master[h] + ni_vnf10_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf11_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm37_0_master{h in Hs}: ni_vnf11_dot_vm37_0_master[h] <= sum{v in {433}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm37_0_master{v in {433}, h in Hs}: ni_vnf11_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf11_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm37_0_slave{h in Hs}: ni_vnf11_dot_vm37_0_slave[h] <= sum{v in {434}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm37_0_slave{v in {434}, h in Hs}: ni_vnf11_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_57_{h in Hs}: ni_vnf11_dot_vm37_0_master[h] + ni_vnf11_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf11_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm38_0_master{h in Hs}: ni_vnf11_dot_vm38_0_master[h] <= sum{v in {435}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm38_0_master{v in {435}, h in Hs}: ni_vnf11_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf11_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm38_0_slave{h in Hs}: ni_vnf11_dot_vm38_0_slave[h] <= sum{v in {436}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm38_0_slave{v in {436}, h in Hs}: ni_vnf11_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_58_{h in Hs}: ni_vnf11_dot_vm38_0_master[h] + ni_vnf11_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf11_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm39_0_master{h in Hs}: ni_vnf11_dot_vm39_0_master[h] <= sum{v in {437}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm39_0_master{v in {437}, h in Hs}: ni_vnf11_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf11_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm39_0_slave{h in Hs}: ni_vnf11_dot_vm39_0_slave[h] <= sum{v in {438}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm39_0_slave{v in {438}, h in Hs}: ni_vnf11_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_59_{h in Hs}: ni_vnf11_dot_vm39_0_master[h] + ni_vnf11_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf11_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm40_0_master{h in Hs}: ni_vnf11_dot_vm40_0_master[h] <= sum{v in {439}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm40_0_master{v in {439}, h in Hs}: ni_vnf11_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf11_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm40_0_slave{h in Hs}: ni_vnf11_dot_vm40_0_slave[h] <= sum{v in {440}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm40_0_slave{v in {440}, h in Hs}: ni_vnf11_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_60_{h in Hs}: ni_vnf11_dot_vm40_0_master[h] + ni_vnf11_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf11_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm7_0_master{h in Hs}: ni_vnf11_dot_vm7_0_master[h] <= sum{v in {441}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm7_0_master{v in {441}, h in Hs}: ni_vnf11_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf11_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf11_dot_vm7_0_slave{h in Hs}: ni_vnf11_dot_vm7_0_slave[h] <= sum{v in {442}} x[v, h];
s.t. or_ineq_ni_vnf11_dot_vm7_0_slave{v in {442}, h in Hs}: ni_vnf11_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_61_{h in Hs}: ni_vnf11_dot_vm7_0_master[h] + ni_vnf11_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm50_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_0_master{h in Hs}: ni_vnf16_dot_vm50_0_master[h] <= sum{v in {474..486}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_0_master{v in {474..486}, h in Hs}: ni_vnf16_dot_vm50_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm50_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_0_slave{h in Hs}: ni_vnf16_dot_vm50_0_slave[h] <= sum{v in {487..499}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_0_slave{v in {487..499}, h in Hs}: ni_vnf16_dot_vm50_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_62_{h in Hs}: ni_vnf16_dot_vm50_0_master[h] + ni_vnf16_dot_vm50_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm50_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_1_master{h in Hs}: ni_vnf16_dot_vm50_1_master[h] <= sum{v in {500..512}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_1_master{v in {500..512}, h in Hs}: ni_vnf16_dot_vm50_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm50_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_1_slave{h in Hs}: ni_vnf16_dot_vm50_1_slave[h] <= sum{v in {513..525}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_1_slave{v in {513..525}, h in Hs}: ni_vnf16_dot_vm50_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_63_{h in Hs}: ni_vnf16_dot_vm50_1_master[h] + ni_vnf16_dot_vm50_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm50_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_2_master{h in Hs}: ni_vnf16_dot_vm50_2_master[h] <= sum{v in {526..538}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_2_master{v in {526..538}, h in Hs}: ni_vnf16_dot_vm50_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm50_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm50_2_slave{h in Hs}: ni_vnf16_dot_vm50_2_slave[h] <= sum{v in {539..551}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm50_2_slave{v in {539..551}, h in Hs}: ni_vnf16_dot_vm50_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_64_{h in Hs}: ni_vnf16_dot_vm50_2_master[h] + ni_vnf16_dot_vm50_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm52_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_0_master{h in Hs}: ni_vnf16_dot_vm52_0_master[h] <= sum{v in {561}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_0_master{v in {561}, h in Hs}: ni_vnf16_dot_vm52_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm52_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_0_slave{h in Hs}: ni_vnf16_dot_vm52_0_slave[h] <= sum{v in {562}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_0_slave{v in {562}, h in Hs}: ni_vnf16_dot_vm52_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_65_{h in Hs}: ni_vnf16_dot_vm52_0_master[h] + ni_vnf16_dot_vm52_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm52_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_1_master{h in Hs}: ni_vnf16_dot_vm52_1_master[h] <= sum{v in {563}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_1_master{v in {563}, h in Hs}: ni_vnf16_dot_vm52_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm52_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_1_slave{h in Hs}: ni_vnf16_dot_vm52_1_slave[h] <= sum{v in {564}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_1_slave{v in {564}, h in Hs}: ni_vnf16_dot_vm52_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_66_{h in Hs}: ni_vnf16_dot_vm52_1_master[h] + ni_vnf16_dot_vm52_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm52_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_2_master{h in Hs}: ni_vnf16_dot_vm52_2_master[h] <= sum{v in {565}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_2_master{v in {565}, h in Hs}: ni_vnf16_dot_vm52_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm52_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm52_2_slave{h in Hs}: ni_vnf16_dot_vm52_2_slave[h] <= sum{v in {566}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm52_2_slave{v in {566}, h in Hs}: ni_vnf16_dot_vm52_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_67_{h in Hs}: ni_vnf16_dot_vm52_2_master[h] + ni_vnf16_dot_vm52_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm53_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_0_master{h in Hs}: ni_vnf16_dot_vm53_0_master[h] <= sum{v in {567..570}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_0_master{v in {567..570}, h in Hs}: ni_vnf16_dot_vm53_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm53_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_0_slave{h in Hs}: ni_vnf16_dot_vm53_0_slave[h] <= sum{v in {571..574}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_0_slave{v in {571..574}, h in Hs}: ni_vnf16_dot_vm53_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_68_{h in Hs}: ni_vnf16_dot_vm53_0_master[h] + ni_vnf16_dot_vm53_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm53_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_1_master{h in Hs}: ni_vnf16_dot_vm53_1_master[h] <= sum{v in {575..578}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_1_master{v in {575..578}, h in Hs}: ni_vnf16_dot_vm53_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm53_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_1_slave{h in Hs}: ni_vnf16_dot_vm53_1_slave[h] <= sum{v in {579..582}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_1_slave{v in {579..582}, h in Hs}: ni_vnf16_dot_vm53_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_69_{h in Hs}: ni_vnf16_dot_vm53_1_master[h] + ni_vnf16_dot_vm53_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm53_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_2_master{h in Hs}: ni_vnf16_dot_vm53_2_master[h] <= sum{v in {583..586}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_2_master{v in {583..586}, h in Hs}: ni_vnf16_dot_vm53_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm53_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm53_2_slave{h in Hs}: ni_vnf16_dot_vm53_2_slave[h] <= sum{v in {587..590}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm53_2_slave{v in {587..590}, h in Hs}: ni_vnf16_dot_vm53_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_70_{h in Hs}: ni_vnf16_dot_vm53_2_master[h] + ni_vnf16_dot_vm53_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm54_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_0_master{h in Hs}: ni_vnf16_dot_vm54_0_master[h] <= sum{v in {591}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_0_master{v in {591}, h in Hs}: ni_vnf16_dot_vm54_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm54_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_0_slave{h in Hs}: ni_vnf16_dot_vm54_0_slave[h] <= sum{v in {592}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_0_slave{v in {592}, h in Hs}: ni_vnf16_dot_vm54_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_71_{h in Hs}: ni_vnf16_dot_vm54_0_master[h] + ni_vnf16_dot_vm54_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm54_1_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_1_master{h in Hs}: ni_vnf16_dot_vm54_1_master[h] <= sum{v in {593}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_1_master{v in {593}, h in Hs}: ni_vnf16_dot_vm54_1_master[h] >= x[v, h];
var ni_vnf16_dot_vm54_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_1_slave{h in Hs}: ni_vnf16_dot_vm54_1_slave[h] <= sum{v in {594}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_1_slave{v in {594}, h in Hs}: ni_vnf16_dot_vm54_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_72_{h in Hs}: ni_vnf16_dot_vm54_1_master[h] + ni_vnf16_dot_vm54_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm54_2_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_2_master{h in Hs}: ni_vnf16_dot_vm54_2_master[h] <= sum{v in {595}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_2_master{v in {595}, h in Hs}: ni_vnf16_dot_vm54_2_master[h] >= x[v, h];
var ni_vnf16_dot_vm54_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm54_2_slave{h in Hs}: ni_vnf16_dot_vm54_2_slave[h] <= sum{v in {596}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm54_2_slave{v in {596}, h in Hs}: ni_vnf16_dot_vm54_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_73_{h in Hs}: ni_vnf16_dot_vm54_2_master[h] + ni_vnf16_dot_vm54_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf16_dot_vm55_0_master{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm55_0_master{h in Hs}: ni_vnf16_dot_vm55_0_master[h] <= sum{v in {597}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm55_0_master{v in {597}, h in Hs}: ni_vnf16_dot_vm55_0_master[h] >= x[v, h];
var ni_vnf16_dot_vm55_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf16_dot_vm55_0_slave{h in Hs}: ni_vnf16_dot_vm55_0_slave[h] <= sum{v in {598}} x[v, h];
s.t. or_ineq_ni_vnf16_dot_vm55_0_slave{v in {598}, h in Hs}: ni_vnf16_dot_vm55_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_74_{h in Hs}: ni_vnf16_dot_vm55_0_master[h] + ni_vnf16_dot_vm55_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm11_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_0_master{h in Hs}: ni_vnf17_dot_vm11_0_master[h] <= sum{v in {629..645}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_0_master{v in {629..645}, h in Hs}: ni_vnf17_dot_vm11_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm11_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_0_slave{h in Hs}: ni_vnf17_dot_vm11_0_slave[h] <= sum{v in {646..662}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_0_slave{v in {646..662}, h in Hs}: ni_vnf17_dot_vm11_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_75_{h in Hs}: ni_vnf17_dot_vm11_0_master[h] + ni_vnf17_dot_vm11_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm11_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_1_master{h in Hs}: ni_vnf17_dot_vm11_1_master[h] <= sum{v in {663..679}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_1_master{v in {663..679}, h in Hs}: ni_vnf17_dot_vm11_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm11_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_1_slave{h in Hs}: ni_vnf17_dot_vm11_1_slave[h] <= sum{v in {680..696}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_1_slave{v in {680..696}, h in Hs}: ni_vnf17_dot_vm11_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_76_{h in Hs}: ni_vnf17_dot_vm11_1_master[h] + ni_vnf17_dot_vm11_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm11_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_2_master{h in Hs}: ni_vnf17_dot_vm11_2_master[h] <= sum{v in {697..713}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_2_master{v in {697..713}, h in Hs}: ni_vnf17_dot_vm11_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm11_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm11_2_slave{h in Hs}: ni_vnf17_dot_vm11_2_slave[h] <= sum{v in {714..730}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm11_2_slave{v in {714..730}, h in Hs}: ni_vnf17_dot_vm11_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_77_{h in Hs}: ni_vnf17_dot_vm11_2_master[h] + ni_vnf17_dot_vm11_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_0_master{h in Hs}: ni_vnf17_dot_vm15_0_master[h] <= sum{v in {767}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_0_master{v in {767}, h in Hs}: ni_vnf17_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_0_slave{h in Hs}: ni_vnf17_dot_vm15_0_slave[h] <= sum{v in {768}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_0_slave{v in {768}, h in Hs}: ni_vnf17_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_78_{h in Hs}: ni_vnf17_dot_vm15_0_master[h] + ni_vnf17_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_1_master{h in Hs}: ni_vnf17_dot_vm15_1_master[h] <= sum{v in {769}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_1_master{v in {769}, h in Hs}: ni_vnf17_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_1_slave{h in Hs}: ni_vnf17_dot_vm15_1_slave[h] <= sum{v in {770}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_1_slave{v in {770}, h in Hs}: ni_vnf17_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_79_{h in Hs}: ni_vnf17_dot_vm15_1_master[h] + ni_vnf17_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm15_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_2_master{h in Hs}: ni_vnf17_dot_vm15_2_master[h] <= sum{v in {771}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_2_master{v in {771}, h in Hs}: ni_vnf17_dot_vm15_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm15_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm15_2_slave{h in Hs}: ni_vnf17_dot_vm15_2_slave[h] <= sum{v in {772}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm15_2_slave{v in {772}, h in Hs}: ni_vnf17_dot_vm15_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_80_{h in Hs}: ni_vnf17_dot_vm15_2_master[h] + ni_vnf17_dot_vm15_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_0_master{h in Hs}: ni_vnf17_dot_vm16_0_master[h] <= sum{v in {773}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_0_master{v in {773}, h in Hs}: ni_vnf17_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_0_slave{h in Hs}: ni_vnf17_dot_vm16_0_slave[h] <= sum{v in {774}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_0_slave{v in {774}, h in Hs}: ni_vnf17_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_81_{h in Hs}: ni_vnf17_dot_vm16_0_master[h] + ni_vnf17_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_1_master{h in Hs}: ni_vnf17_dot_vm16_1_master[h] <= sum{v in {775}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_1_master{v in {775}, h in Hs}: ni_vnf17_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_1_slave{h in Hs}: ni_vnf17_dot_vm16_1_slave[h] <= sum{v in {776}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_1_slave{v in {776}, h in Hs}: ni_vnf17_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_82_{h in Hs}: ni_vnf17_dot_vm16_1_master[h] + ni_vnf17_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm16_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_2_master{h in Hs}: ni_vnf17_dot_vm16_2_master[h] <= sum{v in {777}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_2_master{v in {777}, h in Hs}: ni_vnf17_dot_vm16_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm16_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm16_2_slave{h in Hs}: ni_vnf17_dot_vm16_2_slave[h] <= sum{v in {778}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm16_2_slave{v in {778}, h in Hs}: ni_vnf17_dot_vm16_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_83_{h in Hs}: ni_vnf17_dot_vm16_2_master[h] + ni_vnf17_dot_vm16_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_0_master{h in Hs}: ni_vnf17_dot_vm17_0_master[h] <= sum{v in {779..781}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_0_master{v in {779..781}, h in Hs}: ni_vnf17_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_0_slave{h in Hs}: ni_vnf17_dot_vm17_0_slave[h] <= sum{v in {782..784}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_0_slave{v in {782..784}, h in Hs}: ni_vnf17_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_84_{h in Hs}: ni_vnf17_dot_vm17_0_master[h] + ni_vnf17_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_1_master{h in Hs}: ni_vnf17_dot_vm17_1_master[h] <= sum{v in {785..787}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_1_master{v in {785..787}, h in Hs}: ni_vnf17_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_1_slave{h in Hs}: ni_vnf17_dot_vm17_1_slave[h] <= sum{v in {788..790}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_1_slave{v in {788..790}, h in Hs}: ni_vnf17_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_85_{h in Hs}: ni_vnf17_dot_vm17_1_master[h] + ni_vnf17_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm17_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_2_master{h in Hs}: ni_vnf17_dot_vm17_2_master[h] <= sum{v in {791..793}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_2_master{v in {791..793}, h in Hs}: ni_vnf17_dot_vm17_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm17_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm17_2_slave{h in Hs}: ni_vnf17_dot_vm17_2_slave[h] <= sum{v in {794..796}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm17_2_slave{v in {794..796}, h in Hs}: ni_vnf17_dot_vm17_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_86_{h in Hs}: ni_vnf17_dot_vm17_2_master[h] + ni_vnf17_dot_vm17_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_0_master{h in Hs}: ni_vnf17_dot_vm18_0_master[h] <= sum{v in {797}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_0_master{v in {797}, h in Hs}: ni_vnf17_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_0_slave{h in Hs}: ni_vnf17_dot_vm18_0_slave[h] <= sum{v in {798}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_0_slave{v in {798}, h in Hs}: ni_vnf17_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_87_{h in Hs}: ni_vnf17_dot_vm18_0_master[h] + ni_vnf17_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_1_master{h in Hs}: ni_vnf17_dot_vm18_1_master[h] <= sum{v in {799}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_1_master{v in {799}, h in Hs}: ni_vnf17_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_1_slave{h in Hs}: ni_vnf17_dot_vm18_1_slave[h] <= sum{v in {800}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_1_slave{v in {800}, h in Hs}: ni_vnf17_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_88_{h in Hs}: ni_vnf17_dot_vm18_1_master[h] + ni_vnf17_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm18_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_2_master{h in Hs}: ni_vnf17_dot_vm18_2_master[h] <= sum{v in {801}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_2_master{v in {801}, h in Hs}: ni_vnf17_dot_vm18_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm18_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm18_2_slave{h in Hs}: ni_vnf17_dot_vm18_2_slave[h] <= sum{v in {802}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm18_2_slave{v in {802}, h in Hs}: ni_vnf17_dot_vm18_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_89_{h in Hs}: ni_vnf17_dot_vm18_2_master[h] + ni_vnf17_dot_vm18_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm20_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_0_master{h in Hs}: ni_vnf17_dot_vm20_0_master[h] <= sum{v in {803}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_0_master{v in {803}, h in Hs}: ni_vnf17_dot_vm20_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm20_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_0_slave{h in Hs}: ni_vnf17_dot_vm20_0_slave[h] <= sum{v in {804}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_0_slave{v in {804}, h in Hs}: ni_vnf17_dot_vm20_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_90_{h in Hs}: ni_vnf17_dot_vm20_0_master[h] + ni_vnf17_dot_vm20_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm20_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_1_master{h in Hs}: ni_vnf17_dot_vm20_1_master[h] <= sum{v in {805}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_1_master{v in {805}, h in Hs}: ni_vnf17_dot_vm20_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm20_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_1_slave{h in Hs}: ni_vnf17_dot_vm20_1_slave[h] <= sum{v in {806}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_1_slave{v in {806}, h in Hs}: ni_vnf17_dot_vm20_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_91_{h in Hs}: ni_vnf17_dot_vm20_1_master[h] + ni_vnf17_dot_vm20_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm20_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_2_master{h in Hs}: ni_vnf17_dot_vm20_2_master[h] <= sum{v in {807}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_2_master{v in {807}, h in Hs}: ni_vnf17_dot_vm20_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm20_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm20_2_slave{h in Hs}: ni_vnf17_dot_vm20_2_slave[h] <= sum{v in {808}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm20_2_slave{v in {808}, h in Hs}: ni_vnf17_dot_vm20_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_92_{h in Hs}: ni_vnf17_dot_vm20_2_master[h] + ni_vnf17_dot_vm20_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm21_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_0_master{h in Hs}: ni_vnf17_dot_vm21_0_master[h] <= sum{v in {809,810}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_0_master{v in {809,810}, h in Hs}: ni_vnf17_dot_vm21_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm21_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_0_slave{h in Hs}: ni_vnf17_dot_vm21_0_slave[h] <= sum{v in {811,812}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_0_slave{v in {811,812}, h in Hs}: ni_vnf17_dot_vm21_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_93_{h in Hs}: ni_vnf17_dot_vm21_0_master[h] + ni_vnf17_dot_vm21_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm21_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_1_master{h in Hs}: ni_vnf17_dot_vm21_1_master[h] <= sum{v in {813,814}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_1_master{v in {813,814}, h in Hs}: ni_vnf17_dot_vm21_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm21_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_1_slave{h in Hs}: ni_vnf17_dot_vm21_1_slave[h] <= sum{v in {815,816}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_1_slave{v in {815,816}, h in Hs}: ni_vnf17_dot_vm21_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_94_{h in Hs}: ni_vnf17_dot_vm21_1_master[h] + ni_vnf17_dot_vm21_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm21_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_2_master{h in Hs}: ni_vnf17_dot_vm21_2_master[h] <= sum{v in {817,818}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_2_master{v in {817,818}, h in Hs}: ni_vnf17_dot_vm21_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm21_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm21_2_slave{h in Hs}: ni_vnf17_dot_vm21_2_slave[h] <= sum{v in {819,820}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm21_2_slave{v in {819,820}, h in Hs}: ni_vnf17_dot_vm21_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_95_{h in Hs}: ni_vnf17_dot_vm21_2_master[h] + ni_vnf17_dot_vm21_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm22_0_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_0_master{h in Hs}: ni_vnf17_dot_vm22_0_master[h] <= sum{v in {821}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_0_master{v in {821}, h in Hs}: ni_vnf17_dot_vm22_0_master[h] >= x[v, h];
var ni_vnf17_dot_vm22_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_0_slave{h in Hs}: ni_vnf17_dot_vm22_0_slave[h] <= sum{v in {822}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_0_slave{v in {822}, h in Hs}: ni_vnf17_dot_vm22_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_96_{h in Hs}: ni_vnf17_dot_vm22_0_master[h] + ni_vnf17_dot_vm22_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm22_1_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_1_master{h in Hs}: ni_vnf17_dot_vm22_1_master[h] <= sum{v in {823}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_1_master{v in {823}, h in Hs}: ni_vnf17_dot_vm22_1_master[h] >= x[v, h];
var ni_vnf17_dot_vm22_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_1_slave{h in Hs}: ni_vnf17_dot_vm22_1_slave[h] <= sum{v in {824}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_1_slave{v in {824}, h in Hs}: ni_vnf17_dot_vm22_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_97_{h in Hs}: ni_vnf17_dot_vm22_1_master[h] + ni_vnf17_dot_vm22_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf17_dot_vm22_2_master{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_2_master{h in Hs}: ni_vnf17_dot_vm22_2_master[h] <= sum{v in {825}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_2_master{v in {825}, h in Hs}: ni_vnf17_dot_vm22_2_master[h] >= x[v, h];
var ni_vnf17_dot_vm22_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf17_dot_vm22_2_slave{h in Hs}: ni_vnf17_dot_vm22_2_slave[h] <= sum{v in {826}} x[v, h];
s.t. or_ineq_ni_vnf17_dot_vm22_2_slave{v in {826}, h in Hs}: ni_vnf17_dot_vm22_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_98_{h in Hs}: ni_vnf17_dot_vm22_2_master[h] + ni_vnf17_dot_vm22_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_0_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_0_master{h in Hs}: ni_vnf18_dot_vm65_0_master[h] <= sum{v in {998..1013}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_0_master{v in {998..1013}, h in Hs}: ni_vnf18_dot_vm65_0_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_0_slave{h in Hs}: ni_vnf18_dot_vm65_0_slave[h] <= sum{v in {1014..1029}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_0_slave{v in {1014..1029}, h in Hs}: ni_vnf18_dot_vm65_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_99_{h in Hs}: ni_vnf18_dot_vm65_0_master[h] + ni_vnf18_dot_vm65_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_1_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_1_master{h in Hs}: ni_vnf18_dot_vm65_1_master[h] <= sum{v in {1030..1045}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_1_master{v in {1030..1045}, h in Hs}: ni_vnf18_dot_vm65_1_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_1_slave{h in Hs}: ni_vnf18_dot_vm65_1_slave[h] <= sum{v in {1046..1061}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_1_slave{v in {1046..1061}, h in Hs}: ni_vnf18_dot_vm65_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_100_{h in Hs}: ni_vnf18_dot_vm65_1_master[h] + ni_vnf18_dot_vm65_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_2_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_2_master{h in Hs}: ni_vnf18_dot_vm65_2_master[h] <= sum{v in {1062..1077}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_2_master{v in {1062..1077}, h in Hs}: ni_vnf18_dot_vm65_2_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_2_slave{h in Hs}: ni_vnf18_dot_vm65_2_slave[h] <= sum{v in {1078..1093}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_2_slave{v in {1078..1093}, h in Hs}: ni_vnf18_dot_vm65_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_101_{h in Hs}: ni_vnf18_dot_vm65_2_master[h] + ni_vnf18_dot_vm65_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm65_3_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_3_master{h in Hs}: ni_vnf18_dot_vm65_3_master[h] <= sum{v in {1094..1109}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_3_master{v in {1094..1109}, h in Hs}: ni_vnf18_dot_vm65_3_master[h] >= x[v, h];
var ni_vnf18_dot_vm65_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm65_3_slave{h in Hs}: ni_vnf18_dot_vm65_3_slave[h] <= sum{v in {1110..1125}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm65_3_slave{v in {1110..1125}, h in Hs}: ni_vnf18_dot_vm65_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_102_{h in Hs}: ni_vnf18_dot_vm65_3_master[h] + ni_vnf18_dot_vm65_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_0_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_0_master{h in Hs}: ni_vnf18_dot_vm67_0_master[h] <= sum{v in {1134,1135}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_0_master{v in {1134,1135}, h in Hs}: ni_vnf18_dot_vm67_0_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_0_slave{h in Hs}: ni_vnf18_dot_vm67_0_slave[h] <= sum{v in {1136,1137}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_0_slave{v in {1136,1137}, h in Hs}: ni_vnf18_dot_vm67_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_103_{h in Hs}: ni_vnf18_dot_vm67_0_master[h] + ni_vnf18_dot_vm67_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_1_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_1_master{h in Hs}: ni_vnf18_dot_vm67_1_master[h] <= sum{v in {1138,1139}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_1_master{v in {1138,1139}, h in Hs}: ni_vnf18_dot_vm67_1_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_1_slave{h in Hs}: ni_vnf18_dot_vm67_1_slave[h] <= sum{v in {1140,1141}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_1_slave{v in {1140,1141}, h in Hs}: ni_vnf18_dot_vm67_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_104_{h in Hs}: ni_vnf18_dot_vm67_1_master[h] + ni_vnf18_dot_vm67_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_2_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_2_master{h in Hs}: ni_vnf18_dot_vm67_2_master[h] <= sum{v in {1142,1143}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_2_master{v in {1142,1143}, h in Hs}: ni_vnf18_dot_vm67_2_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_2_slave{h in Hs}: ni_vnf18_dot_vm67_2_slave[h] <= sum{v in {1144,1145}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_2_slave{v in {1144,1145}, h in Hs}: ni_vnf18_dot_vm67_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_105_{h in Hs}: ni_vnf18_dot_vm67_2_master[h] + ni_vnf18_dot_vm67_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm67_3_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_3_master{h in Hs}: ni_vnf18_dot_vm67_3_master[h] <= sum{v in {1146,1147}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_3_master{v in {1146,1147}, h in Hs}: ni_vnf18_dot_vm67_3_master[h] >= x[v, h];
var ni_vnf18_dot_vm67_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm67_3_slave{h in Hs}: ni_vnf18_dot_vm67_3_slave[h] <= sum{v in {1148,1149}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm67_3_slave{v in {1148,1149}, h in Hs}: ni_vnf18_dot_vm67_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_106_{h in Hs}: ni_vnf18_dot_vm67_3_master[h] + ni_vnf18_dot_vm67_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_0_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_0_master{h in Hs}: ni_vnf18_dot_vm68_0_master[h] <= sum{v in {1150..1152}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_0_master{v in {1150..1152}, h in Hs}: ni_vnf18_dot_vm68_0_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_0_slave{h in Hs}: ni_vnf18_dot_vm68_0_slave[h] <= sum{v in {1153..1155}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_0_slave{v in {1153..1155}, h in Hs}: ni_vnf18_dot_vm68_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_107_{h in Hs}: ni_vnf18_dot_vm68_0_master[h] + ni_vnf18_dot_vm68_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_1_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_1_master{h in Hs}: ni_vnf18_dot_vm68_1_master[h] <= sum{v in {1156..1158}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_1_master{v in {1156..1158}, h in Hs}: ni_vnf18_dot_vm68_1_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_1_slave{h in Hs}: ni_vnf18_dot_vm68_1_slave[h] <= sum{v in {1159..1161}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_1_slave{v in {1159..1161}, h in Hs}: ni_vnf18_dot_vm68_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_108_{h in Hs}: ni_vnf18_dot_vm68_1_master[h] + ni_vnf18_dot_vm68_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_2_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_2_master{h in Hs}: ni_vnf18_dot_vm68_2_master[h] <= sum{v in {1162..1164}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_2_master{v in {1162..1164}, h in Hs}: ni_vnf18_dot_vm68_2_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_2_slave{h in Hs}: ni_vnf18_dot_vm68_2_slave[h] <= sum{v in {1165..1167}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_2_slave{v in {1165..1167}, h in Hs}: ni_vnf18_dot_vm68_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_109_{h in Hs}: ni_vnf18_dot_vm68_2_master[h] + ni_vnf18_dot_vm68_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf18_dot_vm68_3_master{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_3_master{h in Hs}: ni_vnf18_dot_vm68_3_master[h] <= sum{v in {1168..1170}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_3_master{v in {1168..1170}, h in Hs}: ni_vnf18_dot_vm68_3_master[h] >= x[v, h];
var ni_vnf18_dot_vm68_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf18_dot_vm68_3_slave{h in Hs}: ni_vnf18_dot_vm68_3_slave[h] <= sum{v in {1171..1173}} x[v, h];
s.t. or_ineq_ni_vnf18_dot_vm68_3_slave{v in {1171..1173}, h in Hs}: ni_vnf18_dot_vm68_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_110_{h in Hs}: ni_vnf18_dot_vm68_3_master[h] + ni_vnf18_dot_vm68_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf19_dot_vm66_0_master{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm66_0_master{h in Hs}: ni_vnf19_dot_vm66_0_master[h] <= sum{v in {1274}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm66_0_master{v in {1274}, h in Hs}: ni_vnf19_dot_vm66_0_master[h] >= x[v, h];
var ni_vnf19_dot_vm66_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm66_0_slave{h in Hs}: ni_vnf19_dot_vm66_0_slave[h] <= sum{v in {1275}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm66_0_slave{v in {1275}, h in Hs}: ni_vnf19_dot_vm66_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_111_{h in Hs}: ni_vnf19_dot_vm66_0_master[h] + ni_vnf19_dot_vm66_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf19_dot_vm67_0_master{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm67_0_master{h in Hs}: ni_vnf19_dot_vm67_0_master[h] <= sum{v in {1276,1277}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm67_0_master{v in {1276,1277}, h in Hs}: ni_vnf19_dot_vm67_0_master[h] >= x[v, h];
var ni_vnf19_dot_vm67_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm67_0_slave{h in Hs}: ni_vnf19_dot_vm67_0_slave[h] <= sum{v in {1278,1279}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm67_0_slave{v in {1278,1279}, h in Hs}: ni_vnf19_dot_vm67_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_112_{h in Hs}: ni_vnf19_dot_vm67_0_master[h] + ni_vnf19_dot_vm67_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf19_dot_vm68_0_master{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm68_0_master{h in Hs}: ni_vnf19_dot_vm68_0_master[h] <= sum{v in {1280,1281}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm68_0_master{v in {1280,1281}, h in Hs}: ni_vnf19_dot_vm68_0_master[h] >= x[v, h];
var ni_vnf19_dot_vm68_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf19_dot_vm68_0_slave{h in Hs}: ni_vnf19_dot_vm68_0_slave[h] <= sum{v in {1282,1283}} x[v, h];
s.t. or_ineq_ni_vnf19_dot_vm68_0_slave{v in {1282,1283}, h in Hs}: ni_vnf19_dot_vm68_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_113_{h in Hs}: ni_vnf19_dot_vm68_0_master[h] + ni_vnf19_dot_vm68_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm11_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_0_master{h in Hs}: ni_vnf20_dot_vm11_0_master[h] <= sum{v in {1319..1332}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_0_master{v in {1319..1332}, h in Hs}: ni_vnf20_dot_vm11_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm11_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_0_slave{h in Hs}: ni_vnf20_dot_vm11_0_slave[h] <= sum{v in {1333..1346}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_0_slave{v in {1333..1346}, h in Hs}: ni_vnf20_dot_vm11_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_114_{h in Hs}: ni_vnf20_dot_vm11_0_master[h] + ni_vnf20_dot_vm11_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm11_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_1_master{h in Hs}: ni_vnf20_dot_vm11_1_master[h] <= sum{v in {1347..1360}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_1_master{v in {1347..1360}, h in Hs}: ni_vnf20_dot_vm11_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm11_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm11_1_slave{h in Hs}: ni_vnf20_dot_vm11_1_slave[h] <= sum{v in {1361..1374}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm11_1_slave{v in {1361..1374}, h in Hs}: ni_vnf20_dot_vm11_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_115_{h in Hs}: ni_vnf20_dot_vm11_1_master[h] + ni_vnf20_dot_vm11_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm15_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_0_master{h in Hs}: ni_vnf20_dot_vm15_0_master[h] <= sum{v in {1401}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_0_master{v in {1401}, h in Hs}: ni_vnf20_dot_vm15_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm15_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_0_slave{h in Hs}: ni_vnf20_dot_vm15_0_slave[h] <= sum{v in {1402}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_0_slave{v in {1402}, h in Hs}: ni_vnf20_dot_vm15_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_116_{h in Hs}: ni_vnf20_dot_vm15_0_master[h] + ni_vnf20_dot_vm15_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm15_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_1_master{h in Hs}: ni_vnf20_dot_vm15_1_master[h] <= sum{v in {1403}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_1_master{v in {1403}, h in Hs}: ni_vnf20_dot_vm15_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm15_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm15_1_slave{h in Hs}: ni_vnf20_dot_vm15_1_slave[h] <= sum{v in {1404}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm15_1_slave{v in {1404}, h in Hs}: ni_vnf20_dot_vm15_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_117_{h in Hs}: ni_vnf20_dot_vm15_1_master[h] + ni_vnf20_dot_vm15_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm16_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_0_master{h in Hs}: ni_vnf20_dot_vm16_0_master[h] <= sum{v in {1405}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_0_master{v in {1405}, h in Hs}: ni_vnf20_dot_vm16_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm16_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_0_slave{h in Hs}: ni_vnf20_dot_vm16_0_slave[h] <= sum{v in {1406}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_0_slave{v in {1406}, h in Hs}: ni_vnf20_dot_vm16_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_118_{h in Hs}: ni_vnf20_dot_vm16_0_master[h] + ni_vnf20_dot_vm16_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm16_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_1_master{h in Hs}: ni_vnf20_dot_vm16_1_master[h] <= sum{v in {1407}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_1_master{v in {1407}, h in Hs}: ni_vnf20_dot_vm16_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm16_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm16_1_slave{h in Hs}: ni_vnf20_dot_vm16_1_slave[h] <= sum{v in {1408}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm16_1_slave{v in {1408}, h in Hs}: ni_vnf20_dot_vm16_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_119_{h in Hs}: ni_vnf20_dot_vm16_1_master[h] + ni_vnf20_dot_vm16_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm17_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_0_master{h in Hs}: ni_vnf20_dot_vm17_0_master[h] <= sum{v in {1409..1411}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_0_master{v in {1409..1411}, h in Hs}: ni_vnf20_dot_vm17_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm17_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_0_slave{h in Hs}: ni_vnf20_dot_vm17_0_slave[h] <= sum{v in {1412..1414}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_0_slave{v in {1412..1414}, h in Hs}: ni_vnf20_dot_vm17_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_120_{h in Hs}: ni_vnf20_dot_vm17_0_master[h] + ni_vnf20_dot_vm17_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm17_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_1_master{h in Hs}: ni_vnf20_dot_vm17_1_master[h] <= sum{v in {1415..1417}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_1_master{v in {1415..1417}, h in Hs}: ni_vnf20_dot_vm17_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm17_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm17_1_slave{h in Hs}: ni_vnf20_dot_vm17_1_slave[h] <= sum{v in {1418..1420}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm17_1_slave{v in {1418..1420}, h in Hs}: ni_vnf20_dot_vm17_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_121_{h in Hs}: ni_vnf20_dot_vm17_1_master[h] + ni_vnf20_dot_vm17_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm18_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_0_master{h in Hs}: ni_vnf20_dot_vm18_0_master[h] <= sum{v in {1421}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_0_master{v in {1421}, h in Hs}: ni_vnf20_dot_vm18_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm18_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_0_slave{h in Hs}: ni_vnf20_dot_vm18_0_slave[h] <= sum{v in {1422}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_0_slave{v in {1422}, h in Hs}: ni_vnf20_dot_vm18_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_122_{h in Hs}: ni_vnf20_dot_vm18_0_master[h] + ni_vnf20_dot_vm18_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm18_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_1_master{h in Hs}: ni_vnf20_dot_vm18_1_master[h] <= sum{v in {1423}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_1_master{v in {1423}, h in Hs}: ni_vnf20_dot_vm18_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm18_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm18_1_slave{h in Hs}: ni_vnf20_dot_vm18_1_slave[h] <= sum{v in {1424}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm18_1_slave{v in {1424}, h in Hs}: ni_vnf20_dot_vm18_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_123_{h in Hs}: ni_vnf20_dot_vm18_1_master[h] + ni_vnf20_dot_vm18_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm20_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_0_master{h in Hs}: ni_vnf20_dot_vm20_0_master[h] <= sum{v in {1425}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_0_master{v in {1425}, h in Hs}: ni_vnf20_dot_vm20_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm20_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_0_slave{h in Hs}: ni_vnf20_dot_vm20_0_slave[h] <= sum{v in {1426}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_0_slave{v in {1426}, h in Hs}: ni_vnf20_dot_vm20_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_124_{h in Hs}: ni_vnf20_dot_vm20_0_master[h] + ni_vnf20_dot_vm20_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm20_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_1_master{h in Hs}: ni_vnf20_dot_vm20_1_master[h] <= sum{v in {1427}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_1_master{v in {1427}, h in Hs}: ni_vnf20_dot_vm20_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm20_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm20_1_slave{h in Hs}: ni_vnf20_dot_vm20_1_slave[h] <= sum{v in {1428}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm20_1_slave{v in {1428}, h in Hs}: ni_vnf20_dot_vm20_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_125_{h in Hs}: ni_vnf20_dot_vm20_1_master[h] + ni_vnf20_dot_vm20_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm21_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_0_master{h in Hs}: ni_vnf20_dot_vm21_0_master[h] <= sum{v in {1429}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_0_master{v in {1429}, h in Hs}: ni_vnf20_dot_vm21_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm21_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_0_slave{h in Hs}: ni_vnf20_dot_vm21_0_slave[h] <= sum{v in {1430}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_0_slave{v in {1430}, h in Hs}: ni_vnf20_dot_vm21_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_126_{h in Hs}: ni_vnf20_dot_vm21_0_master[h] + ni_vnf20_dot_vm21_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm21_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_1_master{h in Hs}: ni_vnf20_dot_vm21_1_master[h] <= sum{v in {1431}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_1_master{v in {1431}, h in Hs}: ni_vnf20_dot_vm21_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm21_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm21_1_slave{h in Hs}: ni_vnf20_dot_vm21_1_slave[h] <= sum{v in {1432}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm21_1_slave{v in {1432}, h in Hs}: ni_vnf20_dot_vm21_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_127_{h in Hs}: ni_vnf20_dot_vm21_1_master[h] + ni_vnf20_dot_vm21_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm22_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_0_master{h in Hs}: ni_vnf20_dot_vm22_0_master[h] <= sum{v in {1433}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_0_master{v in {1433}, h in Hs}: ni_vnf20_dot_vm22_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm22_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_0_slave{h in Hs}: ni_vnf20_dot_vm22_0_slave[h] <= sum{v in {1434}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_0_slave{v in {1434}, h in Hs}: ni_vnf20_dot_vm22_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_128_{h in Hs}: ni_vnf20_dot_vm22_0_master[h] + ni_vnf20_dot_vm22_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm22_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_1_master{h in Hs}: ni_vnf20_dot_vm22_1_master[h] <= sum{v in {1435}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_1_master{v in {1435}, h in Hs}: ni_vnf20_dot_vm22_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm22_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm22_1_slave{h in Hs}: ni_vnf20_dot_vm22_1_slave[h] <= sum{v in {1436}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm22_1_slave{v in {1436}, h in Hs}: ni_vnf20_dot_vm22_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_129_{h in Hs}: ni_vnf20_dot_vm22_1_master[h] + ni_vnf20_dot_vm22_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm76_0_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_0_master{h in Hs}: ni_vnf20_dot_vm76_0_master[h] <= sum{v in {1437}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_0_master{v in {1437}, h in Hs}: ni_vnf20_dot_vm76_0_master[h] >= x[v, h];
var ni_vnf20_dot_vm76_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_0_slave{h in Hs}: ni_vnf20_dot_vm76_0_slave[h] <= sum{v in {1438}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_0_slave{v in {1438}, h in Hs}: ni_vnf20_dot_vm76_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_130_{h in Hs}: ni_vnf20_dot_vm76_0_master[h] + ni_vnf20_dot_vm76_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf20_dot_vm76_1_master{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_1_master{h in Hs}: ni_vnf20_dot_vm76_1_master[h] <= sum{v in {1439}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_1_master{v in {1439}, h in Hs}: ni_vnf20_dot_vm76_1_master[h] >= x[v, h];
var ni_vnf20_dot_vm76_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf20_dot_vm76_1_slave{h in Hs}: ni_vnf20_dot_vm76_1_slave[h] <= sum{v in {1440}} x[v, h];
s.t. or_ineq_ni_vnf20_dot_vm76_1_slave{v in {1440}, h in Hs}: ni_vnf20_dot_vm76_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_131_{h in Hs}: ni_vnf20_dot_vm76_1_master[h] + ni_vnf20_dot_vm76_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_0_master{h in Hs}: ni_vnf21_dot_vm37_0_master[h] <= sum{v in {1459..1463}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_0_master{v in {1459..1463}, h in Hs}: ni_vnf21_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_0_slave{h in Hs}: ni_vnf21_dot_vm37_0_slave[h] <= sum{v in {1464..1468}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_0_slave{v in {1464..1468}, h in Hs}: ni_vnf21_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_132_{h in Hs}: ni_vnf21_dot_vm37_0_master[h] + ni_vnf21_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_1_master{h in Hs}: ni_vnf21_dot_vm37_1_master[h] <= sum{v in {1469..1473}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_1_master{v in {1469..1473}, h in Hs}: ni_vnf21_dot_vm37_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_1_slave{h in Hs}: ni_vnf21_dot_vm37_1_slave[h] <= sum{v in {1474..1478}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_1_slave{v in {1474..1478}, h in Hs}: ni_vnf21_dot_vm37_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_133_{h in Hs}: ni_vnf21_dot_vm37_1_master[h] + ni_vnf21_dot_vm37_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_2_master{h in Hs}: ni_vnf21_dot_vm37_2_master[h] <= sum{v in {1479..1483}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_2_master{v in {1479..1483}, h in Hs}: ni_vnf21_dot_vm37_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_2_slave{h in Hs}: ni_vnf21_dot_vm37_2_slave[h] <= sum{v in {1484..1488}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_2_slave{v in {1484..1488}, h in Hs}: ni_vnf21_dot_vm37_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_134_{h in Hs}: ni_vnf21_dot_vm37_2_master[h] + ni_vnf21_dot_vm37_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm37_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_3_master{h in Hs}: ni_vnf21_dot_vm37_3_master[h] <= sum{v in {1489..1493}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_3_master{v in {1489..1493}, h in Hs}: ni_vnf21_dot_vm37_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm37_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm37_3_slave{h in Hs}: ni_vnf21_dot_vm37_3_slave[h] <= sum{v in {1494..1498}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm37_3_slave{v in {1494..1498}, h in Hs}: ni_vnf21_dot_vm37_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_135_{h in Hs}: ni_vnf21_dot_vm37_3_master[h] + ni_vnf21_dot_vm37_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_0_master{h in Hs}: ni_vnf21_dot_vm38_0_master[h] <= sum{v in {1499}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_0_master{v in {1499}, h in Hs}: ni_vnf21_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_0_slave{h in Hs}: ni_vnf21_dot_vm38_0_slave[h] <= sum{v in {1500}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_0_slave{v in {1500}, h in Hs}: ni_vnf21_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_136_{h in Hs}: ni_vnf21_dot_vm38_0_master[h] + ni_vnf21_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_1_master{h in Hs}: ni_vnf21_dot_vm38_1_master[h] <= sum{v in {1501}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_1_master{v in {1501}, h in Hs}: ni_vnf21_dot_vm38_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_1_slave{h in Hs}: ni_vnf21_dot_vm38_1_slave[h] <= sum{v in {1502}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_1_slave{v in {1502}, h in Hs}: ni_vnf21_dot_vm38_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_137_{h in Hs}: ni_vnf21_dot_vm38_1_master[h] + ni_vnf21_dot_vm38_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_2_master{h in Hs}: ni_vnf21_dot_vm38_2_master[h] <= sum{v in {1503}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_2_master{v in {1503}, h in Hs}: ni_vnf21_dot_vm38_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_2_slave{h in Hs}: ni_vnf21_dot_vm38_2_slave[h] <= sum{v in {1504}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_2_slave{v in {1504}, h in Hs}: ni_vnf21_dot_vm38_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_138_{h in Hs}: ni_vnf21_dot_vm38_2_master[h] + ni_vnf21_dot_vm38_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm38_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_3_master{h in Hs}: ni_vnf21_dot_vm38_3_master[h] <= sum{v in {1505}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_3_master{v in {1505}, h in Hs}: ni_vnf21_dot_vm38_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm38_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm38_3_slave{h in Hs}: ni_vnf21_dot_vm38_3_slave[h] <= sum{v in {1506}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm38_3_slave{v in {1506}, h in Hs}: ni_vnf21_dot_vm38_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_139_{h in Hs}: ni_vnf21_dot_vm38_3_master[h] + ni_vnf21_dot_vm38_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_0_master{h in Hs}: ni_vnf21_dot_vm39_0_master[h] <= sum{v in {1507}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_0_master{v in {1507}, h in Hs}: ni_vnf21_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_0_slave{h in Hs}: ni_vnf21_dot_vm39_0_slave[h] <= sum{v in {1508}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_0_slave{v in {1508}, h in Hs}: ni_vnf21_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_140_{h in Hs}: ni_vnf21_dot_vm39_0_master[h] + ni_vnf21_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_1_master{h in Hs}: ni_vnf21_dot_vm39_1_master[h] <= sum{v in {1509}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_1_master{v in {1509}, h in Hs}: ni_vnf21_dot_vm39_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_1_slave{h in Hs}: ni_vnf21_dot_vm39_1_slave[h] <= sum{v in {1510}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_1_slave{v in {1510}, h in Hs}: ni_vnf21_dot_vm39_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_141_{h in Hs}: ni_vnf21_dot_vm39_1_master[h] + ni_vnf21_dot_vm39_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_2_master{h in Hs}: ni_vnf21_dot_vm39_2_master[h] <= sum{v in {1511}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_2_master{v in {1511}, h in Hs}: ni_vnf21_dot_vm39_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_2_slave{h in Hs}: ni_vnf21_dot_vm39_2_slave[h] <= sum{v in {1512}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_2_slave{v in {1512}, h in Hs}: ni_vnf21_dot_vm39_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_142_{h in Hs}: ni_vnf21_dot_vm39_2_master[h] + ni_vnf21_dot_vm39_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm39_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_3_master{h in Hs}: ni_vnf21_dot_vm39_3_master[h] <= sum{v in {1513}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_3_master{v in {1513}, h in Hs}: ni_vnf21_dot_vm39_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm39_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm39_3_slave{h in Hs}: ni_vnf21_dot_vm39_3_slave[h] <= sum{v in {1514}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm39_3_slave{v in {1514}, h in Hs}: ni_vnf21_dot_vm39_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_143_{h in Hs}: ni_vnf21_dot_vm39_3_master[h] + ni_vnf21_dot_vm39_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_0_master{h in Hs}: ni_vnf21_dot_vm40_0_master[h] <= sum{v in {1515}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_0_master{v in {1515}, h in Hs}: ni_vnf21_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_0_slave{h in Hs}: ni_vnf21_dot_vm40_0_slave[h] <= sum{v in {1516}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_0_slave{v in {1516}, h in Hs}: ni_vnf21_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_144_{h in Hs}: ni_vnf21_dot_vm40_0_master[h] + ni_vnf21_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_1_master{h in Hs}: ni_vnf21_dot_vm40_1_master[h] <= sum{v in {1517}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_1_master{v in {1517}, h in Hs}: ni_vnf21_dot_vm40_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_1_slave{h in Hs}: ni_vnf21_dot_vm40_1_slave[h] <= sum{v in {1518}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_1_slave{v in {1518}, h in Hs}: ni_vnf21_dot_vm40_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_145_{h in Hs}: ni_vnf21_dot_vm40_1_master[h] + ni_vnf21_dot_vm40_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_2_master{h in Hs}: ni_vnf21_dot_vm40_2_master[h] <= sum{v in {1519}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_2_master{v in {1519}, h in Hs}: ni_vnf21_dot_vm40_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_2_slave{h in Hs}: ni_vnf21_dot_vm40_2_slave[h] <= sum{v in {1520}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_2_slave{v in {1520}, h in Hs}: ni_vnf21_dot_vm40_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_146_{h in Hs}: ni_vnf21_dot_vm40_2_master[h] + ni_vnf21_dot_vm40_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm40_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_3_master{h in Hs}: ni_vnf21_dot_vm40_3_master[h] <= sum{v in {1521}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_3_master{v in {1521}, h in Hs}: ni_vnf21_dot_vm40_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm40_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm40_3_slave{h in Hs}: ni_vnf21_dot_vm40_3_slave[h] <= sum{v in {1522}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm40_3_slave{v in {1522}, h in Hs}: ni_vnf21_dot_vm40_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_147_{h in Hs}: ni_vnf21_dot_vm40_3_master[h] + ni_vnf21_dot_vm40_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_0_master{h in Hs}: ni_vnf21_dot_vm7_0_master[h] <= sum{v in {1523}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_0_master{v in {1523}, h in Hs}: ni_vnf21_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_0_slave{h in Hs}: ni_vnf21_dot_vm7_0_slave[h] <= sum{v in {1524}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_0_slave{v in {1524}, h in Hs}: ni_vnf21_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_148_{h in Hs}: ni_vnf21_dot_vm7_0_master[h] + ni_vnf21_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_1_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_1_master{h in Hs}: ni_vnf21_dot_vm7_1_master[h] <= sum{v in {1525}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_1_master{v in {1525}, h in Hs}: ni_vnf21_dot_vm7_1_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_1_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_1_slave{h in Hs}: ni_vnf21_dot_vm7_1_slave[h] <= sum{v in {1526}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_1_slave{v in {1526}, h in Hs}: ni_vnf21_dot_vm7_1_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_149_{h in Hs}: ni_vnf21_dot_vm7_1_master[h] + ni_vnf21_dot_vm7_1_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_2_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_2_master{h in Hs}: ni_vnf21_dot_vm7_2_master[h] <= sum{v in {1527}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_2_master{v in {1527}, h in Hs}: ni_vnf21_dot_vm7_2_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_2_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_2_slave{h in Hs}: ni_vnf21_dot_vm7_2_slave[h] <= sum{v in {1528}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_2_slave{v in {1528}, h in Hs}: ni_vnf21_dot_vm7_2_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_150_{h in Hs}: ni_vnf21_dot_vm7_2_master[h] + ni_vnf21_dot_vm7_2_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf21_dot_vm7_3_master{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_3_master{h in Hs}: ni_vnf21_dot_vm7_3_master[h] <= sum{v in {1529}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_3_master{v in {1529}, h in Hs}: ni_vnf21_dot_vm7_3_master[h] >= x[v, h];
var ni_vnf21_dot_vm7_3_slave{Hs}, binary;
s.t. or_sum_ni_vnf21_dot_vm7_3_slave{h in Hs}: ni_vnf21_dot_vm7_3_slave[h] <= sum{v in {1530}} x[v, h];
s.t. or_ineq_ni_vnf21_dot_vm7_3_slave{v in {1530}, h in Hs}: ni_vnf21_dot_vm7_3_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_151_{h in Hs}: ni_vnf21_dot_vm7_3_master[h] + ni_vnf21_dot_vm7_3_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm37_0_master{h in Hs}: ni_vnf22_dot_vm37_0_master[h] <= sum{v in {1531..1534}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm37_0_master{v in {1531..1534}, h in Hs}: ni_vnf22_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm37_0_slave{h in Hs}: ni_vnf22_dot_vm37_0_slave[h] <= sum{v in {1535..1538}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm37_0_slave{v in {1535..1538}, h in Hs}: ni_vnf22_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_152_{h in Hs}: ni_vnf22_dot_vm37_0_master[h] + ni_vnf22_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm38_0_master{h in Hs}: ni_vnf22_dot_vm38_0_master[h] <= sum{v in {1539,1540}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm38_0_master{v in {1539,1540}, h in Hs}: ni_vnf22_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm38_0_slave{h in Hs}: ni_vnf22_dot_vm38_0_slave[h] <= sum{v in {1541,1542}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm38_0_slave{v in {1541,1542}, h in Hs}: ni_vnf22_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_153_{h in Hs}: ni_vnf22_dot_vm38_0_master[h] + ni_vnf22_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm39_0_master{h in Hs}: ni_vnf22_dot_vm39_0_master[h] <= sum{v in {1543}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm39_0_master{v in {1543}, h in Hs}: ni_vnf22_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm39_0_slave{h in Hs}: ni_vnf22_dot_vm39_0_slave[h] <= sum{v in {1544}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm39_0_slave{v in {1544}, h in Hs}: ni_vnf22_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_154_{h in Hs}: ni_vnf22_dot_vm39_0_master[h] + ni_vnf22_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm40_0_master{h in Hs}: ni_vnf22_dot_vm40_0_master[h] <= sum{v in {1545}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm40_0_master{v in {1545}, h in Hs}: ni_vnf22_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm40_0_slave{h in Hs}: ni_vnf22_dot_vm40_0_slave[h] <= sum{v in {1546}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm40_0_slave{v in {1546}, h in Hs}: ni_vnf22_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_155_{h in Hs}: ni_vnf22_dot_vm40_0_master[h] + ni_vnf22_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm79_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm79_0_master{h in Hs}: ni_vnf22_dot_vm79_0_master[h] <= sum{v in {1547}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm79_0_master{v in {1547}, h in Hs}: ni_vnf22_dot_vm79_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm79_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm79_0_slave{h in Hs}: ni_vnf22_dot_vm79_0_slave[h] <= sum{v in {1548}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm79_0_slave{v in {1548}, h in Hs}: ni_vnf22_dot_vm79_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_156_{h in Hs}: ni_vnf22_dot_vm79_0_master[h] + ni_vnf22_dot_vm79_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf22_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm7_0_master{h in Hs}: ni_vnf22_dot_vm7_0_master[h] <= sum{v in {1549}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm7_0_master{v in {1549}, h in Hs}: ni_vnf22_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf22_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf22_dot_vm7_0_slave{h in Hs}: ni_vnf22_dot_vm7_0_slave[h] <= sum{v in {1550}} x[v, h];
s.t. or_ineq_ni_vnf22_dot_vm7_0_slave{v in {1550}, h in Hs}: ni_vnf22_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_157_{h in Hs}: ni_vnf22_dot_vm7_0_master[h] + ni_vnf22_dot_vm7_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm37_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm37_0_master{h in Hs}: ni_vnf24_dot_vm37_0_master[h] <= sum{v in {1556..1559}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm37_0_master{v in {1556..1559}, h in Hs}: ni_vnf24_dot_vm37_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm37_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm37_0_slave{h in Hs}: ni_vnf24_dot_vm37_0_slave[h] <= sum{v in {1560..1563}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm37_0_slave{v in {1560..1563}, h in Hs}: ni_vnf24_dot_vm37_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_158_{h in Hs}: ni_vnf24_dot_vm37_0_master[h] + ni_vnf24_dot_vm37_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm38_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm38_0_master{h in Hs}: ni_vnf24_dot_vm38_0_master[h] <= sum{v in {1564,1565}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm38_0_master{v in {1564,1565}, h in Hs}: ni_vnf24_dot_vm38_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm38_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm38_0_slave{h in Hs}: ni_vnf24_dot_vm38_0_slave[h] <= sum{v in {1566,1567}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm38_0_slave{v in {1566,1567}, h in Hs}: ni_vnf24_dot_vm38_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_159_{h in Hs}: ni_vnf24_dot_vm38_0_master[h] + ni_vnf24_dot_vm38_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm39_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm39_0_master{h in Hs}: ni_vnf24_dot_vm39_0_master[h] <= sum{v in {1568}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm39_0_master{v in {1568}, h in Hs}: ni_vnf24_dot_vm39_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm39_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm39_0_slave{h in Hs}: ni_vnf24_dot_vm39_0_slave[h] <= sum{v in {1569}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm39_0_slave{v in {1569}, h in Hs}: ni_vnf24_dot_vm39_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_160_{h in Hs}: ni_vnf24_dot_vm39_0_master[h] + ni_vnf24_dot_vm39_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm40_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm40_0_master{h in Hs}: ni_vnf24_dot_vm40_0_master[h] <= sum{v in {1570}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm40_0_master{v in {1570}, h in Hs}: ni_vnf24_dot_vm40_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm40_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm40_0_slave{h in Hs}: ni_vnf24_dot_vm40_0_slave[h] <= sum{v in {1571}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm40_0_slave{v in {1571}, h in Hs}: ni_vnf24_dot_vm40_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_161_{h in Hs}: ni_vnf24_dot_vm40_0_master[h] + ni_vnf24_dot_vm40_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm79_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm79_0_master{h in Hs}: ni_vnf24_dot_vm79_0_master[h] <= sum{v in {1572}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm79_0_master{v in {1572}, h in Hs}: ni_vnf24_dot_vm79_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm79_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm79_0_slave{h in Hs}: ni_vnf24_dot_vm79_0_slave[h] <= sum{v in {1573}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm79_0_slave{v in {1573}, h in Hs}: ni_vnf24_dot_vm79_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_162_{h in Hs}: ni_vnf24_dot_vm79_0_master[h] + ni_vnf24_dot_vm79_0_slave[h] <= 1;
# --- OR variables for CAAF ----
var ni_vnf24_dot_vm7_0_master{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm7_0_master{h in Hs}: ni_vnf24_dot_vm7_0_master[h] <= sum{v in {1574}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm7_0_master{v in {1574}, h in Hs}: ni_vnf24_dot_vm7_0_master[h] >= x[v, h];
var ni_vnf24_dot_vm7_0_slave{Hs}, binary;
s.t. or_sum_ni_vnf24_dot_vm7_0_slave{h in Hs}: ni_vnf24_dot_vm7_0_slave[h] <= sum{v in {1575}} x[v, h];
s.t. or_ineq_ni_vnf24_dot_vm7_0_slave{v in {1575}, h in Hs}: ni_vnf24_dot_vm7_0_slave[h] >= x[v, h];
# --- Cross Anti-Affinity ----
s.t. XAAF_cnstr_163_{h in Hs}: ni_vnf24_dot_vm7_0_master[h] + ni_vnf24_dot_vm7_0_slave[h] <= 1;

#=== Soft Affinity Variables ===
var vnf5_dot_vm30_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm30_0{h in Hs}: vnf5_dot_vm30_0[h] <= sum{v in {323,324}} x[v, h];
s.t. or_ineq_vnf5_dot_vm30_0{v in {323,324}, h in Hs}: vnf5_dot_vm30_0[h] >= x[v, h];
var vnf5_dot_vm31_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm31_0{h in Hs}: vnf5_dot_vm31_0[h] <= sum{v in {325,326}} x[v, h];
s.t. or_ineq_vnf5_dot_vm31_0{v in {325,326}, h in Hs}: vnf5_dot_vm31_0[h] >= x[v, h];
var vnf5_dot_vm32_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm32_0{h in Hs}: vnf5_dot_vm32_0[h] <= sum{v in {327..333}} x[v, h];
s.t. or_ineq_vnf5_dot_vm32_0{v in {327..333}, h in Hs}: vnf5_dot_vm32_0[h] >= x[v, h];
var vnf5_dot_vm33_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm33_0{h in Hs}: vnf5_dot_vm33_0[h] <= sum{v in {334,335}} x[v, h];
s.t. or_ineq_vnf5_dot_vm33_0{v in {334,335}, h in Hs}: vnf5_dot_vm33_0[h] >= x[v, h];
var vnf5_dot_vm34_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm34_0{h in Hs}: vnf5_dot_vm34_0[h] <= sum{v in {336,337}} x[v, h];
s.t. or_ineq_vnf5_dot_vm34_0{v in {336,337}, h in Hs}: vnf5_dot_vm34_0[h] >= x[v, h];
var vnf5_dot_vm35_0{h in Hs}, binary;
s.t. or_sum_vnf5_dot_vm35_0{h in Hs}: vnf5_dot_vm35_0[h] <= sum{v in {338,339}} x[v, h];
s.t. or_ineq_vnf5_dot_vm35_0{v in {338,339}, h in Hs}: vnf5_dot_vm35_0[h] >= x[v, h];
var vnf26_dot_vm82_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm82_0{h in Hs}: vnf26_dot_vm82_0[h] <= sum{v in {1581}} x[v, h];
s.t. or_ineq_vnf26_dot_vm82_0{v in {1581}, h in Hs}: vnf26_dot_vm82_0[h] >= x[v, h];
var vnf26_dot_vm14_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm14_0{h in Hs}: vnf26_dot_vm14_0[h] <= sum{v in {1583}} x[v, h];
s.t. or_ineq_vnf26_dot_vm14_0{v in {1583}, h in Hs}: vnf26_dot_vm14_0[h] >= x[v, h];
var vnf26_dot_vm86_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm86_0{h in Hs}: vnf26_dot_vm86_0[h] <= sum{v in {1586}} x[v, h];
s.t. or_ineq_vnf26_dot_vm86_0{v in {1586}, h in Hs}: vnf26_dot_vm86_0[h] >= x[v, h];
var vnf26_dot_vm87_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm87_0{h in Hs}: vnf26_dot_vm87_0[h] <= sum{v in {1587}} x[v, h];
s.t. or_ineq_vnf26_dot_vm87_0{v in {1587}, h in Hs}: vnf26_dot_vm87_0[h] >= x[v, h];
var vnf26_dot_vm88_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm88_0{h in Hs}: vnf26_dot_vm88_0[h] <= sum{v in {1588}} x[v, h];
s.t. or_ineq_vnf26_dot_vm88_0{v in {1588}, h in Hs}: vnf26_dot_vm88_0[h] >= x[v, h];
var vnf26_dot_vm89_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm89_0{h in Hs}: vnf26_dot_vm89_0[h] <= sum{v in {1589}} x[v, h];
s.t. or_ineq_vnf26_dot_vm89_0{v in {1589}, h in Hs}: vnf26_dot_vm89_0[h] >= x[v, h];
var vnf26_dot_vm91_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm91_0{h in Hs}: vnf26_dot_vm91_0[h] <= sum{v in {1591}} x[v, h];
s.t. or_ineq_vnf26_dot_vm91_0{v in {1591}, h in Hs}: vnf26_dot_vm91_0[h] >= x[v, h];
var vnf26_dot_vm92_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm92_0{h in Hs}: vnf26_dot_vm92_0[h] <= sum{v in {1592}} x[v, h];
s.t. or_ineq_vnf26_dot_vm92_0{v in {1592}, h in Hs}: vnf26_dot_vm92_0[h] >= x[v, h];
var vnf26_dot_vm109_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm109_0{h in Hs}: vnf26_dot_vm109_0[h] <= sum{v in {1609}} x[v, h];
s.t. or_ineq_vnf26_dot_vm109_0{v in {1609}, h in Hs}: vnf26_dot_vm109_0[h] >= x[v, h];
var vnf26_dot_vm110_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm110_0{h in Hs}: vnf26_dot_vm110_0[h] <= sum{v in {1610}} x[v, h];
s.t. or_ineq_vnf26_dot_vm110_0{v in {1610}, h in Hs}: vnf26_dot_vm110_0[h] >= x[v, h];
var vnf26_dot_vm111_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm111_0{h in Hs}: vnf26_dot_vm111_0[h] <= sum{v in {1611}} x[v, h];
s.t. or_ineq_vnf26_dot_vm111_0{v in {1611}, h in Hs}: vnf26_dot_vm111_0[h] >= x[v, h];
var vnf26_dot_vm112_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm112_0{h in Hs}: vnf26_dot_vm112_0[h] <= sum{v in {1612}} x[v, h];
s.t. or_ineq_vnf26_dot_vm112_0{v in {1612}, h in Hs}: vnf26_dot_vm112_0[h] >= x[v, h];
var vnf26_dot_vm113_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm113_0{h in Hs}: vnf26_dot_vm113_0[h] <= sum{v in {1613}} x[v, h];
s.t. or_ineq_vnf26_dot_vm113_0{v in {1613}, h in Hs}: vnf26_dot_vm113_0[h] >= x[v, h];
var vnf26_dot_vm114_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm114_0{h in Hs}: vnf26_dot_vm114_0[h] <= sum{v in {1614}} x[v, h];
s.t. or_ineq_vnf26_dot_vm114_0{v in {1614}, h in Hs}: vnf26_dot_vm114_0[h] >= x[v, h];
var vnf26_dot_vm115_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm115_0{h in Hs}: vnf26_dot_vm115_0[h] <= sum{v in {1615}} x[v, h];
s.t. or_ineq_vnf26_dot_vm115_0{v in {1615}, h in Hs}: vnf26_dot_vm115_0[h] >= x[v, h];
var vnf26_dot_vm116_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm116_0{h in Hs}: vnf26_dot_vm116_0[h] <= sum{v in {1616}} x[v, h];
s.t. or_ineq_vnf26_dot_vm116_0{v in {1616}, h in Hs}: vnf26_dot_vm116_0[h] >= x[v, h];
var vnf26_dot_vm117_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm117_0{h in Hs}: vnf26_dot_vm117_0[h] <= sum{v in {1617}} x[v, h];
s.t. or_ineq_vnf26_dot_vm117_0{v in {1617}, h in Hs}: vnf26_dot_vm117_0[h] >= x[v, h];
var vnf26_dot_vm126_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm126_0{h in Hs}: vnf26_dot_vm126_0[h] <= sum{v in {1626}} x[v, h];
s.t. or_ineq_vnf26_dot_vm126_0{v in {1626}, h in Hs}: vnf26_dot_vm126_0[h] >= x[v, h];
var vnf26_dot_vm127_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm127_0{h in Hs}: vnf26_dot_vm127_0[h] <= sum{v in {1627}} x[v, h];
s.t. or_ineq_vnf26_dot_vm127_0{v in {1627}, h in Hs}: vnf26_dot_vm127_0[h] >= x[v, h];
var vnf26_dot_vm128_0{h in Hs}, binary;
s.t. or_sum_vnf26_dot_vm128_0{h in Hs}: vnf26_dot_vm128_0[h] <= sum{v in {1628}} x[v, h];
s.t. or_ineq_vnf26_dot_vm128_0{v in {1628}, h in Hs}: vnf26_dot_vm128_0[h] >= x[v, h];

#=== Soft Affinity Cost ===
var soft_affinity_penalty;
s.t. soft_affinity_penalty_def: soft_affinity_penalty = 1 * ((sum{h in Hs} vnf5_dot_vm30_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm31_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm32_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm33_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm34_0[h]) - 1 + (sum{h in Hs} vnf5_dot_vm35_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm82_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm14_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm86_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm87_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm88_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm89_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm91_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm92_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm109_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm110_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm111_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm112_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm113_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm114_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm115_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm116_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm117_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm126_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm127_0[h]) - 1 + (sum{h in Hs} vnf26_dot_vm128_0[h]) - 1) / 37;

######################
# COST 
######################
minimize obj:  + 100 * used_hosts + soft_affinity_penalty;

######################
# DATA 
######################
data;
param: CPU := 
0 8.0
1 8.0
2 8.0
3 8.0
4 8.0
5 8.0
6 8.0
7 8.0
8 8.0
9 8.0
10 8.0
11 8.0
12 8.0
13 8.0
14 8.0
15 8.0
16 8.0
17 8.0
18 8.0
19 8.0
20 8.0
21 8.0
22 8.0
23 8.0
24 8.0
25 8.0
26 8.0
27 8.0
28 8.0
29 8.0
30 8.0
31 8.0
32 8.0
33 8.0
34 8.0
35 8.0
36 8.0
37 8.0
38 8.0
39 8.0
40 8.0
41 8.0
42 8.0
43 8.0
44 8.0
45 8.0
46 8.0
47 8.0
48 8.0
49 8.0
50 8.0
51 8.0
52 8.0
53 8.0
54 8.0
55 8.0
56 8.0
57 8.0
58 8.0
59 8.0
60 8.0
61 8.0
62 8.0
63 8.0
64 8.0
65 8.0
66 8.0
67 8.0
68 8.0
69 8.0
70 8.0
71 8.0
72 8.0
73 8.0
74 8.0
75 8.0
76 8.0
77 8.0
78 8.0
79 8.0
80 8.0
81 8.0
82 8.0
83 8.0
84 8.0
85 8.0
86 8.0
87 8.0
88 8.0
89 8.0
90 8.0
91 8.0
92 8.0
93 8.0
94 8.0
95 8.0
96 8.0
97 8.0
98 8.0
99 8.0
100 8.0
101 8.0
102 8.0
103 8.0
104 8.0
105 8.0
106 8.0
107 8.0
108 8.0
109 8.0
110 8.0
111 8.0
112 8.0
113 8.0
114 8.0
115 8.0
116 8.0
117 8.0
118 8.0
119 8.0
120 4.0
121 4.0
122 4.0
123 4.0
124 4.0
125 4.0
126 4.0
127 4.0
128 4.0
129 4.0
130 4.0
131 4.0
132 4.0
133 4.0
134 4.0
135 4.0
136 4.0
137 4.0
138 4.0
139 4.0
140 4.0
141 4.0
142 4.0
143 4.0
144 4.0
145 4.0
146 8.0
147 8.0
148 8.0
149 8.0
150 8.0
151 8.0
152 4.0
153 4.0
154 4.0
155 4.0
156 4.0
157 4.0
158 4.0
159 4.0
160 4.0
161 4.0
162 4.0
163 4.0
164 4.0
165 4.0
166 4.0
167 4.0
168 4.0
169 4.0
170 4.0
171 4.0
172 4.0
173 4.0
174 4.0
175 4.0
176 8.0
177 8.0
178 8.0
179 8.0
180 8.0
181 8.0
182 8.0
183 8.0
184 8.0
185 8.0
186 8.0
187 8.0
188 8.0
189 8.0
190 8.0
191 8.0
192 8.0
193 8.0
194 8.0
195 8.0
196 8.0
197 8.0
198 8.0
199 8.0
200 8.0
201 8.0
202 8.0
203 8.0
204 8.0
205 8.0
206 8.0
207 8.0
208 8.0
209 8.0
210 8.0
211 8.0
212 8.0
213 8.0
214 8.0
215 8.0
216 8.0
217 8.0
218 8.0
219 8.0
220 8.0
221 8.0
222 8.0
223 8.0
224 8.0
225 8.0
226 8.0
227 8.0
228 8.0
229 8.0
230 8.0
231 8.0
232 8.0
233 8.0
234 8.0
235 8.0
236 8.0
237 8.0
238 8.0
239 8.0
240 8.0
241 8.0
242 8.0
243 8.0
244 8.0
245 8.0
246 8.0
247 8.0
248 2.0
249 2.0
250 2.0
251 2.0
252 2.0
253 2.0
254 2.0
255 2.0
256 4.0
257 4.0
258 4.0
259 4.0
260 4.0
261 4.0
262 4.0
263 4.0
264 4.0
265 4.0
266 4.0
267 4.0
268 4.0
269 4.0
270 2.0
271 2.0
272 2.0
273 2.0
274 3.0
275 3.0
276 3.0
277 3.0
278 8.0
279 8.0
280 8.0
281 8.0
282 8.0
283 8.0
284 8.0
285 8.0
286 8.0
287 8.0
288 8.0
289 8.0
290 4.0
291 4.0
292 4.0
293 4.0
294 4.0
295 4.0
296 4.0
297 4.0
298 2.0
299 2.0
300 2.0
301 2.0
302 4.0
303 4.0
304 4.0
305 4.0
306 2.0
307 2.0
308 2.0
309 2.0
310 8.0
311 8.0
312 8.0
313 8.0
314 8.0
315 8.0
316 8.0
317 16.0
318 16.0
319 16.0
320 8.0
321 8.0
322 4.0
323 6.0
324 6.0
325 6.0
326 6.0
327 7.0
328 7.0
329 7.0
330 7.0
331 7.0
332 7.0
333 7.0
334 8.0
335 8.0
336 8.0
337 8.0
338 6.0
339 6.0
340 8.0
341 4.0
342 4.0
343 4.0
344 4.0
345 4.0
346 4.0
347 4.0
348 4.0
349 4.0
350 4.0
351 4.0
352 4.0
353 4.0
354 4.0
355 4.0
356 4.0
357 2.0
358 2.0
359 2.0
360 2.0
361 4.0
362 4.0
363 4.0
364 4.0
365 8.0
366 8.0
367 8.0
368 8.0
369 2.0
370 2.0
371 2.0
372 2.0
373 4.0
374 4.0
375 4.0
376 4.0
377 4.0
378 4.0
379 4.0
380 4.0
381 4.0
382 4.0
383 4.0
384 4.0
385 4.0
386 4.0
387 4.0
388 4.0
389 4.0
390 4.0
391 4.0
392 4.0
393 2.0
394 2.0
395 2.0
396 2.0
397 4.0
398 4.0
399 4.0
400 4.0
401 8.0
402 8.0
403 8.0
404 8.0
405 2.0
406 2.0
407 2.0
408 2.0
409 4.0
410 4.0
411 4.0
412 4.0
413 2.0
414 2.0
415 4.0
416 4.0
417 8.0
418 8.0
419 2.0
420 2.0
421 4.0
422 4.0
423 4.0
424 4.0
425 2.0
426 2.0
427 4.0
428 4.0
429 8.0
430 8.0
431 2.0
432 2.0
433 4.0
434 4.0
435 2.0
436 2.0
437 4.0
438 4.0
439 8.0
440 8.0
441 2.0
442 2.0
443 16.0
444 8.0
445 8.0
446 8.0
447 8.0
448 8.0
449 8.0
450 8.0
451 8.0
452 8.0
453 8.0
454 2.0
455 2.0
456 2.0
457 2.0
458 4.0
459 4.0
460 4.0
461 4.0
462 4.0
463 4.0
464 4.0
465 4.0
466 8.0
467 8.0
468 8.0
469 8.0
470 8.0
471 8.0
472 8.0
473 4.0
474 8.0
475 8.0
476 8.0
477 8.0
478 8.0
479 8.0
480 8.0
481 8.0
482 8.0
483 8.0
484 8.0
485 8.0
486 8.0
487 8.0
488 8.0
489 8.0
490 8.0
491 8.0
492 8.0
493 8.0
494 8.0
495 8.0
496 8.0
497 8.0
498 8.0
499 8.0
500 8.0
501 8.0
502 8.0
503 8.0
504 8.0
505 8.0
506 8.0
507 8.0
508 8.0
509 8.0
510 8.0
511 8.0
512 8.0
513 8.0
514 8.0
515 8.0
516 8.0
517 8.0
518 8.0
519 8.0
520 8.0
521 8.0
522 8.0
523 8.0
524 8.0
525 8.0
526 8.0
527 8.0
528 8.0
529 8.0
530 8.0
531 8.0
532 8.0
533 8.0
534 8.0
535 8.0
536 8.0
537 8.0
538 8.0
539 8.0
540 8.0
541 8.0
542 8.0
543 8.0
544 8.0
545 8.0
546 8.0
547 8.0
548 8.0
549 8.0
550 8.0
551 8.0
552 4.0
553 4.0
554 4.0
555 4.0
556 4.0
557 4.0
558 4.0
559 4.0
560 4.0
561 4.0
562 4.0
563 4.0
564 4.0
565 4.0
566 4.0
567 4.0
568 4.0
569 4.0
570 4.0
571 4.0
572 4.0
573 4.0
574 4.0
575 4.0
576 4.0
577 4.0
578 4.0
579 4.0
580 4.0
581 4.0
582 4.0
583 4.0
584 4.0
585 4.0
586 4.0
587 4.0
588 4.0
589 4.0
590 4.0
591 8.0
592 8.0
593 8.0
594 8.0
595 8.0
596 8.0
597 8.0
598 8.0
599 0.0
600 0.0
601 0.0
602 4.0
603 4.0
604 4.0
605 4.0
606 4.0
607 4.0
608 4.0
609 4.0
610 4.0
611 4.0
612 4.0
613 4.0
614 4.0
615 4.0
616 4.0
617 4.0
618 4.0
619 4.0
620 4.0
621 4.0
622 4.0
623 4.0
624 4.0
625 4.0
626 4.0
627 4.0
628 4.0
629 8.0
630 8.0
631 8.0
632 8.0
633 8.0
634 8.0
635 8.0
636 8.0
637 8.0
638 8.0
639 8.0
640 8.0
641 8.0
642 8.0
643 8.0
644 8.0
645 8.0
646 8.0
647 8.0
648 8.0
649 8.0
650 8.0
651 8.0
652 8.0
653 8.0
654 8.0
655 8.0
656 8.0
657 8.0
658 8.0
659 8.0
660 8.0
661 8.0
662 8.0
663 8.0
664 8.0
665 8.0
666 8.0
667 8.0
668 8.0
669 8.0
670 8.0
671 8.0
672 8.0
673 8.0
674 8.0
675 8.0
676 8.0
677 8.0
678 8.0
679 8.0
680 8.0
681 8.0
682 8.0
683 8.0
684 8.0
685 8.0
686 8.0
687 8.0
688 8.0
689 8.0
690 8.0
691 8.0
692 8.0
693 8.0
694 8.0
695 8.0
696 8.0
697 8.0
698 8.0
699 8.0
700 8.0
701 8.0
702 8.0
703 8.0
704 8.0
705 8.0
706 8.0
707 8.0
708 8.0
709 8.0
710 8.0
711 8.0
712 8.0
713 8.0
714 8.0
715 8.0
716 8.0
717 8.0
718 8.0
719 8.0
720 8.0
721 8.0
722 8.0
723 8.0
724 8.0
725 8.0
726 8.0
727 8.0
728 8.0
729 8.0
730 8.0
731 2.0
732 2.0
733 2.0
734 2.0
735 2.0
736 2.0
737 2.0
738 2.0
739 2.0
740 2.0
741 2.0
742 2.0
743 2.0
744 2.0
745 2.0
746 4.0
747 4.0
748 4.0
749 4.0
750 4.0
751 4.0
752 4.0
753 4.0
754 4.0
755 4.0
756 4.0
757 4.0
758 4.0
759 4.0
760 4.0
761 4.0
762 4.0
763 4.0
764 4.0
765 4.0
766 4.0
767 2.0
768 2.0
769 2.0
770 2.0
771 2.0
772 2.0
773 4.0
774 4.0
775 4.0
776 4.0
777 4.0
778 4.0
779 8.0
780 8.0
781 8.0
782 8.0
783 8.0
784 8.0
785 8.0
786 8.0
787 8.0
788 8.0
789 8.0
790 8.0
791 8.0
792 8.0
793 8.0
794 8.0
795 8.0
796 8.0
797 4.0
798 4.0
799 4.0
800 4.0
801 4.0
802 4.0
803 4.0
804 4.0
805 4.0
806 4.0
807 4.0
808 4.0
809 4.0
810 4.0
811 4.0
812 4.0
813 4.0
814 4.0
815 4.0
816 4.0
817 4.0
818 4.0
819 4.0
820 4.0
821 2.0
822 2.0
823 2.0
824 2.0
825 2.0
826 2.0
827 4.0
828 4.0
829 4.0
830 4.0
831 4.0
832 4.0
833 4.0
834 4.0
835 0.0
836 0.0
837 0.0
838 2.0
839 2.0
840 2.0
841 2.0
842 2.0
843 2.0
844 2.0
845 2.0
846 3.0
847 3.0
848 3.0
849 3.0
850 3.0
851 3.0
852 3.0
853 3.0
854 2.0
855 2.0
856 2.0
857 2.0
858 2.0
859 2.0
860 2.0
861 2.0
862 3.0
863 3.0
864 3.0
865 3.0
866 3.0
867 3.0
868 3.0
869 3.0
870 3.0
871 3.0
872 3.0
873 3.0
874 3.0
875 3.0
876 3.0
877 3.0
878 3.0
879 3.0
880 3.0
881 3.0
882 3.0
883 3.0
884 3.0
885 3.0
886 3.0
887 3.0
888 3.0
889 3.0
890 3.0
891 3.0
892 3.0
893 3.0
894 4.0
895 4.0
896 4.0
897 4.0
898 4.0
899 4.0
900 4.0
901 4.0
902 2.0
903 2.0
904 2.0
905 2.0
906 2.0
907 2.0
908 2.0
909 2.0
910 2.0
911 2.0
912 2.0
913 2.0
914 2.0
915 2.0
916 2.0
917 2.0
918 2.0
919 2.0
920 2.0
921 2.0
922 2.0
923 2.0
924 2.0
925 2.0
926 2.0
927 2.0
928 2.0
929 2.0
930 2.0
931 2.0
932 2.0
933 2.0
934 2.0
935 2.0
936 2.0
937 2.0
938 2.0
939 2.0
940 2.0
941 2.0
942 2.0
943 2.0
944 2.0
945 2.0
946 2.0
947 2.0
948 2.0
949 2.0
950 2.0
951 2.0
952 2.0
953 2.0
954 2.0
955 2.0
956 2.0
957 2.0
958 2.0
959 2.0
960 2.0
961 2.0
962 2.0
963 2.0
964 2.0
965 2.0
966 2.0
967 2.0
968 2.0
969 2.0
970 2.0
971 2.0
972 2.0
973 2.0
974 2.0
975 2.0
976 2.0
977 2.0
978 2.0
979 2.0
980 2.0
981 2.0
982 2.0
983 2.0
984 2.0
985 2.0
986 2.0
987 2.0
988 2.0
989 2.0
990 2.0
991 2.0
992 2.0
993 2.0
994 2.0
995 2.0
996 2.0
997 2.0
998 3.0
999 3.0
1000 3.0
1001 3.0
1002 3.0
1003 3.0
1004 3.0
1005 3.0
1006 3.0
1007 3.0
1008 3.0
1009 3.0
1010 3.0
1011 3.0
1012 3.0
1013 3.0
1014 3.0
1015 3.0
1016 3.0
1017 3.0
1018 3.0
1019 3.0
1020 3.0
1021 3.0
1022 3.0
1023 3.0
1024 3.0
1025 3.0
1026 3.0
1027 3.0
1028 3.0
1029 3.0
1030 3.0
1031 3.0
1032 3.0
1033 3.0
1034 3.0
1035 3.0
1036 3.0
1037 3.0
1038 3.0
1039 3.0
1040 3.0
1041 3.0
1042 3.0
1043 3.0
1044 3.0
1045 3.0
1046 3.0
1047 3.0
1048 3.0
1049 3.0
1050 3.0
1051 3.0
1052 3.0
1053 3.0
1054 3.0
1055 3.0
1056 3.0
1057 3.0
1058 3.0
1059 3.0
1060 3.0
1061 3.0
1062 3.0
1063 3.0
1064 3.0
1065 3.0
1066 3.0
1067 3.0
1068 3.0
1069 3.0
1070 3.0
1071 3.0
1072 3.0
1073 3.0
1074 3.0
1075 3.0
1076 3.0
1077 3.0
1078 3.0
1079 3.0
1080 3.0
1081 3.0
1082 3.0
1083 3.0
1084 3.0
1085 3.0
1086 3.0
1087 3.0
1088 3.0
1089 3.0
1090 3.0
1091 3.0
1092 3.0
1093 3.0
1094 3.0
1095 3.0
1096 3.0
1097 3.0
1098 3.0
1099 3.0
1100 3.0
1101 3.0
1102 3.0
1103 3.0
1104 3.0
1105 3.0
1106 3.0
1107 3.0
1108 3.0
1109 3.0
1110 3.0
1111 3.0
1112 3.0
1113 3.0
1114 3.0
1115 3.0
1116 3.0
1117 3.0
1118 3.0
1119 3.0
1120 3.0
1121 3.0
1122 3.0
1123 3.0
1124 3.0
1125 3.0
1126 2.0
1127 2.0
1128 2.0
1129 2.0
1130 2.0
1131 2.0
1132 2.0
1133 2.0
1134 3.0
1135 3.0
1136 3.0
1137 3.0
1138 3.0
1139 3.0
1140 3.0
1141 3.0
1142 3.0
1143 3.0
1144 3.0
1145 3.0
1146 3.0
1147 3.0
1148 3.0
1149 3.0
1150 6.0
1151 6.0
1152 6.0
1153 6.0
1154 6.0
1155 6.0
1156 6.0
1157 6.0
1158 6.0
1159 6.0
1160 6.0
1161 6.0
1162 6.0
1163 6.0
1164 6.0
1165 6.0
1166 6.0
1167 6.0
1168 6.0
1169 6.0
1170 6.0
1171 6.0
1172 6.0
1173 6.0
1174 1.0
1175 1.0
1176 1.0
1177 1.0
1178 1.0
1179 1.0
1180 1.0
1181 1.0
1182 1.0
1183 1.0
1184 1.0
1185 1.0
1186 1.0
1187 1.0
1188 1.0
1189 1.0
1190 2.0
1191 2.0
1192 2.0
1193 2.0
1194 2.0
1195 2.0
1196 2.0
1197 2.0
1198 2.0
1199 2.0
1200 2.0
1201 2.0
1202 2.0
1203 2.0
1204 2.0
1205 2.0
1206 4.0
1207 4.0
1208 4.0
1209 4.0
1210 4.0
1211 4.0
1212 4.0
1213 4.0
1214 4.0
1215 4.0
1216 4.0
1217 4.0
1218 4.0
1219 4.0
1220 4.0
1221 4.0
1222 4.0
1223 4.0
1224 4.0
1225 4.0
1226 4.0
1227 4.0
1228 4.0
1229 4.0
1230 0.0
1231 0.0
1232 0.0
1233 0.0
1234 0.0
1235 0.0
1236 0.0
1237 0.0
1238 0.0
1239 2.0
1240 2.0
1241 3.0
1242 3.0
1243 2.0
1244 2.0
1245 3.0
1246 3.0
1247 3.0
1248 3.0
1249 4.0
1250 4.0
1251 3.0
1252 3.0
1253 3.0
1254 3.0
1255 3.0
1256 3.0
1257 3.0
1258 3.0
1259 3.0
1260 3.0
1261 3.0
1262 3.0
1263 3.0
1264 3.0
1265 3.0
1266 3.0
1267 3.0
1268 3.0
1269 3.0
1270 3.0
1271 3.0
1272 3.0
1273 3.0
1274 2.0
1275 2.0
1276 3.0
1277 3.0
1278 3.0
1279 3.0
1280 6.0
1281 6.0
1282 6.0
1283 6.0
1284 1.0
1285 1.0
1286 1.0
1287 1.0
1288 1.0
1289 1.0
1290 2.0
1291 2.0
1292 4.0
1293 4.0
1294 4.0
1295 4.0
1296 4.0
1297 4.0
1298 0.0
1299 0.0
1300 0.0
1301 4.0
1302 4.0
1303 4.0
1304 4.0
1305 4.0
1306 4.0
1307 4.0
1308 4.0
1309 4.0
1310 4.0
1311 4.0
1312 4.0
1313 4.0
1314 4.0
1315 4.0
1316 4.0
1317 4.0
1318 4.0
1319 8.0
1320 8.0
1321 8.0
1322 8.0
1323 8.0
1324 8.0
1325 8.0
1326 8.0
1327 8.0
1328 8.0
1329 8.0
1330 8.0
1331 8.0
1332 8.0
1333 8.0
1334 8.0
1335 8.0
1336 8.0
1337 8.0
1338 8.0
1339 8.0
1340 8.0
1341 8.0
1342 8.0
1343 8.0
1344 8.0
1345 8.0
1346 8.0
1347 8.0
1348 8.0
1349 8.0
1350 8.0
1351 8.0
1352 8.0
1353 8.0
1354 8.0
1355 8.0
1356 8.0
1357 8.0
1358 8.0
1359 8.0
1360 8.0
1361 8.0
1362 8.0
1363 8.0
1364 8.0
1365 8.0
1366 8.0
1367 8.0
1368 8.0
1369 8.0
1370 8.0
1371 8.0
1372 8.0
1373 8.0
1374 8.0
1375 2.0
1376 2.0
1377 2.0
1378 2.0
1379 2.0
1380 2.0
1381 2.0
1382 2.0
1383 4.0
1384 4.0
1385 4.0
1386 4.0
1387 4.0
1388 4.0
1389 4.0
1390 4.0
1391 4.0
1392 4.0
1393 4.0
1394 4.0
1395 4.0
1396 4.0
1397 4.0
1398 4.0
1399 4.0
1400 4.0
1401 2.0
1402 2.0
1403 2.0
1404 2.0
1405 3.0
1406 3.0
1407 3.0
1408 3.0
1409 8.0
1410 8.0
1411 8.0
1412 8.0
1413 8.0
1414 8.0
1415 8.0
1416 8.0
1417 8.0
1418 8.0
1419 8.0
1420 8.0
1421 4.0
1422 4.0
1423 4.0
1424 4.0
1425 2.0
1426 2.0
1427 2.0
1428 2.0
1429 4.0
1430 4.0
1431 4.0
1432 4.0
1433 2.0
1434 2.0
1435 2.0
1436 2.0
1437 4.0
1438 4.0
1439 4.0
1440 4.0
1441 4.0
1442 4.0
1443 4.0
1444 4.0
1445 4.0
1446 4.0
1447 4.0
1448 4.0
1449 2.0
1450 2.0
1451 2.0
1452 2.0
1453 8.0
1454 8.0
1455 8.0
1456 8.0
1457 0.0
1458 0.0
1459 4.0
1460 4.0
1461 4.0
1462 4.0
1463 4.0
1464 4.0
1465 4.0
1466 4.0
1467 4.0
1468 4.0
1469 4.0
1470 4.0
1471 4.0
1472 4.0
1473 4.0
1474 4.0
1475 4.0
1476 4.0
1477 4.0
1478 4.0
1479 4.0
1480 4.0
1481 4.0
1482 4.0
1483 4.0
1484 4.0
1485 4.0
1486 4.0
1487 4.0
1488 4.0
1489 4.0
1490 4.0
1491 4.0
1492 4.0
1493 4.0
1494 4.0
1495 4.0
1496 4.0
1497 4.0
1498 4.0
1499 2.0
1500 2.0
1501 2.0
1502 2.0
1503 2.0
1504 2.0
1505 2.0
1506 2.0
1507 4.0
1508 4.0
1509 4.0
1510 4.0
1511 4.0
1512 4.0
1513 4.0
1514 4.0
1515 8.0
1516 8.0
1517 8.0
1518 8.0
1519 8.0
1520 8.0
1521 8.0
1522 8.0
1523 2.0
1524 2.0
1525 2.0
1526 2.0
1527 2.0
1528 2.0
1529 2.0
1530 2.0
1531 4.0
1532 4.0
1533 4.0
1534 4.0
1535 4.0
1536 4.0
1537 4.0
1538 4.0
1539 2.0
1540 2.0
1541 2.0
1542 2.0
1543 4.0
1544 4.0
1545 8.0
1546 8.0
1547 4.0
1548 4.0
1549 2.0
1550 2.0
1551 8.0
1552 8.0
1553 8.0
1554 4.0
1555 16.0
1556 4.0
1557 4.0
1558 4.0
1559 4.0
1560 4.0
1561 4.0
1562 4.0
1563 4.0
1564 2.0
1565 2.0
1566 2.0
1567 2.0
1568 4.0
1569 4.0
1570 8.0
1571 8.0
1572 4.0
1573 4.0
1574 2.0
1575 2.0
1576 8.0
1577 8.0
1578 8.0
1579 4.0
1580 16.0
1581 4.0
1582 6.0
1583 16.0
1584 8.0
1585 2.0
1586 2.0
1587 3.0
1588 5.0
1589 4.0
1590 4.0
1591 8.0
1592 6.0
1593 6.0
1594 6.0
1595 6.0
1596 6.0
1597 5.0
1598 5.0
1599 8.0
1600 8.0
1601 8.0
1602 8.0
1603 8.0
1604 8.0
1605 8.0
1606 8.0
1607 4.0
1608 4.0
1609 12.0
1610 12.0
1611 12.0
1612 12.0
1613 8.0
1614 8.0
1615 8.0
1616 8.0
1617 8.0
1618 4.0
1619 4.0
1620 9.0
1621 9.0
1622 9.0
1623 9.0
1624 4.0
1625 4.0
1626 8.0
1627 8.0
1628 8.0;
param: MEM := 
0 24.0
1 24.0
2 24.0
3 24.0
4 24.0
5 24.0
6 24.0
7 24.0
8 24.0
9 24.0
10 24.0
11 24.0
12 24.0
13 24.0
14 24.0
15 24.0
16 24.0
17 24.0
18 24.0
19 24.0
20 24.0
21 24.0
22 24.0
23 24.0
24 24.0
25 24.0
26 24.0
27 24.0
28 24.0
29 24.0
30 24.0
31 24.0
32 24.0
33 24.0
34 24.0
35 24.0
36 24.0
37 24.0
38 24.0
39 24.0
40 24.0
41 24.0
42 24.0
43 24.0
44 24.0
45 24.0
46 24.0
47 24.0
48 24.0
49 24.0
50 24.0
51 24.0
52 24.0
53 24.0
54 24.0
55 24.0
56 24.0
57 24.0
58 24.0
59 24.0
60 24.0
61 24.0
62 24.0
63 24.0
64 24.0
65 24.0
66 24.0
67 24.0
68 24.0
69 24.0
70 24.0
71 24.0
72 24.0
73 24.0
74 24.0
75 24.0
76 24.0
77 24.0
78 24.0
79 24.0
80 24.0
81 24.0
82 24.0
83 24.0
84 24.0
85 24.0
86 24.0
87 24.0
88 24.0
89 24.0
90 24.0
91 24.0
92 24.0
93 24.0
94 24.0
95 24.0
96 24.0
97 24.0
98 24.0
99 24.0
100 24.0
101 24.0
102 24.0
103 24.0
104 24.0
105 24.0
106 24.0
107 24.0
108 24.0
109 24.0
110 24.0
111 24.0
112 24.0
113 24.0
114 24.0
115 24.0
116 24.0
117 24.0
118 24.0
119 24.0
120 32.0
121 32.0
122 32.0
123 32.0
124 32.0
125 32.0
126 16.0
127 16.0
128 16.0
129 16.0
130 16.0
131 16.0
132 16.0
133 16.0
134 16.0
135 16.0
136 16.0
137 16.0
138 16.0
139 16.0
140 16.0
141 16.0
142 16.0
143 16.0
144 16.0
145 16.0
146 24.0
147 24.0
148 24.0
149 24.0
150 16.0
151 16.0
152 32.0
153 32.0
154 32.0
155 32.0
156 32.0
157 32.0
158 16.0
159 16.0
160 16.0
161 16.0
162 16.0
163 16.0
164 16.0
165 16.0
166 16.0
167 16.0
168 16.0
169 16.0
170 8.0
171 8.0
172 8.0
173 8.0
174 8.0
175 8.0
176 18.0
177 18.0
178 18.0
179 18.0
180 18.0
181 18.0
182 18.0
183 18.0
184 18.0
185 18.0
186 18.0
187 18.0
188 18.0
189 18.0
190 18.0
191 18.0
192 18.0
193 18.0
194 18.0
195 18.0
196 18.0
197 18.0
198 18.0
199 18.0
200 18.0
201 18.0
202 18.0
203 18.0
204 18.0
205 18.0
206 18.0
207 18.0
208 18.0
209 18.0
210 18.0
211 18.0
212 18.0
213 18.0
214 18.0
215 18.0
216 18.0
217 18.0
218 18.0
219 18.0
220 18.0
221 18.0
222 18.0
223 18.0
224 18.0
225 18.0
226 18.0
227 18.0
228 18.0
229 18.0
230 18.0
231 18.0
232 18.0
233 18.0
234 18.0
235 18.0
236 18.0
237 18.0
238 18.0
239 18.0
240 18.0
241 18.0
242 18.0
243 18.0
244 18.0
245 18.0
246 18.0
247 18.0
248 8.0
249 8.0
250 8.0
251 8.0
252 4.0
253 4.0
254 4.0
255 4.0
256 48.0
257 48.0
258 48.0
259 48.0
260 48.0
261 48.0
262 48.0
263 48.0
264 48.0
265 48.0
266 48.0
267 48.0
268 48.0
269 48.0
270 8.0
271 8.0
272 8.0
273 8.0
274 4.0
275 4.0
276 4.0
277 4.0
278 32.0
279 32.0
280 32.0
281 32.0
282 32.0
283 32.0
284 32.0
285 32.0
286 32.0
287 32.0
288 32.0
289 32.0
290 8.0
291 8.0
292 8.0
293 8.0
294 8.0
295 8.0
296 8.0
297 8.0
298 4.0
299 4.0
300 4.0
301 4.0
302 4.0
303 4.0
304 4.0
305 4.0
306 4.0
307 4.0
308 4.0
309 4.0
310 128.0
311 128.0
312 128.0
313 128.0
314 128.0
315 128.0
316 128.0
317 32.0
318 32.0
319 32.0
320 4.0
321 8.0
322 4.0
323 8.0
324 8.0
325 8.0
326 8.0
327 6.0
328 6.0
329 6.0
330 6.0
331 6.0
332 6.0
333 6.0
334 32.0
335 32.0
336 8.0
337 8.0
338 16.0
339 16.0
340 32.0
341 24.0
342 24.0
343 24.0
344 24.0
345 24.0
346 24.0
347 24.0
348 24.0
349 24.0
350 24.0
351 24.0
352 24.0
353 24.0
354 24.0
355 24.0
356 24.0
357 8.0
358 8.0
359 8.0
360 8.0
361 16.0
362 16.0
363 16.0
364 16.0
365 32.0
366 32.0
367 32.0
368 32.0
369 8.0
370 8.0
371 8.0
372 8.0
373 24.0
374 24.0
375 24.0
376 24.0
377 24.0
378 24.0
379 24.0
380 24.0
381 24.0
382 24.0
383 24.0
384 24.0
385 24.0
386 24.0
387 24.0
388 24.0
389 24.0
390 24.0
391 24.0
392 24.0
393 8.0
394 8.0
395 8.0
396 8.0
397 16.0
398 16.0
399 16.0
400 16.0
401 32.0
402 32.0
403 32.0
404 32.0
405 8.0
406 8.0
407 8.0
408 8.0
409 24.0
410 24.0
411 24.0
412 24.0
413 8.0
414 8.0
415 16.0
416 16.0
417 32.0
418 32.0
419 8.0
420 8.0
421 24.0
422 24.0
423 24.0
424 24.0
425 8.0
426 8.0
427 16.0
428 16.0
429 32.0
430 32.0
431 8.0
432 8.0
433 24.0
434 24.0
435 8.0
436 8.0
437 16.0
438 16.0
439 32.0
440 32.0
441 8.0
442 8.0
443 32.0
444 32.0
445 32.0
446 32.0
447 32.0
448 32.0
449 32.0
450 32.0
451 32.0
452 32.0
453 32.0
454 4.0
455 4.0
456 4.0
457 4.0
458 8.0
459 8.0
460 8.0
461 8.0
462 8.0
463 8.0
464 8.0
465 8.0
466 64.0
467 64.0
468 64.0
469 64.0
470 64.0
471 64.0
472 64.0
473 16.0
474 24.0
475 24.0
476 24.0
477 24.0
478 24.0
479 24.0
480 24.0
481 24.0
482 24.0
483 24.0
484 24.0
485 24.0
486 24.0
487 24.0
488 24.0
489 24.0
490 24.0
491 24.0
492 24.0
493 24.0
494 24.0
495 24.0
496 24.0
497 24.0
498 24.0
499 24.0
500 24.0
501 24.0
502 24.0
503 24.0
504 24.0
505 24.0
506 24.0
507 24.0
508 24.0
509 24.0
510 24.0
511 24.0
512 24.0
513 24.0
514 24.0
515 24.0
516 24.0
517 24.0
518 24.0
519 24.0
520 24.0
521 24.0
522 24.0
523 24.0
524 24.0
525 24.0
526 24.0
527 24.0
528 24.0
529 24.0
530 24.0
531 24.0
532 24.0
533 24.0
534 24.0
535 24.0
536 24.0
537 24.0
538 24.0
539 24.0
540 24.0
541 24.0
542 24.0
543 24.0
544 24.0
545 24.0
546 24.0
547 24.0
548 24.0
549 24.0
550 24.0
551 24.0
552 32.0
553 32.0
554 32.0
555 32.0
556 32.0
557 32.0
558 32.0
559 32.0
560 32.0
561 16.0
562 16.0
563 16.0
564 16.0
565 16.0
566 16.0
567 16.0
568 16.0
569 16.0
570 16.0
571 16.0
572 16.0
573 16.0
574 16.0
575 16.0
576 16.0
577 16.0
578 16.0
579 16.0
580 16.0
581 16.0
582 16.0
583 16.0
584 16.0
585 16.0
586 16.0
587 16.0
588 16.0
589 16.0
590 16.0
591 24.0
592 24.0
593 24.0
594 24.0
595 24.0
596 24.0
597 16.0
598 16.0
599 0.0
600 0.0
601 0.0
602 32.0
603 32.0
604 32.0
605 32.0
606 32.0
607 32.0
608 32.0
609 32.0
610 32.0
611 16.0
612 16.0
613 16.0
614 16.0
615 16.0
616 16.0
617 16.0
618 16.0
619 16.0
620 16.0
621 16.0
622 16.0
623 16.0
624 16.0
625 16.0
626 16.0
627 16.0
628 16.0
629 18.0
630 18.0
631 18.0
632 18.0
633 18.0
634 18.0
635 18.0
636 18.0
637 18.0
638 18.0
639 18.0
640 18.0
641 18.0
642 18.0
643 18.0
644 18.0
645 18.0
646 18.0
647 18.0
648 18.0
649 18.0
650 18.0
651 18.0
652 18.0
653 18.0
654 18.0
655 18.0
656 18.0
657 18.0
658 18.0
659 18.0
660 18.0
661 18.0
662 18.0
663 18.0
664 18.0
665 18.0
666 18.0
667 18.0
668 18.0
669 18.0
670 18.0
671 18.0
672 18.0
673 18.0
674 18.0
675 18.0
676 18.0
677 18.0
678 18.0
679 18.0
680 18.0
681 18.0
682 18.0
683 18.0
684 18.0
685 18.0
686 18.0
687 18.0
688 18.0
689 18.0
690 18.0
691 18.0
692 18.0
693 18.0
694 18.0
695 18.0
696 18.0
697 18.0
698 18.0
699 18.0
700 18.0
701 18.0
702 18.0
703 18.0
704 18.0
705 18.0
706 18.0
707 18.0
708 18.0
709 18.0
710 18.0
711 18.0
712 18.0
713 18.0
714 18.0
715 18.0
716 18.0
717 18.0
718 18.0
719 18.0
720 18.0
721 18.0
722 18.0
723 18.0
724 18.0
725 18.0
726 18.0
727 18.0
728 18.0
729 18.0
730 18.0
731 8.0
732 8.0
733 8.0
734 8.0
735 8.0
736 8.0
737 8.0
738 8.0
739 8.0
740 4.0
741 4.0
742 4.0
743 4.0
744 4.0
745 4.0
746 48.0
747 48.0
748 48.0
749 48.0
750 48.0
751 48.0
752 48.0
753 48.0
754 48.0
755 48.0
756 48.0
757 48.0
758 48.0
759 48.0
760 48.0
761 48.0
762 48.0
763 48.0
764 48.0
765 48.0
766 48.0
767 8.0
768 8.0
769 8.0
770 8.0
771 8.0
772 8.0
773 4.0
774 4.0
775 4.0
776 4.0
777 4.0
778 4.0
779 32.0
780 32.0
781 32.0
782 32.0
783 32.0
784 32.0
785 32.0
786 32.0
787 32.0
788 32.0
789 32.0
790 32.0
791 32.0
792 32.0
793 32.0
794 32.0
795 32.0
796 32.0
797 8.0
798 8.0
799 8.0
800 8.0
801 8.0
802 8.0
803 4.0
804 4.0
805 4.0
806 4.0
807 4.0
808 4.0
809 4.0
810 4.0
811 4.0
812 4.0
813 4.0
814 4.0
815 4.0
816 4.0
817 4.0
818 4.0
819 4.0
820 4.0
821 4.0
822 4.0
823 4.0
824 4.0
825 4.0
826 4.0
827 8.0
828 8.0
829 8.0
830 8.0
831 8.0
832 8.0
833 8.0
834 8.0
835 0.0
836 0.0
837 0.0
838 2.0
839 2.0
840 2.0
841 2.0
842 2.0
843 2.0
844 2.0
845 2.0
846 2.0
847 2.0
848 2.0
849 2.0
850 2.0
851 2.0
852 2.0
853 2.0
854 2.0
855 2.0
856 2.0
857 2.0
858 2.0
859 2.0
860 2.0
861 2.0
862 2.0
863 2.0
864 2.0
865 2.0
866 2.0
867 2.0
868 2.0
869 2.0
870 2.0
871 2.0
872 2.0
873 2.0
874 2.0
875 2.0
876 2.0
877 2.0
878 2.0
879 2.0
880 2.0
881 2.0
882 2.0
883 2.0
884 2.0
885 2.0
886 2.0
887 2.0
888 2.0
889 2.0
890 2.0
891 2.0
892 2.0
893 2.0
894 1.0
895 1.0
896 1.0
897 1.0
898 1.0
899 1.0
900 1.0
901 1.0
902 3.0
903 3.0
904 3.0
905 3.0
906 3.0
907 3.0
908 3.0
909 3.0
910 3.0
911 3.0
912 3.0
913 3.0
914 3.0
915 3.0
916 3.0
917 3.0
918 3.0
919 3.0
920 3.0
921 3.0
922 3.0
923 3.0
924 3.0
925 3.0
926 3.0
927 3.0
928 3.0
929 3.0
930 3.0
931 3.0
932 3.0
933 3.0
934 3.0
935 3.0
936 3.0
937 3.0
938 3.0
939 3.0
940 3.0
941 3.0
942 3.0
943 3.0
944 3.0
945 3.0
946 3.0
947 3.0
948 3.0
949 3.0
950 3.0
951 3.0
952 3.0
953 3.0
954 3.0
955 3.0
956 3.0
957 3.0
958 3.0
959 3.0
960 3.0
961 3.0
962 3.0
963 3.0
964 3.0
965 3.0
966 3.0
967 3.0
968 3.0
969 3.0
970 3.0
971 3.0
972 3.0
973 3.0
974 3.0
975 3.0
976 3.0
977 3.0
978 3.0
979 3.0
980 3.0
981 3.0
982 3.0
983 3.0
984 3.0
985 3.0
986 3.0
987 3.0
988 3.0
989 3.0
990 3.0
991 3.0
992 3.0
993 3.0
994 3.0
995 3.0
996 3.0
997 3.0
998 3.0
999 3.0
1000 3.0
1001 3.0
1002 3.0
1003 3.0
1004 3.0
1005 3.0
1006 3.0
1007 3.0
1008 3.0
1009 3.0
1010 3.0
1011 3.0
1012 3.0
1013 3.0
1014 3.0
1015 3.0
1016 3.0
1017 3.0
1018 3.0
1019 3.0
1020 3.0
1021 3.0
1022 3.0
1023 3.0
1024 3.0
1025 3.0
1026 3.0
1027 3.0
1028 3.0
1029 3.0
1030 3.0
1031 3.0
1032 3.0
1033 3.0
1034 3.0
1035 3.0
1036 3.0
1037 3.0
1038 3.0
1039 3.0
1040 3.0
1041 3.0
1042 3.0
1043 3.0
1044 3.0
1045 3.0
1046 3.0
1047 3.0
1048 3.0
1049 3.0
1050 3.0
1051 3.0
1052 3.0
1053 3.0
1054 3.0
1055 3.0
1056 3.0
1057 3.0
1058 3.0
1059 3.0
1060 3.0
1061 3.0
1062 3.0
1063 3.0
1064 3.0
1065 3.0
1066 3.0
1067 3.0
1068 3.0
1069 3.0
1070 3.0
1071 3.0
1072 3.0
1073 3.0
1074 3.0
1075 3.0
1076 3.0
1077 3.0
1078 3.0
1079 3.0
1080 3.0
1081 3.0
1082 3.0
1083 3.0
1084 3.0
1085 3.0
1086 3.0
1087 3.0
1088 3.0
1089 3.0
1090 3.0
1091 3.0
1092 3.0
1093 3.0
1094 3.0
1095 3.0
1096 3.0
1097 3.0
1098 3.0
1099 3.0
1100 3.0
1101 3.0
1102 3.0
1103 3.0
1104 3.0
1105 3.0
1106 3.0
1107 3.0
1108 3.0
1109 3.0
1110 3.0
1111 3.0
1112 3.0
1113 3.0
1114 3.0
1115 3.0
1116 3.0
1117 3.0
1118 3.0
1119 3.0
1120 3.0
1121 3.0
1122 3.0
1123 3.0
1124 3.0
1125 3.0
1126 1.0
1127 1.0
1128 1.0
1129 1.0
1130 1.0
1131 1.0
1132 1.0
1133 1.0
1134 2.0
1135 2.0
1136 2.0
1137 2.0
1138 2.0
1139 2.0
1140 2.0
1141 2.0
1142 2.0
1143 2.0
1144 2.0
1145 2.0
1146 2.0
1147 2.0
1148 2.0
1149 2.0
1150 2.0
1151 2.0
1152 2.0
1153 2.0
1154 2.0
1155 2.0
1156 2.0
1157 2.0
1158 2.0
1159 2.0
1160 2.0
1161 2.0
1162 2.0
1163 2.0
1164 2.0
1165 2.0
1166 2.0
1167 2.0
1168 2.0
1169 2.0
1170 2.0
1171 2.0
1172 2.0
1173 2.0
1174 0.5
1175 0.5
1176 0.5
1177 0.5
1178 0.5
1179 0.5
1180 0.5
1181 0.5
1182 0.5
1183 0.5
1184 0.5
1185 0.5
1186 0.5
1187 0.5
1188 0.5
1189 0.5
1190 1.0
1191 1.0
1192 1.0
1193 1.0
1194 1.0
1195 1.0
1196 1.0
1197 1.0
1198 1.0
1199 1.0
1200 1.0
1201 1.0
1202 1.0
1203 1.0
1204 1.0
1205 1.0
1206 1.0
1207 1.0
1208 1.0
1209 1.0
1210 1.0
1211 1.0
1212 1.0
1213 1.0
1214 1.0
1215 1.0
1216 1.0
1217 1.0
1218 1.0
1219 1.0
1220 1.0
1221 1.0
1222 1.0
1223 1.0
1224 1.0
1225 1.0
1226 1.0
1227 1.0
1228 1.0
1229 1.0
1230 0.0
1231 0.0
1232 0.0
1233 0.0
1234 0.0
1235 0.0
1236 0.0
1237 0.0
1238 0.0
1239 2.0
1240 2.0
1241 2.0
1242 2.0
1243 2.0
1244 2.0
1245 2.0
1246 2.0
1247 2.0
1248 2.0
1249 1.0
1250 1.0
1251 3.0
1252 3.0
1253 3.0
1254 3.0
1255 3.0
1256 3.0
1257 3.0
1258 3.0
1259 3.0
1260 3.0
1261 3.0
1262 3.0
1263 3.0
1264 3.0
1265 3.0
1266 3.0
1267 3.0
1268 3.0
1269 3.0
1270 3.0
1271 3.0
1272 3.0
1273 3.0
1274 1.0
1275 1.0
1276 2.0
1277 2.0
1278 2.0
1279 2.0
1280 2.0
1281 2.0
1282 2.0
1283 2.0
1284 0.5
1285 0.5
1286 0.5
1287 0.5
1288 0.5
1289 0.5
1290 1.0
1291 1.0
1292 1.0
1293 1.0
1294 1.0
1295 1.0
1296 1.0
1297 1.0
1298 0.0
1299 0.0
1300 0.0
1301 32.0
1302 32.0
1303 32.0
1304 32.0
1305 32.0
1306 32.0
1307 16.0
1308 16.0
1309 16.0
1310 16.0
1311 16.0
1312 16.0
1313 16.0
1314 16.0
1315 16.0
1316 16.0
1317 16.0
1318 16.0
1319 18.0
1320 18.0
1321 18.0
1322 18.0
1323 18.0
1324 18.0
1325 18.0
1326 18.0
1327 18.0
1328 18.0
1329 18.0
1330 18.0
1331 18.0
1332 18.0
1333 18.0
1334 18.0
1335 18.0
1336 18.0
1337 18.0
1338 18.0
1339 18.0
1340 18.0
1341 18.0
1342 18.0
1343 18.0
1344 18.0
1345 18.0
1346 18.0
1347 18.0
1348 18.0
1349 18.0
1350 18.0
1351 18.0
1352 18.0
1353 18.0
1354 18.0
1355 18.0
1356 18.0
1357 18.0
1358 18.0
1359 18.0
1360 18.0
1361 18.0
1362 18.0
1363 18.0
1364 18.0
1365 18.0
1366 18.0
1367 18.0
1368 18.0
1369 18.0
1370 18.0
1371 18.0
1372 18.0
1373 18.0
1374 18.0
1375 8.0
1376 8.0
1377 8.0
1378 8.0
1379 4.0
1380 4.0
1381 4.0
1382 4.0
1383 48.0
1384 48.0
1385 48.0
1386 48.0
1387 48.0
1388 48.0
1389 48.0
1390 48.0
1391 48.0
1392 48.0
1393 48.0
1394 48.0
1395 48.0
1396 48.0
1397 48.0
1398 48.0
1399 48.0
1400 48.0
1401 8.0
1402 8.0
1403 8.0
1404 8.0
1405 4.0
1406 4.0
1407 4.0
1408 4.0
1409 32.0
1410 32.0
1411 32.0
1412 32.0
1413 32.0
1414 32.0
1415 32.0
1416 32.0
1417 32.0
1418 32.0
1419 32.0
1420 32.0
1421 8.0
1422 8.0
1423 8.0
1424 8.0
1425 4.0
1426 4.0
1427 4.0
1428 4.0
1429 4.0
1430 4.0
1431 4.0
1432 4.0
1433 4.0
1434 4.0
1435 4.0
1436 4.0
1437 4.0
1438 4.0
1439 4.0
1440 4.0
1441 8.0
1442 8.0
1443 8.0
1444 8.0
1445 8.0
1446 8.0
1447 8.0
1448 8.0
1449 8.0
1450 8.0
1451 8.0
1452 8.0
1453 32.0
1454 32.0
1455 32.0
1456 32.0
1457 0.0
1458 0.0
1459 24.0
1460 24.0
1461 24.0
1462 24.0
1463 24.0
1464 24.0
1465 24.0
1466 24.0
1467 24.0
1468 24.0
1469 24.0
1470 24.0
1471 24.0
1472 24.0
1473 24.0
1474 24.0
1475 24.0
1476 24.0
1477 24.0
1478 24.0
1479 24.0
1480 24.0
1481 24.0
1482 24.0
1483 24.0
1484 24.0
1485 24.0
1486 24.0
1487 24.0
1488 24.0
1489 24.0
1490 24.0
1491 24.0
1492 24.0
1493 24.0
1494 24.0
1495 24.0
1496 24.0
1497 24.0
1498 24.0
1499 8.0
1500 8.0
1501 8.0
1502 8.0
1503 8.0
1504 8.0
1505 8.0
1506 8.0
1507 16.0
1508 16.0
1509 16.0
1510 16.0
1511 16.0
1512 16.0
1513 16.0
1514 16.0
1515 32.0
1516 32.0
1517 32.0
1518 32.0
1519 32.0
1520 32.0
1521 32.0
1522 32.0
1523 8.0
1524 8.0
1525 8.0
1526 8.0
1527 8.0
1528 8.0
1529 8.0
1530 8.0
1531 24.0
1532 24.0
1533 24.0
1534 24.0
1535 24.0
1536 24.0
1537 24.0
1538 24.0
1539 8.0
1540 8.0
1541 8.0
1542 8.0
1543 16.0
1544 16.0
1545 32.0
1546 32.0
1547 16.0
1548 16.0
1549 8.0
1550 8.0
1551 128.0
1552 4.0
1553 8.0
1554 4.0
1555 32.0
1556 24.0
1557 24.0
1558 24.0
1559 24.0
1560 24.0
1561 24.0
1562 24.0
1563 24.0
1564 8.0
1565 8.0
1566 8.0
1567 8.0
1568 16.0
1569 16.0
1570 32.0
1571 32.0
1572 16.0
1573 16.0
1574 8.0
1575 8.0
1576 128.0
1577 4.0
1578 8.0
1579 4.0
1580 32.0
1581 4.0
1582 6.0
1583 90.0
1584 6.0
1585 2.0
1586 5.0
1587 6.0
1588 7.0
1589 6.0
1590 4.0
1591 10.0
1592 6.0
1593 7.0
1594 7.0
1595 13.0
1596 13.0
1597 7.0
1598 7.0
1599 34.0
1600 34.0
1601 34.0
1602 34.0
1603 34.0
1604 34.0
1605 34.0
1606 34.0
1607 4.0
1608 4.0
1609 134.0
1610 134.0
1611 134.0
1612 134.0
1613 32.0
1614 32.0
1615 32.0
1616 32.0
1617 32.0
1618 14.0
1619 14.0
1620 12.0
1621 12.0
1622 12.0
1623 12.0
1624 4.0
1625 4.0
1626 32.0
1627 32.0
1628 32.0;
param: NET := 
0 0.0
1 0.0
2 0.0
3 0.0
4 0.0
5 0.0
6 0.0
7 0.0
8 0.0
9 0.0
10 0.0
11 0.0
12 0.0
13 0.0
14 0.0
15 0.0
16 0.0
17 0.0
18 0.0
19 0.0
20 0.0
21 0.0
22 0.0
23 0.0
24 0.0
25 0.0
26 0.0
27 0.0
28 0.0
29 0.0
30 0.0
31 0.0
32 0.0
33 0.0
34 0.0
35 0.0
36 0.0
37 0.0
38 0.0
39 0.0
40 0.0
41 0.0
42 0.0
43 0.0
44 0.0
45 0.0
46 0.0
47 0.0
48 0.0
49 0.0
50 0.0
51 0.0
52 0.0
53 0.0
54 0.0
55 0.0
56 0.0
57 0.0
58 0.0
59 0.0
60 0.0
61 0.0
62 0.0
63 0.0
64 0.0
65 0.0
66 0.0
67 0.0
68 0.0
69 0.0
70 0.0
71 0.0
72 0.0
73 0.0
74 0.0
75 0.0
76 0.0
77 0.0
78 0.0
79 0.0
80 0.0
81 0.0
82 0.0
83 0.0
84 0.0
85 0.0
86 0.0
87 0.0
88 0.0
89 0.0
90 0.0
91 0.0
92 0.0
93 0.0
94 0.0
95 0.0
96 0.0
97 0.0
98 0.0
99 0.0
100 0.0
101 0.0
102 0.0
103 0.0
104 0.0
105 0.0
106 0.0
107 0.0
108 0.0
109 0.0
110 0.0
111 0.0
112 0.0
113 0.0
114 0.0
115 0.0
116 0.0
117 0.0
118 0.0
119 0.0
120 0.0
121 0.0
122 0.0
123 0.0
124 0.0
125 0.0
126 0.0
127 0.0
128 0.0
129 0.0
130 0.0
131 0.0
132 0.0
133 0.0
134 0.0
135 0.0
136 0.0
137 0.0
138 0.0
139 0.0
140 0.0
141 0.0
142 0.0
143 0.0
144 0.0
145 0.0
146 0.0
147 0.0
148 0.0
149 0.0
150 0.0
151 0.0
152 0.0
153 0.0
154 0.0
155 0.0
156 0.0
157 0.0
158 0.0
159 0.0
160 0.0
161 0.0
162 0.0
163 0.0
164 0.0
165 0.0
166 0.0
167 0.0
168 0.0
169 0.0
170 0.0
171 0.0
172 0.0
173 0.0
174 0.0
175 0.0
176 0.0
177 0.0
178 0.0
179 0.0
180 0.0
181 0.0
182 0.0
183 0.0
184 0.0
185 0.0
186 0.0
187 0.0
188 0.0
189 0.0
190 0.0
191 0.0
192 0.0
193 0.0
194 0.0
195 0.0
196 0.0
197 0.0
198 0.0
199 0.0
200 0.0
201 0.0
202 0.0
203 0.0
204 0.0
205 0.0
206 0.0
207 0.0
208 0.0
209 0.0
210 0.0
211 0.0
212 0.0
213 0.0
214 0.0
215 0.0
216 0.0
217 0.0
218 0.0
219 0.0
220 0.0
221 0.0
222 0.0
223 0.0
224 0.0
225 0.0
226 0.0
227 0.0
228 0.0
229 0.0
230 0.0
231 0.0
232 0.0
233 0.0
234 0.0
235 0.0
236 0.0
237 0.0
238 0.0
239 0.0
240 0.0
241 0.0
242 0.0
243 0.0
244 0.0
245 0.0
246 0.0
247 0.0
248 0.0
249 0.0
250 0.0
251 0.0
252 0.0
253 0.0
254 0.0
255 0.0
256 0.0
257 0.0
258 0.0
259 0.0
260 0.0
261 0.0
262 0.0
263 0.0
264 0.0
265 0.0
266 0.0
267 0.0
268 0.0
269 0.0
270 0.0
271 0.0
272 0.0
273 0.0
274 0.0
275 0.0
276 0.0
277 0.0
278 0.0
279 0.0
280 0.0
281 0.0
282 0.0
283 0.0
284 0.0
285 0.0
286 0.0
287 0.0
288 0.0
289 0.0
290 0.0
291 0.0
292 0.0
293 0.0
294 0.0
295 0.0
296 0.0
297 0.0
298 0.0
299 0.0
300 0.0
301 0.0
302 0.0
303 0.0
304 0.0
305 0.0
306 0.0
307 0.0
308 0.0
309 0.0
310 0.0
311 0.0
312 0.0
313 0.0
314 0.0
315 0.0
316 0.0
317 0.0
318 0.0
319 0.0
320 0.0
321 0.0
322 0.0
323 0.0
324 0.0
325 0.0
326 0.0
327 0.0
328 0.0
329 0.0
330 0.0
331 0.0
332 0.0
333 0.0
334 0.0
335 0.0
336 0.0
337 0.0
338 0.0
339 0.0
340 0.0
341 0.0
342 0.0
343 0.0
344 0.0
345 0.0
346 0.0
347 0.0
348 0.0
349 0.0
350 0.0
351 0.0
352 0.0
353 0.0
354 0.0
355 0.0
356 0.0
357 0.0
358 0.0
359 0.0
360 0.0
361 0.0
362 0.0
363 0.0
364 0.0
365 0.0
366 0.0
367 0.0
368 0.0
369 0.0
370 0.0
371 0.0
372 0.0
373 0.0
374 0.0
375 0.0
376 0.0
377 0.0
378 0.0
379 0.0
380 0.0
381 0.0
382 0.0
383 0.0
384 0.0
385 0.0
386 0.0
387 0.0
388 0.0
389 0.0
390 0.0
391 0.0
392 0.0
393 0.0
394 0.0
395 0.0
396 0.0
397 0.0
398 0.0
399 0.0
400 0.0
401 0.0
402 0.0
403 0.0
404 0.0
405 0.0
406 0.0
407 0.0
408 0.0
409 0.0
410 0.0
411 0.0
412 0.0
413 0.0
414 0.0
415 0.0
416 0.0
417 0.0
418 0.0
419 0.0
420 0.0
421 0.0
422 0.0
423 0.0
424 0.0
425 0.0
426 0.0
427 0.0
428 0.0
429 0.0
430 0.0
431 0.0
432 0.0
433 0.0
434 0.0
435 0.0
436 0.0
437 0.0
438 0.0
439 0.0
440 0.0
441 0.0
442 0.0
443 0.0
444 0.0
445 0.0
446 0.0
447 0.0
448 0.0
449 0.0
450 0.0
451 0.0
452 0.0
453 0.0
454 0.0
455 0.0
456 0.0
457 0.0
458 0.0
459 0.0
460 0.0
461 0.0
462 0.0
463 0.0
464 0.0
465 0.0
466 0.0
467 0.0
468 0.0
469 0.0
470 0.0
471 0.0
472 0.0
473 0.0
474 0.0
475 0.0
476 0.0
477 0.0
478 0.0
479 0.0
480 0.0
481 0.0
482 0.0
483 0.0
484 0.0
485 0.0
486 0.0
487 0.0
488 0.0
489 0.0
490 0.0
491 0.0
492 0.0
493 0.0
494 0.0
495 0.0
496 0.0
497 0.0
498 0.0
499 0.0
500 0.0
501 0.0
502 0.0
503 0.0
504 0.0
505 0.0
506 0.0
507 0.0
508 0.0
509 0.0
510 0.0
511 0.0
512 0.0
513 0.0
514 0.0
515 0.0
516 0.0
517 0.0
518 0.0
519 0.0
520 0.0
521 0.0
522 0.0
523 0.0
524 0.0
525 0.0
526 0.0
527 0.0
528 0.0
529 0.0
530 0.0
531 0.0
532 0.0
533 0.0
534 0.0
535 0.0
536 0.0
537 0.0
538 0.0
539 0.0
540 0.0
541 0.0
542 0.0
543 0.0
544 0.0
545 0.0
546 0.0
547 0.0
548 0.0
549 0.0
550 0.0
551 0.0
552 0.0
553 0.0
554 0.0
555 0.0
556 0.0
557 0.0
558 0.0
559 0.0
560 0.0
561 0.0
562 0.0
563 0.0
564 0.0
565 0.0
566 0.0
567 0.0
568 0.0
569 0.0
570 0.0
571 0.0
572 0.0
573 0.0
574 0.0
575 0.0
576 0.0
577 0.0
578 0.0
579 0.0
580 0.0
581 0.0
582 0.0
583 0.0
584 0.0
585 0.0
586 0.0
587 0.0
588 0.0
589 0.0
590 0.0
591 0.0
592 0.0
593 0.0
594 0.0
595 0.0
596 0.0
597 0.0
598 0.0
599 0.0
600 0.0
601 0.0
602 0.0
603 0.0
604 0.0
605 0.0
606 0.0
607 0.0
608 0.0
609 0.0
610 0.0
611 0.0
612 0.0
613 0.0
614 0.0
615 0.0
616 0.0
617 0.0
618 0.0
619 0.0
620 0.0
621 0.0
622 0.0
623 0.0
624 0.0
625 0.0
626 0.0
627 0.0
628 0.0
629 0.0
630 0.0
631 0.0
632 0.0
633 0.0
634 0.0
635 0.0
636 0.0
637 0.0
638 0.0
639 0.0
640 0.0
641 0.0
642 0.0
643 0.0
644 0.0
645 0.0
646 0.0
647 0.0
648 0.0
649 0.0
650 0.0
651 0.0
652 0.0
653 0.0
654 0.0
655 0.0
656 0.0
657 0.0
658 0.0
659 0.0
660 0.0
661 0.0
662 0.0
663 0.0
664 0.0
665 0.0
666 0.0
667 0.0
668 0.0
669 0.0
670 0.0
671 0.0
672 0.0
673 0.0
674 0.0
675 0.0
676 0.0
677 0.0
678 0.0
679 0.0
680 0.0
681 0.0
682 0.0
683 0.0
684 0.0
685 0.0
686 0.0
687 0.0
688 0.0
689 0.0
690 0.0
691 0.0
692 0.0
693 0.0
694 0.0
695 0.0
696 0.0
697 0.0
698 0.0
699 0.0
700 0.0
701 0.0
702 0.0
703 0.0
704 0.0
705 0.0
706 0.0
707 0.0
708 0.0
709 0.0
710 0.0
711 0.0
712 0.0
713 0.0
714 0.0
715 0.0
716 0.0
717 0.0
718 0.0
719 0.0
720 0.0
721 0.0
722 0.0
723 0.0
724 0.0
725 0.0
726 0.0
727 0.0
728 0.0
729 0.0
730 0.0
731 0.0
732 0.0
733 0.0
734 0.0
735 0.0
736 0.0
737 0.0
738 0.0
739 0.0
740 0.0
741 0.0
742 0.0
743 0.0
744 0.0
745 0.0
746 0.0
747 0.0
748 0.0
749 0.0
750 0.0
751 0.0
752 0.0
753 0.0
754 0.0
755 0.0
756 0.0
757 0.0
758 0.0
759 0.0
760 0.0
761 0.0
762 0.0
763 0.0
764 0.0
765 0.0
766 0.0
767 0.0
768 0.0
769 0.0
770 0.0
771 0.0
772 0.0
773 0.0
774 0.0
775 0.0
776 0.0
777 0.0
778 0.0
779 0.0
780 0.0
781 0.0
782 0.0
783 0.0
784 0.0
785 0.0
786 0.0
787 0.0
788 0.0
789 0.0
790 0.0
791 0.0
792 0.0
793 0.0
794 0.0
795 0.0
796 0.0
797 0.0
798 0.0
799 0.0
800 0.0
801 0.0
802 0.0
803 0.0
804 0.0
805 0.0
806 0.0
807 0.0
808 0.0
809 0.0
810 0.0
811 0.0
812 0.0
813 0.0
814 0.0
815 0.0
816 0.0
817 0.0
818 0.0
819 0.0
820 0.0
821 0.0
822 0.0
823 0.0
824 0.0
825 0.0
826 0.0
827 0.0
828 0.0
829 0.0
830 0.0
831 0.0
832 0.0
833 0.0
834 0.0
835 0.0
836 0.0
837 0.0
838 0.0
839 0.0
840 0.0
841 0.0
842 0.0
843 0.0
844 0.0
845 0.0
846 0.0
847 0.0
848 0.0
849 0.0
850 0.0
851 0.0
852 0.0
853 0.0
854 0.0
855 0.0
856 0.0
857 0.0
858 0.0
859 0.0
860 0.0
861 0.0
862 0.0
863 0.0
864 0.0
865 0.0
866 0.0
867 0.0
868 0.0
869 0.0
870 0.0
871 0.0
872 0.0
873 0.0
874 0.0
875 0.0
876 0.0
877 0.0
878 0.0
879 0.0
880 0.0
881 0.0
882 0.0
883 0.0
884 0.0
885 0.0
886 0.0
887 0.0
888 0.0
889 0.0
890 0.0
891 0.0
892 0.0
893 0.0
894 0.0
895 0.0
896 0.0
897 0.0
898 0.0
899 0.0
900 0.0
901 0.0
902 0.0
903 0.0
904 0.0
905 0.0
906 0.0
907 0.0
908 0.0
909 0.0
910 0.0
911 0.0
912 0.0
913 0.0
914 0.0
915 0.0
916 0.0
917 0.0
918 0.0
919 0.0
920 0.0
921 0.0
922 0.0
923 0.0
924 0.0
925 0.0
926 0.0
927 0.0
928 0.0
929 0.0
930 0.0
931 0.0
932 0.0
933 0.0
934 0.0
935 0.0
936 0.0
937 0.0
938 0.0
939 0.0
940 0.0
941 0.0
942 0.0
943 0.0
944 0.0
945 0.0
946 0.0
947 0.0
948 0.0
949 0.0
950 0.0
951 0.0
952 0.0
953 0.0
954 0.0
955 0.0
956 0.0
957 0.0
958 0.0
959 0.0
960 0.0
961 0.0
962 0.0
963 0.0
964 0.0
965 0.0
966 0.0
967 0.0
968 0.0
969 0.0
970 0.0
971 0.0
972 0.0
973 0.0
974 0.0
975 0.0
976 0.0
977 0.0
978 0.0
979 0.0
980 0.0
981 0.0
982 0.0
983 0.0
984 0.0
985 0.0
986 0.0
987 0.0
988 0.0
989 0.0
990 0.0
991 0.0
992 0.0
993 0.0
994 0.0
995 0.0
996 0.0
997 0.0
998 0.0
999 0.0
1000 0.0
1001 0.0
1002 0.0
1003 0.0
1004 0.0
1005 0.0
1006 0.0
1007 0.0
1008 0.0
1009 0.0
1010 0.0
1011 0.0
1012 0.0
1013 0.0
1014 0.0
1015 0.0
1016 0.0
1017 0.0
1018 0.0
1019 0.0
1020 0.0
1021 0.0
1022 0.0
1023 0.0
1024 0.0
1025 0.0
1026 0.0
1027 0.0
1028 0.0
1029 0.0
1030 0.0
1031 0.0
1032 0.0
1033 0.0
1034 0.0
1035 0.0
1036 0.0
1037 0.0
1038 0.0
1039 0.0
1040 0.0
1041 0.0
1042 0.0
1043 0.0
1044 0.0
1045 0.0
1046 0.0
1047 0.0
1048 0.0
1049 0.0
1050 0.0
1051 0.0
1052 0.0
1053 0.0
1054 0.0
1055 0.0
1056 0.0
1057 0.0
1058 0.0
1059 0.0
1060 0.0
1061 0.0
1062 0.0
1063 0.0
1064 0.0
1065 0.0
1066 0.0
1067 0.0
1068 0.0
1069 0.0
1070 0.0
1071 0.0
1072 0.0
1073 0.0
1074 0.0
1075 0.0
1076 0.0
1077 0.0
1078 0.0
1079 0.0
1080 0.0
1081 0.0
1082 0.0
1083 0.0
1084 0.0
1085 0.0
1086 0.0
1087 0.0
1088 0.0
1089 0.0
1090 0.0
1091 0.0
1092 0.0
1093 0.0
1094 0.0
1095 0.0
1096 0.0
1097 0.0
1098 0.0
1099 0.0
1100 0.0
1101 0.0
1102 0.0
1103 0.0
1104 0.0
1105 0.0
1106 0.0
1107 0.0
1108 0.0
1109 0.0
1110 0.0
1111 0.0
1112 0.0
1113 0.0
1114 0.0
1115 0.0
1116 0.0
1117 0.0
1118 0.0
1119 0.0
1120 0.0
1121 0.0
1122 0.0
1123 0.0
1124 0.0
1125 0.0
1126 0.0
1127 0.0
1128 0.0
1129 0.0
1130 0.0
1131 0.0
1132 0.0
1133 0.0
1134 0.0
1135 0.0
1136 0.0
1137 0.0
1138 0.0
1139 0.0
1140 0.0
1141 0.0
1142 0.0
1143 0.0
1144 0.0
1145 0.0
1146 0.0
1147 0.0
1148 0.0
1149 0.0
1150 0.0
1151 0.0
1152 0.0
1153 0.0
1154 0.0
1155 0.0
1156 0.0
1157 0.0
1158 0.0
1159 0.0
1160 0.0
1161 0.0
1162 0.0
1163 0.0
1164 0.0
1165 0.0
1166 0.0
1167 0.0
1168 0.0
1169 0.0
1170 0.0
1171 0.0
1172 0.0
1173 0.0
1174 0.0
1175 0.0
1176 0.0
1177 0.0
1178 0.0
1179 0.0
1180 0.0
1181 0.0
1182 0.0
1183 0.0
1184 0.0
1185 0.0
1186 0.0
1187 0.0
1188 0.0
1189 0.0
1190 0.0
1191 0.0
1192 0.0
1193 0.0
1194 0.0
1195 0.0
1196 0.0
1197 0.0
1198 0.0
1199 0.0
1200 0.0
1201 0.0
1202 0.0
1203 0.0
1204 0.0
1205 0.0
1206 0.0
1207 0.0
1208 0.0
1209 0.0
1210 0.0
1211 0.0
1212 0.0
1213 0.0
1214 0.0
1215 0.0
1216 0.0
1217 0.0
1218 0.0
1219 0.0
1220 0.0
1221 0.0
1222 0.0
1223 0.0
1224 0.0
1225 0.0
1226 0.0
1227 0.0
1228 0.0
1229 0.0
1230 0.0
1231 0.0
1232 0.0
1233 0.0
1234 0.0
1235 0.0
1236 0.0
1237 0.0
1238 0.0
1239 0.0
1240 0.0
1241 0.0
1242 0.0
1243 0.0
1244 0.0
1245 0.0
1246 0.0
1247 0.0
1248 0.0
1249 0.0
1250 0.0
1251 0.0
1252 0.0
1253 0.0
1254 0.0
1255 0.0
1256 0.0
1257 0.0
1258 0.0
1259 0.0
1260 0.0
1261 0.0
1262 0.0
1263 0.0
1264 0.0
1265 0.0
1266 0.0
1267 0.0
1268 0.0
1269 0.0
1270 0.0
1271 0.0
1272 0.0
1273 0.0
1274 0.0
1275 0.0
1276 0.0
1277 0.0
1278 0.0
1279 0.0
1280 0.0
1281 0.0
1282 0.0
1283 0.0
1284 0.0
1285 0.0
1286 0.0
1287 0.0
1288 0.0
1289 0.0
1290 0.0
1291 0.0
1292 0.0
1293 0.0
1294 0.0
1295 0.0
1296 0.0
1297 0.0
1298 0.0
1299 0.0
1300 0.0
1301 0.0
1302 0.0
1303 0.0
1304 0.0
1305 0.0
1306 0.0
1307 0.0
1308 0.0
1309 0.0
1310 0.0
1311 0.0
1312 0.0
1313 0.0
1314 0.0
1315 0.0
1316 0.0
1317 0.0
1318 0.0
1319 0.0
1320 0.0
1321 0.0
1322 0.0
1323 0.0
1324 0.0
1325 0.0
1326 0.0
1327 0.0
1328 0.0
1329 0.0
1330 0.0
1331 0.0
1332 0.0
1333 0.0
1334 0.0
1335 0.0
1336 0.0
1337 0.0
1338 0.0
1339 0.0
1340 0.0
1341 0.0
1342 0.0
1343 0.0
1344 0.0
1345 0.0
1346 0.0
1347 0.0
1348 0.0
1349 0.0
1350 0.0
1351 0.0
1352 0.0
1353 0.0
1354 0.0
1355 0.0
1356 0.0
1357 0.0
1358 0.0
1359 0.0
1360 0.0
1361 0.0
1362 0.0
1363 0.0
1364 0.0
1365 0.0
1366 0.0
1367 0.0
1368 0.0
1369 0.0
1370 0.0
1371 0.0
1372 0.0
1373 0.0
1374 0.0
1375 0.0
1376 0.0
1377 0.0
1378 0.0
1379 0.0
1380 0.0
1381 0.0
1382 0.0
1383 0.0
1384 0.0
1385 0.0
1386 0.0
1387 0.0
1388 0.0
1389 0.0
1390 0.0
1391 0.0
1392 0.0
1393 0.0
1394 0.0
1395 0.0
1396 0.0
1397 0.0
1398 0.0
1399 0.0
1400 0.0
1401 0.0
1402 0.0
1403 0.0
1404 0.0
1405 0.0
1406 0.0
1407 0.0
1408 0.0
1409 0.0
1410 0.0
1411 0.0
1412 0.0
1413 0.0
1414 0.0
1415 0.0
1416 0.0
1417 0.0
1418 0.0
1419 0.0
1420 0.0
1421 0.0
1422 0.0
1423 0.0
1424 0.0
1425 0.0
1426 0.0
1427 0.0
1428 0.0
1429 0.0
1430 0.0
1431 0.0
1432 0.0
1433 0.0
1434 0.0
1435 0.0
1436 0.0
1437 0.0
1438 0.0
1439 0.0
1440 0.0
1441 0.0
1442 0.0
1443 0.0
1444 0.0
1445 0.0
1446 0.0
1447 0.0
1448 0.0
1449 0.0
1450 0.0
1451 0.0
1452 0.0
1453 0.0
1454 0.0
1455 0.0
1456 0.0
1457 0.0
1458 0.0
1459 0.0
1460 0.0
1461 0.0
1462 0.0
1463 0.0
1464 0.0
1465 0.0
1466 0.0
1467 0.0
1468 0.0
1469 0.0
1470 0.0
1471 0.0
1472 0.0
1473 0.0
1474 0.0
1475 0.0
1476 0.0
1477 0.0
1478 0.0
1479 0.0
1480 0.0
1481 0.0
1482 0.0
1483 0.0
1484 0.0
1485 0.0
1486 0.0
1487 0.0
1488 0.0
1489 0.0
1490 0.0
1491 0.0
1492 0.0
1493 0.0
1494 0.0
1495 0.0
1496 0.0
1497 0.0
1498 0.0
1499 0.0
1500 0.0
1501 0.0
1502 0.0
1503 0.0
1504 0.0
1505 0.0
1506 0.0
1507 0.0
1508 0.0
1509 0.0
1510 0.0
1511 0.0
1512 0.0
1513 0.0
1514 0.0
1515 0.0
1516 0.0
1517 0.0
1518 0.0
1519 0.0
1520 0.0
1521 0.0
1522 0.0
1523 0.0
1524 0.0
1525 0.0
1526 0.0
1527 0.0
1528 0.0
1529 0.0
1530 0.0
1531 0.0
1532 0.0
1533 0.0
1534 0.0
1535 0.0
1536 0.0
1537 0.0
1538 0.0
1539 0.0
1540 0.0
1541 0.0
1542 0.0
1543 0.0
1544 0.0
1545 0.0
1546 0.0
1547 0.0
1548 0.0
1549 0.0
1550 0.0
1551 0.0
1552 0.0
1553 0.0
1554 0.0
1555 0.0
1556 0.0
1557 0.0
1558 0.0
1559 0.0
1560 0.0
1561 0.0
1562 0.0
1563 0.0
1564 0.0
1565 0.0
1566 0.0
1567 0.0
1568 0.0
1569 0.0
1570 0.0
1571 0.0
1572 0.0
1573 0.0
1574 0.0
1575 0.0
1576 0.0
1577 0.0
1578 0.0
1579 0.0
1580 0.0
1581 0.0
1582 0.0
1583 0.0
1584 0.0
1585 0.0
1586 0.0
1587 0.0
1588 0.0
1589 0.0
1590 0.0
1591 0.0
1592 0.0
1593 0.0
1594 0.0
1595 0.0
1596 0.0
1597 0.0
1598 0.0
1599 0.0
1600 0.0
1601 0.0
1602 0.0
1603 0.0
1604 0.0
1605 0.0
1606 0.0
1607 0.0
1608 0.0
1609 0.0
1610 0.0
1611 0.0
1612 0.0
1613 0.0
1614 0.0
1615 0.0
1616 0.0
1617 0.0
1618 0.0
1619 0.0
1620 0.0
1621 0.0
1622 0.0
1623 0.0
1624 0.0
1625 0.0
1626 0.0
1627 0.0
1628 0.0;


######################
# SOLVE 
######################
end;
